Function  GetDefaultEnvironmentVariables() : GetDefaultEnvironmentVariables = Array( _
	"RGA_ProgramFolder",  GetFullPath( ".", Empty ), _
	"Tools",  "scriptlib\Tools", _
_
Empty ) : End Function


Sub  Main( Opt, AppKey )
	SetDefaultEnvironmentVariables
	g_Copyright = "Copyright(c) 2012-2014 Renesas Electronics Corporation"
	Set o = new InputCommandOpt
		o.Lead = "RGA Tools - "+ g_Copyright
		Set o.CommandReplace = Dict(Array( _
			"1","RunImagePackager",_
			"2","SearchErrorInformation",_
			"3","ConvertBin",_
			_
			"BinToSRec", "BinToSRec_sth",_
			"RunImagePackager", "RunImagePackager_sth",_
			"RunBinaryGenerator", "RunBinaryGenerator_sth",_
			"SearchErrorInformation", "SearchErrorInformation_sth",_
			"SRecToBin", "SRecToBin_sth",_
			"UpdateErrorInformation", "UpdateErrorInformation" ))
		Set o.MenuCaption = Dict(Array(_
			"1","Convert image format [%name%]",_
			"2","Search error information [%name%]",_
			"3","Convert to binary [%name%]" ))
	InputCommand  o, Empty, Opt, AppKey
End Sub

Dim  g_Copyright


 
'********************************************************************************
'  <<< [RunImagePackager_sth] >>> 
'********************************************************************************
Sub  RunImagePackager_sth( Opt, AppKey )
	Set c = g_VBS_Lib


	'// Create "packager" as "ImagePackagerClass"
	If IsEmpty( g_ImagePackagerLib_vbs ) Then _
		include  env("%Tools%\ImagePackager\ImagePackagerLib.vbs")
	Set packager = new ImagePackagerClass


	'// User interface
	echo  ""
	echo  "Renesas Image Packager - "+ g_Copyright
	echo  "Make one binary file from many image files and other files."
	echo  "Enter only : Open sample folder."
	Do
		input_path = InputPath( "Setting file(ImagePackagerConfig.xml) >", c.CheckFileExists or c.AllowEnterOnly )
		If input_path <> "" Then  Exit Do

		Setting_openFolder  packager.RootFolderPath +"\Sample"
	Loop
	input_folder = g_fs.GetParentFolderName( input_path )
	echo  ""


	'// Call "NewWritable"
	packager.LoadConfig  input_path
	Dim writables : ReDim  writables( 2 + packager.OutputBinaryPaths.Count + packager.OutputHeaderPaths.Count - 1 )
	writables(0) = input_path
	writables(1) = packager.WorkFolderPath
	AddArrElem  writables, packager.OutputBinaryPaths
	AddArrElem  writables, packager.OutputHeaderPaths
	Set w_= AppKey.NewWritable( writables ).Enable()


	'// Call "RunImagePackager"
	packager.Run  input_path


	'// Finish message
	echo  ""
	For i=0 To packager.OutputBinaryPaths.UBound_
		echo  "Binary: "+ GetStepPath( packager.OutputBinaryPaths(i), input_folder )
	Next
	For i=0 To packager.OutputHeaderPaths.UBound_
		echo  "Header: "+ GetStepPath( packager.OutputHeaderPaths(i), input_folder )
	Next
End Sub


 
'********************************************************************************
'  <<< [RunBinaryGenerator_sth] >>> 
'********************************************************************************
Sub  RunBinaryGenerator_sth( Opt, AppKey )
	Set c = g_VBS_Lib
	If not IsDefined( "RunBinaryGenerator" ) Then _
		include  env("%Tools%\BinaryGenerator\BinaryGeneratorLib.vbs")

	Set work = new RunBinaryGeneratorWork

	echo  "Make one binary file from many image files and other files."
	echo  "Enter only : Open sample folder."
	Do
		input_path = InputPath( "Setting file(BinaryGenerateConfig.xml) >", c.CheckFileExists or c.AllowEnterOnly )
		If input_path <> "" Then  Exit Do

		Setting_openFolder  work.ToolFolderPath +"\Sample_PC"
	Loop
	echo  ""


	'// Call "NewWritable"
	work.LoadConfig  input_path
	Dim  writables : ReDim  writables( 2 + UBound( work.OutputBinaryPaths )  )
	writables(0) = work.UnzipedFolderPath
	writables(1) = work.OutputHeaderPath
	AddArrElem  writables, work.OutputBinaryPaths
	Set w_= AppKey.NewWritable( writables ).Enable()


	'// Change directory
	Set ec = new EchoOff
	cd  g_start_in_path
	ec = Empty


	'// Call "RunBinaryGenerator"
	RunBinaryGenerator  input_path


	'// Finish message
	echo  ""
	For i=0 To UBound( work.OutputBinaryPaths )
		echo  "Binary: "+ GetStepPath( work.OutputBinaryPaths(i), Empty )
	Next
	echo  "Header: "+ GetStepPath( work.OutputHeaderPath, Empty )
End Sub


 
'********************************************************************************
'  <<< [SearchErrorInformation_sth] >>> 
'********************************************************************************
Sub  SearchErrorInformation_sth( Opt, AppKey )
	path = Input( "Path or file name raising error >" )
	Set jumps = GetTagJumpParams( path )
	If IsEmpty( jumps.LineNum ) Then
		line_num = CInt2( Input( "Line number raising error >" ) )
	Else
		path = jumps.Path
		line_num = jumps.LineNum
	End If

	If not IsDefined( "get_ErrorInformationClass" ) Then _
		include  env("%Tools%\ErrorInformation\ErrorInformationLib.vbs")
	Set info = get_ErrorInformationClass()
	Set ec = new EchoOff
	If Right( path, 4 ) = ".c.h" Then _
		path = Left( path, Len( path ) - 2 )  '// "*.c.h" -> "*.c"
	info.Search  path, line_num
	ec = Empty
	echo  info.SearchResultString
End Sub


 
'********************************************************************************
'  <<< [ConvertBin] >>> 
'********************************************************************************
Sub  ConvertBin( Opt, AppKey )
	echo  "1. Binary to C language [BinToC]"
	echo  "2. Binary to S-record format [BinToSRec]"
	echo  "3. S-record format to binary [SRecToBin]"
	key = Input( "Number or command >" )
	Select Case  key
		Case "1" : key = "BinToC"
		Case "2" : key = "BinToSRec"
		Case "3" : key = "SRecToBin"
	End Select

	Select Case  LCase( key )
		Case "bintoc"    : GenerateCSourceFromBinaryFile_sth  Opt, AppKey
		Case "bintosrec" : BinToSRec_sth  Opt, AppKey
		Case "srectobin" : SRecToBin_sth  Opt, AppKey
		Case Else  echo_v  "ERROR: Unknown command"
	End Select
End Sub


 
'********************************************************************************
'  <<< [GenerateCSourceFromBinaryFile_sth] >>> 
'********************************************************************************
Sub  GenerateCSourceFromBinaryFile_sth( Opt, AppKey )
	Set c = g_VBS_Lib
	echo  "Converts from binary file to C language source."
	binary_path = InputPath( "Path of binary file >", c.CheckFileExists )
	source_path = InputPath( "Path of output source file >", Empty )
	header_path = InputPath( "Path of output header file >", Empty )
	name = Input( "Variable name >" )

	Set w_=AppKey.NewWritable( Array( source_path, header_path ) ).Enable()

	GenerateCSourceFromBinaryFile  binary_path,  name,  source_path,  header_path

	echo  "Done output."
End Sub


 
'********************************************************************************
'  <<< [SRecToBin_sth] >>> 
'********************************************************************************
Sub  SRecToBin_sth( Opt, AppKey )
	Set c = g_VBS_Lib

	echo  "Converts from S-record format file to binary file."
	input_path = InputPath( "Path of input S-record format file >", c.CheckFileExists )

	echo  ""
	echo  "Enter only : the file name changed to bin in the same folder"
	output_path = InputPath( "Path of output binary file (#overwriting) >", c.AllowEnterOnly )
	If output_path = "" Then
		output_path = AddLastOfFileName( input_path, ".bin" )
	End  If

	Set w_= AppKey.NewWritable( output_path ).Enable()

	SRecToBin  input_path, output_path, Empty

	echo  "Done output."
End Sub


 
'********************************************************************************
'  <<< [BinToSRec_sth] >>> 
'********************************************************************************
Sub  BinToSRec_sth( Opt, AppKey )
	Set c = g_VBS_Lib

	echo  "Converts from binary file to S-record format file."
	input_path = InputPath( "Path of input binary file >", c.CheckFileExists )

	echo  ""
	echo  "Enter only : the file name changed to srec in the same folder"
	output_path = InputPath( "Path of output S-record format file (#overwriting) >", c.AllowEnterOnly )
	If output_path = "" Then
		output_path = AddLastOfFileName( input_path, ".srec" )
	End  If

	comment = Input( "Comment at first line >" )
	load_address = Input( "Load address (e.g. 00002F00)>" )
	execute_address = Input( "Execute address (0=Data)>" )

	Set w_= AppKey.NewWritable( output_path ).Enable()

	BinToSRec  input_path, output_path, comment, load_address, execute_address, Empty

	echo  "Done output."
End Sub


 







'--- start of vbslib include ------------------------------------------------------ 

'// Calling "Main" function from inside of here.
'// And searching "scriptlib" folder and include vbslib.

'// vbslib is provided under 3-clause BSD license.
'// Copyright (C) 2007-2014 Sofrware Design Gallery "Sage Plaisir 21" All Rights Reserved.

Dim  g_Vers : If IsEmpty( g_Vers ) Then
Set  g_Vers = CreateObject("Scripting.Dictionary") : g_Vers("vbslib") = 99.99
Dim  g_debug, g_debug_params, g_admin, g_vbslib_path, g_CommandPrompt, g_fs, g_sh, g_AppKey
Dim  g_MainPath, g_SrcPath, g_f, g_include_path, g_i, g_debug_tree, g_debug_process, g_is_compile_debug
Dim  g_is64bitWSH
g_SrcPath = WScript.ScriptFullName : g_MainPath = g_SrcPath
SetupVbslibParameters
Set  g_fs = CreateObject( "Scripting.FileSystemObject" )
Set  g_sh = WScript.CreateObject("WScript.Shell") : g_f = g_sh.CurrentDirectory
g_sh.CurrentDirectory = g_fs.GetParentFolderName( WScript.ScriptFullName )
For g_i = 20 To 1 Step -1 : If g_fs.FileExists(g_vbslib_path) Then  Exit For
g_vbslib_path = "..\" + g_vbslib_path  : Next
If g_fs.FileExists(g_vbslib_path) Then  g_vbslib_path = g_fs.GetAbsolutePathName( g_vbslib_path )
g_sh.CurrentDirectory = g_f
If g_i=0 Then WScript.Echo "Not found " + g_fs.GetFileName( g_vbslib_path ) +vbCR+vbLF+_
	"Let's download vbslib and Copy scriptlib folder." : Stop : WScript.Quit 1
Set g_f = g_fs.OpenTextFile( g_vbslib_path,,,-2 ): Execute g_f.ReadAll() : g_f = Empty
If ResumePush Then  On Error Resume Next
	CallMainFromVbsLib
ResumePop : On Error GoTo 0
End If
'---------------------------------------------------------------------------------

Sub  SetupDebugTools()
	set_input  ""
	SetBreakByFName  Empty
	SetStartSectionTree  ""
End Sub

Sub  SetupVbslibParameters()
	'--- start of parameters for vbslib include -------------------------------
	'// g_Vers("OldMain") = 1
	g_vbslib_path = "scriptlib\vbs_inc.vbs"
	g_CommandPrompt = 1

	g_debug = 0   '// release:0, debug:99, -1:call SetupDebugTools
	              '// You can debug, if g_debug = 1 and installed MS Office or Visual Studio
	              '// Write "Stop" at start of step running or the place of watching the value of variable.

	'--- end of parameters for vbslib include ---------------------------------
End Sub
'--- end of vbslib include --------------------------------------------------------


 
