'*******************************************************************************
'* DISCLAIMER
'* This software is supplied by Renesas Electronics Corporation and is only
'* intended for use with Renesas products. No other uses are authorized. This
'* software is owned by Renesas Electronics Corporation and is protected under
'* all applicable laws, including copyright laws.
'* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
'* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
'* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
'* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
'* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
'* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
'* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
'* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
'* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
'* Renesas reserves the right, without notice, to make changes to this software
'* and to discontinue the availability of this software. By using this software,
'* you agree to the additional terms and conditions found by accessing the
'* following link:
'* http://www.renesas.com/disclaimer
'* Copyright (C) 2012-2014 Renesas Electronics Corporation. All rights reserved.
'*******************************************************************************

Dim  g_SrcPath
If IsEmpty( g_SrcPath ) Then  MsgBox "This is vbslib library file." : WScript.Quit
Dim  g_ImagePackagerLib_vbs : get_DefineInfoObject  g_ImagePackagerLib_vbs, g_SrcPath


 
'-------------------------------------------------------------------------
' ### <<<< [ImagePackagerClass] >>>> 
'-------------------------------------------------------------------------
Class  ImagePackagerClass
	Public  RootFolderPath
	Public  WorkFolderPath
	Public  ImageConvertTool_exe
	Public  LoadedConfigFilePath
	Public  OutputBinaryPaths  '// ArrayClass of string
	Public  OutputHeaderPaths  '// ArrayClass of string
	Public  XML_Root
	Public  LinkedXML  '// LinkedXMLs
	Public  Variables  '// LazyDictionaryClass


Sub  Class_Initialize()
	Me.RootFolderPath  = g_fs.GetParentFolderName( g_ImagePackagerLib_vbs.FullPath )
	Me.WorkFolderPath  = Me.RootFolderPath +"\work"
	Me.ImageConvertTool_exe = Me.RootFolderPath +"\ImageConvertTool.exe"
	Set Me.LinkedXML = new LinkedXMLs
	Me.LinkedXML.XmlTagNamesHavingIdName = Array( "SuperClass", "SourceTemplate" )
End Sub


 
'********************************************************************************
'  <<< [ImagePackagerClass::LoadConfig] >>> 
'********************************************************************************
Sub  LoadConfig( ConfigFilePath )

	Set xml_root = LoadXML( ConfigFilePath, Empty )
	base_path = GetParentFullPath( ConfigFilePath )
	Me.LoadedConfigFilePath = GetFullPath( ConfigFilePath, Empty )


	'// Set "Me.OutputBinaryPaths" for set writable folders
	Set Me.OutputBinaryPaths = new ArrayClass
	For Each output_bin_tag  In xml_root.selectNodes( "./OutputBinary" )

		value = output_bin_tag.getAttribute( "path" )
		If IsNull( value ) Then  Me.RaiseError _
			Array( "XPath", "/ImagePackager/OutputBinary/@path" )
		Me.OutputBinaryPaths.Add  GetFullPath( value, base_path )
	Next


	'// Set "Me.OutputHeaderPaths" for set writable folders
	Set Me.OutputHeaderPaths = new ArrayClass
	For Each output_header_tag  In xml_root.selectNodes( "./OutputHeader" )

		value = output_header_tag.getAttribute( "path" )
		If IsNull( value ) Then  Me.RaiseError _
			Array( "XPath", "/ImagePackager/OutputHeader/@path" )
		Me.OutputHeaderPaths.Add  GetFullPath( value, base_path )
	Next
	For Each output_header_tag  In xml_root.selectNodes( "./OutputBinary/OutputHeader" )

		value = output_header_tag.getAttribute( "path" )
		If IsNull( value ) Then  Me.RaiseError _
			Array( "XPath", "/ImagePackager/OutputBinary/OutputHeader/@path" )
		Me.OutputHeaderPaths.Add  GetFullPath( value, base_path )
	Next
End Sub


 
'********************************************************************************
'  <<< [ImagePackagerClass::Run] >>> 
'********************************************************************************
Sub  Run( ConfigFilePath )
	Set ds = new CurDirStack
	Set c = g_VBS_Lib
	Set fin = new FinObj : fin.SetFunc "ImagePackagerClass_run_Finally"
	fin.SetVar "Me", Me

	echo  g_sh.CurrentDirectory +">RunImagePackager  """+ ConfigFilePath +""""
	Set ec = new EchoOff

	If IsEmpty( Me.LoadedConfigFilePath ) Then _
		Me.LoadConfig  ConfigFilePath


	'// Clean work folder
	del  Me.WorkFolderPath


	'// Read "ConfigFilePath"
	Set Me.XML_Root = LoadXML( ConfigFilePath, Empty )
	base_path = GetParentFullPath( ConfigFilePath )


	'// ...
	Set Me.Variables = new_LazyDictionaryClass( Me.XML_Root )
	Me.Variables( "${ImagePackagerLib}" ) = g_fs.GetParentFolderName( g_ImagePackagerLib_vbs.FullPath )


	'// Set "input_base_folder"
	value = XmlRead( Me.XML_Root, "./InputFiles/@base_folder" )
	If IsEmpty( value ) Then  value = "."
	input_base_folder = GetFullPath( value, GetParentFullPath( ConfigFilePath ) )


	output_count = 0
	For Each output_bin_tag  In Me.XML_Root.selectNodes( "./OutputBinary" )


		'// Set "output_header_tag"
		Set output_header_tag = output_bin_tag.selectSingleNode( "./OutputHeader" )
		If output_header_tag is Nothing Then

			Set output_header_tag = Me.XML_Root.selectNodes( "./OutputHeader" )
			If output_header_tag.Length >= 2 Then _
				echo_v  "<Warning msg=""Multi /ImagePackager/OutputHeader was moved to "+_
					"/ImagePackager/OutputBinary/OutputHeader""/>"

			Set output_header_tag = Me.XML_Root.selectSingleNode( "./OutputHeader" )
			If output_header_tag is Nothing Then
				Me.RaiseError  Array( "XPath", "/ImagePackager/OutputBinary/OutputHeader" )
			End If

			If IsEmpty( same_offset_checker ) Then
				Set same_offset_checker = new ArrayClass
				Assert  output_count = 0
			End If
		End If


		'// Set "include_define"
		include_define = output_header_tag.getAttribute( "include_define" )
		If IsNull( include_define ) Then  include_define = "__BINARY_IMAGE__"
		Me.Variables( "${IncludeDefine}" ) = include_define


		'// Set "output_binary"

		Set output_binary = new OutputBinaryClass


		'// Set "super_class_ID"
		super_class_ID = output_bin_tag.getAttribute( "super_class" )
		If not IsNull( super_class_ID ) Then

			If InStr( super_class_ID, "#" ) = 0 Then
				super_class_ID = "#"+ super_class_ID
			End If
			super_class_ID = Me.Variables( super_class_ID )

			'// Set "super_class"
			Me.LinkedXML.StartNavigation  Me.LoadedConfigFilePath, Me.XML_Root
			Set super_class = Me.LinkedXML.GetLinkTargetNode( super_class_ID )
			Me.LinkedXML.EndNavigation
			If super_class is Nothing Then _
				Me.RaiseError  Array( "XPath", xpath )

			Set output_bin_2_tag = super_class.selectSingleNode( "OutputBinary" )
		Else
			Set output_bin_2_tag = Nothing
		End If
		If output_bin_2_tag is Nothing Then
			output_binary.Endian = Null
			output_binary.RawImageAlignment = Null
			output_binary.RawStrideAlignment = Null
			output_binary.RawStrideAlignment4_PixelFormats = Null
		Else
			output_binary.Endian = output_bin_2_tag.getAttribute( "endian" )
			output_binary.RawImageAlignment = output_bin_2_tag.getAttribute( "raw_image_alignment" )
			output_binary.RawStrideAlignment = output_bin_2_tag.getAttribute( "raw_stride_alignment" )
			output_binary.RawStrideAlignment4_PixelFormats = output_bin_2_tag.getAttribute( "raw_stride_alignment_4" )
		End If


		If IsNull( output_binary.Endian ) Then _
			output_binary.Endian = output_bin_tag.getAttribute( "endian" )
		If IsNull( output_binary.Endian ) Then  output_binary.Endian = "LittleEndian"

		value = output_binary.RawImageAlignment
		If IsNull( value ) Then _
			value = output_bin_tag.getAttribute( "raw_image_alignment" )
		If IsNull( value ) Then  value = 4
		value = Replace( value, "0x", "&h" )
		output_binary.RawImageAlignment = value

		value = output_bin_tag.getAttribute( "raw_image_alignment_symbol" )
		If not IsNull( value ) Then
			raw_image_alignment_symbol = value
		End If

		value = output_binary.RawStrideAlignment
		If IsNull( value ) Then _
			value = output_bin_tag.getAttribute( "raw_stride_alignment" )
		If IsNull( value ) Then  value = 1
		output_binary.RawStrideAlignment = value

		value = output_binary.RawStrideAlignment4_PixelFormats
		If IsNull( value ) Then _
			value = output_bin_tag.getAttribute( "raw_stride_alignment_4" )
		If IsNull( value ) Then  value = ""
		output_binary.RawStrideAlignment4_PixelFormats = value

		value = output_bin_tag.getAttribute( "alpha_raw_image_width" )
		If IsNull( value ) Then _
			value = Empty
		output_binary.AlphaRawImageWidth = value

		If not IsBaseOfALogarithm2( CLng( output_binary.RawImageAlignment ) ) Then
			Me.RaiseError  Array( "NotBaseOfALogarithm2", "/ImagePackager/OutputBinary/@raw_image_alignment", _
				output_binary.RawImageAlignment )
		End If

		value = output_bin_tag.getAttribute( "language" )
		If IsNull( value ) Then  value = "Binary"
		output_binary.Language = value

		value = output_bin_tag.getAttribute( "table_format" )
		If IsNull( value ) Then
			If output_binary.Language = "Binary" Then
				value = "Offset"
			Else
				value = "Embed"
			End If
		End If
		output_binary.TableFormat = value


		'// Set "variables_file"
		Set variables_file = new StringStream


		'// Append from InputFiles to "output_binary.BinaryParts"
		'// Loop of "/ImagePackager/InputFiles/Image" in XML
		files_text = ""
		input_file_num = 1
		For Each image_tag  In  Me.XML_Root.selectNodes( "./InputFiles/Image | ./InputFiles/File | ./InputFiles/Var" )

			If image_tag.tagName = "Var" Then
				Set input_file = new InputFileClass
				Set o = input_file
					o.InputFullPath = input_base_folder
					Set o.ImageTag = image_tag
				o = Empty
				Me.RunByInputPath  input_file, output_binary
			Else
				path = image_tag.getAttribute( "path" )
				input_abs_path = GetFullPath( path, input_base_folder )
				If not IsWildcard( path ) Then
					AssertExist  input_abs_path
					If g_fs.FolderExists( input_abs_path ) Then _
						input_abs_path = input_abs_path + "\*"
				End If
				ExpandWildcard  input_abs_path,_
					c.File or c.SubFolder or c.AbsPath,_
					dummy, input_abs_paths

				For Each input_abs_path  In input_abs_paths

					'// Call "Me.RunByInputPath"
					Set input_file = new InputFileClass
					Set o = input_file
						o.InputFileNum = input_file_num
						o.InputFullPath = input_abs_path
						Set o.ImageTag = image_tag
					o = Empty
					ec = Empty
					Me.RunByInputPath  input_file, output_binary
					Set ec = new EchoOff

					input_file_num = input_file_num + 1
				Next
			End If
		Next


		'// Append "offset" to "image"
		Set image = new BinaryArray
		Set arr   = new BinaryArray
		is_offset_table_format = ( LCase( output_binary.TableFormat ) = "offset" )

		output_binary_offset = 0
		If is_offset_table_format Then
			For Each bin  In output_binary.BinaryParts.Items  '// as BinaryPartClass
				If bin.IsOffsetable Then
					If bin.IsVariableInHeader Then

						'// Set "OffsetOfOffsetOfSelfOffsetInPackage"
						bin.OffsetOfOffsetOfSelfOffsetInPackage = output_binary_offset
						output_binary_offset = output_binary_offset + 4
					End If
				Else
					'// Set "bin_alignment"
					bin_alignment = bin.Alignment
					If IsEmpty( bin_alignment ) Then
						bin_alignment = Me.GetDefaultFileAlignment_sub( output_bin_tag, bin )
					End If

					'// Adjust alignment of the position(offset) in "image"
					If ( output_binary_offset mod bin_alignment ) <> 0 Then _
						output_binary_offset = ( output_binary_offset + bin_alignment - 1 ) and _
							not ( bin_alignment - 1 )

					'// Append a binary to "image"
					image.WriteFromBinaryArray  output_binary_offset, bin.BinaryData, 0, bin.Size
					bin.BinaryData = Empty
					bin.SelfOffsetInPackage = output_binary_offset

					output_binary_offset = output_binary_offset + bin.Size
				End If
			Next
		End If


		'// Append "output_binary.BinaryParts" to "image"
		For Each bin  In output_binary.BinaryParts.Items  '// as BinaryPartClass
			is_output = False

			'// Set "bin_alignment"
			bin_alignment = bin.Alignment
			If IsEmpty( bin_alignment ) Then
				bin_alignment = Me.GetDefaultFileAlignment_sub( output_bin_tag, bin )
			End If

			'// Adjust alignment of the position(offset) in "image"
			If ( output_binary_offset mod bin_alignment ) <> 0 Then _
				output_binary_offset = ( output_binary_offset + bin_alignment - 1 ) and _
					not ( bin_alignment - 1 )

			'// Append a binary to "image"
			If IsEmpty( bin.Path ) Then
				If not IsEmpty( bin.BinaryData ) Then
					image.WriteFromBinaryArray  output_binary_offset, bin.BinaryData, 0, bin.Size
					is_output = True
				End If
			Else
				arr.Load  bin.Path
				image.WriteFromBinaryArray  output_binary_offset, arr, 0, bin.Size
				is_output = True
			End If

			If not IsEmpty( bin.OffsetOfOffsetOfSelfOffsetInPackage ) Then
				offset_member_index = bin.OffsetOfOffsetOfSelfOffsetInPackage
				image.WriteStruct  offset_member_index, Array( vbLong, output_binary_offset )

				If LCase( output_binary.Endian ) = "bigendian" Then
					image.SwapEndian  offset_member_index, 4, 4
				End If
			End If


			'// Next of loop
			If is_output Then
				bin.SelfOffsetInPackage = output_binary_offset
				output_binary_offset = output_binary_offset + bin.Size
			End If


			'// Append a binary to "OutputHeaderFile"
			If not IsEmpty( bin.Symbol ) Then

				If is_offset_table_format  and  bin.IsOffsetable Then
					variables_file.WriteLine  "#define  "+ bin.Symbol +_
						"  ("+ bin.TypeCast +"( &${Symbol}[ *(int32_t*)( ${Symbol} + 0x"+ _
						Right( "0000000"+ Hex( bin.OffsetOfOffsetOfSelfOffsetInPackage ), 8 ) +" ) ] ))"
				Else
					variables_file.WriteLine  "#define  "+ bin.Symbol +_
						"  ("+ bin.TypeCast +"( ${Symbol} + 0x"+ _
						Right( "0000000"+ Hex( bin.SelfOffsetInPackage ), 8 ) +" ))"
				End If
			End If
		Next


		'// Adjust offset data in binaries
		'// e.g. Modify the value of variables of offset in "bin" to the posigion put in the package.
		'// e.g. Modify the offset to Raw data in image header
		For bin_num = 0  To output_binary.BinaryParts.UBound_
			Set bin = output_binary.BinaryParts( bin_num )  '// as BinaryPartClass
			If not IsEmpty( bin.OffsetAdjuster ) Then

				Set target_bin = output_binary.BinaryParts( bin_num + bin.OffsetAdjuster.StepIndexOfTargetBinary )
					'// as BinaryPartClass
					'// Example of "target_bin": next of "bin"
				offset_member_index = bin.SelfOffsetInPackage +_
					bin.OffsetAdjuster.OffsetOfOffsetMemberVariable
				offset = target_bin.SelfOffsetInPackage - bin.SelfOffsetInPackage


				'// Write "offset" at "offset_member_index" in "image"
				image.WriteStruct  offset_member_index, Array( vbLong, offset )

				If LCase( output_binary.Endian ) = "bigendian" Then
					image.SwapEndian  offset_member_index, 4, 4
				End If
			End If
		Next


		'// Check by "same_offset_checker"
		If not IsEmpty( same_offset_checker ) Then
			If output_count = 0 Then
				For bin_num = 0  To output_binary.BinaryParts.UBound_
					Set bin = output_binary.BinaryParts( bin_num )  '// as BinaryPartClass
					same_offset_checker.Add  bin.SelfOffsetInPackage
				Next
			Else
				For bin_num = 0  To output_binary.BinaryParts.UBound_
					Set bin = output_binary.BinaryParts( bin_num )  '// as BinaryPartClass
					If same_offset_checker( bin_num ) <> bin.SelfOffsetInPackage Then
						Me.RaiseError  Array( "MultiOutNotSame" )
					End If
				Next
			End If
		End If


		'// Set "section"
		section = output_bin_tag.getAttribute( "section" )
		If IsNull( section ) Then
			section = Empty
			Me.Variables( "${Section}" ) = ""
		Else
			Me.Variables( "${Section}" ) = section
		End If


		'// Set "bin_image_symbol"
		bin_image_symbol = output_bin_tag.getAttribute( "symbol" )
		If IsNull( bin_image_symbol ) Then  Me.RaiseError _
			Array( "XPath", "/ImagePackager/OutputBinary/@symbol" )
		Me.Variables( "${Symbol}" ) = bin_image_symbol


		'// Set "bin_size", "bin_size_str"
		bin_size = output_binary_offset
		bin_size_str = "0x"+ Right( "0000000"+ Hex( bin_size ), 8 )
		Me.Variables( "${Size}" ) = bin_size_str


		'// Set "src_tmp" : Load "@source_template"
		Set src_tmp = Me.GetSourceTemplate_sub( output_bin_tag, _
			source_template, bin_image_symbol )  '// SourceTemplateClass


		'// Make binary "image" file
		ec = Empty
		echo  "Output """+ GetStepPath( Me.OutputBinaryPaths( output_count ), Empty ) +""""
		Set ec = new EchoOff

		address = output_header_tag.getAttribute( "address" )

		Select Case  LCase( output_binary.Language )
		 Case  "binary"
			image.Save  Me.OutputBinaryPaths( output_count )

		 Case  "c"
			'// Before Data
			temporary_path_1_c = GetTempPath( "ImagePackager_1_*.c" )
			If IsEmpty( section ) Then
				text_1 = src_tmp.Source
			Else
				text_1 = src_tmp.SourceWithSection
			End If

			half_position = InStr( text_1, "${BinaryData}" )
			If half_position = 0 Then  Raise 1, "<ERROR msg=""There is not ${BinaryData}""/>"
			half_position = InStr( text_1, "${BinaryData}"+vbCRLF )
			If half_position = 0 Then  Raise 1, "<ERROR msg=""Insert return after ${BinaryData}""/>"
			text_1 = Left( text_1, half_position - 1 )

			text_1 = Me.Variables( text_1 )
			CreateFile  temporary_path_1_c,  text_1


			'// Data
			temporary_path_2_bin = GetTempPath( "ImagePackager_2_*.bin" )
			image.Save  temporary_path_2_bin

			temporary_path_2_c = GetTempPath( "ImagePackager_2_*.c" )
			temporary_path_2_h = GetTempPath( "ImagePackager_2_*.h" )
			GenerateCSourceFromBinaryFile  temporary_path_2_bin,  Empty,_
				temporary_path_2_c, temporary_path_2_h


			'// After Data
			temporary_path_3_c = GetTempPath( "ImagePackager_3_*.c" )
			If IsEmpty( section ) Then
				text_3 = src_tmp.Source
			Else
				text_3 = src_tmp.SourceWithSection
			End If

			half_position = InStr( text_3, "${BinaryData}"+vbCRLF )
			text_3 = Mid( text_3, half_position + Len( "${BinaryData}"+vbCRLF ) )

			text_3 = Me.Variables( text_3 )
			CreateFile  temporary_path_3_c, text_3


			'// Connect
			log_txt = GetTempPath( "ImagePackager_Log_*.txt" )

			mkdir_for  Me.OutputBinaryPaths( output_count )
			command_line = "copy /B """+ _
				temporary_path_1_c +"""+"""+_
				temporary_path_2_c +"""+"""+_
				temporary_path_3_c +""" """+_
				Me.OutputBinaryPaths( output_count ) +""""
			r= RunBat( command_line, log_txt )
			Assert  r = 0
			AssertExist  Me.OutputBinaryPaths( output_count )

			del  temporary_path_1_c
			del  temporary_path_2_bin
			del  temporary_path_2_c
			del  temporary_path_2_h
			del  temporary_path_3_c
			del  log_txt

		 Case  "srec"
			If IsNull( address ) Then  address = "0x00000000"
			If Left( address, 2 ) <> "0x" Then  Me.RaiseError _
				Array( "XPathBadValue", "/ImagePackager/OutputHeader/@address", ConfigFilePath )

			temporary_path_2_bin = GetTempPath( "ImagePackager_2_*.bin" )
			image.Save  temporary_path_2_bin

			BinToSRec  temporary_path_2_bin,  Me.OutputBinaryPaths( output_count ), _
				"",  Mid( address, 3 ), "0", Empty

			del  temporary_path_2_bin

		 Case Else
			Me.RaiseError  Array( "XPathBadValue", _
				"/ImagePackager/OutputBinary/@language", ConfigFilePath )
		End Select

		output_count = output_count + 1


		'// Replace "${DeclareBinaryImageSymbol}"
		If IsEmpty( section ) Then
			text = src_tmp.Header
		Else
			text = src_tmp.HeaderWithSection
		End If

		variables_text = variables_file.ReadAll()
		CutLastOf  variables_text, vbLF, Empty
		CutLastOf  variables_text, vbCR, Empty
		Me.Variables( "${Variables}" ) = variables_text

		If IsNull( address ) Then
			Me.Variables( "${StartAddress}" ) = ""
		Else
			Me.Variables( "${StartAddress}" ) = address
		End If
		Me.Variables( "${LastAddress}" ) = ""

		If IsEmpty( raw_image_alignment_symbol ) Then
			define_text = vbCRLF
		Else
			define_text = vbCRLF +"#define  "+ raw_image_alignment_symbol +_
				"  "& output_binary.RawImageAlignment & vbCRLF
		End If
		If InStr( text, "${DeclareBinaryImageSymbol}" ) > 0 Then
			If InStr( text, "${DeclareBinaryImageSymbol}"+ vbCRLF ) = 0 Then
				Raise 1, "<ERROR msg=""Insert return after ${DeclareBinaryImageSymbol}""/>"
			End If
		End If

		If IsNull( address ) Then
			text = Replace( text, "${DeclareBinaryImageSymbol}"+ vbCRLF, _
				src_tmp.DeclareVariable + define_text )
		Else
			If Left( address, 2 ) <> "0x" Then  Me.RaiseError _
				Array( "XPathBadValue", "/ImagePackager/OutputHeader/@address", ConfigFilePath )

			last_address = CLng( "&h" & Mid( address, 3 ) ) + bin_size - 1
			last_address = "0x"+ Right( "0000000"+ Hex( last_address ), 8 )
			Me.Variables( "${LastAddress}" ) = last_address

			text = Replace( text, "${DeclareBinaryImageSymbol}"+ vbCRLF, _
				src_tmp.DeclareAddress + define_text )
		End If

		text = Me.Variables( text )


		'// Save header file
		CreateFile  GetFullPath( output_header_tag.getAttribute( "path" ), base_path ), text


		Me.Variables.Remove  "${Section}"
		Me.Variables.Remove  "${Symbol}"
		Me.Variables.Remove  "${Size}"
		Me.Variables.Remove  "${Variables}"
		Me.Variables.Remove  "${StartAddress}"
		Me.Variables.Remove  "${LastAddress}"
		Me.Variables.Remove  "${IncludeDefine}"
	Next


	del  Me.WorkFolderPath
End Sub


 
'********************************************************************************
'  <<< [ImagePackagerClass::RunByInputPath] >>> 
' - input_file.ImageTag : "Image" tag or "File" tag
'********************************************************************************
Sub  RunByInputPath( input_file, output_binary )
	Const  symbol_width = 29
	Const  info_bin_alignment = 4

	If not g_is_debug Then  Set ec = new EchoOff
	Set at = AlignmentTypeEnum
	file_name = g_fs.GetFileName( input_file.InputFullPath )
	command_line_options = ""

	'// Set "symbol"
	symbol = input_file.ImageTag.getAttribute( "symbol" )
	If IsNull( symbol ) Then  Me.RaiseError _
		Array( "XPath", "/ImagePackager/InputFiles/Image/@symbol" )
	Me.Variables( "${BaseName}" ) = g_fs.GetBaseName( file_name )
	Me.Variables( "${Extension}" ) = g_fs.GetExtensionName( file_name )


	If LCase( input_file.ImageTag.tagName ) = "image" Then


		'// Set "header_type"
		header_type = input_file.ImageTag.getAttribute( "type" )
		If IsNull( header_type ) Then  Me.RaiseError _
			Array( "XPath", "/ImagePackager/InputFiles/Image/@type" )
		asterisk = "*"
		If Len( header_type ) >= 2 Then
			If Right( header_type, 1 ) = "*" Then
				header_type = Left( header_type, Len( header_type ) - 1 )
				asterisk = ""
			End If
		End If


		'// Set "command_line_options" for @type
		command_line_options = command_line_options +" /HeaderType:"+ header_type


		'// Set "output_formats"
		output_formats = input_file.ImageTag.getAttribute( "output_format" )
		If IsNull( output_formats ) Then  Me.RaiseError _
			Array( "XPath", "/ImagePackager/InputFiles/Image/@output_format" )
		output_formats = ArrayFromCSV( output_formats )


		'// Set "output_format_params"
		ReDim  output_format_params( UBound( output_formats ) )
		For format_num = 0  To UBound( output_formats )
			output_format = output_formats( format_num )
			Select Case  output_format
				Case  "ARGB8888" : output_format_params( format_num ) = "-8888"
				Case  "XRGB8888" : output_format_params( format_num ) = "-888"
				Case  "RGB565"   : output_format_params( format_num ) = "-565"
				Case  "ARGB1555" : output_format_params( format_num ) = "-1555"
				Case  "ARGB4444" : output_format_params( format_num ) = "-4444"
				Case  "YUV422"   : output_format_params( format_num ) = "-422"
				Case  "CLUT8"    : output_format_params( format_num ) = "-32PAL256"
				Case  "CLUT4"    : output_format_params( format_num ) = "-32PAL16"
				Case  "CLUT1"    : output_format_params( format_num ) = "-32PAL2"
				Case  "A8"       : output_format_params( format_num ) = "-A8"
				Case  "A4"       : output_format_params( format_num ) = "-A4"
				Case  "A1"       : output_format_params( format_num ) = "-A1"
				Case Else  Me.RaiseError  Array( "Format", output_format )
			End Select
		Next


		'// Set "input_file_type"
		input_file_type = Me.GetInputFileTypeAsExeParam( file_name )


		'// Set "command_line_options" for @premultiplied_alpha
		Select Case  input_file.ImageTag.getAttribute( "premultiplied_alpha" )
		 Case  "yes"
			command_line_options = command_line_options +" /PremultipliedAlpha:yes"
		 Case  "already_yes"
			command_line_options = command_line_options +" /PremultipliedAlpha:already_yes"
		End Select


		'// Loop of "format_num"
		For format_num = 0  To UBound( output_formats )
			output_format       = output_formats( format_num )
			output_format_param = output_format_params( format_num )


			'// Set "stride_align" for @raw_stride_alignment, @raw_stride_alignment_4
			stride_align = output_binary.RawStrideAlignment
			For Each  format  In ArrayFromCSV( output_binary.RawStrideAlignment4_PixelFormats )
				If format = output_format Then _
					stride_align = 4
			Next
			stride_align_option = " /StrideAlign:" & stride_align


			'// Set "stride_align" for @alpha_raw_image_width
			width_option = Empty
			If not IsEmpty( output_binary.AlphaRawImageWidth ) Then
				Select Case  output_format
					Case  "A1", "A4", "A8"
						width_option = output_binary.AlphaRawImageWidth
				End Select
			End If
			If IsEmpty( width_option ) Then
				width_option = ""
			Else
				width_option = " /Width:" & width_option
			End If


			'// Set "bit_field_endian_option"
			Select Case  output_format
				Case  "A1", "A4", "A8"
					bit_field_endian_option = " /BitFieldEndian:Little"
				Case Else
					bit_field_endian_option = ""
			End Select


			'// Set "symbol2"
			Me.Variables( "${Format}" ) = output_format
			symbol2 = Me.Variables( symbol )
			Me.Variables.Remove  "${Format}"

			If Len( symbol2 ) < symbol_width Then _
				symbol2 = symbol2 + String( symbol_width - Len( symbol2 ), " " )


			'// Set "endian_str"
			If LCase( output_binary.Endian ) = "littleendian" Then
				endian_str = "1"
			Else
				endian_str = "0"
			End If


			'// Set "work_folder"
			work_folder = Me.GetWorkFolder( output_binary.Endian, output_format, _
				input_file.InputFileNum )


			'// Copy input files
			copy  input_file.InputFullPath,  work_folder


			'// Run "Me.ImageConvertTool_exe"
			ec = Empty
			echo  file_name +" : "+ output_format
			If not g_is_debug Then  Set ec = new EchoOff
			pushd  work_folder
			command_line = """"+ Me.ImageConvertTool_exe +""" "+ output_format_param +" """+_
				file_name +""" 2 0 "+ input_file_type +" 0 0 "+ endian_str +_
				command_line_options + stride_align_option + bit_field_endian_option + _
				width_option
			If g_is_debug Then
				echo_v  ">"+ command_line
			End If
			redirect_path = "_converter_out.txt" : If g_is_debug Then  redirect_path = ""
			r= RunProg( command_line, redirect_path )
			popd

'// For debugging ImageConvertTool_exe
'// If g_count_up(0)=3 Then  Pause : Stop
'// echo_v "g_count(0)=" & g_count(0)

			If r <> 0 Then  Me.RaiseError  Array( "Exe", r, _
				ReadFile( work_folder +"\_converter_out.txt" ) )


			'// Add to "output_binary.BinaryParts"
			If input_file_type = "0" Then
				file_name_bmp = file_name
			Else
				file_name_bmp = AddLastOfFileName( file_name, ".bmp" )
			End If

			Set bin = new BinaryPartClass
			bin.Path = work_folder +"\"+ file_name_bmp +".info.bin"
			bin.Symbol = symbol2
			bin.TypeCast = asterisk +"(const "+ header_type +"*)"
			bin.AlignmentType = at.RawInfoBinAlignmentType
			bin.Alignment = info_bin_alignment
			bin.Size = g_fs.GetFile( bin.Path ).Size
			bin.IsVariableInHeader = True
			bin.IsOffsetable = True
			Set bin.OffsetAdjuster = new OffsetAdjusterClass
			bin.OffsetAdjuster.OffsetOfOffsetMemberVariable = 4
			bin.OffsetAdjuster.StepIndexOfTargetBinary = 1
			output_binary.BinaryParts.Add  bin

			Set bin = new BinaryPartClass
			bin.Path = work_folder +"\"+ file_name_bmp +".raw.bin"
			bin.AlignmentType = at.RawImageAlignmentType
			bin.Alignment = output_binary.RawImageAlignment
			bin.Size = g_fs.GetFile( bin.Path ).Size
			bin.IsVariableInHeader = False
			bin.IsOffsetable = True
			output_binary.BinaryParts.Add  bin
		Next

	ElseIf LCase( input_file.ImageTag.tagName ) = "var" Then

		'// Set "header_type"
		header_type = input_file.ImageTag.getAttribute( "type" )
		If IsNull( header_type ) Then
			header_type = "uint32_t"
		End If
		Select Case  header_type
			Case  "uint32_t" : var_size = 4
			Case  "uint16_t" : var_size = 2
			Case  "uint8_t"  : var_size = 1
			Case  "int32_t"  : var_size = 4
			Case  "int16_t"  : var_size = 2
			Case  "int8_t"   : var_size = 1
			Case Else
				Raise  1, "<ERROR msg=""Not supported type"" tag_symbol="""+ symbol +"""/>"
		End Select

		'// Set "var_value"
		var_value = input_file.ImageTag.getAttribute( "value" )
		If IsNull( var_value ) Then
			Raise  1, "<ERROR msg=""Not found value attribute"" tag_symbol="""+ symbol +"""/>"
		End If
		If InStr( var_value, "(new Image('" ) > 0 Then
			path = sscanf( var_value, "Image('%s')" )
			path = GetFullPath( path, input_file.InputFullPath )
			Set image_information = Me.ParseImageInformation( path )
			If InStr( var_value, ".width" ) > 0 Then
				var_value = CStr( image_information( "Width" ) )
			Else
				var_value = CStr( image_information( "Height" ) )
			End If
		End If


		'// Set "symbol"
		symbol = Me.Variables( symbol )
		If Len( symbol ) < symbol_width Then _
			symbol = symbol + String( symbol_width - Len( symbol ), " " )

		'// Add to "output_binary.BinaryParts"
		Set bin = new BinaryPartClass
		Set bin.BinaryData = new BinaryArray
		bin.BinaryData.WriteStruct  0, Array( vbLong, CInt2( Replace( var_value, "0x", "&h" ) ) )
		If LCase( output_binary.Endian ) = "bigendian" Then _
			bin.BinaryData.SwapEndian  0, var_size, var_size
		bin.Symbol = symbol
		bin.TypeCast = "*(const "+ header_type +"*)"
		bin.AlignmentType = Empty
		bin.Alignment = var_size
		bin.Size = var_size
		bin.IsVariableInHeader = True
		bin.IsOffsetable = False
		output_binary.BinaryParts.Add  bin

	Else  '// LCase( input_file.ImageTag.tagName ) = "file"

		ec = Empty
		echo  file_name +" : Binary"
		If not g_is_debug Then  Set ec = new EchoOff


		'// Set "header_type"
		header_type = input_file.ImageTag.getAttribute( "type" )
		If IsNull( header_type ) Then
			header_type = "uint8_t[]"
		End If
		asterisk = "*"
		If Len( header_type ) >= 2 Then
			If Right( header_type, 1 ) = "*" Then
				header_type = Left( header_type, Len( header_type ) - 1 )
				asterisk = ""
			End If
		End If
		If Len( header_type ) >= 3 Then
			If Right( header_type, 2 ) = "[]" Then
				header_type = Left( header_type, Len( header_type ) - 2 )
				asterisk = ""
			End If
		End If


		'// Set "file_alignment"
		file_alignment = input_file.ImageTag.getAttribute( "alignment" )
		If IsNull( file_alignment ) Then
			file_alignment = Empty
		Else
			file_alignment = CInt2( file_alignment )
		End If


		'// Set "symbol"
		symbol = Me.Variables( symbol )
		If Len( symbol ) < symbol_width Then _
			symbol = symbol + String( symbol_width - Len( symbol ), " " )


		'// Add to "output_binary.BinaryParts"
		Set bin = new BinaryPartClass
		bin.Path = input_file.InputFullPath
		bin.Symbol = symbol
		bin.TypeCast = asterisk +"(const "+ header_type +"*)"
		bin.AlignmentType = at.BinaryFileAlignmentType
		bin.Alignment = file_alignment
		bin.Size = g_fs.GetFile( bin.Path ).Size
		bin.IsVariableInHeader = True
		bin.IsOffsetable = True
		output_binary.BinaryParts.Add  bin
	End If

	Me.Variables.Remove  "${BaseName}"
	Me.Variables.Remove  "${Extension}"
End Sub


 
'********************************************************************************
'  <<< [ImagePackagerClass::GetDefaultFileAlignment_sub] >>> 
'********************************************************************************
Function  GetDefaultFileAlignment_sub( output_bin_tag, bin )

	bin_alignment = 4  '// Default

	'// Set "super_class_ID"
	super_class_ID = output_bin_tag.getAttribute( "super_class" )
	If IsNull( super_class_ID ) Then _
		super_class_ID = output_bin_tag.getAttribute( "super_class_id" )

	If not IsNull( super_class_ID ) Then

		If InStr( super_class_ID, "#" ) = 0 Then
			super_class_ID = "#"+ super_class_ID
		End If
		super_class_ID = Me.Variables( super_class_ID )

		'// Set "super_class"
		Me.LinkedXML.StartNavigation  Me.LoadedConfigFilePath, Me.XML_Root
		Set super_class = Me.LinkedXML.GetLinkTargetNode( super_class_ID )
		Me.LinkedXML.EndNavigation
		If super_class is Nothing Then _
			Me.RaiseError  Array( "XPath", xpath )

		'// Set "file_tag"
		For Each file_tag  In super_class.selectNodes( "InputFiles/File" )
			is_matched = False

			'// Set "is_matched" : if bin was not matched with File/@path
			path = file_tag.getAttribute( "path" )
			If IsNull( path ) Then _
				Me.RaiseError  Array( "XPath", file_tag_xpath +"/@path" )

			is_matched = IsMatchedWithWildcard( bin.Path, path )

			'// Set "bin_alignment" : from File/@alignment
			If is_matched Then
				bin_alignment = file_tag.getAttribute( "alignment" )
				If not IsNull( bin_alignment ) Then
					bin_alignment = CInt2( bin_alignment )
					Exit For
				End If
			End If
		Next
	End If
	GetDefaultFileAlignment_sub = bin_alignment
End Function


 
'********************************************************************************
'  <<< [ImagePackagerClass::GetSourceTemplate_sub] >>> 
'********************************************************************************
Function  GetSourceTemplate_sub( output_bin_tag, source_template, bin_image_symbol )

	'// Set "source_template_ID"
	source_template_ID = output_bin_tag.getAttribute( "source_template" )
	If IsNull( source_template_ID ) Then
		Set source_template = output_bin_tag.ownerDocument.createElement( "Dummy" )
	Else
		'// Set "source_template"
		source_template_ID = Me.Variables( source_template_ID )
		Me.LinkedXML.StartNavigation  Me.LoadedConfigFilePath, Me.XML_Root
		Set source_template = Me.LinkedXML.GetLinkTargetNode( source_template_ID )
		Me.LinkedXML.EndNavigation
		If source_template is Nothing Then _
			Me.RaiseError  Array( "XPath", xpath )
	End If


	'// Set "src_tmp"
	Set src_tmp = new  SourceTemplateClass

	Set tag = source_template.selectSingleNode( "Source" )
	If tag is Nothing Then
		src_tmp.Source = _
			"typedef unsigned char  uint8_t;"+ vbCRLF +_
			"const uint8_t  ${Symbol}[${Size}] = {"+ vbCRLF +_
			"${BinaryData}"+ vbCRLF +_
			"};"+ vbCRLF
		is_default_template = True
	Else
		src_tmp.Source = Replace( tag.text, vbLF, vbCRLF )
		is_default_template = False
	End If

	Set tag = source_template.selectSingleNode( "SourceWithSection" )
	If tag is Nothing Then
		If is_default_template Then
			src_tmp.SourceWithSection = _
				"#pragma section  ${Section}"+ vbCRLF +_
				"typedef unsigned char  uint8_t;"+ vbCRLF +_
				"const uint8_t  ${Symbol}[${Size}] = {"+ vbCRLF +_
				"${BinaryData}"+ vbCRLF +_
				"};"+ vbCRLF +_
				"#pragma section"+ vbCRLF
		Else
			src_tmp.SourceWithSection = src_tmp.Source
		End If
	Else
		src_tmp.SourceWithSection = Replace( tag.text, vbLF, vbCRLF )
	End If

	Set tag = source_template.selectSingleNode( "Header" )
	If tag is Nothing Then
		src_tmp.Header = _
			"#ifndef  ${IncludeDefine}"+ vbCRLF +_
			"#define  ${IncludeDefine}"+ vbCRLF +_
			""+ vbCRLF +_
			"#ifdef  __cplusplus"+ vbCRLF +_
			"extern ""C"" {  /* Start of C Symbol */"+ vbCRLF +_
			"#endif"+ vbCRLF +_
			""+ vbCRLF +_
			"${DeclareBinaryImageSymbol}"+ vbCRLF +_
			""+ vbCRLF +_
			"${Variables}"+ vbCRLF +_
			""+ vbCRLF +_
			"#ifdef  __cplusplus"+ vbCRLF +_
			" }  /* End of C Symbol */ "+ vbCRLF +_
			"#endif"+ vbCRLF +_
			""+ vbCRLF +_
			"#endif"+ vbCRLF
	Else
		src_tmp.Header = Replace( tag.text, vbLF, vbCRLF )
	End If

	Set tag = source_template.selectSingleNode( "HeaderWithSection" )
	If tag is Nothing Then
		src_tmp.HeaderWithSection = src_tmp.Header
	Else
		src_tmp.HeaderWithSection = Replace( tag.text, vbLF, vbCRLF )
	End If

	Set tag = source_template.selectSingleNode( "DeclareVariable" )
	If tag is Nothing Then
		src_tmp.DeclareVariable = "extern const uint8_t  ${Symbol}[ ${Size} ];"
	Else
		src_tmp.DeclareVariable = Replace( tag.text, vbLF, vbCRLF )
	End If

	Set tag = source_template.selectSingleNode( "DeclareAddress" )
	If tag is Nothing Then
		src_tmp.DeclareAddress = "#define  ${Symbol}  ( (const uint8_t*) ${StartAddress} )"+ vbCRLF+_
			"         "+ String( Len( bin_image_symbol ), " " ) +"  /* Last address is ${LastAddress} */"
	Else
		src_tmp.DeclareAddress = Replace( tag.text, vbLF, vbCRLF )
	End If

	Set GetSourceTemplate_sub = src_tmp
End Function


 
'********************************************************************************
'  <<< [ImagePackagerClass::ParseImageInformation] >>> 
' - Return value is dictionary type. Look at "ReadStruct" in this function.
'********************************************************************************
Function  ParseImageInformation( ImagePath )
	file_name = g_fs.GetFileName( ImagePath )
	work_folder = Me.GetWorkFolder( "LittleEndian", "ForInfo", 1 )
	input_file_type = Me.GetInputFileTypeAsExeParam( file_name )
	If input_file_type = "0" Then
		file_name_bmp = file_name
	Else
		file_name_bmp = AddLastOfFileName( file_name, ".bmp" )
	End If


	copy_ren  ImagePath, work_folder +"\"+ file_name


	'// Run "Me.ImageConvertTool_exe"
	mkdir  work_folder
	pushd  work_folder
	command_line = """"+ Me.ImageConvertTool_exe +""" -565 """+_
		file_name +""" 2 0 "+ input_file_type +" 0 0 1 "+_
		"/HeaderType:GraphicsImageClass /StrideAlign:1"
	If g_is_debug Then
		echo_v  ">"+ command_line
	End If
	redirect_path = "_converter_out.txt" : If g_is_debug Then  redirect_path = ""
	r= RunProg( command_line, redirect_path )
	popd
	If r <> 0 Then  Me.RaiseError  Array( "Exe", r, _
		ReadFile( work_folder +"\_converter_out.txt" ) )


	'// Parse "*.info.bin" file
	Set info_bin = ReadBinaryFile( work_folder +"\"+ file_name_bmp +".info.bin" )
	info_bin.ReadStruct  &h0C, ParseImageInformation, _
		Array( "Width", vbInteger,  "Height", vbInteger )
End Function


 
'********************************************************************************
'  <<< [ImagePackagerClass::GetWorkFolder] >>> 
'********************************************************************************
Function  GetWorkFolder( Endian, OutputFormat, InputFileNum )
	GetWorkFolder = Me.WorkFolderPath +"\"+ Endian +"\"+ OutputFormat +"\"& InputFileNum
End Function


 
'********************************************************************************
'  <<< [ImagePackagerClass::GetInputFileTypeAsExeParam] >>> 
'********************************************************************************
Function  GetInputFileTypeAsExeParam( Path )
	Select Case  LCase( g_fs.GetExtensionName( Path ) )
		Case  "png" : GetInputFileTypeAsExeParam = "1"
		Case  "jpg" : GetInputFileTypeAsExeParam = "2"
		Case Else   : GetInputFileTypeAsExeParam = "0"
	End Select
End Function


 
'********************************************************************************
'  <<< [ImagePackagerClass::RaiseError] >>> 
'********************************************************************************
Sub  RaiseError( Arguments )
	args = Arguments
	Select Case  args(0)
	 Case "XPath"
		Raise  1,"<ERROR msg=""There is not setting in XML"" xpath="""+ args(1) +""" path="""+ _
			Me.LoadedConfigFilePath +"""/>"
	 Case "XPathBadValue"
		Raise  1,"<ERROR msg=""Invalid value in XML"" path="""+ args(1) +""" xpath="""+ _
			args(2) +"""/>"
	 Case "Exe"
		Raise  1,"<ERROR msg="""+ Me.ImageConvertTool_exe +""" raised error"" return_value="""& _
			args(1) & """>"+ vbCRLF + args(2) + vbCRLF +"</ERROR>"
	 Case "Format"
		Raise  1,"<ERROR msg=""The format convert is not supported"" format="""+ args(1) +"""/>"

	 Case "MultiOutNotSame"
		Raise  1,"<ERROR msg=""Put OutputHeader tag in OutputBinary tag, if multiple output.""/>"

	 Case "NotBaseOfALogarithm2"
		Raise  1,"<ERROR msg=""alignment must be exponent of 2"""+_
			" xpath="""+ args(1) +""" value="""& args(2) &"""/>"
	 Case Else
		Error
	End Select
End Sub


 
End Class
' ### <<<< End of Class implement >>>> 


 
'********************************************************************************
'  <<< [ImagePackagerClass_run_Finally] >>> 
'********************************************************************************
Sub  ImagePackagerClass_run_Finally( Vars )
  Dim  en,ed : en = Err.Number : ed = Err.Description : On Error Resume Next  '// This clears error

  Set Me_ = Vars.Item("Me")
  Me_.LoadedConfigFilePath = Empty

  ErrorCheckInTerminate  en : On Error GoTo 0 : If en <> 0 Then  Err.Raise en,,ed  '// This sets en again
End Sub


 
'-------------------------------------------------------------------------
' ### <<<< [OutputBinaryClass] >>>> 
'-------------------------------------------------------------------------
Class  OutputBinaryClass
	Public  BinaryParts  '// as ArrayClass of BinaryPartClass

	Public  Language  '// as string
	Public  Endian  '// as string
	Public  RawImageAlignment   '// Unit is byte
	Public  RawStrideAlignment
	Public  RawStrideAlignment4_PixelFormats
	Public  AlphaRawImageWidth  '// as integer or Empty
	Public  TableFormat  '// as string


	Private Sub  Class_Initialize()
		Set BinaryParts = new ArrayClass
	End Sub
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [BinaryPartClass] >>>> 
'-------------------------------------------------------------------------
Class  BinaryPartClass
	'// for Step 1
	Public  Path
	Public  BinaryData '// as BinaryArray, If IsEmpty( Path )
	Public  Symbol     '// Empty if not output to header file
	Public  TypeCast   '// as string
	Public  AlignmentType  '// as AlignmentTypeEnum
	Public  Alignment
	Public  Size
	Public  IsVariableInHeader
	Public  IsOffsetable
	Public  OffsetOfOffsetOfSelfOffsetInPackage
	Public  OffsetAdjuster  '// as Empty or OffsetAdjusterClass

	'// for Step 2
	Public  SelfOffsetInPackage
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [InputFileClass] >>>> 
'-------------------------------------------------------------------------
Class  InputFileClass
	Public  InputFileNum
	Public  InputFullPath
	Public  ImageTag
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [OffsetAdjusterClass] >>>> 
'-------------------------------------------------------------------------
Class  OffsetAdjusterClass
	'// for Step 1
	Public  OffsetOfOffsetMemberVariable
	Public  StepIndexOfTargetBinary
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [SourceTemplateClass] >>>> 
'-------------------------------------------------------------------------
Class  SourceTemplateClass
	Public  Source
	Public  SourceWithSection
	Public  Header
	Public  HeaderWithSection
	Public  DeclareVariable
	Public  DeclareAddress
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [AlignmentTypeEnum] >>>> 
'-------------------------------------------------------------------------
Class  AlignmentTypeEnumClass
	Public  RawInfoBinAlignmentType
	Public  RawImageAlignmentType
	Public  RawStrideAlignmentType
	Public  BinaryFileAlignmentType

	Private Sub  Class_Initialize()
		RawInfoBinAlignmentType = 1
		RawImageAlignmentType   = 2
		RawStrideAlignmentType  = 3
		BinaryFileAlignmentType = 4
	End Sub

	Public Function  Str( EnumValue )
		Select Case  EnumValue
			Case  RawInfoBinAlignmentType : Str = "RawInfoBinAlignmentType"
			Case  RawImageAlignmentType   : Str = "RawImageAlignmentType"
			Case  RawStrideAlignmentType  : Str = "RawStrideAlignmentType"
			Case  BinaryFileAlignmentType : Str = "BinaryFileAlignmentType"
		End Select
	End Function
End Class

Dim  AlignmentTypeEnum : Set AlignmentTypeEnum = new AlignmentTypeEnumClass


 
