'// vbslib - VBScript ShortHand Library  ver4.91  Oct.13, 2014
'// vbslib is provided under 3-clause BSD license.
'// Copyright (C) 2007-2014 Sofrware Design Gallery "Sage Plaisir 21" All Rights Reserved.

Dim  g_SrcPath
Dim  g_vbslib_main_Path
     g_vbslib_main_Path = g_SrcPath


 
'*************************************************************************
'  <<< Global variables >>> 
'*************************************************************************

Dim  g_WritablePathes
Dim  g_Err2, g_EchoObj
Dim  g_AppKey
Dim  g_Test
Dim  g_CUI
Dim  g_ChildHead
Dim  g_CurrentWritables
Dim  g_FileSystemRetryMSec
Dim  g_FileSystemMaxRetryMSec
Dim  g_Player  ' as Vbslib_Player
Dim  g_BreakByFName  ' as string
Dim  g_debug_tree
Dim  g_IsAutoTest : g_IsAutoTest = False
Dim  g_StdRegProv


Function  InitializeModule( ThisPath )
	ReDim  g_WritablePathes(-1)
	get_TempFile
	g_Vers("CutPropertyM") = True
	Set o = new VarWithParameterClass
		o.VarNameAndStartBracket = "XML_File("
		Set o.GetVar = GetRef( "VarWithParameter_XML_FileClass_GetVar" )
	Set g_Vers( o.VarNameAndStartBracket ) = o
	Set o = new VarWithParameterClass
		o.VarNameAndStartBracket = "ArrayFromXML_File("
		Set o.GetVar = GetRef( "VarWithParameter_ArrayFromXML_FileClass_GetVar" )
	Set g_Vers( o.VarNameAndStartBracket ) = o
	Set o = new VarWithParameterClass
		o.VarNameAndStartBracket = "SpecialFolders("
		Set o.GetVar = GetRef( "VarWithParameter_SpecialFoldersClass_GetVar" )
	Set g_Vers( o.VarNameAndStartBracket ) = o

	Set g_CurrentWritables = new CurrentWritables : ErrCheck

	Set g_CustomEchoStrGenerators = CreateObject( "Scripting.Dictionary" )
	Set g_CustomEchoStrGenerators( "Dictionary" ) = GetRef( "Dictionary_xml_sub" )
	Set g_CustomEchoStrGenerators( "SWbemObjectEx" ) = GetRef( "SWbemObjectEx_xml_sub" )

	Set g_EchoObj = new EchoObj : ErrCheck
	Set g_Err2 = new Err2 : ErrCheck
	g_Err2.BreakErrID = g_debug
	g_Err2.BreakNestPos = g_debug_tree
	If IsNumeric( g_debug_process ) Then
		g_Err2.Break_ChildProcess_CallID.Add  g_debug_process
	ElseIf IsArray( g_debug_process ) Then
		g_Err2.Break_ChildProcess_CallID.Copy  g_debug_process
	End If

	Set g_FileOptions = new FileOptionsClass

	Set g_CUI = new CUI : ErrCheck
	g_FileSystemRetryMSec = 4*1000
	g_FileSystemMaxRetryMSec = 60*1000
	Set g_InterProcess = new InterProcess : ErrCheck
	new_ChildProcess_ifChildProcess
	If not g_ChildProcess is Nothing Then  g_InterProcess.OnCallInChild  g_ChildProcess

	g_InterProcess.InterProcessDataArray.Add  g_Err2

	Dim pr : Set pr = get_ChildProcess()
	If not pr is Nothing Then _
		g_Err2.loadXML  pr.m_InXML.selectSingleNode( "Err2" )

	If not IsObject( g_Vers("TextFileExtension") ) Then
		a_array = g_Vers("TextFileExtension")
		Set g_Vers("TextFileExtension") = CreateObject( "Scripting.Dictionary" )
		g_Vers("TextFileExtension").CompareMode = 1
		If not IsEmpty( a_array ) Then
			Dic_addFromArray  g_Vers("TextFileExtension"), a_array, True
		End If
	End If
End Function
Dim  g_InitializeModule
Set  g_InitializeModule = GetRef( "InitializeModule" )


Function  FinalizeModule( ThisPath, Reason )
	If not IsEmpty( g_Err2 ) Then
		If Reason = 0 Then
			g_Err2.OnSuccessFinish
		Else
			g_Err2.OnErrorFinish
		End If
	End If

	If not IsEmpty( g_ChildProcess ) Then
		If not g_ChildProcess is Nothing Then  g_ChildProcess.Finish
	End If

	If not IsEmpty( g_EchoObj ) Then  echo_flush
End Function
Dim g_FinalizeModule: Set g_FinalizeModule = GetRef( "FinalizeModule" )
Dim g_FinalizeLevel:      g_FinalizeLevel  = 100  ' If smaller, called early


Const  F_File      = 1
Const  F_Folder    = 2
Const  F_SubFolder = 4

Const  g_PauseMsg = "Press Enter key to continue ... "
Const  g_PauseMsgStone = 24
Const  g_ConnectDebugMsg = "If you open following file using text editor and modify value of variable, it stops at problem point connected debugger."
Const  g_ConnectDebugMsgV5 = ""


 
'*************************************************************************
'  <<< [g_VBS_Lib] >>> 
'*************************************************************************
Dim  g_VBS_Lib : Set g_VBS_Lib = new VbsLibConsts : ErrCheck


Class  VbsLibConsts
	Public  CutTag, ExistOnly, MakeShortcutFiles, ForMkDir, CopyV4, CopyRen, EnvVar, Range
	Public  WholeWord, CaseSensitive, EchoV_NotSame, Rev, LastNextPos, LineHead, LineTail
	Public  Reverse, NotEchoStartCommand, NotInstalled, ErrorIfNotExistBoth
	Public  Shift_JIS,  EUC_JP,  Unicode,  UTF_8,  UTF_8_No_BOM,  No_BOM,  ISO_2022_JP,  Append
	Public  NODE_ELEMENT, NODE_ATTRIBUTE, NODE_TEXT, NODE_DOCUMENT, NODE_PROCESSING_INSTRUCTION
	Public  KeepLineSeparators, CutLineSeparators, LF, CRLF
	Public  SkipLeftSpaceLine, RightHasPercentFunction, WithDollarVariable, ErrorIfNotSame
	Public  DstIsBackup,  CheckFileExists,  CheckFolderExists,  AllowEnterOnly
	Public  UseArgument1,  UseArgument2
	Public  File, Folder, SubFolder, NotSubFolder, SubFolderIfWildcard, AbsPath, FullPath, ArrayOfArray
	Public  NotToEmptyArray
	Public  CompareOnlyExistB, StopIsNotSame
	Public  NotCaseSensitive, NoError, NoConfirm
	Public  NoRound, ToTrashBox
	Public  XML_DOM_ReadCache  '// as dictionary key=path, item=IXMLDOMElement class
	Public  TextReadCache  '// as dictionary key=path, item=string
	Public  WshRunning, WshFinished, Forever, NoRootXML, StringData, InheritSuperClass

	Private Sub  Class_Initialize()
		CutTag = 1
		ExistOnly = 2
		MakeShortcutFiles = 4
		ForMkDir = 1
		CopyV4 = 4
		CopyRen = 5
		Set EnvVar = new EnumType : EnvVar.Name = "EnvVar"
		Range = 2
		CaseSensitive = &h01
		WholeWord     = &h04
		Rev           = &h08
		LastNextPos   = &h10
		LineHead      = &h20
		LineTail      = &h40
		EchoV_NotSame = &h100
		ErrorIfNotExistBoth = &h200
		Reverse = -1
		NotEchoStartCommand = 1
		NotInstalled = 16

		Shift_JIS    =   &h1000
		EUC_JP       =   &h2000
		ISO_2022_JP  =   &h4000
		Unicode      =        2
		UTF_8        =        8
		UTF_8_No_BOM =     &h80
		No_BOM       =     &h40

		Append       =        4

		NODE_ELEMENT = 1 :  NODE_ATTRIBUTE = 2  : NODE_TEXT = 3
		NODE_DOCUMENT = 9 : NODE_PROCESSING_INSTRUCTION = 7

		KeepLineSeparators = -2 : CutLineSeparators = 4 :  LF = 10 : CRLF = -1
		SkipLeftSpaceLine = 2 : RightHasPercentFunction = 4 : WithDollarVariable = 8
		ErrorIfNotSame = &h4000

		DstIsBackup = 2

		CheckFileExists =   1
		CheckFolderExists = 2
		AllowEnterOnly =    4
		UseArgument1 =   &h10
		UseArgument2 =   &h20

		File                = &h01
		Folder              = &h02
		SubFolder           = &h04
		NotSubFolder        = &h08
		SubFolderIfWildcard = &h20
		AbsPath             = &h08
		FullPath            = &h08
		ArrayOfArray        = &h10

		NotToEmptyArray = &h4000

		CompareOnlyExistB = 1
		StopIsNotSame     = 2

		NotCaseSensitive = 1

		NoError          = &h4000
		NoConfirm        = &h2000

		NoRound          = 1
		ToTrashBox       = 1

		WshRunning       = 0
		WshFinished      = 1
		Forever          = -1

		NoRootXML = 1       '// F_NoRoot
		StringData = &h8000  '// F_Str
		InheritSuperClass = 2


		Set XML_DOM_ReadCache = CreateObject( "Scripting.Dictionary" )
		XML_DOM_ReadCache.CompareMode = NotCaseSensitive
		Set TextReadCache = CreateObject( "Scripting.Dictionary" )
		TextReadCache.CompareMode = NotCaseSensitive
	End Sub

	Private  m_XML_ElementBase64
	Public Property Get  XML_ElementBase64()
		If IsEmpty( m_XML_ElementBase64 ) Then
			Set xml = CreateObject( "MSXML2.DOMDocument" )
			Set m_XML_ElementBase64 = xml.createElement( "DummyTag" )
			m_XML_ElementBase64.dataType = "bin.base64"
		End If
		Set XML_ElementBase64 = m_XML_ElementBase64
	End Property
End Class


Class EnumType : Public Name : End Class


 
'*************************************************************************
'  <<< Error Code >>> 
'*************************************************************************

Dim E_Others            : E_Others            = 1

Dim E_TestPass          : E_TestPass          = 21
Dim E_TestSkip          : E_TestSkip          = 22
Dim E_TestFail          : E_TestFail          = 23
Dim E_UserCancel        : E_UserCancel        = 24

Dim E_BuildFail         : E_BuildFail         = &h80041004
Dim E_OutOfWritable     : E_OutOfWritable     = &h80041005
Dim E_NotFoundSymbol    : E_NotFoundSymbol    = &h80041006
Dim E_ProgRetNotZero    : E_ProgRetNotZero    = &h80041007
Dim E_Unexpected        : E_Unexpected        = &h80041008
Dim E_TimeOut           : E_TimeOut           = &h80041009

Dim E_WIN32_FILE_NOT_FOUND: E_WIN32_FILE_NOT_FOUND = &h80070002
Dim E_WIN32_DIRECTORY     : E_WIN32_DIRECTORY      = &h8007010B
Dim E_WIN32_INVALID_NAME  : E_WIN32_INVALID_NAME   = &h8007007B

Dim E_ProgTerminated    : E_ProgTerminated    = &hC0000005

Dim E_BadType           : E_BadType           = 13
Dim E_FileNotExist      : E_FileNotExist      = 53
Dim E_EndOfFile         : E_EndOfFile         = 62
Dim E_WriteAccessDenied : E_WriteAccessDenied = 70
Dim E_ReadAccessDenied  : E_ReadAccessDenied  = 70
Dim E_PathNotFound      : E_PathNotFound      = 76
Dim E_AlreadyExist      : E_AlreadyExist      = 58
Dim E_NoObjectProvider  : E_NoObjectProvider  = 3251


 
'*************************************************************************
'  <<< File Object >>> 
'*************************************************************************

Const ReadOnly  = 1


 
'*-------------------------------------------------------------------------*
'* ### <<<< Debugging >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [g_count_up] >>> 
'*************************************************************************
Redim  g_count(-1)

Function  g_count_up( i )
	If i > UBound( g_count ) Then  Redim Preserve  g_count(i)
	g_count_up = g_count(i) + 1
	g_count(i) = g_count_up
End Function


 
'*************************************************************************
'  <<< [SetTestMode] >>> 
'*************************************************************************
Dim  F_NotRandom : F_NotRandom = 1
Dim  g_TestModeFlags

Sub  SetTestMode( Flags )
	g_TestModeFlags = Flags
End Sub


 
'*-------------------------------------------------------------------------*
'* ### <<<< User Interface >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [EchoObj] Class >>> 
'*************************************************************************
Class  EchoObj
	Public  m_bEchoOff
	Public  m_bDisableEchoOff
	Public  m_Buf
	Public  m_BufN

	Private Sub  Class_Initialize()
		Me.m_bEchoOff = False : Me.m_bDisableEchoOff = False
	End Sub
End Class
'// g_EchoObj


 
'*************************************************************************
'  <<< [echo] >>> 
'*************************************************************************
Function  echo( ByVal msg )
	If g_EchoObj.m_bEchoOff Then  Exit Function

	If not IsEmpty( msg ) Then
		msg = GetEchoStr( msg )

		If g_CommandPrompt = 0 Then
			If IsEmpty( g_EchoObj.m_Buf ) Then
				g_EchoObj.m_Buf = msg
			Else
				g_EchoObj.m_Buf = g_EchoObj.m_Buf & vbCRLF & msg
			End If
			g_EchoObj.m_BufN = g_EchoObj.m_BufN + 1
			If g_EchoObj.m_BufN >= 20 Then  echo_flush
		Else
			WScript.Echo  msg
		End If

		If not IsEmpty( g_Test ) Then  g_Test.WriteLogLine  msg
	End If
	echo = msg
End Function


 
'*************************************************************************
'  <<< [GetEchoStr] >>> 
'*************************************************************************
Function  GetEchoStr( ByVal msg )
	GetEchoStr = GetEchoStr_sub( msg, 0 ) : CutLastOf  GetEchoStr, vbCRLF, Empty
End Function

Dim  g_CustomEchoStrGenerators  '// as Dictionary of Function. Key=TypeName

Function  GetEchoStr_sub( ByVal msg, ByVal Level )
	Dim  s, e, i, ss

	If IsObject( msg ) Then
		If g_CustomEchoStrGenerators.Exists( TypeName( msg ) ) Then
			s = g_CustomEchoStrGenerators.Item( TypeName( msg ) )( msg, Level )
		ElseIf msg is Nothing Then
			s = "Nothing" + vbCRLF
		Else
			s = "Class " & TypeName( msg )
			ErrCheck : On Error Resume Next
				s = msg.xml_sub( Level+1 )
				e = Err.Number
				If e = 438 Then  s = s + vbCRLF + msg.xml()
			On Error GoTo 0
			If e <> 0 And e <> 438 Then  Err.Raise e
		End If

		GetEchoStr_sub = s
		Exit Function
	End If

	If IsNull( msg ) Then
		msg = GetTab( Level )+ "(null)"
	ElseIf VarType( msg ) = vbBoolean Then
		If msg Then  msg = GetTab( Level )+ "True" _
		Else         msg = GetTab( Level )+ "False"
	ElseIf IsArray( msg ) Then
		s = GetTab( Level )+ "<Array ubound="""& UBound( msg ) &""">" +vbCRLF
		For i=0 To UBound( msg )
			s = s + GetTab( Level+1 )+ "<Item id="""& i &""">"& Trim( GetEchoStr_sub( msg(i), Level+1 ) )
			CutLastOf  s, vbCRLF, Empty
			s = s + "</Item>" +vbCRLF
		Next
		s = s + GetTab( Level )+ "</Array>" +vbCRLF
		GetEchoStr_sub = s
		Exit Function
	End If

	GetEchoStr_sub = GetTab( Level ) & msg & vbCRLF
End Function


 
'*************************************************************************
'  <<< [echo_flush] >>> 
'*************************************************************************
Sub  echo_flush()
	If g_CommandPrompt = 0 Then
	 If ( g_EchoObj.m_BufN > 0 ) and ( Err.Number <> E_UserCancel ) and ( g_Err2.num <> E_UserCancel ) Then
		Do
			If MsgBox( g_EchoObj.m_Buf, vbOKCancel, WScript.ScriptName ) = vbCancel Then
				If MsgBox( "Exit this script?", vbYesNo, WScript.ScriptName ) = vbYes Then
					g_EchoObj.m_Buf = Empty
					g_EchoObj.m_BufN = 0
					If g_IsFinalizing Then
						Exit Sub
					Else
						Raise  E_UserCancel, "User cancel"
					End If
				End If
			Else
				Exit Do
			End If
		Loop
		g_EchoObj.m_Buf = Empty
		g_EchoObj.m_BufN = 0
	 End If
	End If
End Sub


 
'*************************************************************************
'  <<< [echo_line] >>> 
'*************************************************************************
Sub  echo_line()
	echo  "-------------------------------------------------------------------------------"
End Sub


 
'*************************************************************************
'  <<< [EchoOff] >>> 
'*************************************************************************
Class  EchoOff
	Public  m_Prev

	Private Sub Class_Initialize
		m_Prev = g_EchoObj.m_bEchoOff
		g_EchoObj.m_bEchoOff = not g_EchoObj.m_bDisableEchoOff
	End Sub

	Private Sub Class_Terminate
		g_EchoObj.m_bEchoOff = m_Prev
	End Sub
End Class


 
'*************************************************************************
'  <<< [DisableEchoOff] >>> 
'*************************************************************************
Sub  DisableEchoOff()
	g_EchoObj.m_bDisableEchoOff = True
End Sub


 
'*************************************************************************
'  <<< [echo_r] >>> 
' return: output message
'*************************************************************************
Function  echo_r( ByVal msg, redirect_path )
	Dim  f

	If IsObject( msg ) Then  msg = msg.xml

	If g_is_debug Then  WScript.Echo  msg

	If IsEmpty( redirect_path ) Then
	ElseIf redirect_path = "" Then
		If not g_is_debug Then  WScript.Echo  msg
	Else
		Set f = g_fs.OpenTextFile( redirect_path, 2, True, False )
		f.WriteLine  msg
	End If

	echo_r = msg
End Function


 
'*************************************************************************
'  <<< [type_] >>> 
'*************************************************************************
Sub  type_( path )
	Dim  f

	Set f = g_fs.OpenTextFile( path,,,-2 )

	Do Until f.AtEndOfStream
		echo f.ReadLine
	Loop
End Sub


 
'*************************************************************************
'  <<< [Pause] >>> 
'*************************************************************************
Sub  Pause()
	If g_CommandPrompt = 0 Then
		echo  "Press Enter key to continue ..."
		echo_flush
	Else
		g_CUI.input_sub  g_PauseMsg, False
	End If
End Sub


 
'*************************************************************************
'  <<< [pause2] >>> 
'*************************************************************************
Sub  pause2()
	If WScript.Arguments.Named("wscript")=1 Then input g_PauseMsg
End Sub


 
'*************************************************************************
'  <<< [Input] >>> 
'*************************************************************************
Function  Input( ByVal msg )

	Input = g_CUI.input( msg )

End Function


 
'*************************************************************************
'  <<< [SetInput] >>> 
'*************************************************************************
Sub  SetInput( Keys )
	g_CUI.m_Auto_Keys = Keys
End Sub


 
'*************************************************************************
'  <<< [set_input] >>> 
'*************************************************************************
Sub  set_input( Keys )
	g_CUI.m_Auto_Keys = Keys
End Sub


 
'*************************************************************************
'  <<< [InputPath] >>> 
'*************************************************************************
Const  F_ChkFileExists   = 1
Const  F_ChkFolderExists = 2
Const  F_AllowEnterOnly  = 4

Function  InputPath( Prompt, Flags )
	Dim  path,  path_and_num
	Dim  c : Set c = g_VBS_Lib

	If Flags and ( c.UseArgument1 or c.UseArgument2 ) Then
		Dim  args : Set args = GetWScriptArgumentsUnnamed()
		Dim  num
		If Flags and c.UseArgument1 Then
			num = 1
		Else
			num = 2
		End If

		If args.Count >= num Then
			path = args( num - 1 )
		End If
	End If

	Do
		If IsEmpty( path ) Then _
			path = input( Prompt )
		path = Trim2( path )

		If path = "" Then
			If  Flags and c.AllowEnterOnly  Then  path_and_num = "" : Exit Do
			echo  "Input Path"
		Else
			If Left( path, 1 ) = """" and Right( path, 1 ) = """" Then _
				path = Mid( path, 2, Len( path ) - 2 )

			path_and_num = GetFullPath( path, g_start_in_path )
			path = GetTagJumpParams( path_and_num ).Path

			If ( Flags and ( c.CheckFileExists or c.CheckFolderExists ) ) = 0 Then  Exit Do
			If Flags and c.CheckFileExists Then
				If g_fs.FileExists( path ) Then  Exit Do
			End If
			If Flags and c.CheckFolderExists Then
				If g_fs.FolderExists( path ) Then  Exit Do
			End If
			echo  "Not found : "+ path
		End If
		path = Empty
	Loop
	InputPath = path_and_num
End Function


 
'*************************************************************************
'  <<< [InputCommand] >>> 
'*************************************************************************
Dim  g_InputCommand_Args

Sub  InputCommand( ByVal LeadOrOpt, ByVal Prompt, Opt, AppKey )
	Dim  key, keys, func, e, ds, s, i,  is_show_name

	If IsEmpty( Prompt ) Then  Prompt = "Number or command >"

	If IsEmpty( g_InputCommand_Args ) Then
		Set g_InputCommand_Args = GetWScriptArgumentsUnnamed()

		If VarType( LeadOrOpt ) = vbString Then
			s = LeadOrOpt
			Set LeadOrOpt = new InputCommandOpt : LeadOrOpt.Lead = s
		End If

		If g_InputCommand_Args.Count > 0 Then
			key = g_InputCommand_Args(0)

			s = ""
			For i = 1 To g_InputCommand_Args.Count - 1
				s = s + g_InputCommand_Args(i) + vbLF
			Next
			g_CUI.m_Auto_KeyEnter = vbLF
			If g_CUI.m_Auto_Keys <> "" Then _
				Raise 1, "If InputCommand was used, do not call set_input or SetAutoKeysFromMainArg"
			set_input  s
		End If
	Else
		i = InStr( g_CUI.m_Auto_Keys, vbLF )
		If i = 0 Then
			key = g_CUI.m_Auto_Keys
			If key = "" Then  Set g_InputCommand_Args = new ArrayClass  '// empty array
		Else
			key = Left( g_CUI.m_Auto_Keys, i-1 )
			g_CUI.m_Auto_Keys = Mid( g_CUI.m_Auto_Keys, i+1 )
		End If
	End If

	Do
		is_show_name = False

		If g_InputCommand_Args.Count = 0 Then
			If g_CommandPrompt = 0 Then
				echo_flush
			Else
				echo_line
			End If

			Do
				If not IsEmpty( LeadOrOpt.Lead ) Then _
					echo  LeadOrOpt.Lead

				For Each key  In LeadOrOpt.CommandReplace.Keys
					If IsNumeric( key ) Then
						If LeadOrOpt.MenuCaption.Exists( key ) Then
							s = Replace( LeadOrOpt.MenuCaption.Item( key ), "%name%", LeadOrOpt.CommandReplace.Item( key ) )
							echo  " "+ key +". " + env( s )
						Else
							echo  " "+ key +". " + LeadOrOpt.CommandReplace.Item( key )
						End If
					End If
				Next

				key = input( Prompt )

				If ( key = "" ) and ( g_CommandPrompt = 0 ) Then
					If MsgBox( "Exit this script?", vbYesNo, WScript.ScriptName ) = vbYes Then
						If g_IsFinalizing Then
							Exit Sub
						Else
							Raise  E_UserCancel, "User cancel"
						End If
					End If
				Else
					Exit Do
				End If
			Loop
		End If

		If LeadOrOpt.CommandReplace.Exists( key ) Then
			If IsNumeric( key ) Then  is_show_name = True
			key = LeadOrOpt.CommandReplace.Item( key )
			If LeadOrOpt.CommandReplace.Exists( key ) Then _
				key = LeadOrOpt.CommandReplace.Item( key )
		End If

		If not IsDefined( key ) Then
			If StrComp( key, LeadOrOpt.AllTestName, 1 ) = 0 Then
				keys = LeadOrOpt.CommandReplace.Items
				is_show_name = True
			Else
				If key <> "" Then
					s = "<ERROR msg="""+ key +" function is not found""/>"
					If g_InputCommand_Args.Count = 0 Then  echo  s  Else  Raise  E_NotFoundSymbol, s
				End If
				keys = Array( )
			End If
		Else
			keys = Array( key )
		End If

		For Each key  In keys
			If LeadOrOpt.CommandReplace.Exists( key ) Then _
				key = LeadOrOpt.CommandReplace.Item( key )

			If g_InputCommand_Args.Count = 0 Then  echo_line
			If is_show_name Then  echo  "  ((( ["& key &"] )))"

			If TryStart(e) Then  On Error Resume Next
				Set ds = new CurDirStack
				Set func = GetRef( key )
				func  Opt, AppKey
			If TryEnd Then  On Error GoTo 0
			If e.num <> 0  and  e.num <> E_TestPass Then
				If g_InputCommand_Args.Count > 0 Then
					e.Raise
				Else
					echo  e.DebugHint()
				End If
			End If
			e.Clear
			ds = Empty
		Next

		If g_InputCommand_Args.Count > 0 Then  Exit Do
	Loop
End Sub


Class  InputCommandOpt
	Public  Lead  '// as string
	Public  CommandReplace  '// as Dictionary of string
	Public  MenuCaption     '// as Dictionary of string
	Public  AllTestName     '// as string

	Private Sub  Class_Initialize()
		Set Me.CommandReplace = CreateObject( "Scripting.Dictionary" )
		Set Me.MenuCaption    = CreateObject( "Scripting.Dictionary" )
		Me.AllTestName = "AllTest"
	End Sub
End Class


 
'*************************************************************************
'  <<< [SendKeys] Send keyboard code stroke to OS >>> 
'*************************************************************************
Sub  SendKeys( ByVal window_title, ByVal keycords, ByVal late_time )
	WScript.Sleep late_time
	If window_title <> "" Then
		If not g_sh.AppActivate( window_title ) Then _
			Raise E_NotFoundSymbol, "<ERROR msg='Not found window title' title='"& window_title &"'/>"
	End If
	WScript.Sleep 100
	g_sh.SendKeys keycords
End Sub


 
'*-------------------------------------------------------------------------*
'* ### <<<< [CUI] Class >>>> 
'*-------------------------------------------------------------------------*

Class  CUI

	Public  m_Auto_InputFunc    ' as string of auto input function name
	Public  m_Auto_Src          ' as string of path
	Public  m_Auto_Keys         ' as string of auto input keys
	Public  m_Auto_KeyEnter     ' as string of the character of replacing to enter key
	Public  m_Auto_DebugCount   ' as integer


 
'*************************************************************************
'  <<< [CUI::Class_Initialize] >>> 
'*************************************************************************
Private Sub Class_Initialize
	Me.m_Auto_Keys = ""
	Me.m_Auto_KeyEnter = "."
	Me.m_Auto_DebugCount = Empty
End Sub


 
'*************************************************************************
'  <<< [CUI::input] >>> 
'*************************************************************************
Public Function  input( ByVal msg )
	input = input_sub( msg,  not IsEmpty( GetWScriptArgumentsNamed("GUI_input") ) )
End Function

Public Function  input_sub( ByVal msg, bGUI_input )
	Dim e, en, ed
	Dim InputFunc

	en = Err.Number : ed = Err.Description

	If not IsEmpty( g_EchoObj.m_Buf ) Then  msg = g_EchoObj.m_Buf + vbCRLF + msg
	g_EchoObj.m_Buf = Empty
	g_EchoObj.m_BufN = 0

	If ( msg = g_PauseMsg ) and ( not IsEmpty( m_Auto_Keys ) ) and ( m_Auto_Keys <> "" ) Then
		'// Owner process does not wait in EchoStream
		WScript.StdOut.Write  Left( g_PauseMsg, g_PauseMsgStone )+"*"+Chr(8)+_
		                       Mid( g_PauseMsg, g_PauseMsgStone+1 )
	Else
		If g_CommandPrompt <> 0 Then _
			WScript.StdOut.Write  msg
	End If

	On Error Resume Next

	If ( not IsEmpty( m_Auto_Keys ) ) and ( m_Auto_Keys <> "" ) Then
		If Not IsEmpty( m_Auto_KeyEnter ) Then
			e = InStr( m_Auto_Keys, m_Auto_KeyEnter )
			If e = 0 Then
				input_sub = m_Auto_Keys
				m_Auto_Keys = Empty
			Else
				input_sub = Left( m_Auto_Keys, e - 1 )
				m_Auto_Keys = Mid( m_Auto_Keys, e + 1 )
			End If
		Else
			input_sub = m_Auto_Keys
			m_Auto_Keys = Empty
		End If

		If IsEmpty( m_Auto_DebugCount ) Then
			WScript.StdOut.WriteLine  input_sub
		ElseIf  m_Auto_DebugCount > 1 Then
			WScript.StdOut.WriteLine  input_sub
			m_Auto_DebugCount = m_Auto_DebugCount - 1
		Else
			WScript.StdOut.Write  input_sub
			If bGUI_input Then
				input_sub = InputBox( msg, WScript.ScriptName, "" )
				WScript.StdOut.WriteLine  input_sub
			Else
				input_sub = StdIn_ReadLine_ForJP()
			End If
			WScript.StdOut.WriteLine ""
		End If

	ElseIf IsEmpty( m_Auto_InputFunc ) Then
		If bGUI_input or ( g_CommandPrompt = 0 ) Then
			input_sub = InputBox( msg, WScript.ScriptName, "" )
			If g_CommandPrompt <> 0 Then _
				WScript.StdOut.WriteLine  input_sub
		Else
			input_sub = StdIn_ReadLine_ForJP()
		End If
	Else
		If IsEmpty( m_Auto_Src ) Then
			Set InputFunc = GetRef( m_Auto_InputFunc )
			If Err.Number = 5 Then WScript.StdOut.WriteLine vbCR+vbLF+"Not found function of """+_
				m_Auto_InputFunc +"""": Err.Clear
			If Not IsEmpty( InputFunc ) Then  input_sub = InputFunc( msg )
		Else
			input_sub = call_vbs_t( m_Auto_Src, m_Auto_InputFunc, msg )
			If Err.Number = 5 Then WScript.StdOut.WriteLine vbCR+vbLF+"Not found function of """+_
				m_Auto_InputFunc +""" in """+m_Auto_Src+"""" : Err.Clear
			If IsEmpty( input_sub ) Then  WScript.StdOut.Write  msg : input_sub = StdIn_ReadLine_ForJP()
		End If
	End If

	e = Err.Number : Err.Clear : On Error GoTo 0
	If e <> 0 Then
		If e <> 62 Then Err.Raise e  '62= End Of File (StdIn, ^C)
		Stop:WScript.Quit 1
	End If

	'//=== resume err
	If en <> 0 Then  On Error Resume Next : Err.Raise  en,, ed  '// : On Error GoTo 0
		'// This comment out means not clear Err.Number
End Function


 
'*************************************************************************
'  <<< [CUI::SetAutoKeysFromMainArg] >>> 
'*************************************************************************
Public Sub  SetAutoKeysFromMainArg
	If not IsEmpty( Me.m_Auto_Keys ) and Me.m_Auto_Keys = "" Then
		Me.m_Auto_Keys = GetWScriptArgumentsNamed("set_input")
		Me.m_Auto_KeyEnter = GetWScriptArgumentsNamed("set_input_enter")
		If IsEmpty( Me.m_Auto_KeyEnter ) Then  Me.m_Auto_KeyEnter = "."
		Me.m_Auto_DebugCount = GetWScriptArgumentsNamed("set_input_debug")
	End If
End Sub


 
End Class 


 
'*************************************************************************
'  <<< [StdIn_ReadLine_ForJP] >>> 
'*************************************************************************
Function  StdIn_ReadLine_ForJP()
	Dim  r, i, a
	Const  msg1 = "Command prompt or InputBox can not input over 254 characters."
	Const  msg2 = "Command prompt can not input over 128 characters in not English."
	Const  msg3 = "Input again"

	Do
		r = WScript.StdIn.ReadLine()

		If Len( r ) >= 254 Then
			WScript.StdOut.WriteLine  msg1
			WScript.StdOut.Write  msg3 + ">"
		ElseIf Len( r ) > 128 Then
			For i=1 To 128
				a = Asc( Mid( r, i, 1 ) )
				If a < 0 or a > 127 Then
					r = InputBox( msg2+msg3, WScript.ScriptName )
					While  Len( r ) >= 254
						r = InputBox( msg1+msg3, WScript.ScriptName )
					WEnd
					WScript.StdOut.Write  msg3 +">"+ r +vbCRLF
					Exit For
				End If
			Next
			Exit Do
		Else
			Exit Do
		End If
	Loop

	StdIn_ReadLine_ForJP = r
End Function


 
'*************************************************************************
'  <<< [GetTextFromClipboard] >>> 
'*************************************************************************
Function  GetTextFromClipboard()
	Dim  ec: Set ec = new EchoOff
	Dim  temp_path : temp_path = GetPathOfClipboardText()
	GetTextFromClipboard = ReadFile( temp_path )
	del  temp_path
End Function


 
'*************************************************************************
'  <<< [GetPathOfClipboardText] >>> 
'*************************************************************************
Function  GetPathOfClipboardText()
	Dim  temp_path : temp_path = GetTempPath( "clip_*.txt" )
	Dim  r
	Dim  ec: Set ec = new EchoOff

	AssertExist  g_vbslib_ver_folder +"vbslib_helper.exe"
	mkdir_for  temp_path
	r= RunProg( """"+ g_vbslib_ver_folder +"vbslib_helper.exe"" GetTextFromClipboard /Out:"""+_
		temp_path +"""", "" )
	If r = &h415 Then  Raise E_BadType, "There is not any text in clipboard"
	Assert  r = 0

	GetPathOfClipboardText = temp_path
End Function


 
'*************************************************************************
'  <<< [SetTextToClipboard] >>> 
'*************************************************************************
Sub  SetTextToClipboard( Text )
	Dim  temp_path : temp_path = GetTempPath( "clip_*.txt" )
	Dim  r
	Dim  ec: Set ec = new EchoOff

	AssertExist  g_vbslib_ver_folder +"vbslib_helper.exe"

	CreateFile  temp_path, Text
	r= RunProg( """"+ g_vbslib_ver_folder +"vbslib_helper.exe"" SetTextToClipboard /In:"""+_
		temp_path +"""", "" )
	Assert  r = 0
	del  temp_path
End Sub


 
'*************************************************************************
'  <<< [EditAndRunScript] >>> 
'*************************************************************************
Sub  EditAndRunScript( ByVal ScriptPath, IsKeepThisProcessIfRun )
	If IsEmpty( ScriptPath ) Then  ScriptPath = WScript.ScriptFullName
	ScriptPath = GetFullPath( ScriptPath, Empty )
	echo  ">EditAndRunScript """ + ScriptPath + """"

	RunProg  GetEditorCmdLine( ScriptPath ), ""

	Dim  folder : folder = g_fs.GetParentFolderName( ScriptPath )
	Dim  file_name : file_name = g_fs.GetFileName( ScriptPath )

	If MsgBox( "Closed script will be run."+ vbCRLF+vbCRLF +_
		file_name +vbCRLF+vbCRLF +"folder: "+ folder,_
		vbOKCancel, g_fs.GetFileName( ScriptPath ) ) = vbCancel Then _
			Exit Sub

	g_sh.Run  "wscript.exe //nologo """+ScriptPath+""""
	If not IsKeepThisProcessIfRun Then  WScript.Quit  21
End Sub


 
'*-------------------------------------------------------------------------*
'* ### <<<< File >>>> 
'*-------------------------------------------------------------------------*


 
'-------------------------------------------------------------------------
' ### <<<< [AppKeyClass] Class >>>> 
'-------------------------------------------------------------------------
Const  F_AskIfWarn = 0
Const  F_ErrIfWarn = 1
Const  F_IgnoreIfWarn = 2

Class  AppKeyClass
	Private  m_Key
	Private  m_bAppKey
	Private  m_WritableMode  ' as Flags
	Private  m_PlusWritables()

	Private Sub Class_Initialize()
		m_WritableMode = F_AskIfWarn
		ReDim  m_PlusWritables(-1)
	End Sub

	Public Function  SetKey( Key )
		If not IsEmpty( m_Key ) Then  Err.Raise 1,,"Double Key"
		Set m_Key = Key
		Key.SetKey_sub  Me
		Set SetKey = Key
	End Function
	Public Sub  SetKey_sub( Key )
		Dim  b, pathes

		If not IsEmpty( m_Key ) Then  Err.Raise 1,,"Double Key"
		m_bAppKey = ( Key Is g_AppKey )
		Set m_Key = Key

		b=( IsArray( g_debug_process ) )  'or
		If not b Then  b=( g_debug_process > 0 )
		If (g_is_debug  or g_debug = -1  or  b ) _
				and  IsDefined( "SetupDebugTools" ) Then
			Dim  mode_bk : mode_bk = m_WritableMode : m_WritableMode = F_IgnoreIfWarn
			SetupDebugTools
			m_WritableMode = mode_bk
		End If

		Set pathes = new ArrayClass
		pathes.Add  g_TempFile.m_FolderPath + "\"
		g_CurrentWritables.PushPathes  Me, pathes
	End Sub

	Public Function  IsSame( Key )
		IsSame = ( m_Key Is Key ) and Key.IsSame_sub( Me )
	End Function
	Public Function  IsSame_sub( Key )
		IsSame_sub = ( m_Key Is Key )
	End Function

	Public Sub  CheckGlobalAppKey()
		If not m_bAppKey Then _
			MsgBox "[ERROR] This is not AppKey from main2"
		If not IsSame( g_AppKey ) Then _
			MsgBox "[ERROR] g_AppKey was overrided by unknown"
	End Sub
	Private Sub  Class_Terminate()
		If m_bAppKey Then  CheckGlobalAppKey
	End Sub


 
'*************************************************************************
'  <<< [AppKeyClass::NewWritable] >>> 
'*************************************************************************
Public Function  NewWritable( Pathes )
	Me.CheckGlobalAppKey
	Dim  m : Set m = new Writables : ErrCheck
	m.SetPathes  Me, Pathes
	Set  NewWritable = m
End Function


 
'*************************************************************************
'  <<< [AppKeyClass::SetWritableMode] >>> 
'*************************************************************************
Public Sub  SetWritableMode( Flags )
	If g_AppKey Is Me Then
		If Flags = F_IgnoreIfWarn Then
			Err.Raise  1
		Else
			m_Key.SetWritableMode( Flags )
			Exit Sub
		End If
	End If

	Select Case  Flags
		Case  F_AskIfWarn : echo  ">SetWritableMode  F_AskIfWarn"
		Case  F_ErrIfWarn : echo  ">SetWritableMode  F_ErrIfWarn"
		Case  F_IgnoreIfWarn:echo ">SetWritableMode  F_IgnoreIfWarn"
		Case Else : Err.Raise  1
	End Select

	m_WritableMode = Flags
End Sub

Public Function  GetWritableMode()
	If g_AppKey Is Me Then
		GetWritableMode = m_Key.GetWritableMode()
	Else
		GetWritableMode = m_WritableMode
	End If
End Function


 
'*************************************************************************
'  <<< [AppKeyClass::CheckWritable] >>> 
'*************************************************************************
Public Sub  CheckWritable( Path, Opt )
	Dim  full_path, plus_path, b, i

	If g_AppKey Is Me Then  m_Key.CheckWritable Path, Opt : Exit Sub

	'//=== Stop for debug
	If g_debug_or_test Then
		If StrComp( g_BreakByFName, g_fs.GetFileName( Path ), vbTextCompare ) = 0 Then
			echo_v  "Break by """ + g_BreakByFName + """"
			Stop  '// Look at caller function
		End If
	End If


	'//=== If the folder in writable folder, Exit Sub
	'// If InStr( Path, "sub1\sub2" ) > 0 Then  Stop  '// for debug
	full_path = g_CurrentWritables.CheckWritablePart( Path, Opt )
	If IsEmpty( full_path ) Then  Exit Sub

	full_path = Me.CheckPlusWritablePart( full_path )
	If IsEmpty( full_path ) Then  Exit Sub


	'//=== Ask to add "plus writable"
	If not g_CurrentWritables.CanPlusNoAsk( full_path, Opt, plus_path ) Then  '//[out] plus_path
		plus_path = full_path
		Me.Ask  plus_path, Empty
	End If


	'//=== add "plus writable"
	If Right( plus_path, 2 ) = "\." Then
		plus_path = Left( plus_path, Len( plus_path ) - 1 )
	ElseIf Right( plus_path, 1 ) <> "\" Then
		plus_path = plus_path + "\"
	End If
	ReDim Preserve  m_PlusWritables( UBound( m_PlusWritables ) + 1 )
	m_PlusWritables( UBound( m_PlusWritables ) ) = plus_path

End Sub


 
'*************************************************************************
'  <<< [AppKeyClass::CheckPlusWritablePart] >>> 
'*************************************************************************
Public Function  CheckPlusWritablePart( AbsPath )
	Dim  writable, full_path

	If g_AppKey Is Me Then  CheckPlusWritablePart = m_Key.CheckPlusWritablePart( AbsPath ) : Exit Function

	full_path = AbsPath
	If Right( full_path, 2 ) <> "\." Then  full_path = full_path + "\."

	For Each writable  In m_PlusWritables
		If StrComp( writable, Left( full_path, Len( writable ) ), 1 ) = 0 Then  Exit Function
	Next
	CheckPlusWritablePart = AbsPath
End Function


 
'*************************************************************************
'  <<< [AppKeyClass::Ask] >>> 
'*************************************************************************
Public Sub  Ask( CheckPath, Message )
	If g_AppKey Is Me Then  m_Key.Ask  CheckPath, Message : Exit Sub

	Dim  s, msg2
	If IsEmpty( Message ) Then
		msg2 = "" : If exist( CheckPath ) Then  msg2 = "Cannot overwrite, "
	Else
		msg2 = Message +", "
	End If

	Dim  writable
	For Each writable  In g_CurrentWritables.CurrentPathes
		If Right( writable, 3 ) = "\*\" Then
			If Left( writable, Len(writable) - 2 ) = Left( CheckPath, Len( writable ) - 2 ) or _
				 Left( writable, Len(writable) - 3 ) = CheckPath Then
				If g_fs.FileExists( CheckPath ) Then
					msg2 = "Cannot overwrite NOT NEW file, "
				Else
					msg2 = "Cannot overwrite NOT NEW folder, "
				End If
			End If
		End If
	Next

	If m_WritableMode <> F_ErrIfWarn Then
		echo_v  Me.GetWarningMessage( msg2, CheckPath )
	End If

	If m_WritableMode = F_AskIfWarn Then
		Do
			echo_flush
			If g_CommandPrompt = 0 Then
				s = InputBox( "File is be output at the path displayed at previous dialog window." & vbCRLF & _
						"(Y) Yes. Enable to output file" & vbCRLF & "(A) Yes all. Enable to output any files" & vbCRLF & _
						"(N) No. Program is finished" & vbCRLF & "(R) Path is shown again", _
						"[WARNING] " +msg2+ "Out of Writable", "Y" )
			Else
				s = InputBox( "File is output at the path displayed at command prompt." & vbCRLF & _
						"(Y) Yes. Enable to output file" & vbCRLF & "(A) Yes all. Enable to output any files" & vbCRLF & _
						"(N) No. Program is finished" & vbCRLF & "(R) Path is shown again", _
						"[WARNING] " +msg2+ "Out of Writable", "Y" )
			End If

			If s="Y" or s="y" Then
				Exit Do
			ElseIf s="A" or s="a" Then
				If MsgBox( "A system may be broken and data may be lost. All OK?", _
						vbYesNo + vbDefaultButton2, "WARNING" ) = vbNo Then
					Me.Raise  "Out of Writable """ & CheckPath & """"
				End If
				SetWritableMode  F_IgnoreIfWarn
				Exit Do
			ElseIf s="R" or s="r" Then
				MsgBox  CheckPath, vbOKOnly, "[WARNING] Out of Writable"
			Else
				Me.Raise  "Out of Writable """ & CheckPath & """"
			End If
		Loop
	End If

	If m_WritableMode = F_ErrIfWarn Then
		echo_v  Me.GetWarningMessage( msg2, CheckPath )
		Me.Raise  msg2+"Out of Writable """ & CheckPath & """"
	End If
End Sub


 
'*************************************************************************
'  <<< [AppKeyClass::Raise] >>> 
'*************************************************************************
Public Sub  Raise( Message )
	If Err.Number = 0 Then
		Err.Raise  E_OutOfWritable,, Message
			' Watch  g_CurrentWritables.CurrentPathes and Path (CheckPath)
			' or call "g_AppKey.Watch"
	Else
		Err.Raise  Err.Number, Err.Source, Err.Description
	End If
End Sub


 
'*************************************************************************
'  <<< [AppKeyClass::InPath] >>> 
'*************************************************************************
Public Function  InPath( ChkPathes, WritablePathes )
	If TypeName( ChkPathes ) = "ArrayClass" Then
		InPath = InPath( ChkPathes.Items, WritablePathes )
		Exit Function
	End If
	If TypeName( WritablePathes ) = "ArrayClass" Then
		InPath = InPath( ChkPathes, WritablePathes.Items )
		Exit Function
	End If

	Dim  c, w, b, cs, ws

	'// ChkPathes To abs path
	If IsArray( ChkPathes ) Then
		ReDim  cs( UBound( ChkPathes ) )
		For i=0 To UBound( cs )
			cs(i) = g_fs.GetAbsolutePathName( ChkPathes(i) ) + "\"
		Next
	Else
		ReDim cs(0)
		cs(0) = g_fs.GetAbsolutePathName( ChkPathes ) + "\"
	End If


	'// WritablePathes To abs path
	If IsArray( WritablePathes ) Then
		ReDim  ws( UBound( WritablePathes ) )
		For i=0 To UBound( ws )
			ws(i) = g_fs.GetAbsolutePathName( WritablePathes(i) ) + "\"
		Next
	Else
		ReDim ws(0)
		ws(0) = g_fs.GetAbsolutePathName( WritablePathes ) + "\"
	End If


	'// Compare path
	For Each c  In cs
		b = False
		For Each w  In ws
			If Left( c, Len(w) ) = w Then  b = True : Exit For
		Next
		If not b Then  InPath = False : Exit Function
	Next
	InPath = True
End Function


 
'*************************************************************************
'  <<< [AppKeyClass::BreakByPath] >>> 
'*************************************************************************
Public Function  BreakByPath( Path )
	If StrComp( g_BreakByFName, g_fs.GetFileName( Path ), vbTextCompare ) = 0 Then
		echo_v  "Break by """ + g_BreakByFName + """"
		Stop  '// Look at caller function
	End If
End Function


 
'*************************************************************************
'  <<< [AppKeyClass::BreakByWildcard] >>> 
'*************************************************************************
Public Sub  BreakByWildcard( Path, Flags )
	Dim  folder, fnames()
	Dim  fname

	If IsEmpty( g_BreakByFName ) Then  Exit Sub

	ExpandWildcard  Path, Flags, folder, fnames
	For Each fname in fnames
		If StrComp( g_BreakByFName, g_fs.GetFileName( fname ), vbTextCompare ) = 0 Then
			echo_v  "Break by """ + g_BreakByFName + """"
			Stop  '// Look at caller function
		End If
	Next
End Sub


 
'*************************************************************************
'  <<< [AppKeyClass::GetWarningMessage] >>> 
'*************************************************************************
Public Function  GetWarningMessage( msg2, CheckPath )

	s = ""

	If Err.Number <> 0 Then
		s=s+ vbCRLF + GetErrStr( Err.Number, Err.Description ) + vbCRLF + vbCRLF +_
			"The following files may be written until middle of range" + vbCRLF
	End If

	s=s+ "<Warning msg=""" +msg2+ "Out of Writable, see the help of SetWritableMode."">"+vbCRLF

	For Each writable  In g_CurrentWritables.CurrentPathes
		s=s+ "  <Writable path="""+ Left( writable, Len( writable ) - 1 ) +"""/>"+vbCRLF
	Next
	For Each writable  In m_PlusWritables
		s=s+ "  <PlusWritable path="""+ Left( writable, Len( writable ) - 1 ) +"""/>"+vbCRLF
	Next

	s=s+ vbCRLF+vbCRLF +"  <Requested path=""" & CheckPath & """/>"+vbCRLF


	GetWarningMessage = s+ "</Warning>"
End Function


 
'*************************************************************************
'  <<< [AppKeyClass::Watch] >>> 
'*************************************************************************
Public Sub  Watch()
	If g_AppKey Is Me Then  m_Key.Watch : Exit Sub

	echo  Me.GetWarningMessage( "Watch of ", "" )
End Sub

 
End Class 


 
'*************************************************************************
'  <<< [CurrentWritables] Class >>> 
'*************************************************************************
Class  CurrentWritables
	Private  m_PathesStack ' as ArrayClass of ArrayClass

	Private  m_ProgramFiles
	Private  m_windir
	Private  m_APPDATA
	Private  m_LOCALAPPDATA

	Public Property Get  CurrentPathes()
		If m_PathesStack.Count > 0 Then
			CurrentPathes = m_PathesStack( m_PathesStack.Count-1 ).Items
		Else
			CurrentPathes = m_PathesStack.Items
		End If
	End Property
	Public Property Get  PathesStack : Set PathesStack = m_PathesStack : End Property


	Private Sub  Class_Initialize()
		Set  m_PathesStack = new ArrayClass : ErrCheck

		m_ProgramFiles = g_sh.ExpandEnvironmentStrings( "%ProgramFiles%" )
		m_windir       = g_sh.ExpandEnvironmentStrings( "%windir%" )
		m_APPDATA      = g_sh.ExpandEnvironmentStrings( "%APPDATA%" )
		m_LOCALAPPDATA = g_sh.ExpandEnvironmentStrings( "%LOCALAPPDATA%" )

		If m_ProgramFiles = "%ProgramFiles%" Then  m_ProgramFiles = Empty
		If m_windir       = "%windir%"       Then  m_windir       = Empty
		If m_APPDATA      = "%APPDATA%"      Then  m_APPDATA      = Empty
		If m_LOCALAPPDATA = "%LOCALAPPDATA%" Then  m_LOCALAPPDATA = Empty
	End Sub


	Public Sub  PushPathes( AppKey, Pathes )
		Dim  i
		If not g_AppKey.IsSame( AppKey ) Then  Err.Raise 1,,"Invalied AppKey"
		Dim  new_pathes : Set new_pathes = new ArrayClass : ErrCheck
		new_pathes.Copy  Pathes
		m_PathesStack.Push  new_pathes
	End Sub


	Public Sub  PopPathes( AppKey, Pathes )
		Dim  i,j

		If not g_AppKey.IsSame( AppKey ) Then  Err.Raise 1,,"Invalied AppKey"

		For i=m_PathesStack.Count-1 To 0 Step -1
			If Pathes.Count = m_PathesStack(i).Count Then
				For j=0 To Pathes.Count-1
					If Pathes(j) <> m_PathesStack(i)(j) Then  Exit For
				Next
				If j = Pathes.Count Then  Exit For  '// If same all Pathes
			End If
		Next
		If i = -1 Then  Err.Raise 1

		For i=i To m_PathesStack.Count-2
			Set m_PathesStack(i) = m_PathesStack(i+1)
		Next
		m_PathesStack.Pop
	End Sub


	Public Function  CheckWritablePart( Path, Opt )
	'// return value = Empty(=Pass), abs path out of writable
		Dim  full_path, writable, s
		Dim  c : Set c = g_VBS_Lib

		full_path = g_fs.GetAbsolutePathName( Path )
		If g_TempFile.m_FolderPath = full_path Then  Exit Function
		If Right( full_path, 2 ) <> "\." Then  If Right( full_path, 2 ) <> ":\" Then  full_path = full_path + "\."

		For Each writable  In Me.CurrentPathes
			If StrComp( writable, Left( full_path, Len( writable ) ), 1 ) = 0 Then  Exit Function
		Next

		full_path = Left( full_path, Len( full_path ) - 1 )

		If Opt = c.ForMkDir Then
			For Each writable  In Me.CurrentPathes
				If StrComp( full_path, Left( writable, Len( full_path ) ), 1 ) = 0 Then  Exit Function
			Next
		End If

		full_path = Left( full_path, Len( full_path ) - 1 )
		CheckWritablePart = full_path
	End Function


	Public Function  CanPlusNoAsk( AbsPath, Opt, out_PlusPath )
		Dim  full_path, writable, i

		full_path = AbsPath
		If Right( full_path, 2 ) <> "\." Then  full_path = full_path + "\."

		If not exist( full_path ) Then
		 '// If the folder already exists, do not writable

			For Each writable  In Me.CurrentPathes
				If Right( writable, 3 ) = "\*\" Then
					If Left( writable, Len(writable) - 2 ) = Left( full_path, Len( writable ) - 2 ) or _
						 Left( writable, Len(writable) - 3 ) = full_path Then

						out_PlusPath = Left( full_path, InStr( Len( writable ) - 1, full_path, "\" ) )
							'// writable="fo\*\", full_path="fo\sub\sub2\f.txt", out_PlusPath="fo\sub\"
						CanPlusNoAsk = True
						Exit Function
					End If
				End If
			Next
		End If
		CanPlusNoAsk = False
	End Function


	Public Sub  AskFileAccess( AbsPath )
		Const  s = "System folder access"

		If Left( AbsPath, Len( g_TempFile.m_FolderPath ) + 1 ) = g_TempFile.m_FolderPath + "\" Then _
			Exit Sub

		If not IsEmpty( m_ProgramFiles ) Then _
			If Left( AbsPath, Len( m_ProgramFiles ) ) = m_ProgramFiles or _
				 Left( m_ProgramFiles, Len( AbsPath ) ) = AbsPath Then _
				g_AppKey.Ask  AbsPath, s

		If not IsEmpty( m_windir ) Then _
			If Left( AbsPath, Len( m_windir ) ) = m_windir or _
				 Left( m_windir, Len( AbsPath ) ) = AbsPath Then _
				g_AppKey.Ask  AbsPath, s

		If not IsEmpty( m_APPDATA ) Then _
			If Left( AbsPath, Len( m_APPDATA ) ) = m_APPDATA or _
				 Left( m_APPDATA, Len( AbsPath ) ) = AbsPath Then _
				g_AppKey.Ask  AbsPath, s

		If not IsEmpty( m_LOCALAPPDATA ) Then _
			If Left( AbsPath, Len( m_LOCALAPPDATA ) ) = m_LOCALAPPDATA or _
				 Left( m_LOCALAPPDATA, Len( AbsPath ) ) = AbsPath Then _
				g_AppKey.Ask  AbsPath, s
	End Sub

End Class


 
'*************************************************************************
'  <<< [WritablesStack] Class >>> 
'*************************************************************************
Class  WritablesStack
	Private  m_AppKey
	Private  m_Pathes

	Public Sub  PushPathes( AppKey, Pathes )
		Set  m_Pathes = new ArrayClass : ErrCheck
		m_Pathes.Copy  Pathes
		Set  m_AppKey = AppKey
		g_CurrentWritables.PushPathes  AppKey, Pathes
	End Sub

	Private Sub Class_Terminate()
		g_CurrentWritables.PopPathes  m_AppKey, m_Pathes
	End Sub
End Class


 
'*************************************************************************
'  <<< [Writables] Class >>> 
'*************************************************************************
Class  Writables
	Private  m_Pathes()
	Private  m_AppKey

	Public Sub  SetPathes( AppKey, ByVal Pathes )
		Dim  full_path, i, j

		If not IsEmpty( m_AppKey ) Then  Err.Raise 1,,"Double key"
		If not g_AppKey.IsSame( AppKey ) Then  Err.Raise 1,,"Invalied AppKey"

		If TypeName( Pathes ) = "ArrayClass" Then
			Pathes = Pathes.Items
		ElseIf not IsArray( Pathes ) Then
			Pathes = Array( Pathes )
		End If

		ReDim  m_Pathes( UBound( Pathes ) + 1 )
		j=0
		For i=0 To UBound( Pathes )
			full_path = env( Pathes(i) )
			If not IsEmpty( full_path ) Then
				full_path = GetFullPath( full_path, Empty )
				g_CurrentWritables.AskFileAccess  full_path
				m_Pathes(j) = full_path + "\"
				j=j+1
			End If
		Next
		ReDim Preserve  m_Pathes( j )
		m_Pathes( j ) = g_TempFile.m_FolderPath + "\"  '// Last is Temp

		Set m_AppKey = AppKey
	End Sub

	Public Function  Enable()
		Dim  st : Set st = new WritablesStack : ErrCheck
		st.PushPathes  m_AppKey, m_Pathes
		Set Enable = st
	End Function
End Class


 
'*************************************************************************
'  <<< [SetWritableMode] >>> 
'*************************************************************************
Sub  SetWritableMode( Flags )
	g_AppKey.SetWritableMode  Flags
End Sub


 
'*************************************************************************
'  <<< [CheckWritable] Check not to modify out of working folder >>> 
' comment
'  - If path is out of workfolder, raise error of E_OutOfWritable.
'  - This function is overritable, because other APIs calling this and g_CurrentWritables
'    can overrite.
'*************************************************************************
'Sub  CheckWritable( Path )
'  Dim  full_path
'
'  full_path = g_CurrentWritables.CheckWritable( Path )
'  If IsEmpty( full_path ) Then  Exit Sub
'  g_AppKey.Ask  full_path, Empty
'End Sub


 
'*************************************************************************
'  <<< [set_workfolder] old function >>> 
'*************************************************************************
Sub  set_workfolder( ByVal dir )
	ThisIsOldSpec
End Sub


Class  WorkFolderStack
	Private Sub Class_Initialize()
		ThisIsOldSpec
	End Sub
	Public Sub  set_( x ) : End Sub
End Class


 
'*************************************************************************
'  <<< [SetBreakByFName] >>> 
'*************************************************************************
Sub  SetBreakByFName( FName )
	g_BreakByFName = FName
End Sub


 
'*************************************************************************
'  <<< [IsWriteAccessDenied] >>> 
'*************************************************************************
Function  IsWriteAccessDenied( ErrNumber, Path, FolderOrFile, in_out_nRetry )
	If ErrNumber = E_WriteAccessDenied Then
		Const  msg = "Write access denied.(70)"
		Set c = g_VBS_Lib
		max_retry = g_FileSystemMaxRetryMSec / g_FileSystemRetryMSec


		'//=== error of overwrite file on folder or folder on file
		If (FolderOrFile and c.File)=0 Then
			If g_fs.FileExists( Path ) Then  Raise  ErrNumber, msg
		End If
		If (FolderOrFile and c.Folder)=0 Then
			If g_fs.FolderExists( Path ) Then  Raise  ErrNumber, msg
		End If


		'//=== If read only
		is_read_only = False
		If g_fs.FileExists( Path ) Then
			If g_fs.GetFile( Path ).Attributes and ReadOnly Then
				is_read_only = True
			End If
		End If
		If g_fs.FolderExists( Path ) Then
			If g_fs.GetFolder( Path ).Attributes and ReadOnly Then
				is_read_only = True
			End If
		End If
		If is_read_only Then
			Err.Raise  70,, "<ERROR msg=""This is read only"" path="""+ Path +"""/>"
		End If


		'//=== echo warning
		If g_CommandPrompt = 0 Then
			If MsgBox( msg, vbRetryCancel ) = vbCancel Then   Raise  ErrNumber, msg
		Else
			in_out_nRetry = in_out_nRetry + 1
			If in_out_nRetry = 1 Then
				echo_v  "<WARNING msg="""+ msg +""" msg2=""Retrying"" retry_count=""1"""+ _
					vbCRLF+ " path="""+ Path +"""/>"
			ElseIf in_out_nRetry <= max_retry Then
				echo_v  "<WARNING msg="""+ msg +""" msg2=""Retrying"" retry_count="""& _
					in_out_nRetry &"""/>"
			Else
				Raise  ErrNumber, msg
			End If

			WScript.Sleep  g_FileSystemRetryMSec
		End If
		IsWriteAccessDenied = True
	Else
		IsWriteAccessDenied = False
	End If
End Function


 
'*************************************************************************
'  <<< [cd] >>> 
'*************************************************************************
Sub  cd( dir )
	echo  ">cd  """ & dir & """"
	cd_sub  dir
End Sub

Sub  cd_sub( dir )
	old_current = g_sh.CurrentDirectory
	If Mid( old_current, 2 ) = ":" Then
		g_sh.CurrentDirectory = Left( old_current, 3 )
			'// Change to root folder of current drive for unlock
	End If

	ErrCheck : On Error Resume Next
		g_sh.CurrentDirectory = dir
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = E_WIN32_FILE_NOT_FOUND or en = E_WIN32_DIRECTORY or en = E_WIN32_INVALID_NAME Then _
		Err.Raise en,, "<ERROR msg=""It is not folder."" path=""" + dir +_
			""" current="""+ g_sh.CurrentDirectory +"""/>"
	If en <> 0 Then  Err.Raise en,,ed

	If not IsFullPath( dir ) Then  echo  g_sh.CurrentDirectory +">"
End Sub


 
'*************************************************************************
'  <<< [CurDirStack] >>> 
'*************************************************************************
Class  CurDirStack

	Public  m_Prev

	Private Sub Class_Initialize
		m_Prev = g_sh.CurrentDirectory
	End Sub

	Private Sub Class_Terminate
		g_sh.CurrentDirectory = m_Prev
	End Sub
End Class


 
'*************************************************************************
'  <<< [pushd] >>> 
'*************************************************************************
Dim  g_pushd_stack()
Dim  g_pushd_stack_n

Sub  pushd( dir )
	echo  ">pushd  """ & dir & """"

	g_pushd_stack_n = g_pushd_stack_n + 1
	Redim Preserve  g_pushd_stack( g_pushd_stack_n )

	g_pushd_stack( g_pushd_stack_n ) = g_sh.CurrentDirectory
	cd_sub  dir
End Sub


 
'*************************************************************************
'  <<< [popd] >>> 
'*************************************************************************
Sub  popd
	echo  ">popd"
	Dim  sh

	If g_pushd_stack_n < 1 Then Exit Sub

	Set sh = WScript.CreateObject("WScript.Shell")
	sh.CurrentDirectory = g_pushd_stack( g_pushd_stack_n )

	g_pushd_stack_n = g_pushd_stack_n - 1
End Sub


 
'*************************************************************************
'  <<< [cd_UpperCaseDrive] >>> 
'*************************************************************************
Sub  cd_UpperCaseDrive()
	current = g_sh.CurrentDirectory
	g_sh.CurrentDirectory = "\"
	g_sh.CurrentDirectory = UCase( Left( current, 1 ) ) + Mid( current, 2 )
End Sub


 
'*************************************************************************
'  <<< [copy] >>> 
'*************************************************************************
Sub  copy( SrcPath, DstFolderPath )
	Dim  c : Set c = g_VBS_Lib

	If g_Vers("CopyTargetIsFileOrFolder") Then
		copy_sub  SrcPath, DstFolderPath, Empty
	Else
		copy_sub  SrcPath, DstFolderPath, c.CopyV4
	End If
End Sub


Sub  copy_sub( SrcPath, DstPath, Opt )
	Dim  en, ed, n_retry, fo, src, dst, dst_fo, command_str
	Dim  c : Set c = g_VBS_Lib
	If Opt = c.CopyRen Then  command_str = ">copy_ren  """  Else  command_str = ">copy  """

	echo  command_str & SrcPath & """, """ & DstPath & """"
	Set en = new EchoOff


	'//=== If SrcPath had Wild card
	If IsWildcard( SrcPath )  or  ( Opt = c.CopyRen  and  g_fs.FolderExists( SrcPath ) ) Then
		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath, c.File or c.SubFolder

		src = SrcPath : CutLastOf  src, "\", Empty : CutLastOf  src, "/", Empty
		If Opt = c.CopyRen Then  src = src +"\*"
		If not g_fs.FolderExists( GetParentFullPath( src ) ) Then _
			Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		g_AppKey.CheckWritable  DstPath + "\.", Empty  '// "\." is for able to make writable folder

		If Opt = c.CopyRen Then  del  DstPath : mkdir  DstPath

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFile  src, DstPath, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.File, n_retry ) Then  Exit Do
		Loop
		If en = E_FileNotExist Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFolder  src, DstPath, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en = E_PathNotFound Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed


	'//=== If SrcPath is file
	ElseIf g_fs.FileExists( SrcPath ) Then
		If g_debug_or_test Then  g_AppKey.BreakByPath DstPath

		If g_fs.FolderExists( DstPath )  and  Opt <> c.CopyRen  Then
			dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		Else
			If Opt = c.CopyV4 Then  '// vbslib ver4
				dst_fo = DstPath
				dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
			Else  '// vbslib ver3 or c.CopyRen
				dst_fo = GetParentFullPath( DstPath )
				dst = DstPath
				If IsEmpty( Opt ) Then  ThisIsOldSpec  '// use copy_ren
			End If
			If not g_fs.FolderExists( dst_fo ) Then  mkdir  dst_fo
		End If

		If Opt = c.CopyV4 Then  '// vbslib ver4
			g_AppKey.CheckWritable  dst, Empty  '// "\." is for able to make writable folder
			If g_fs.FolderExists( dst ) Then  del  dst  '// for Replace from folder to file
		Else
			If not g_fs.FileExists( dst ) Then
				g_AppKey.CheckWritable  dst + "\.", Empty  '// "\." is for able to make writable folder
			Else
				g_AppKey.CheckWritable  dst, Empty
				If g_fs.FolderExists( dst ) Then  del  dst  '// for Replace from folder to file
			End If
		End If

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFile  SrcPath, dst, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.File, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' If SrcPath is folder
	ElseIf g_fs.FolderExists( SrcPath ) Then
		If g_debug_or_test Then  g_AppKey.BreakByPath  DstPath

		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath+"\*", c.File or c.SubFolder

		dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		g_AppKey.CheckWritable  dst, Empty

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFolder  SrcPath, dst, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' not found
	Else
		echo  command_str & SrcPath & """, """ & DstPath & """"
		Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
	End If
End Sub


 
'*************************************************************************
'  <<< [copy_ren] >>> 
'*************************************************************************
Sub  copy_ren( SrcPath, DstPath )
	Dim  c : Set c = g_VBS_Lib
	copy_sub  SrcPath, DstPath, c.CopyRen
End Sub


 
'*************************************************************************
'  <<< [copy_ren_tmp] >>> 
'*************************************************************************
Sub  copy_ren_tmp( SrcPath, DstPath )
	Dim  c : Set c = g_VBS_Lib
	copy_sub_tmp  SrcPath, DstPath, c.CopyRen
End Sub

Sub  copy_sub_tmp( SrcPath, DstPath, Opt )
	Dim  en, ed, n_retry, fo, src, dst, dst_fo, command_str
	Dim  c : Set c = g_VBS_Lib
	If Opt = c.CopyRen Then  command_str = ">copy_ren  """  Else  command_str = ">copy  """

	echo  command_str & SrcPath & """, """ & DstPath & """"
	Dim ec : Set ec = new EchoOff


	'//=== If DstPath had Wild card
	If IsWildcard( DstPath )  and  Opt = c.CopyRen Then
		Dim  src_path_wild,  dst_path_wild,  src_fo_path,  dst_fo_path,  src_fname,  dst_fname,  dst_path
		Dim  src_i,  src_left,  src_mid,  src_right,  src_left_len,  src_right_len,  src_len_not_wild
		Dim  dst_i,  dst_left,  dst_mid,  dst_right

		If not IsWildcard( SrcPath ) Then  Error

		src_path_wild = g_fs.GetFileName( SrcPath )
		dst_path_wild = g_fs.GetFileName( DstPath )
		src_fo_path = GetParentFullPath( SrcPath ) : CutLastOf  src_fo_path, "\", Empty : If IsWildcard( src_fo_path ) Then Error
		dst_fo_path = GetParentFullPath( DstPath ) : CutLastOf  dst_fo_path, "\", Empty : If IsWildcard( dst_fo_path ) Then Error
		src_i            = InStr( src_path_wild, "*" )
		src_left         = Left(  src_path_wild,  src_i - 1 )
		src_right        = Right( src_path_wild,  Len( src_path_wild ) - src_i )
		If InStr( src_i + 1,  src_path_wild,  "*" ) > 0 Then
			If src_i <> 1 Then Error
			If Right( src_path_wild, 1 ) <> "*" Then Error
			src_mid = Mid( src_path_wild, 2, Len( src_path_wild ) - 2 )
		End If
		src_left_len     = Len( src_left )
		src_right_len    = Len( src_right )
		src_len_not_wild = Len( src_path_wild ) - 1
		dst_i            = InStr( dst_path_wild, "*" )
		dst_left         = Left(  dst_path_wild,  dst_i - 1 )
		dst_right        = Right( dst_path_wild,  Len( dst_path_wild ) - dst_i )
		If InStr( dst_i + 1,  dst_path_wild,  "*" ) > 0 Then
			If dst_i <> 1 Then Error
			If Right( dst_path_wild, 1 ) <> "*" Then Error
			dst_mid = Mid( dst_path_wild, 2, Len( dst_path_wild ) - 2 )
		End If
		If IsEmpty( src_mid ) <> IsEmpty( dst_mid ) Then Error

		mkdir  dst_fo_path

		Set fo = g_fs.GetFolder( src_fo_path )
		For Each src  In fo.Files
			src_fname = src.Name
			dst_fname = Empty
			If IsEmpty( dst_mid ) Then
				If Left( src_fname, src_left_len ) = src_left  and  Right( src_fname, src_right_len ) = src_right Then

					dst_fname = dst_left + _
											Mid( src_fname,  src_i,  Len( src_fname ) - src_len_not_wild ) + _
											dst_right
						'// (sample) src_path_wild = "AB*CD" : dst_path_wild = "abcd*efgh"
						'//               1234567                 12345678901
						'//  src_fname = "AB123CD" : dst_fname = "abcd123efgh"
				End If
			Else
				If InStr( src_fname, src_mid ) > 0 Then
					dst_fname = Replace( src_fname, src_mid, dst_mid )
				End If
			End If
			If not IsEmpty( dst_fname ) Then
				dst_path = dst_fo_path +"\"+ dst_fname
				If g_fs.FolderExists( dst_path ) Then  Error
				g_fs.CopyFile  src_fo_path +"\"+ src_fname,  dst_path
			End If
		Next

		For Each src  In fo.SubFolders
			src_fname = src.Name
			dst_fname = Empty
			If IsEmpty( dst_mid ) Then
				If Left( src_fname, src_left_len ) = src_left  and  Right( src_fname, src_right_len ) = src_right Then
					dst_fname = dst_left + _
											Mid( src_fname,  src_i,  Len( src_fname ) - src_len_not_wild ) + _
											dst_right
				End If
			Else
				If InStr( src_fname, src_mid ) > 0 Then
					dst_fname = Replace( src_fname, src_mid, dst_mid )
				End If
			End If
			If not IsEmpty( dst_fname ) Then
				copy_sub_tmp  src_fo_path +"\"+ src_fname +"\*",  dst_fo_path +"\"+ dst_fname,  c.CopyRen
			End If
		Next


	'//=== If SrcPath had Wild card
	ElseIf IsWildcard( SrcPath )  or  ( Opt = c.CopyRen  and  g_fs.FolderExists( SrcPath ) ) Then
		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath, c.File or c.SubFolder

		src = SrcPath : CutLastOf  src, "\", Empty : CutLastOf  src, "/", Empty
		If Opt = c.CopyRen  and  g_fs.FolderExists( src ) Then  Assert( not IsWildcard( src ) ) : src = src +"\*"
		If not g_fs.FolderExists( GetParentFullPath( src ) ) Then _
			Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		g_AppKey.CheckWritable  DstPath + "\.", Empty  '// "\." is for able to make writable folder

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFile  src, DstPath, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.File, n_retry ) Then  Exit Do
		Loop
		If en = E_FileNotExist Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFolder  src, DstPath, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en = E_PathNotFound Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed


	'//=== If SrcPath is file
	ElseIf g_fs.FileExists( SrcPath ) Then
		If g_debug_or_test Then  g_AppKey.BreakByPath DstPath

		If g_fs.FolderExists( DstPath )  and  Opt <> c.CopyRen  Then
			dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		Else
			If Opt = c.CopyV4 Then  '// vbslib ver4
				dst_fo = DstPath
				dst = GetAbsPatg( g_fs.GetFileName( SrcPath ), DstPath )
			Else  '// vbslib ver3 or c.CopyRen
				dst_fo = GetParentFullPath( DstPath )
				dst = DstPath
				If IsEmpty( Opt ) Then  ThisIsOldSpec  '// use copy_ren
			End If
			If not g_fs.FolderExists( dst_fo ) Then  mkdir  dst_fo
		End If

		If Opt = c.CopyV4 Then  '// vbslib ver4
			g_AppKey.CheckWritable  dst, Empty  '// "\." is for able to make writable folder
			If g_fs.FolderExists( dst ) Then  del  dst  '// for Replace from folder to file
		Else
			If not g_fs.FileExists( dst ) Then
				g_AppKey.CheckWritable  dst + "\.", Empty  '// "\." is for able to make writable folder
			Else
				g_AppKey.CheckWritable  dst, Empty
				If g_fs.FolderExists( dst ) Then  del  dst  '// for Replace from folder to file
			End If
		End If

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFile  SrcPath, dst, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.File, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' If SrcPath is folder
	ElseIf g_fs.FolderExists( SrcPath ) Then
		If g_debug_or_test Then  g_AppKey.BreakByPath  DstPath

		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath+"\*", c.File or c.SubFolder

		dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		g_AppKey.CheckWritable  dst, Empty

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CopyFolder  SrcPath, dst, True
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' not found
	Else
		echo  command_str & SrcPath & """, """ & DstPath & """"
		Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
	End If
End Sub

 
'*************************************************************************
'  <<< [copy_ex] >>> 
'*************************************************************************
Sub  copy_ex( ByVal SrcPath, DstPath, Option_ )
	Dim  folder, fnames(), fname, fname2, tag, to_tag, dst, fo, ec
	Dim  c : Set c = g_VBS_Lib

	echo  ">copy_ex  """+ SrcPath +""" """+ DstPath +""", "& Option_

	g_AppKey.CheckWritable  DstPath +"\.", Empty  '// "\." is for able to make writable folder


	Select Case  Option_

	 Case c.CutTag
		tag = Replace( g_fs.GetFileName( SrcPath ), "*", "" )
		If Right( tag, 1 ) = "." Then
			to_tag = "."
		Else
			to_tag = ""
		End If

	 Case c.MakeShortcutFiles
		SrcPath = GetFullPath( SrcPath, Empty )

	End Select


	ExpandWildcard  SrcPath, c.File or c.SubFolder, folder, fnames
	For Each fname in fnames

		Select Case  Option_

		 Case  c.CutTag
			echo  fname
			dst = DstPath +"\"+ g_fs.GetParentFolderName( fname ) +"\"
			fname2 = g_fs.GetFileName( fname )
			If InStr( fname2, "." ) = 0 Then
				dst = dst + Left( fname2, Len( fname2 ) - Len( tag ) + 1 )
			Else
				dst = dst + Replace( fname2, tag, to_tag )
			End If
			fo = g_fs.GetParentFolderName( dst )
			If not g_fs.FolderExists( fo ) Then  mkdir  fo
			g_fs.CopyFile  folder +"\"+ fname, dst

		 Case  c.ExistOnly
			dst = DstPath +"\"+ fname
			If g_fs.FileExists( dst ) Then
				echo  fname
				g_fs.CopyFile  folder +"\"+ fname,  dst
			End If

		 Case  c.MakeShortcutFiles
			lnk_path = DstPath +"\"+ AddLastOfFileName( fname, ".lnk" )
			Set shcut = g_sh.CreateShortcut( lnk_path )
			shcut.TargetPath = folder +"\"+ fname
			mkdir_for  lnk_path
			shcut.Save

		 Case Else
			echo  fname
			g_fs.CopyFile  folder +"\"+ fname, DstPath +"\"+ fname
		End Select
	Next
End Sub


 
'*************************************************************************
'  <<< [cat] >>> 
'*************************************************************************
Sub  cat( Files, OutputPath )
	If IsArray( Files ) Then
		cat_sub  Files, OutputPath
	Else
		cat_sub  Array( Files ), OutputPath
	End If
End Sub

Sub  cat_sub( Files, OutputPath )
	Dim  path, f, text, ec, is_append

	is_append = ( Files(0) = OutputPath  and  UBound( Files ) = 1 )
	If is_append Then
		text = ">cat """+ Files(1) +""" >> """+ OutputPath +""""
	Else
		text = ">cat (array)"
		If not IsEmpty( OutputPath ) Then _
			text = text + " > """+ OutputPath +""""
	End If
	echo  text

	text = ""
	For Each path  In Files
		If not is_append Then  echo  "  """+ path +""""
		text = text + ReadFile( path )
	Next

	Set ec = new EchoOff
	If IsEmpty( OutputPath ) Then
		type_  path
	Else
		CreateFile  OutputPath, text
	End If
End Sub

 
'*************************************************************************
'  <<< [move] >>> 
'*************************************************************************
Sub  move( SrcPath, DstFolderPath )
	Dim  c : Set c = g_VBS_Lib

	If g_Vers("CopyTargetIsFileOrFolder") Then
		move_or_copy_del_sub  SrcPath, DstFolderPath, Empty
	Else
		move_or_copy_del_sub  SrcPath, DstFolderPath, c.CopyV4
	End If
End Sub


Sub  move_or_copy_del_sub( SrcPath, DstPath, Opt )
	If IsMovablePathToPath( SrcPath, DstPath, True ) Then
		move_sub  SrcPath, DstPath, Opt
	Else
		copy_sub  SrcPath, DstPath, Opt

		If TryStart(e) Then  On Error Resume Next
			del  SrcPath
		If TryEnd Then  On Error GoTo 0
		If e.num = E_WriteAccessDenied  Then  Raise 17, e.Description
		If e.num <> 0 Then  e.Raise
	End If
End Sub


Sub  move_sub( SrcPath, DstPath, Opt )
	Dim  en, en_, ed, n_retry, fo, src, dst, dst_fo, command_str
	Dim  c : Set c = g_VBS_Lib
	If Opt = c.CopyRen Then  command_str = ">move_ren  """  Else  command_str = ">move  """

	echo  command_str & SrcPath & """, """ & DstPath & """"
	Set en_ = new EchoOff


	'//=== If SrcPath had Wild card
	If IsWildcard( SrcPath )  or  ( Opt = c.CopyRen  and  g_fs.FolderExists( SrcPath ) ) Then
		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath, c.File or c.SubFolder

		src = SrcPath : CutLastOf  src, "\", Empty : CutLastOf  src, "/", Empty
		If Opt = c.CopyRen Then  src = src +"\*"
		If not g_fs.FolderExists( GetParentFullPath( src ) ) Then _
			Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ _
				SrcPath +"""/>"
		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		g_AppKey.CheckWritable  DstPath + "\.", Empty  '// "\." is for able to make writable folder
		is_parent_dst = False


		'//=== Delete dst files and folders
		If Opt = c.CopyRen Then
			del  DstPath +"\*"
		Else
			Dim  folder, fnames(), fname, parent
			parent = g_fs.GetFileName( g_fs.GetParentFolderName( src ) )

			ExpandWildcard  src, c.File or c.Folder, folder, fnames
			For Each fname  In fnames
				If parent <> fname Then
					del  DstPath +"\"+ fname
				ElseIf StrCompHeadOf( GetParentFullPath( src ), GetFullPath( DstPath, Empty ), _
						Empty ) = 0 Then
					temporary_dst = DstPath +"\"+ parent +"_"
					While  exist( temporary_dst )
						temporary_dst = temporary_dst +"_"
					WEnd
					is_parent_dst = True
				Else
					del  DstPath +"\"+ fname
				End If
			Next
		End If
		en = Empty


		'//=== Move files
		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFile  src, DstPath
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.File, n_retry ) Then  Exit Do
		Loop
		If en = E_FileNotExist Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed


		'//=== Move folders
		n_retry = 0
		If is_parent_dst Then
			mkdir  temporary_dst
			Do
				ErrCheck : On Error Resume Next
					g_fs.MoveFile  g_fs.GetParentFolderName( src ) +"\"+ parent +"\*", _
						temporary_dst
				en = Err.Number : ed = Err.Description : On Error GoTo 0
				If not IsWriteAccessDenied( en, DstPath, c.Folder, n_retry ) Then  Exit Do
			Loop
			Do
				ErrCheck : On Error Resume Next
					g_fs.MoveFolder  g_fs.GetParentFolderName( src ) +"\"+ parent +"\*", _
						temporary_dst
				en = Err.Number : ed = Err.Description : On Error GoTo 0
				If not IsWriteAccessDenied( en, DstPath, c.Folder, n_retry ) Then  Exit Do
			Loop
			g_fs.DeleteFolder  g_fs.GetParentFolderName( src ) +"\"+ parent
		End If
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFolder  src, DstPath
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.Folder, n_retry ) Then  Exit Do
		Loop
		If is_parent_dst Then
			g_fs.DeleteFolder  g_fs.GetParentFolderName( src )
			ren  temporary_dst,  parent
		End If
		If en = E_PathNotFound Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed

		If Opt = c.CopyRen Then  del  SrcPath


	'//=== If SrcPath is file
	ElseIf g_fs.FileExists( SrcPath ) Then
		If g_debug_or_test Then
			g_AppKey.BreakByPath SrcPath
			g_AppKey.BreakByPath DstPath
		End If

		If g_fs.FolderExists( DstPath )  and  Opt <> c.CopyV4 Then
			If Opt = c.CopyRen Then  Err.Raise  E_WriteAccessDenied
			dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		Else
			If Opt = c.CopyV4 Then  '// vbslib ver4
				dst_fo = DstPath
				dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
			Else  '// vbslib ver3 or c.CopyRen
				dst_fo = GetParentFullPath( DstPath )
				dst = DstPath
				If IsEmpty( Opt ) Then  ThisIsOldSpec  '// use move_ren
			End If
			If not g_fs.FolderExists( dst_fo ) Then
				g_AppKey.CheckWritable  SrcPath, Empty  '// If src cannot write, Don't make folder
				mkdir  dst_fo
			End If
		End If

		g_AppKey.CheckWritable  SrcPath, Empty
		If Opt = c.CopyV4 Then  '// vbslib ver4
			g_AppKey.CheckWritable  dst, Empty  '// "\." is for able to make writable folder
			del  dst  '// for Replace from folder to file
		Else
			If not g_fs.FileExists( dst ) Then
				g_AppKey.CheckWritable  dst + "\.", Empty  '// "\." is for able to make writable folder
			Else
				g_AppKey.CheckWritable  dst, Empty
				del  dst  '// for Replace from folder to file
			End If
		End If

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFile  SrcPath, dst
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.File, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' If SrcPath is folder
	ElseIf g_fs.FolderExists( SrcPath ) Then
		If g_debug_or_test Then
			g_AppKey.BreakByPath  SrcPath
			g_AppKey.BreakByPath  DstPath
		End If

		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath+"\*", c.File or c.SubFolder

		dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		g_AppKey.CheckWritable  dst, Empty

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFolder  SrcPath, dst
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' not found
	Else
		echo  command_str & SrcPath & """, """ & DstPath & """"
		Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
	End If
End Sub


 
'*************************************************************************
'  <<< [move_ren] >>> 
'*************************************************************************
Sub  move_ren( SrcPath, DstPath )
	Dim  c : Set c = g_VBS_Lib
	move_or_copy_del_sub  SrcPath, DstPath, c.CopyRen
End Sub


 
'*************************************************************************
'  <<< [move_ren_tmp] >>> 
'*************************************************************************
Sub  move_ren_tmp( SrcPath, DstPath )
	Dim  c : Set c = g_VBS_Lib
	move_sub_tmp  SrcPath, DstPath, c.CopyRen
End Sub

Sub  move_sub_tmp( SrcPath, DstPath, Opt )
	Dim  en, ed, n_retry, fo, src, dst, dst_fo, command_str
	Dim  c : Set c = g_VBS_Lib
	If Opt = c.CopyRen Then  command_str = ">move_ren  """  Else  command_str = ">move  """

	echo  command_str & SrcPath & """, """ & DstPath & """"
	Dim ec : Set ec = new EchoOff


	'//=== If DstPath had Wild card
	If IsWildcard( DstPath )  and  Opt = c.CopyRen Then
		Dim  src_path_wild,  dst_path_wild,  src_fo_path,  dst_fo_path,  src_fname,  dst_fname,  dst_path
		Dim  src_i,  src_left,  src_mid,  src_right,  src_left_len,  src_right_len,  src_len_not_wild
		Dim  dst_i,  dst_left,  dst_mid,  dst_right

		If not IsWildcard( SrcPath ) Then  Error

		src_path_wild = g_fs.GetFileName( SrcPath )
		dst_path_wild = g_fs.GetFileName( DstPath )
		src_fo_path = GetParentFullPath( SrcPath ) : CutLastOf  src_fo_path, "\", Empty : If IsWildcard( src_fo_path ) Then Error
		dst_fo_path = GetParentFullPath( DstPath ) : CutLastOf  dst_fo_path, "\", Empty : If IsWildcard( dst_fo_path ) Then Error
		src_i            = InStr( src_path_wild, "*" )
		src_left         = Left(  src_path_wild,  src_i - 1 )
		src_right        = Right( src_path_wild,  Len( src_path_wild ) - src_i )
		If InStr( src_i + 1,  src_path_wild,  "*" ) > 0 Then
			If src_i <> 1 Then Error
			If Right( src_path_wild, 1 ) <> "*" Then Error
			src_mid = Mid( src_path_wild, 2, Len( src_path_wild ) - 2 )
		End If
		src_left_len     = Len( src_left )
		src_right_len    = Len( src_right )
		src_len_not_wild = Len( src_path_wild ) - 1
		dst_i            = InStr( dst_path_wild, "*" )
		dst_left         = Left(  dst_path_wild,  dst_i - 1 )
		dst_right        = Right( dst_path_wild,  Len( dst_path_wild ) - dst_i )
		If InStr( dst_i + 1,  dst_path_wild,  "*" ) > 0 Then
			If dst_i <> 1 Then Error
			If Right( dst_path_wild, 1 ) <> "*" Then Error
			dst_mid = Mid( dst_path_wild, 2, Len( dst_path_wild ) - 2 )
		End If
		If IsEmpty( src_mid ) <> IsEmpty( dst_mid ) Then Error

		mkdir  dst_fo_path

		Set fo = g_fs.GetFolder( src_fo_path )
		For Each src  In fo.Files
			src_fname = src.Name
			dst_fname = Empty
			If IsEmpty( dst_mid ) Then
				If Left( src_fname, src_left_len ) = src_left  and  Right( src_fname, src_right_len ) = src_right Then

					dst_fname = dst_left + _
						Mid( src_fname,  src_i,  Len( src_fname ) - src_len_not_wild ) + _
						dst_right
						'// (sample) src_path_wild = "AB*CD" : dst_path_wild = "abcd*efgh"
						'//               1234567                 12345678901
						'//  src_fname = "AB123CD" : dst_fname = "abcd123efgh"
				End If
			Else
				If InStr( src_fname, src_mid ) > 0 Then
					dst_fname = Replace( src_fname, src_mid, dst_mid )
				End If
			End If
			If not IsEmpty( dst_fname ) Then
				dst_path = dst_fo_path +"\"+ dst_fname
				If g_fs.FolderExists( dst_path ) Then  Error
				g_fs.MoveFile  src_fo_path +"\"+ src_fname,  dst_path
			End If
		Next

		For Each src  In fo.SubFolders
			src_fname = src.Name
			dst_fname = Empty
			If IsEmpty( dst_mid ) Then
				If Left( src_fname, src_left_len ) = src_left  and  Right( src_fname, src_right_len ) = src_right Then
					dst_fname = dst_left + _
											Mid( src_fname,  src_i,  Len( src_fname ) - src_len_not_wild ) + _
											dst_right
				End If
			Else
				If InStr( src_fname, src_mid ) > 0 Then
					dst_fname = Replace( src_fname, src_mid, dst_mid )
				End If
			End If
			If not IsEmpty( dst_fname ) Then
				move_sub_tmp  src_fo_path +"\"+ src_fname +"\*",  dst_fo_path +"\"+ dst_fname,  c.CopyRen
			End If
		Next


	'//=== If SrcPath had Wild card
	ElseIf IsWildcard( SrcPath )  or  ( Opt = c.CopyRen  and  g_fs.FolderExists( SrcPath ) ) Then
		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath, c.File or c.SubFolder

		src = SrcPath : CutLastOf  src, "\", Empty : CutLastOf  src, "/", Empty
		If Opt = c.CopyRen  and  g_fs.FolderExists( src ) Then  Assert( not IsWildcard( src ) ) : src = src +"\*"
		If not g_fs.FolderExists( GetParentFullPath( src ) ) Then _
			Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		g_AppKey.CheckWritable  DstPath + "\.", Empty  '// "\." is for able to make writable folder


		'//=== Delete dst files and folders
		If Opt = c.CopyRen Then
			del  DstPath +"\*"
		Else
			Dim  folder, fnames(), fname

			ExpandWildcard  src, c.File or c.Folder, folder, fnames
			For Each fname  In fnames
				del  DstPath +"\"+ fname
			Next
		End If


		'//=== Move files
		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFile  src, DstPath
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.File, n_retry ) Then  Exit Do
		Loop
		If en = E_FileNotExist Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed


		'//=== Move folders
		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFolder  src, DstPath
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, DstPath, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en = E_PathNotFound Then  en = 0
		If en <> 0 Then  Err.Raise en,,ed

		If Opt = c.CopyRen Then  del  SrcPath


	'//=== If SrcPath is file
	ElseIf g_fs.FileExists( SrcPath ) Then
		If g_debug_or_test Then
			g_AppKey.BreakByPath SrcPath
			g_AppKey.BreakByPath DstPath
		End If

		If g_fs.FolderExists( DstPath )  and  Opt <> c.CopyV4 Then
			dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		Else
			If Opt = c.CopyV4 Then  '// vbslib ver4
				dst_fo = DstPath
				dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
			Else  '// vbslib ver3 or c.CopyRen
				dst_fo = GetParentFullPath( DstPath )
				dst = DstPath
				If IsEmpty( Opt ) Then  ThisIsOldSpec  '// use move_ren
			End If
			If not g_fs.FolderExists( dst_fo ) Then
				g_AppKey.CheckWritable  SrcPath, Empty  '// If src cannot write, Don't make folder
				mkdir  dst_fo
			End If
		End If

		g_AppKey.CheckWritable  SrcPath, Empty
		If Opt = c.CopyV4 Then  '// vbslib ver4
			g_AppKey.CheckWritable  dst, Empty  '// "\." is for able to make writable folder
			del  dst  '// for Replace from folder to file
		Else
			If not g_fs.FileExists( dst ) Then
				g_AppKey.CheckWritable  dst + "\.", Empty  '// "\." is for able to make writable folder
			Else
				g_AppKey.CheckWritable  dst, Empty
				del  dst  '// for Replace from folder to file
			End If
		End If

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFile  SrcPath, dst
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.File, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' If SrcPath is folder
	ElseIf g_fs.FolderExists( SrcPath ) Then
		If g_debug_or_test Then
			g_AppKey.BreakByPath  SrcPath
			g_AppKey.BreakByPath  DstPath
		End If

		If not g_fs.FolderExists( DstPath ) Then  mkdir  DstPath

		If g_debug_or_test Then  g_AppKey.BreakByWildcard  SrcPath+"\*", c.File or c.SubFolder

		dst = GetFullPath( g_fs.GetFileName( SrcPath ), GetFullPath( DstPath, Empty ) )
		g_AppKey.CheckWritable  dst, Empty

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.MoveFolder  SrcPath, dst
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, dst, c.Folder, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


	' not found
	Else
		echo  command_str & SrcPath & """, """ & DstPath & """"
		Raise  E_PathNotFound, "<ERROR msg=""Not found file or folder."" path="""+ SrcPath +"""/>"
	End If
End Sub


 
'*************************************************************************
'  <<< [ren] >>> 
'*************************************************************************
Sub  ren( src, dst )
	echo  ">ren  """ & src & """, """ & dst & """"
	Set c = g_VBS_Lib

	dst_name = g_fs.GetFileName( dst )

	If StrComp( g_fs.GetFileName( src ), dst_name, 1 ) = 0 Then  Exit Sub

	If g_fs.FileExists( src ) Then
		g_AppKey.CheckWritable  src, Empty
		Set f = g_fs.GetFile( src )
		flag = c.File
	Else
		g_AppKey.CheckWritable  src + "\.", Empty  '// "\." is for able to make writable folder
		Set f = g_fs.GetFolder( src )
		flag = c.Folder
	End If

	n_retry = 0
	Do
		On Error Resume Next
			f.Name = dst_name
		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If not IsWriteAccessDenied( en, src, flag, n_retry ) Then  Exit Do
	Loop
	If en <> 0 Then  Err.Raise en,,ed
End Sub


 
'*************************************************************************
'  <<< [ReplaceShortcutFilesToFiles] >>> 
'*************************************************************************
Sub  ReplaceShortcutFilesToFiles( Path, Option_ )
	Set c = g_VBS_Lib
	echo  ">ReplaceShortcutFilesToFiles  """+ Path +""""
	Set ec = new EchoOff

	ExpandWildcard  Path +"\*.lnk", c.File or c.SubFolder, folder, step_paths
	copy_count = 0
	not_copy_count = 0
	For Each step_path  In step_paths
		lnk_path = GetFullPath( step_path, folder )
		target_path = g_sh.CreateShortcut( lnk_path ).TargetPath
		If exist( target_path ) Then
			destination_path = AddLastOfFileName( lnk_path, "."+ _
				g_fs.GetExtensionName( target_path ) )


			If exist( destination_path ) Then
				count = 2
				Do
					path = AddLastOfFileName( destination_path, " ("& count &")" )
					If not exist( path ) Then _
						Exit Do
					count = count + 1
				Loop
				destination_path = path
			End If


			copy_ren  target_path, destination_path
				
			del  lnk_path
			copy_count = copy_count + 1
		Else
			not_copy_count = not_copy_count + 1
		End If
	Next
	ec = Empty

	echo  copy_count & "files copied."
	If not_copy_count >= 1 Then
		echo  not_copy_count & "links target was not found."
	End If
End Sub


 
'*************************************************************************
'  <<< [IsMovablePathToPath] >>> 
'*************************************************************************
Function  IsMovablePathToPath( SrcPath, DstPath, IsDstFolder )
	'// src = soruce, dst = destination, pos = position

	src_path = Replace( SrcPath, "/", "\" )
	dst_path = Replace( DstPath, "/", "\" )

	If not IsFullPath( src_path ) Then  src_path = g_fs.GetAbsolutePathName( src_path )
	If not IsFullPath( dst_path ) Then  dst_path = g_fs.GetAbsolutePathName( dst_path )
	If IsDstFolder Then  dst_path = GetFullPath( "a", dst_path )

	If Left( src_path, 2 ) = "\\" Then
		If Left( dst_path, 2 ) = "\\" Then
			slash_pos = InStr( 3, src_path, "\" )
			slash_pos = InStr( slash_pos + 1, src_path, "\" )
			IsMovablePathToPath = ( UCase( Left( src_path, slash_pos ) ) =_
				UCase( Left( dst_path, slash_pos ) ) )
		Else
			IsMovablePathToPath = False
		End If
	Else
		If Left( dst_path, 2 ) = "\\" Then
			IsMovablePathToPath = False
		Else
			IsMovablePathToPath = ( UCase( Left( src_path, 1 ) ) = _
				 UCase( Left( dst_path, 1 ) ) )
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [SafeFileUpdate] >>> 
'*************************************************************************
Sub  SafeFileUpdate( FromTmpFilePath, ToUpdateFilePath )
	SafeFileUpdateEx  FromTmpFilePath, ToUpdateFilePath, g_VBS_Lib.ToTrashBox
End Sub


 
'*************************************************************************
'  <<< [SafeFileUpdateEx] >>> 
'*************************************************************************
Sub  SafeFileUpdateEx( FromTmpFilePath, ToUpdateFilePath, Option_ )
	echo  ">SafeFileUpdate  """ & FromTmpFilePath & """, """ & ToUpdateFilePath & """"

	If not g_fs.FileExists( ToUpdateFilePath ) Then
		move_ren  FromTmpFilePath, ToUpdateFilePath
		Exit Sub
	End If

	For i=1 To 999
		back_up_path = GetParentFullPath( ToUpdateFilePath ) + "\" + _
			g_fs.GetBaseName( ToUpdateFilePath ) + "." & i & "." + g_fs.GetExtensionName( ToUpdateFilePath )
		If not exist( back_up_path ) Then  Exit For
	Next
	If exist( back_up_path ) Then  Err.Raise E_Other,,"Back up file name can not be made." + ToUpdateFilePath

	ErrCheck : On Error Resume Next
		g_fs.CopyFile  ToUpdateFilePath, back_up_path, False
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en <> 0 Then  Err.Raise en,,"Back up copy was failed."+vbCR+vbLF+_
		 "back up from:"+ToUpdateFilePath+vbCR+vbLF+ "back up to:"+ back_up_path +vbCR+vbLF+ ed

	If Option_ = g_VBS_Lib.ToTrashBox Then
		del_to_trashbox  back_up_path
	End If

	ErrCheck : On Error Resume Next
		g_fs.CopyFile  FromTmpFilePath, ToUpdateFilePath, True
	en2 = Err.Number : ed2 = Err.Description : On Error GoTo 0

	ErrCheck : On Error Resume Next
		g_fs.DeleteFile  FromTmpFilePath
	en = Err.Number : ed = Err.Description : On Error GoTo 0

	If Option_ = g_VBS_Lib.ToTrashBox Then
	Else
		ErrCheck : On Error Resume Next
			g_fs.DeleteFile  back_up_path
		en3 = Err.Number : ed = Err.Description : On Error GoTo 0
	End If

	If en2 <> 0 Then
		If Option_ = g_VBS_Lib.ToTrashBox Then
			Err.Raise  en2,,"Filed to overwrite copy"+_
				"Restore source file thrown to trash box"+vbCR+vbLF+_
				"Source:"+ FromTmpFilePath +vbCR+vbLF+ _
				"Destination:"+ ToUpdateFilePath +vbCR+vbLF
		Else
			Err.Raise  en2,,"Filed to overwrite copy"+_
				"Source file is exist at following place."+vbCR+vbLF+_
				"Source:"+ FromTmpFilePath +vbCR+vbLF+ _
				"Destination:"+ ToUpdateFilePath +vbCR+vbLF+ _
				"Source file:"+ back_up_path +vbCR+vbLF
		End If
	End If

	If en <> 0  or  en3 <> 0 Then
		If en = 0 Then  en = en3

		message = ""
		If en <> 0 Then
			message = message + "temporary file:"+ FromTmpFilePath +vbCR+vbLF
		End If
		If en3 <> 0 Then
			message = message + "Back up file:"+ back_up_path +vbCR+vbLF
		End If

		Err.Raise  en,, "Updating was successed, but following files was failed to delete."+vbCR+vbLF+_
			 message + "Updated file:"+ ToUpdateFilePath +vbCR+vbLF
	End If
End Sub


 
'*************************************************************************
'  <<< [del] >>> 
'*************************************************************************
Sub  del( ByVal path )
	If TypeName( path ) = "PathDictionaryClass" Then
		del_by_PathDictionaryClass  path

	' If path had Wild card
	Else
		echo  ">del  """ & path & """"
		Set ec = new EchoOff
		Set c = g_VBS_Lib

		If IsWildCard( path ) Then
			ExpandWildcard  path, c.File, folder, fnames
			For Each fname in fnames
				del  folder +"\"+ fname
			Next

			ExpandWildcard  path, c.Folder, folder, fnames
			For Each fname in fnames
				del  folder +"\"+ fname
			Next

		' If path was file or folder path
		Else

			If g_fs.FileExists( path ) Then
				g_AppKey.CheckWritable  path, Empty

				n_retry = 0
				Do
					ErrCheck : On Error Resume Next
						g_fs.DeleteFile  path
					en = Err.Number : ed = Err.Description : On Error GoTo 0
					If not IsWriteAccessDenied( en, path, c.File, n_retry ) Then  Exit Do
				Loop
				If en <> 0 Then  Err.Raise en,,ed
			ElseIf g_fs.FolderExists( path ) Then
				rmdir  path
			End If
		End If
	End If
End Sub


 
'*************************************************************************
'  <<< [del_subfolder] >>> 
'*************************************************************************
Sub  del_subfolder( path )
	Set c = g_VBS_Lib
	Set arr = new ArrayClass : arr.AddElems  path
	echo  ">del_subfolder  """ & arr.CSV & """"

	For i = 0  To arr.UBound_
		If Left( arr(i), 2 ) = "*\" Then _
			arr(i) = Mid( arr(i), 3 )
		If Right( arr(i), 2 ) = "\*" Then _
			arr(i) = Left( arr(i), Len( arr(i) ) - 2 )
	Next

	ExpandWildcard  arr.Items, c.File or c.Folder or c.SubFolder, folder, fnames
	For Each fname in fnames
		del  folder +"\"+ fname
	Next
End Sub


 
'*************************************************************************
'  <<< [del_to_trashbox] >>> 
'*************************************************************************
Sub  del_to_trashbox( path )
	If False Then
		'// This script is not used, because it is necessary to request to input with user.
		del_to_trashbox_old  path
	Else
		g_AppKey.CheckWritable  path + "\.", Empty  '// "\." is for able to make writable folder
		If exist( path ) Then
			folder_in_tmp = GetTempPath( "_RecycleBin\*" )
			If TryStart(e) Then  On Error Resume Next
				move  path,  folder_in_tmp
			If TryEnd Then  On Error GoTo 0
			If e.num = 70 Then  Err.Raise 17,,"Fail to move to trash box : " + path
			If e.num <> 0 Then  e.Raise
		End If
	End If
End Sub



Sub  del_to_trashbox_old( ByVal path )
	echo  ">del_to_trashbox  """ & path & """"
	Dim  ec : Set ec = new EchoOff
	Dim  en,ed
	Dim  sh_ap, TrashBox, folder, item, fname

	g_AppKey.CheckWritable  path + "\.", Empty  '// "\." is for able to make writable folder

	Set  sh_ap = CreateObject("Shell.Application")
	Const  ssfBITBUCKET = 10


	'//=== Check deletable by rename for Windows XP
	ErrCheck : On Error Resume Next
		ren  path, g_fs.GetFileName( path ) + "_deleting"
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = 70 Then  Err.Raise 17,,"Fail to move to trash box : " + path
	If en = 76 Then  Exit Sub  ' not found path
	If en <> 0 Then  Err.Raise en,,ed
	ErrCheck : On Error Resume Next
		ren  path + "_deleting", g_fs.GetFileName( path )
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en <> 0 and en <> E_OutOfWritable Then  Err.Raise en,,ed


	'//=== move to trashbox
	path = g_fs.GetAbsolutePathName( path )
	fname = g_fs.GetFileName( path )
	Set  folder = sh_ap.NameSpace( g_fs.GetParentFolderName( path ) )
	If folder is Nothing Then  Exit Sub
	Set  item = folder.Items.Item( fname )
	If item is Nothing Then  Exit Sub

	Set  TrashBox = sh_ap.NameSpace( ssfBITBUCKET )
	TrashBox.MoveHere  item, &h40


	'//=== for Windows Vista
	' If exist( path ) Then  Err.Raise 17,,"Fail to move to trash box : " + path


	'//=== for Windows XP
	Do
		WScript.Sleep 300
		Set  item = folder.Items.Item( fname )
		If item is Nothing Then Exit Do
		item = Empty
	Loop
End Sub


 
'*************************************************************************
'  <<< [del_confirmed] >>> 
'*************************************************************************
Function  del_confirmed( Path )
	echo  ""
	If exist( Path ) Then
		r = input( "Are you delete ? : """ + Path + """ (Y/N)" )
		del_confirmed = ( r="Y" or r="y" )
		If del_confirmed Then  del  Path
	Else
		del_confirmed = True
	End If
End Function


 
'*************************************************************************
'  <<< [del_empty_folder] >>> 
'*************************************************************************
Sub  del_empty_folder( FolderPath )
	Dim  root_foler_paths,  root_folder_path,  folder_paths,  folder_path,  folder
	Dim  c : Set c = g_VBS_Lib

	If IsArray( FolderPath ) Then  root_foler_paths = FolderPath _
	Else  root_foler_paths = Array( FolderPath )

	For Each root_folder_path  In root_foler_paths
		echo  ">del_empty_folder  """+ root_folder_path +""""

		ExpandWildcard  root_folder_path+"\*", c.Folder or c.SubFolder or c.AbsPath,_
			Empty, folder_paths

		folder_paths = ArrayToNameOnlyClassArray( folder_paths )
		QuickSort  folder_paths, 0, UBound( folder_paths ), GetRef("LengthNameCompare"), -1

		ReDim Preserve  folder_paths( UBound( folder_paths ) + 1 )
		Set folder_paths( UBound( folder_paths ) ) = new NameOnlyClass
		folder_paths( UBound( folder_paths ) ).Name = GetFullPath( root_folder_path, Empty )

		For Each folder_path  In folder_paths
			If g_fs.FolderExists( folder_path.Name ) Then
				Set  folder = g_fs.GetFolder( folder_path.Name )
				If folder.Files.Count = 0 Then
					If folder.SubFolders.Count = 0 Then
						g_fs.DeleteFolder  folder_path.Name
					End If
				End If
			End If
		Next
	Next
End Sub


 
'*************************************************************************
'  <<< [del_by_PathDictionaryClass] >>> 
'*************************************************************************
Sub  del_by_PathDictionaryClass( in_PathDictionary )
	count = 0

	For Each  step_path  In  in_PathDictionary.FilePaths
		del  GetFullPath( step_path, in_PathDictionary.BasePath )

		count = count + 1
		If count = 2 Then
			echo  ">del  ..."
			Set ec = new EchoOff
		End If
	Next

	For Each  step_path  In  in_PathDictionary.DeleteFolderPaths
		del  GetFullPath( step_path, in_PathDictionary.BasePath )
	Next
End Sub


 
'*************************************************************************
'  <<< [mkdir] >>> 
'*************************************************************************
Function  mkdir( ByVal Path )
	Dim  i, n, names(), fo2, n_retry, en, ed
	Dim  c : Set c = g_VBS_Lib

	If Right( Path, 2 ) = ":\" Then
		g_AppKey.CheckWritable  Path + ".",  c.ForMkDir
	Else
		g_AppKey.CheckWritable  Path + "\.",  c.ForMkDir
	End If

	If g_fs.FolderExists( Path ) Then  mkdir = 0 : Exit Function

	n = 0
	fo2 = g_fs.GetAbsolutePathName( Path )
	Do
		If g_fs.FolderExists( fo2 ) Then Exit Do

		n = n + 1
		Redim Preserve  names(n)
		names(n) = g_fs.GetFileName( fo2 )
		fo2 = g_fs.GetParentFolderName( fo2 )
	Loop

	mkdir = n

	For n=n To 1 Step -1
		fo2 = GetFullPath( names(n), fo2 )

		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				g_fs.CreateFolder  fo2
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, path, c.File, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then
			If g_fs.FileExists( fo2 ) Then _
				Raise  E_AlreadyExist, "<ERROR msg=""It is not able to make folder at file exists"""+_
					" path="""+ fo2 +"""/>"
			Err.Raise en,,ed
		End If
	Next
End Function


 
'*************************************************************************
'  <<< [mkdir_for] >>> 
'*************************************************************************
Sub  mkdir_for( Path )
	Dim  s

	s = GetParentFullPath( Path )
	If s = "" Then  Exit Sub
	mkdir  s
End Sub


 
'*************************************************************************
'  <<< [rmdir] >>> 
'*************************************************************************
Sub  rmdir( ByVal Path )
	echo  ">rmdir  """ & Path & """"
	Set c = g_VBS_Lib

	If Not g_fs.FolderExists( Path ) Then Exit Sub
	g_AppKey.CheckWritable  Path + "\.", Empty  '// "\." is for able to make writable folder


	' Cut last \
	path2 = Path
	If Right( path2, 1 ) = "\" Then  path2 = Left( path2, Len( path2 ) - 1 )

	nFolder = 1
	ReDim folderPathes(nFolder)
	folderPathes(nFolder) = path2

	' Enum sub folders
	iFolder = 1
	While iFolder <= nFolder
		Set fo = g_fs.GetFolder( folderPathes(iFolder) )
		For Each subf in fo.SubFolders
			nFolder = nFolder + 1
			ReDim Preserve folderPathes(nFolder)
			folderPathes(nFolder) = subf.Path
		Next
		iFolder = iFolder + 1
	WEnd

	' Remove read only attribute of all files in sub folders
	For iFolder = 1 To nFolder
		Set fo = g_fs.GetFolder( folderPathes(iFolder) )
		For Each f in fo.Files
			Set file = g_fs.GetFile( f.Path )
			If g_debug_or_test Then  g_AppKey.BreakByPath( f.Path )
			file.Attributes = file.Attributes and not ReadOnly
		Next
	Next

	' Delete folders
	n_retry = 0
	Do
		ErrCheck : On Error Resume Next
			g_fs.DeleteFolder( Path )
		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If not IsWriteAccessDenied( en, Path, c.Folder, n_retry ) Then  Exit Do
	Loop
	If en <> 0 Then  Err.Raise en,,ed
End Sub


 
'*************************************************************************
'  <<< [exist] >>> 
'*************************************************************************
Function  exist( ByVal path )
	If IsWildcard( path ) Then
		Set c = g_VBS_Lib

		If TryStart(e) Then  On Error Resume Next
			ExpandWildcard  path, c.File or c.Folder, folder, fnames
		If TryEnd Then  On Error GoTo 0
		If e.Num = E_PathNotFound  Then  ReDim fnames(-1) : e.Clear
		If e.Num <> 0 Then  e.Raise

		exist = UBound( fnames ) <> -1
	Else
		exist = ( g_fs.FileExists( path ) = True ) Or ( g_fs.FolderExists( path ) = True )
	End If
End Function


 
'*************************************************************************
'  <<< [exist_ex] >>> 
'*************************************************************************
Function  exist_ex( Path, Opt )
	If IsEmpty( Opt ) Then
		exist_ex = exist( Path )
	Else
		If exist( Path ) Then
			cs_path = GetCaseSensitiveFullPath( Path )
			If Left( cs_path, 2 ) = "\\" Then
				full_path = GetFullPath( Path, Empty )
				root_pos = GetRootSeparatorPosition( full_path )
				full_path = LCase( Left( full_path, root_pos ) ) + _
					Mid( full_path, root_pos + 1 )

				exist_ex = ( cs_path = full_path )
			Else
				exist_ex = ( cs_path = GetFullPath( Path, Empty ) )
			End If
		Else
			exist_ex = False
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [GetCaseSensitiveFullPath] >>> 
'*************************************************************************
Function  GetCaseSensitiveFullPath( Path )
	If g_fs.FileExists( Path ) Then
		GetCaseSensitiveFullPath = g_fs.GetFile( Path ).Path
	ElseIf g_fs.FolderExists( Path ) Then
		GetCaseSensitiveFullPath = g_fs.GetFolder( Path ).Path
	Else
		GetCaseSensitiveFullPath = GetFullPath( Path, Empty )
	End If
End Function


 
'*************************************************************************
'  <<< [CheckExist] >>> 
'*************************************************************************
Sub  CheckExist( Path, Flags )
	Set c = g_VBS_Lib
	If Flags and c.File Then
		If not g_fs.FileExists( Path ) Then
			Raise  E_FileNotExist, "<ERROR msg='Not found file' path='"+ XmlAttrA( Path ) +"'/>"
		End If
	ElseIf Flags and c.Folder Then
		If not g_fs.FolderExists( Path ) Then
			Raise  E_PathNotFound, "<ERROR msg='Not found folder' path='"+ XmlAttrA( Path ) +"'/>"
		End If
	Else
		Error
	End If
End Sub


 
'*************************************************************************
'  <<< [fc] file compare as binary >>> 
' argument
'  - return : True=same, False=different
'*************************************************************************
Function  fc( path_a, path_b )
	fc = fc_r( path_a, path_b, "" )
End Function


 
'*************************************************************************
'  <<< [fc_r] file compare as binary >>> 
' argument
'  - return : True=same, False=different
'*************************************************************************
Function  fc_r( path_a, path_b, redirect_path )
	Dim  opt : Set opt = new fc_option : ErrCheck

	opt.FcResultRedirectPath = redirect_path
	fc_r = fc_ex( path_a, path_b, opt )
End Function


 
'*************************************************************************
'  <<< [fc_ex] file compare as binary >>> 
'*************************************************************************
Function  fc_ex( PathA, PathB, Opt )
	Dim  cmdline, opt_echo, redirect_path, b_stdout, is_compare_xml
	Dim  s, is_not_redir


	'//=== set cmdline from Opt.FcOptionIniPath
	cmdline = """" + g_vbslib_ver_folder + "feq.exe"""
	If not IsEmpty( Opt ) Then
		If not IsEmpty( Opt.FcOptionIniPath ) Then
			cmdline = cmdline + " /ini:""" + Opt.FcOptionIniPath + """"
			opt_echo = " /ini:" + g_fs.GetFileName( Opt.FcOptionIniPath )
		End If
		is_compare_xml = not Opt.IsXmlComparedAsBinary  and  not IsEmpty( Opt.IsXmlComparedAsBinary )
	End If
	cmdline = cmdline + " """ + PathA + """ """ + PathB + """"


	'//=== set redirect_path from Opt.FcResultRedirectPath
	If not IsEmpty( Opt ) Then
		redirect_path = Opt.FcResultRedirectPath
		b_stdout = Opt.m_bStdOut
	End If


	'//=== echo
	is_not_redir = True : If Not IsEmpty( Opt ) Then  is_not_redir = (Opt.FcResultRedirectPath = "")
	If is_not_redir Then '// IsEmpty or
		echo  ">fc " + opt_echo + " """ + PathA + """, """ + PathB + """"
	End If


	If is_compare_xml Then
		Dim  root1, root2

		If not g_fs.FileExists( PathA )  or  not g_fs.FileExists( PathB ) Then
			If g_fs.FolderExists( PathA )  or  g_fs.FolderExists( PathB ) Then  Fail
			echo  "different"
			fc_ex = False
		Else
			Set root1 = LoadXML( PathA, Empty ) '// as IXMLDOMElement
			Set root2 = LoadXML( PathB, Empty ) '// as IXMLDOMElement
			fc_ex = ( root1.xml = root2.xml )
			If fc_ex Then  echo  "same"  Else  echo  "different"
		End If
	Else
		'//=== Exec
		Dim  ex
		chk_exist_in_lib  "feq.exe"
		Set  ex = g_sh.Exec( cmdline )
		If not IsEmpty( redirect_path ) Then  redirect_path = g_sh.ExpandEnvironmentStrings( redirect_path )
		fc_ex = ( WaitForFinishAndRedirect( ex, redirect_path ) = 0 )

		If not is_not_redir Then
			s = ReadFile( redirect_path )
			Dim f : Set f = g_fs.OpenTextFile( redirect_path, 2, True, False )
			f.WriteLine  ">fc " + opt_echo + " """ + PathA + """, """ + PathB + """"
			f.Write      s
			f = Empty
		End If
	End IF
End Function


 
'*************************************************************************
'  <<< [fc_option] >>> 
'*************************************************************************
Class  fc_option
	Public  FcOptionIniPath
	Public  FcResultRedirectPath
	Public  IsXmlComparedAsBinary
	Public  m_bStdOut
End Class


 
'*************************************************************************
'  <<< [IsSameTextFile] >>> 
'*************************************************************************
Function  IsSameTextFile( PathA, PathB, in_out_Options )
	Set c = g_VBS_Lib

	echo  ">IsSameTextFile  """+ PathA +""", """+ PathB +""""

	ParseOptionArguments  in_out_Options

	option_flags = in_out_Options( "Integer" )
	If in_out_Options.Exists("OptionsFor_IsSameTextFile_Class") Then
		Set options = in_out_Options("OptionsFor_IsSameTextFile_Class")
	Else
		Set options = new OptionsFor_IsSameTextFile_Class
	End If

	Set in_out_Options( "PercentStringClass" ) = new PercentStringClass

	If option_flags and c.ErrorIfNotSame Then
		is_err_if_not_same = True
		option_flags = option_flags - c.ErrorIfNotSame
	End If

	If option_flags and c.CaseSensitive Then
		strcomp_opt = 0
		option_flags = option_flags - c.CaseSensitive
	Else
		strcomp_opt = 1
	End If

	err_msg = ""

	If option_flags = 0 Then
		Dim  text1, text2

		Set cs1 = new_TextFileCharSetStack( options.CharSetA )
		text1 = ReadFile( PathA ) : text1 = Replace( text1, vbCRLF, vbLF )
		cs1 = Empty

		Set cs2 = new_TextFileCharSetStack( options.CharSetB )
		text2 = ReadFile( PathB ) : text2 = Replace( text2, vbCRLF, vbLF )
		cs2 = Empty

		If in_out_Options.Exists( "StringReplaceSetClass" ) Then
			Set rep = in_out_Options( "StringReplaceSetClass" )
			text1 = rep.DoReplace( text1 )
			text2 = rep.DoReplace( text2 )
		End If


		IsSameTextFile = ( StrComp( text1, text2, strcomp_opt ) = 0 )


		If not IsSameTextFile Then
		If is_err_if_not_same Then

			Set f  = new StringStream :  f.SetString  text1
			Set f2 = new StringStream : f2.SetString  text2

			Do Until f.AtEndOfStream
				line = f.ReadLine() : line2 = f2.ReadLine()
				If line <> line2 Then  Exit Do
			Loop
			Raise  E_TestFail, "<ERROR msg=""Not Same"" file1="""+_
				PathA +""" file2="""+ PathB +""" line=""" & ( f.Line - 1 ) & """/>"
		End If
		End If
	Else
		IsSameTextFile = False
		base_path = GetParentFullPath( PathB )

		Set cs1 = new_TextFileCharSetStack( options.CharSetA )
		Set cs2 = new_TextFileCharSetStack( options.CharSetB )

		Set f  = OpenForRead( PathA )
		Set f2 = OpenForRead( PathB )
		Do Until  f.AtEndOfStream
			line = f.ReadLine()
			If IsEmpty( multi_line ) Then
				If f2.AtEndOfStream Then
					If is_err_if_not_same Then
						Raise  E_TestFail, "<ERROR msg=""Not Same"" file1="""+ PathA +_
							""" file2="""+ PathB +_
							""" line1_num=""" & ( f.Line - 1 ) & _
							""" line2_num=""" & ( f2.Line - 1 ) & _
							""" msg2=""Whole of file2 is same as until part of file1""/>"
					End If
					Exit Function
				End If
				line2 = f2.ReadLine()
			End If

			If option_flags and c.RightHasPercentFunction Then

				'// read multi_line
				If StrCompHeadOf( line2, "%MultiLine%", Empty ) = 0 Then
						'// future is "%MultiLine(2)%%RegExp(var\[.\]=.*)%"

					multi_line = Mid( line2, Len("%MultiLine%") + 1 )
					If multi_line = "" Then  multi_line = "%RegExp(.*)%"
					If not f2.AtEndOfStream Then
						line2 = f2.ReadLine()
						Assert  StrCompHeadOf( line2, "%MultiLine%", Empty ) <> 0
					Else
						line2 = vbCRLF  '// not match any lines
					End If
				End If

				'// compare a line
				If IsSameTextLineByPercentFunction( line, line2, base_path, in_out_Options ) Then
					If not IsEmpty( multi_line ) Then  multi_line = Empty

				ElseIf IsEmpty( multi_line ) Then
					is_err = True : Exit Do

				'// compare multi_line
				Else
					If not IsSameTextLineByPercentFunction( line, multi_line, base_path, in_out_Options ) Then _
						is_err = True : Exit Do
				End If
				in_out_Options( "PercentStringClass" ).ExpandedString = Empty
			Else
				While  StrComp( text1, text2, strcomp_opt ) <> 0
					If option_flags = c.SkipLeftSpaceLine Then
						If Trim2( line ) <> "" Then  is_err = True : Exit Do
						If f.AtEndOfStream Then  is_err = True : Exit Do
						line = f.ReadLine()
					Else
						is_err = True : Exit Do
					End If
				WEnd
			End If
		Loop

		If IsEmpty( is_err ) Then
			If IsEmpty( multi_line ) Then
				If not f2.AtEndOfStream Then
					is_err = True
					err_msg = " msg2=""Whole of file1 is same as until part of file2"""
				End If
			ElseIf line2 = vbCRLF Then
				is_err = False
			Else
				is_err = True
				err_msg = " msg2=""Not found next line text of %MultiLine%"" next_line="""+ _
					XmlAttr( line2 ) +""""
			End If
		End If

		If is_err Then
			If is_err_if_not_same Then
				If IsEmpty( in_out_Options( "PercentStringClass" ).ExpandedString ) Then
					line2_expanded_message = "line2="
				Else
					line2_expanded_message = "line2="""+ XmlAttr( _
						in_out_Options( "PercentStringClass" ).ExpandedString ) +""""+ _
						vbCRLF +"line2_source="
				End If

				Raise  E_TestFail, "<ERROR msg=""Not Same"" file1="""+ PathA +_
					""" file2="""+ PathB +_
					""" line1_num=""" & ( f.Line - 1 ) & """ line2_num=""" & ( f2.Line - 1 ) & """"+ _
					err_msg +" current_folder="""+ g_sh.CurrentDirectory +""""+ vbCRLF + _
					"line1="""+ XmlAttr( line ) +""""+ vbCRLF + _
					line2_expanded_message + _
					""""+ XmlAttr( line2 ) +""""+ _
					 +"/>"
			End If
			Exit Function
		End If
		IsSameTextFile = True
	End If
End Function


'//[IsSameTextFile_Old]
Function  IsSameTextFile_Old( PathA, CharSetA, PathB, CharSetB, in_out_Options )
	ThisIsOldSpec
	If IsEmpty( CharSetA )  and  IsEmpty( CharSetB ) Then
		IsSameTextFile_Old = IsSameTextFile( PathA, PathB, in_out_Options )
	Else
		Set proxy_options = new OptionsFor_IsSameTextFile_Class
		proxy_options.CharSetA = CharSetA
		proxy_options.CharSetB = CharSetB
		IsSameTextFile_Old = IsSameTextFile( PathA, PathB, Array( proxy_options, in_out_Options ) )
	End If
End Function


 
'*************************************************************************
'  <<< [OptionsFor_IsSameTextFile_Class] >>> 
'*************************************************************************
Class  OptionsFor_IsSameTextFile_Class
	Public  CharSetA
	Public  CharSetB
End Class


 
'*************************************************************************
'  <<< [IsSameTextLineByPercentFunction] >>> 
'*************************************************************************
Function  IsSameTextLineByPercentFunction( TextA, TextBWithPercent, BasePath, in_out_Options )
	Set c = g_VBS_Lib

	option_flags = in_out_Options( "Integer" )

	If option_flags and c.CaseSensitive Then  strcomp_opt = 0  Else  strcomp_opt = 1

	If InStr( TextBWithPercent, "%" ) = 0 Then _
		IsSameTextLineByPercentFunction = ( TextA = TextBWithPercent ) : Exit Function

	IsSameTextLineByPercentFunction = False

	line = g_sh.ExpandEnvironmentStrings( TextBWithPercent )

	Do
		p1 = InStr( 1, line, "%FullPath(", 1 ) : length = 10
		If p1 = 0 Then  p1 = InStr( line, "%AbsPath(" ) : length = 9
		If p1 = 0 Then  Exit Do
		p2 = InStr( p1 + length, line, ")%" )
		path = Mid( line,  p1 + length,  p2 - p1 - length )
		path = GetFullPath( path, GetFullPath( BasePath, Empty ) )
		line = Left( line, p1 - 1 ) + path + Mid( line, p2 + 2 )
	Loop

	line = Replace( line, "%DesktopPath%", g_sh.SpecialFolders("Desktop"), 1, -1, 1 )

	Do
		p1 = InStr( 1, line, "%Env(", 1 ) : length = 5
		If p1 = 0 Then  Exit Do
		p2 = InStr( p1 + length, line, ")%" )
		environment_name = Mid( line,  p1 + length,  p2 - p1 - length )
		line = Left( line, p1 - 1 ) + env( "%"+ environment_name +"%" ) + Mid( line, p2 + 2 )
	Loop

	If in_out_Options.Exists( "PercentStringClass" ) Then
		If line <> TextBWithPercent Then _
			in_out_Options( "PercentStringClass" ).ExpandedString = line
	End If

	p1 = InStr( 1, line, "%RegExp(", 1 )
	If p1 > 0 Then
		If Left( TextA, p1 - 1 ) <> Left( line, p1 - 1 ) Then  Exit Function
		p2 = InStr( p1 + 8, line, ")%" )
		part_b = Mid( line, p2 + 2 )
		If Right( TextA, Len( part_b ) ) <> part_b Then  Exit Function
		part_a = Mid( TextA, p1 )
		part_a = Left( part_a, Len( part_a ) - Len( part_b ) )

		Set re = CreateObject("VBScript.RegExp")
		re.Pattern = Mid( line, p1+8, p2-p1-8 )
		re.Global = False
		If re.Replace( part_a, "") <> "" Then  Exit Function
	Else
		If StrComp( TextA, line, strcomp_opt ) <> 0  Then  Exit Function
	End If
	IsSameTextLineByPercentFunction = True
End Function


'//[PercentStringClass]
Class  PercentStringClass
	Public  ExpandedString
End Class


 
'*************************************************************************
'  <<< [ADODB_Stream_loadFromFile] >>> 
'*************************************************************************
Sub  ADODB_Stream_loadFromFile( Stream, Path )
	AssertExist  Path

	If TryStart(e) Then  On Error Resume Next
		Stream.LoadFromFile  Path
	If TryEnd Then  On Error GoTo 0
	If e.num = 3002  Then
		e.OverRaise  e.num, _
			"<ERROR msg=""Cannot open file"" path="""+ GetFullPath( Path, Empty ) +"""/>"
	End If
	If e.num <> 0 Then  e.Raise
End Sub


 
'*************************************************************************
'  <<< [IsSameBinaryFile] >>> 
'*************************************************************************
Function  IsSameBinaryFile( PathA, PathB, Opt )

	If not g_fs.FileExists( PathA ) Then
		IsSameBinaryFile = not g_fs.FileExists( PathB )

		If g_fs.FolderExists( PathA ) Then _
			If g_fs.FolderExists( PathB ) Then _
				Error

		Exit Function
	Else
		If not g_fs.FileExists( PathB ) Then
			IsSameBinaryFile = False
			Exit Function
		End If
	End If

	Set f = CreateObject( "ADODB.Stream" )
	f.Type = 1

	f.Open
	ADODB_Stream_loadFromFile  f, NormalizePath( PathA )
	a_bin = f.Read( -1 )
	f.Close

	f.Open
	ADODB_Stream_loadFromFile  f, NormalizePath( PathB )
	b_bin = f.Read( -1 )
	f.Close

	IsSameBinaryFile = ( StrComp( a_bin, b_bin, 0 ) = 0 )

	If IsNull( IsSameBinaryFile ) Then
		IsSameBinaryFile = ( IsNull( a_bin )  and  IsNull( b_bin ) )
	End If
End Function


 
'*************************************************************************
'  <<< [IsSameFolder] >>> 
'*************************************************************************
Function  IsSameFolder( PathA, PathB, in_out_Options )
	Set c = g_VBS_Lib

	echo  ">IsSameFolder  """+ PathA +""", """+ PathB +""""
	Set ec = new EchoOff

	ParseOptionArguments  in_out_Options

	option_flags = in_out_Options("integer")
	If in_out_Options.Exists("OptionsFor_IsSameFolder_Class") Then
		Set options = in_out_Options("OptionsFor_IsSameFolder_Class")
	Else
		Set options = new OptionsFor_IsSameFolder_Class
	End If
	If IsEmpty( options.IsSameFileFunction ) Then
		Set compare_function = GetRef( "IsSameBinaryFile" )
	Else
		Set compare_function = options.IsSameFileFunction
	End If


	'// Check exist root folders
	If exist( PathA ) Then
		If exist( PathB ) Then
			If g_fs.FileExists( PathA ) Then
				If g_fs.FileExists( PathB ) Then
					IsSameFolder = compare_function( PathA, PathB, _
						options.IsSameFileFunction_Parameter )
					If not IsSameFolder Then
						If IsBitSet( option_flags, c.EchoV_NotSame ) Then
							echo_v  "<ERROR msg=""File contents are not same"" "+_
								"path_A="""+ PathA +""" path_B="""+ PathB +"""/>"
						End If
					End If
				Else
					IsSameFolder = False
					If IsBitSet( option_flags, c.EchoV_NotSame ) Then
							echo_v  "<ERROR msg=""Different between file and folder"" "+_
						"path_A="""+ PathA +""" path_B="""+ PathB +"""/>"
					End If
				End If
				Exit Function
			Else
				If g_fs.FileExists( PathB ) Then
					IsSameFolder = False
					If IsBitSet( option_flags, c.EchoV_NotSame ) Then
						echo_v  "<ERROR msg=""Different between file and folder"" "+_
							"path_A="""+ PathA +""" path_B="""+ PathB +"""/>"
					End If
					Exit Function
				Else
				End If
			End If
		Else
			If IsBitSet( option_flags, c.EchoV_NotSame ) Then
				echo_v  "<ERROR msg=""Not found"" "+_
					"path="""+ PathB +"""/>"
			End If

			IsSameFolder = False
			Exit Function
		End If
	Else
		If exist( PathB ) Then
			If IsBitSet( option_flags, c.EchoV_NotSame ) Then
				echo_v  "<ERROR msg=""Not found"" "+_
					"path="""+ PathA +"""/>"
			End If

			IsSameFolder = False
		Else
			If IsBitSet( option_flags, c.ErrorIfNotExistBoth ) Then
				Raise  E_PathNotFound, "<ERROR msg=""Not found both"" "+_
					"path_A="""+ PathA +""" path_B="""+ PathB +"""/>"
			Else
				IsSameFolder = True
			End If
		End If
		Exit Function
	End If


	'// Compare file contents at root
	is_same_type = True
	If g_fs.FileExists( PathA ) Then
		If g_fs.FileExists( PathB ) Then
			If IsSameBinaryFile( PathA, PathB, Empty ) Then
				IsSameFolder = True
			Else
				If IsBitSet( option_flags, c.EchoV_NotSame ) Then
					echo_v  "<ERROR msg=""File contents are not same"" "+_
						"path_A="""+ PathA +""" path_B="""+ PathB +"""/>"
				End If

				IsSameFolder = False
			End If
			Exit Function
		Else
			is_same_type = False
		End If
	Else
		If g_fs.FileExists( PathB ) Then
			is_same_type = False
		Else
		End If
	End If
	If not is_same_type Then
		If IsBitSet( option_flags, c.EchoV_NotSame ) Then
			echo_v  "<ERROR msg=""Different between file and folder"" "+_
				"path_A="""+ PathA +""" path_B="""+ PathB +"""/>"
		End If

		IsSameFolder = False
		Exit Function
	End If


	'// Set "except_reg_exps"
	If IsEmpty( options.ExceptNames ) Then
		except_reg_exps = Array( )
	Else
		ReDim  except_reg_exps( UBound( options.ExceptNames ) )
		i = 0
		For Each name  In options.ExceptNames
			Set re = CreateObject("VBScript.RegExp")
			re.Pattern = name
			re.IgnoreCase = True

			Set except_reg_exps( i ) = re
			i = i + 1
		Next
	End If


	'// Compare folder exists
	If IsBitSet( option_flags, c.NotSubFolder ) Then
		Set  folders_A = CreateObject( "Scripting.Dictionary" )
		folders_A.Add  ".", g_fs.GetFolder( PathA )
		Set  folders_B = CreateObject( "Scripting.Dictionary" )
		folders_B.Add  ".", g_fs.GetFolder( PathB )
	Else
		EnumFolderObjectDic  PathA, Empty, folders_A  '// [out] folders_A
		EnumFolderObjectDic  PathB, Empty, folders_B  '// [out] folders_B
	End If

	IsSameFolder_removeExceptInDic_Sub  folders_A, except_reg_exps
	IsSameFolder_removeExceptInDic_Sub  folders_B, except_reg_exps

	If IsSameFolder_isExistOneSideOnly_Sub( PathA, PathB, folders_A, folders_B, in_out_Options ) Then
		IsSameFolder = False
		Exit Function
	End If


	'// Compare file exists
	For Each  step_path  In folders_A.Keys
		step_path_A = PathA +"\"+ step_path
		step_path_B = PathB +"\"+ step_path
		Set folder_A = folders_A( step_path )
		Set folder_B = folders_B( step_path )

		EnumFileObjectDic  folder_A, files_A  '// [out] files_A
		EnumFileObjectDic  folder_B, files_B  '// [out] files_B

		IsSameFolder_removeExceptInDic_Sub  files_A, except_reg_exps
		IsSameFolder_removeExceptInDic_Sub  files_B, except_reg_exps

		If IsSameFolder_isExistOneSideOnly_Sub( step_path_A, step_path_B, _
				files_A, files_B, in_out_Options ) Then
			IsSameFolder = False
			Exit Function
		End If


		'// Compare file contents in a folder
		For Each  file_name  In files_A.Keys
			If not compare_function( _
					step_path_A +"\"+ file_name, _
					step_path_B +"\"+ file_name, _
					options.IsSameFileFunction_Parameter ) Then
				If IsBitSet( option_flags, c.EchoV_NotSame ) Then
					echo_v  "<ERROR msg=""File contents are not same"" "+_
						"path="""+ NormalizePath( step_path +"\"+ file_name ) +"""/>"
				End If

				IsSameFolder = False
				Exit Function
			End If
		Next
	Next

	IsSameFolder = True
End Function


'//[IsSameFolder_removeExceptInDic_Sub]
Sub  IsSameFolder_removeExceptInDic_Sub( a_Dictionary, ExceptRegExps )
	For Each step_path  In a_Dictionary.Keys
		For Each  re  In ExceptRegExps
			If re.Test( g_fs.GetFileName( step_path ) ) Then
				a_Dictionary.Remove  step_path
			End If
		Next
	Next
End Sub


'//[IsSameFolder_isExistOneSideOnly_Sub]
Function  IsSameFolder_isExistOneSideOnly_Sub( PathA, PathB, objects_A, objects_B, in_out_Options )
	Set c = g_VBS_Lib
	option_flags = in_out_Options("integer")

	For Each step_path  In objects_A.Keys
		If not objects_B.Exists( step_path ) Then
			If IsBitSet( option_flags, c.EchoV_NotSame ) Then
				echo_v  "<ERROR msg=""There is only in one side folder"" "+_
					"path="""+ NormalizePath( PathA +"\"+ step_path ) +"""/>"
			End If

			IsSameFolder_isExistOneSideOnly_Sub = True
			Exit Function
		End If
	Next

	For Each step_path  In objects_B.Keys
		If not objects_A.Exists( step_path ) Then
			If IsBitSet( option_flags, c.EchoV_NotSame ) Then
				echo_v  "<ERROR msg=""There is only in one side folder"" "+_
					"path="""+ NormalizePath( PathB +"\"+ step_path ) +"""/>"
			End If

			IsSameFolder_isExistOneSideOnly_Sub = True
			Exit Function
		End If
	Next

	IsSameFolder_isExistOneSideOnly_Sub = False
End Function


'//[IsSameFolder_Old]
Function  IsSameFolder_Old( PathA, PathB, Options_, ExceptNames )
	ThisIsOldSpec
	If not IsEmpty( ExceptNames ) Then
		Set proxy_options = new OptionsFor_IsSameFolder_Class
		proxy_options.ExceptNames = ExceptNames
		IsSameFolder_Old = IsSameFolder( PathA, PathB, Array( Options_, proxy_options ) )
	Else
		IsSameFolder_Old = IsSameFolder( PathA, PathB, Options_ )
	End If
End Function


 
'*************************************************************************
'  <<< [OptionsFor_IsSameFolder_Class] >>> 
'*************************************************************************
Class  OptionsFor_IsSameFolder_Class
	Public  ExceptNames
	Public  IsSameFileFunction
	Public  IsSameFileFunction_Parameter
End Class


 
'*************************************************************************
'  <<< [find] find lines including keyword >>> 
'*************************************************************************
Function  find( ByVal keyword, ByVal path )
	Dim  f, line, ret
	Set  f = g_fs.OpenTextFile( path,,,-2 )

	ret = ""
	Do Until f.AtEndOfStream
		line = f.ReadLine
		If InStr( line, keyword ) > 0 Then  ret = ret + line
	Loop

	f.Close

	find = ret
End Function


 
'*************************************************************************
'  <<< [find_c] find lines count including keyword >>> 
'*************************************************************************
Function  find_c( ByVal keyword, ByVal path )
	Dim  f, line, ret
	Set  f = g_fs.OpenTextFile( path,,,-2 )

	ret = 0
	Do Until f.AtEndOfStream
		line = f.ReadLine
		If InStr( line, keyword ) > 0 Then  ret = ret + 1
	Loop

	f.Close

	find_c = ret
End Function


 
'*************************************************************************
'  <<< [grep] >>> 
'*************************************************************************
Function  grep( ByVal Params, ByVal OutPath )
	Set c = g_VBS_Lib

	out_is_empty = IsEmpty( OutPath )
	is_echo_start = True
	If IsNumeric( OutPath ) Then
		If OutPath = c.NotEchoStartCommand Then
			is_echo_start = False
			out_is_empty = True
		End If
	End If

	If is_echo_start Then
		If out_is_empty  or  OutPath = "" Then
			echo  ">grep "+ Params
		Else
			echo  ">grep "+ Params +" > """+ OutPath +""""
		End If
	End If


	'//
	params_arr = ArrayFromBashCmdLine( Params )  '// bash cmdline -> string

	For expr_pos = 0  To UBound( params_arr )
		Select Case  params_arr( expr_pos )
			Case  "-u", "-L", "-c"
				grep_unicode_sub  params_arr, OutPath, grep
				Exit Function
		End Select
	Next

	If out_is_empty Then
		OutPath = GetTempPath( "grep_out_*.txt" )
	End If

	For expr_pos = 0  To UBound( params_arr )
		If Left( params_arr( expr_pos ), 1 ) <> "-" Then  Exit For
	Next
	If expr_pos >= UBound( params_arr ) Then
		Raise  1, "Few grep parameters"
	End If

	expression = params_arr( expr_pos )

	'// first char "/" -> "\/"
	If Left( expression, 1 ) = "/" Then  expression = "\"+ expression

	'// space /C option
	If InStr( expression, " " ) > 0 Then _
		expression = "/C:"+ expression

	'// "+" -> "*"
	plus_pos = InStr( expression, "+" )
	While  plus_pos > 0

		'// Set "not_meta_character"
		ReDim  not_meta_character( Len( expression ) )
		pos = 1
		Do
			pos = InStr( pos, expression, "\" )
			If pos = 0 Then  Exit Do
			not_meta_character( pos + 1 ) = True
			pos = pos + 2
		Loop

		'// Set "start_of_repeat"
		If Mid( expression, plus_pos - 1, 1 ) = "]"  and _
				not not_meta_character( plus_pos - 1 ) Then
			Do
				pos = InStrRev( expression, "[", plus_pos )
				Assert  pos > 0
				If not not_meta_character( pos - 1 ) Then _
					Exit Do
				pos = pos - 1
			Loop
			start_of_repeat = pos
		Else
			start_of_repeat = plus_pos - 1
		End If

		'// Set "expression"
		expression = Left( expression, plus_pos - 1 ) + _
			Mid( expression,  start_of_repeat,  plus_pos - start_of_repeat ) + _
			"*" + _
			Mid( expression,  plus_pos + 1 )

		'// Next
		plus_pos = InStr( plus_pos, expression, "+" )
	WEnd

	'// "|" vertical line
	expression = Replace( expression, "\|", "\v" )
	expression = Replace( expression, "|", " " )
	expression = Replace( expression, "\v", "|" )

	'// Set "expression"
	If expression = "" Then  Raise  1, "No search keyword."
	If InStr( expression, vbLF ) > 0 Then  Raise  1, "There is a return character in the keyword. Use -u option"
	params_arr( expr_pos ) = expression


	'// target folder + "\*"
	target_path = params_arr( expr_pos + 1 )
	If InStr( target_path, "*" ) = 0 Then
		If g_fs.FolderExists( target_path ) Then _
			params_arr( expr_pos + 1 ) = target_path +"\*"
	End If
	If LenK( target_path ) <> Len( target_path ) Then
		Raise  1, "The file path has not ascii character. Use -u option."
	End If

	'// --include option
	For i = 0  To UBound( params_arr )
		If Left( params_arr( i ), 10 ) = "--include=" Then
			If InStr( target_path, "*" ) > 0 Then
				target_path = g_fs.GetParentFolderName( target_path )
				If target_path = "" Then
					params_arr( expr_pos + 1 ) = Mid( params_arr( i ), 11 )
				Else
					params_arr( expr_pos + 1 ) = target_path +"\"+ Mid( params_arr( i ), 11 )
				End If
			Else
				params_arr( expr_pos + 1 ) = target_path +"\"+ Mid( params_arr( i ), 11 )
			End If

			params_arr( i ) = ""
			Exit For
		End If
	Next


	'// Resume Params : string -> dos cmdline
	Params = CmdLineFromStr( params_arr )


	If g_is_debug Then  echo  ">>> "+ Params


	'//
	Params = Replace( Params, "-i", "/I" )
	Params = Replace( Params, "-r", "/S" )
	Params = Replace( Params, "-l", "/M" )


	If OutPath <> "" Then _
		mkdir_for  OutPath


	is_use_safetee = False
	If out_is_empty Then
		If g_EchoObj.m_bEchoOff Then
			RunProg  "cmd /c findstr /R /N "+ Params +" > """+ OutPath +"""", c.NotEchoStartCommand
		Else
			is_use_safetee = True
		End If
	Else
		If OutPath = "" Then
			RunProg  "cmd /c findstr /R /N "+ Params, c.NotEchoStartCommand
		Else
			is_use_safetee = True
		End If
	End If
	If is_use_safetee Then
		AssertExist  g_vbslib_ver_folder +"safetee\safetee.exe"
		RunProg  "cmd /c findstr /R /N "+ Params +" | """+ g_vbslib_ver_folder +_
			"safetee\safetee.exe"" -o """+ OutPath +"""", c.NotEchoStartCommand
	End If


	'//=== //
	If out_is_empty Then
		Set founds = new ArrayClass

		params_arr = ArrayFromCmdLine( Params )
		is_exist_colon = ( InStr( params_arr( UBound( params_arr ) ), ":" ) > 0 )
		is_exist_slashM = ( InStr( Params, "/M " ) > 0 )

		Set f = OpenForRead( OutPath )
		Do Until  f.AtEndOfStream
			line = f.ReadLine()
			If is_exist_slashM Then
				Set o = new GrepFound
					o.Path = line
				founds.Add  o
				o = Empty
			Else
				i = InStr( line, ":" )
				If is_exist_colon Then  i = InStr( i+1, line, ":" )
				If i > 0 Then
					i2 = InStr( i+1, line, ":" )

					Set o = new GrepFound
					If i2 > 0 Then
						o.Path = Left( line, i-1 )
						o.LineNum = Mid( line, i+1, i2-i-1 )
						o.LineText = Mid( line, i2+1 )
					Else
						o.Path = target_path
						o.LineNum = Left( line, i-1 )
						o.LineText = Mid( line, i+1 )
					End If
					founds.Add  o
					o = Empty
				End If
			End If
		Loop
		f = Empty

		Set ec_= new EchoOff
		del  OutPath
	End If

	If not IsEmpty( founds ) Then
		grep = founds.Items
		If g_Vers("ExpandWildcard_Sort") Then  SortGrepFoundArray  grep
	End If
End Function


Class  GrepFound
	Public  Path  '// folder path in 2nd parameter in grep + step path
	Public  LineNum
	Public  LineText

	Public Sub  CalcLineCount()
		If IsArray( LineText ) Then
			text = ReadFile( Path )

			If IsEmpty( LineNum ) Then
				LineNum = GetLineCount( Left( text, LineText(0) ), Empty )
			End If

			text = Mid( text, LineText(0), LineText(1) - LineText(0) )
			LineText = text
		End If
	End Sub

	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		CalcLineCount
		xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Path='"+ XmlAttrA( Path ) + _
			"' LineNum='"& XmlAttrA( LineNum ) &"'>"+ _
			LineText +"</"+TypeName(Me)+">"+ vbCRLF
	End Function
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [GrepClass] >>>> 
'-------------------------------------------------------------------------

Class  GrepClass
	Private  m_RegExp
	Public   IsRecurseSubFolders  '// as boolean. Ignored for PathDictionaryClass
	Public   IsOutFileNameOnly    '// as boolean
	Public   IsInvertMatch        '// as boolean
	Public   IsHitCount           '// as boolean

	Private Sub  Class_Initialize()
		Set m_RegExp = CreateObject("VBScript.RegExp")
		m_RegExp.MultiLine = True

		Me.IsRecurseSubFolders = False
		Me.IsOutFileNameOnly = False
		Me.IsInvertMatch = False
		Me.IsHitCount = False
	End Sub

	Public Property Let  Pattern( x ) : m_RegExp.Pattern = x : End Property
	Public Property Get  Pattern() : Pattern = m_RegExp.Pattern : End Property
	Public Property Let  IgnoreCase( x ) : m_RegExp.IgnoreCase = x : End Property
	Public Property Get  IgnoreCase() : IgnoreCase = m_RegExp.IgnoreCase : End Property
	Public Property Get  RegExp() : Set RegExp = m_RegExp : End Property


	Function  Execute( TargetPath )
		Set c = g_VBS_Lib
		out_founds = Array( )
		m_RegExp.Global = not Me.IsOutFileNameOnly

		If Me.IsRecurseSubFolders Then
			Set target_paths = ArrayFromWildcard( TargetPath )
		ElseIf TypeName( TargetPath ) = "PathDictionaryClass" Then
			Set target_paths = TargetPath
		Else
			If IsArray( TargetPath ) Then
				target_path_array = TargetPath
			Else
				target_path_array = Array( TargetPath )
			End If
			For index = 0  To UBound( target_path_array )
				target_path = target_path_array( index )

				CutLastOf  target_path, "\", c.CaseSensitive
				CutLastOf  target_path, "/", c.CaseSensitive

				'// Add ".\"
				sp = InStrEx( target_path, Array( "\", "/" ), 0, c.Rev )  '// Separator Position
				If sp = 0 Then
					target_path = ".\"+ target_path
				Else
					target_path = Left( target_path, sp ) +"."+ Mid( target_path, sp )
				End If

				target_path_array( index ) = target_path
			Next

			Set target_paths = ArrayFromWildcard( target_path_array )
		End If

		base_path = GetPathWithSeparator( target_paths.BasePath )
		For Each  target_path  In target_paths.FilePaths
			If IsFullPath( target_path ) Then
				target_full_path = target_path
			Else
				target_full_path = base_path + target_path
			End If

			Me.Execute_Sub  target_path,  target_full_path,  out_founds,  Empty
		Next

		If g_Vers("ExpandWildcard_Sort") Then  SortGrepFoundArray  out_founds

		Execute = out_founds
	End Function


	'// Share code with grep (unicode)
	Sub  Execute_Sub( target_path,  target_full_path,  out_founds,  out_file )
		Set c = g_VBS_Lib
		text = ReadFile( target_full_path )
		Set matches = m_RegExp.Execute( text )

		If Me.IsOutFileNameOnly Then
			found = Empty
			If not Me.IsInvertMatch  and  matches.Count >= 1 Then
				Set found = new GrepFound
			ElseIf Me.IsInvertMatch  and  matches.Count = 0 Then
				Set found = new GrepFound
			End If
			If not IsEmpty( found ) Then
				found.Path = target_path

				If not IsEmpty( out_founds ) Then
					ReDim Preserve  out_founds( UBound( out_founds ) + 1 )
					Set out_founds( UBound( out_founds ) ) = found
				End If

				echo  found.Path
				If not IsEmpty( out_file ) Then
					out_file.WriteLine  found.Path
				End If
			End If
		Else
			If not IsEmpty( out_founds ) Then
				old_ubound = UBound( out_founds )
				ReDim Preserve  out_founds( old_ubound + matches.Count )
			End If
			previous_line_num = -1

			For Each  match  In matches
				Set found = new GrepFound

				found.Path = target_path
				found.LineNum = GetLineCount( Left( text, match.FirstIndex ), Empty )

				If found.LineNum <> previous_line_num Then
						'// When hit one line, not make multi
					previous_line_num = found.LineNum

					previous_lf = InStrRev( text, vbLF, match.FirstIndex + 1 )
					next_lf = InStr( match.FirstIndex + 1, text, vbLF )
					If next_lf = 0 Then  next_lf = Len( text ) + 1
					line = Mid( text, previous_lf + 1, next_lf - previous_lf - 2 )
					CutLastOf  line, vbCR, c.CaseSensitive
					found.LineText = line

					If not IsEmpty( out_founds ) Then
						old_ubound = old_ubound + 1
						Set out_founds( old_ubound ) = found
					End If

					line_string = found.Path +":" & found.LineNum & ":"+ line

					echo  line_string
					If not IsEmpty( out_file ) Then
						out_file.WriteLine  line_string
					End If
				End If
			Next
			If Me.IsHitCount  and  matches.Count = 0 Then
				Set found = new GrepFound

				found.Path = target_path
				found.LineNum = 0

				If not IsEmpty( out_founds ) Then
					old_ubound = old_ubound + 1
					ReDim Preserve  out_founds( old_ubound )
					Set out_founds( old_ubound ) = found
				End If

				line_string = found.Path +":" & found.LineNum & ":"

				echo  line_string
				If not IsEmpty( out_file ) Then
					out_file.WriteLine  line_string
				End If
			End If

			If not IsEmpty( out_founds ) Then
				If old_ubound < UBound( out_founds ) Then
					ReDim Preserve  out_founds( old_ubound )
				End If
			End If
		End If
	End Sub
End Class


 
Sub  grep_unicode_sub( params_arr, OutPath, out_founds )
	Set c = g_VBS_Lib

	Set a_grep = new GrepClass

	flags = c.File or c.FullPath
	unnamed_index = 0
	name_mask = "*"
	Set targets = new ArrayClass


	For Each a_parameter  In params_arr
		If a_parameter = "-u" Then
			'// ignore
		ElseIf a_parameter = "-r" Then
			flags = flags or c.SubFolderIfWildcard or c.NoError
		ElseIf a_parameter = "-i" Then
			a_grep.IgnoreCase = True
		ElseIf a_parameter = "-l" Then
			a_grep.IsOutFileNameOnly = True
			a_grep.IsInvertMatch = False
		ElseIf a_parameter = "-L" Then
			a_grep.IsOutFileNameOnly = True
			a_grep.IsInvertMatch = True
		ElseIf a_parameter = "-c" Then
			a_grep.IsHitCount = True
		ElseIf Left( a_parameter, 10 ) = "--include=" Then
			Assert  name_mask = "*"
			name_mask = MeltCmdLine( Mid( a_parameter, 11 ), 1 )
		Else
			Assert  Left( a_parameter, 1 ) <> "-"
			If unnamed_index = 0 Then
				expression = a_parameter
				If expression = "" Then  Raise  1, "No search keyword."
			Else
				targets.Add  a_parameter
			End If
			unnamed_index = unnamed_index + 1
		End If
	Next


	If targets.Count = 0 Then
		Raise  1, "Few grep parameters"
	End If
	ReDim  path_starts( targets.UBound_ )
	ReDim  path_heads( targets.UBound_ )

	path_start = Len( g_sh.CurrentDirectory ) + 2

	For index = 0 To targets.UBound_
		target_path = targets( index )

		base_path = ""
		If target_path = "*" Then
			target_path = name_mask
		Else
			If Right( target_path, 2 ) = "\*" Then
				target_path = Left( target_path, Len( target_path ) - 2 )
			ElseIf Right( target_path, 1 ) = "\" Then
				target_path = Left( target_path, Len( target_path ) - 1 )
			End If

			If not IsWildcard( target_path ) Then
				If g_fs.FolderExists( target_path ) Then
					base_path = target_path +"\"
					target_path = base_path + name_mask
				End If
			End If
		End If

		targets( index ) = target_path

		If IsFullPath( target_path ) Then
			path_starts( index ) = 1
			path_heads( index ) = ""
		ElseIf InStr( target_path, "\..\" ) = 0  and  Left( target_path, 3 ) <> "..\" Then
			path_starts( index ) = path_start
			path_heads( index ) = ""
		Else
			path_heads( index ) = g_fs.GetParentFolderName( target_path ) +"\"
			path_starts( index ) = Len( GetFullPath( path_heads( index ), Empty ) ) + 2
		End If
	Next


	a_grep.Pattern = expression
	a_grep.RegExp.Global = not a_grep.IsOutFileNameOnly
	out_founds = Array( )


	If VarType( OutPath ) = vbString  and  OutPath <> "" Then
		Set ec = new EchoOff
		Set out_file = OpenForWrite( OutPath, c.Unicode )
		ec = Empty
	Else
		out_file = Empty
	End If


	For index = 0 To targets.UBound_

		ExpandWildcard  targets( index ), flags, Empty, paths

		path_start = path_starts( index )
		path_head = path_heads( index )

		For Each  target_full_path  In paths
			target_path = path_head + Mid( target_full_path, path_start )

			a_grep.Execute_Sub  target_path,  target_full_path,  out_founds,  out_file
		Next
	Next

	If g_Vers("ExpandWildcard_Sort") Then  SortGrepFoundArray  out_founds

	echo  ""
End Sub


 
'*************************************************************************
'  <<< [SortGrepFoundArray] >>> 
'*************************************************************************
Sub  SortGrepFoundArray( GrepFounds )
	QuickSort  GrepFounds, 0, UBound( GrepFounds ), GetRef("SortGrepFoundArray_Sub"), Empty
End Sub

Function  SortGrepFoundArray_Sub( Left, Right, Param )
	sign = PathCompare( Left.Path, Right.Path, Param )
	If sign = 0 Then  sign = Left.LineNum - Right.LineNum
	SortGrepFoundArray_Sub = sign
End Function

 
'*************************************************************************
'  <<< [GrepKeyword] >>> 
'*************************************************************************
Function  GrepKeyword( Keyword )
	GrepKeyword = Keyword
	GrepKeyword = Replace( GrepKeyword, "\", "\\\\" )
	If Left( Keyword, 1 ) = "-" Then  If Len( Keyword ) >= 2 Then  GrepKeyword = "\"+ GrepKeyword
End Function


 
'*************************************************************************
'  <<< [EGrepKeyword] >>> 
'*************************************************************************
Function  EGrepKeyword( Keyword )
	EGrepKeyword = Keyword
	EGrepKeyword = Replace( EGrepKeyword, "\", "\\\\" )
	If Left( Keyword, 1 ) = "-" Then  If Len( Keyword ) >= 2 Then  EGrepKeyword = "\"+ EGrepKeyword
	EGrepKeyword = Replace( EGrepKeyword, "|", "\|" )
End Function


 
'*************************************************************************
'  <<< [GrepExpression] >>> 
'*************************************************************************
Function  GrepExpression( Keyword )
	GrepExpression = Keyword
	GrepExpression = Replace( GrepExpression, "\", "\\" )
	If Left( Keyword, 1 ) = "-" Then  If Len( Keyword ) >= 2 Then  GrepExpression = "\"+ GrepExpression
End Function


 
'*************************************************************************
'  <<< [sort] >>> 
'*************************************************************************
Sub  sort( InPath, OutPath )
	RunProg  "cmd /C sort """ + InPath + """ /o """ + OutPath + """", ""
End Sub


 
'*************************************************************************
'  <<< [CreateFile] >>> 
'*************************************************************************
Function  CreateFile( ByVal Path, ByVal Text )
	Dim  t,  f,  f2,  is_unicode,  folder,  n_retry,  org_path,  en, ed
	Dim  c : Set c = g_VBS_Lib

	If VarType( Text ) <> vbString Then _
		Text = CStr( Text )
	t = InStr( Text, vbCRLF )
	If t = 0 Then  t = Text+""""  Else  t = Left( Text, t-1 ) + """+vbCRLF+..."
	echo  ">CreateFile  """ & Path & """, """ & t

	If IsWildcard( Path ) Then  Path = GetTempPath( Path ) : echo  "Create  """ & Path & """"

	If g_FileOptions.IsSafeFileUpdate Then
		org_path = Path
		Path = GetTempPath( "CreateFile_*.txt" )
	End If

	Dim  ec : Set ec = new EchoOff : ErrCheck

	g_AppKey.CheckWritable  Path, Empty

	Path = g_fs.GetAbsolutePathName( Path )
	folder = g_fs.GetParentFolderName( Path )
	If not g_fs.FolderExists( folder ) Then  mkdir  folder


	Select Case  g_FileOptions.CharSet
		Case  "unicode"                       : is_unicode = True
		Case  Empty, "shift_jis", "shift-jis" : is_unicode = False
		Case  Else
			Set f2 = CreateObject( "ADODB.Stream" )
			f2.Charset = g_FileOptions.CharSet
			f2.Open
			f2.WriteText  Text

			If g_FileOptions.CharSetEx = c.UTF_8_No_BOM Then
				f2.Position = 0
				f2.Type = 1  '// adTypeBinary
				f2.Position = 3  '// skip BOM
				t = f2.Read()  '// read all after BOM
				f2.Position = 0
				f2.Write  t
				f2.SetEOS
			End If
	End Select


	n_retry = 0
	Do
		ErrCheck : On Error Resume Next

			'//####################### main
			If IsEmpty( is_unicode ) Then
				f2.SaveToFile  Path, 2
				f2.Close
			Else
				Set f = g_fs.CreateTextFile( Path, True, is_unicode )
			End If

		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If not IsWriteAccessDenied( en, Path, c.File, n_retry ) Then  Exit Do
	Loop
	If en <> 0 Then  Err.Raise en,,ed


	If not IsEmpty( f ) Then
		f.Write  Text
		f.Close
	End If


	If g_FileOptions.IsSafeFileUpdate Then
		SafeFileUpdate  Path, org_path
		del  Path
		Path = org_path
	End If

	CreateFile = Path
End Function


 
'*************************************************************************
'  <<< [touch] >>> 
'*************************************************************************
Sub  touch( Path )
	Set ec = new EchoOff
	If exist( Path ) Then
		g_AppKey.CheckWritable  Path, Empty
		r= RunProg( """"+ g_vbslib_ver_folder +"vbslib_helper.exe"" touch """+ Path +"""", "" )
		Assert  r = 0
	Else
		CreateFile  Path, ""
	End IF
End Sub


 
'*************************************************************************
'  <<< [ReadFile] >>> 
'*************************************************************************
Function  ReadFile( Path )
	Set c = g_VBS_Lib

	If TypeName( Path ) = "FilePathClass" Then
		ReadFile = Path.Text
		Exit Function
	End If

	ReadFile = ""

	ErrCheck : On Error Resume Next

		Select Case  g_FileOptions.CharSet
			Case  "unicode"
				Set f = g_fs.OpenTextFile( Path, 1, False, True )

			Case  Empty, "shift_jis", "shift-jis"
				Select Case  ReadUnicodeFileBOM( Path )
					Case  c.Unicode
						Set f = g_fs.OpenTextFile( Path, 1, False, True )
					Case  c.UTF_8
						Set cs = new_TextFileCharSetStack( "UTF-8" )
					Case Else
						Set f = g_fs.OpenTextFile( Path, 1, False, False )
				End Select
		End Select

		If IsEmpty( f )  and  Err.Number = 0 Then
			Set f2 = CreateObject( "ADODB.Stream" )
			f2.Charset = g_FileOptions.CharSet
			f2.Open
			f2.LoadFromFile( Path )
		End If

	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = E_FileNotExist  or  en = E_PathNotFound  or _
		 en = E_EndOfFile  or  en = 3002  Then  Exit Function
			'// E_PathNotFound is not found parent folder
			'// 3002 is E_FileNotExist or E_PathNotFound from LoadFromFile

	If en <> 0 Then
		Raise  en, "<ERROR msg="""+ ed +""" path="""+ GetFullPath( Path, Empty ) +"""/>"
	End If

	If IsEmpty( f2 ) Then
		ReadFile = ReadAll( f )
	Else
		ReadFile = f2.ReadText( -1 )  '// adReadAll
	End If
End Function


 
'*************************************************************************
'  <<< [ReadFileInTag] >>> 
'*************************************************************************
Function  ReadFileInTag( Path, StartOfText, EndOfText )
	ReadFileInTag = sscanf( ReadFile( Path ), StartOfText +"%s"+ EndOfText )
	While Left(  ReadFileInTag, 2 ) = vbCRLF : ReadFileInTag = Mid(  ReadFileInTag, 3 ) : WEnd
	While Right( ReadFileInTag, 2 ) = vbCRLF : ReadFileInTag = Left( ReadFileInTag, Len(ReadFileInTag) - 2 ) : WEnd
End Function


 
'*************************************************************************
'  <<< [type_] >>> 
'*************************************************************************
Sub  type_( Path )
	echo  ">type_  """ & Path & """"
	echo  ReadFile( Path )
End Sub


 
'*************************************************************************
'  <<< [OpenForRead] >>> 
'*************************************************************************
Function  OpenForRead( Path )
	Dim  f, f2, en, ed, cs
	Dim  c : Set c = g_VBS_Lib

	ErrCheck : On Error Resume Next

		Select Case  g_FileOptions.CharSet
			Case  "unicode"
				If IsEmpty( g_FileOptions.LineSeparator ) Then
					AssertExist  Path
					Set f = g_fs.OpenTextFile( Path, 1, False, True )
				Else
					Set cs = new_TextFileCharSetStack( "Unicode" )
				End IF

			Case  Empty, "shift_jis", "shift-jis"
				Select Case  ReadUnicodeFileBOM( Path )
					Case  c.Unicode
						If IsEmpty( g_FileOptions.LineSeparator ) Then
							Set f = g_fs.OpenTextFile( Path, 1, False, True )
						Else
							Set cs = new_TextFileCharSetStack( "Unicode" )  '// to g_FileOptions.CharSet
						End IF
					Case  c.UTF_8
						Set cs = new_TextFileCharSetStack( "UTF-8" )
					Case Else
						If IsEmpty( g_FileOptions.LineSeparator ) Then
							Set f = g_fs.OpenTextFile( Path, 1, False, False )
						Else
							Set cs = new_TextFileCharSetStack( "Shift_JIS" )
						End If
				End Select
		End Select

		If IsEmpty( f )  and  Err.Number = 0 Then
			AssertExist  Path
			Set f2 = CreateObject( "ADODB.Stream" )
			f2.Charset = g_FileOptions.CharSet
			f2.Open
			f2.LoadFromFile( Path )
		End If

	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = E_EndOfFile Then  en = 0  '// This is no problem, when FileSize=0
	If en <> 0 Then
		If en = 3002 Then
			If g_fs.FileExists( Path ) Then  en = E_ReadAccessDenied  Else  en = E_FileNotExist
				'// 3002 is E_FileNotExist or E_PathNotFound from LoadFromFile
		End If
		If en = E_FileNotExist or en = E_PathNotFound Then  ed = ed +" : "+ GetFullPath( Path, Empty )
		If en = E_ReadAccessDenied Then  ed = "Access denied : "+ GetFullPath( Path, Empty )
		Err.Raise en,,ed
	End If

	If not IsEmpty( f2 ) Then
		Set f = new TextStreamDelegateToADODBStream
		Set f.aADODBStream = f2
		If g_FileOptions.LineSeparator = g_VBS_Lib.KeepLineSeparators Then _
			f.LineSeparator = Empty  '// //
		If IsEmpty( g_FileOptions.LineSeparator ) Then _
			f.LineSeparator = c.CutLineSeparators
	End If
	Set OpenForRead = f
End Function


 
'*************************************************************************
'  <<< [OpenForWrite] >>> 
'*************************************************************************
Function  OpenForWrite( ByVal Path, Flags )
	echo  ">OpenForWrite  """ & Path & """"
	Dim  en, ed,  o,  n_retry,  charset,  charset_stack,  is_unicode,  text
	Dim  c : Set c = g_VBS_Lib
	Dim  is_append  : is_append  = ((Flags and c.Append)  = c.Append)

	If IsWildcard( Path ) Then  Path = GetTempPath( Path ) : echo  "Create  """ & Path & """"

	g_AppKey.CheckWritable  Path, Empty


	If is_append Then
		Select Case  ReadUnicodeFileBOM( Path )
			Case  c.Unicode : charset = "unicode"
			Case  c.UTF_8   : charset = "utf-8"
			Case  c.No_BOM  '// Do nothing
			Case  Empty     : is_append = False
		End Select
	End If

	If Flags and c.Shift_JIS Then    charset = "shift_jis"
	If Flags and c.Unicode Then      charset = "unicode"
	If Flags and c.UTF_8 Then        charset = "utf-8"
	If Flags and c.UTF_8_No_BOM Then charset = "utf-8-no-bom"
	If Flags and c.EUC_JP Then       charset = "euc-jp"
	If Flags and c.ISO_2022_JP Then  charset = "ISO-2022-JP"
	If not IsEmpty( charset ) Then  Set charset_stack = new_TextFileCharSetStack( charset )

	If IsEmpty( g_FileOptions.LineSeparator ) Then
		Select Case  g_FileOptions.CharSet
			Case  "unicode"                : is_unicode = True
			Case  "shift_jis", "shift-jis" : is_unicode = False
			Case  Empty
				Select Case  ReadUnicodeFileBOM( Path )
					Case  c.Unicode : is_unicode = True
					Case  c.UTF_8   : charset = "utf-8"
					Case  c.No_BOM  : is_unicode = False
					Case  Empty       '// Do nothing
				End Select
		End Select
	End If

	If IsEmpty( is_unicode ) Then
		If is_append Then
			Set o = CreateObject( "ADODB.Stream" )
			o.Charset = g_FileOptions.CharSet
			o.Open
			ADODB_Stream_loadFromFile  o, Path
			text = o.ReadText( -1 )  '// -1 = adReadAll
			o.Close
		End If

		Set o = CreateObject( "ADODB.Stream" )
			If IsEmpty( g_FileOptions.CharSet ) or g_FileOptions.CharSet = "shift-jis" Then
				o.Charset = "shift_jis"
				'// Err.Number = //
				'// //
			Else
				o.Charset = g_FileOptions.CharSet
			End If
			o.Open
			If is_append Then  o.WriteText  text
		If not IsEmpty( g_FileOptions.LineSeparator ) Then _
			o.LineSeparator = g_VBS_Lib.LF
		Set OpenForWrite = new TextStreamDelegateToADODBStream
		Set OpenForWrite.aADODBStream = o
		OpenForWrite.WriteAbsPath = GetFullPath( Path, Empty )
		OpenForWrite.CharSetEx = g_FileOptions.CharSetEx
		OpenForWrite.LineSeparator = vbCRLF
	End If


	'//=== open
	n_retry = 0
	Do
		ErrCheck : On Error Resume Next

			If not IsEmpty( is_unicode ) Then
				If is_append Then
					Set OpenForWrite = g_fs.OpenTextFile( Path, 8, True, -2 )
				Else
					Set OpenForWrite = g_fs.CreateTextFile( Path, True, is_unicode )
				End If
			End If

		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If en = 0 Then  Exit Function
		If not IsWriteAccessDenied( en, Path, c.File, n_retry ) Then  Exit Do
	Loop


	'//=== create folder and open
	If en = E_PathNotFound Then
		Dim  fo : fo = g_fs.GetParentFolderName( Path )
		If not g_fs.FolderExists( fo ) Then
			mkdir  fo

			n_retry = 0
			Do
				ErrCheck : On Error Resume Next

					Set OpenForWrite = g_fs.CreateTextFile( Path, True, is_unicode )

				en = Err.Number : ed = Err.Description : On Error GoTo 0
				If en = 0 Then  Exit Function
				If not IsWriteAccessDenied( en, Path, c.File, n_retry ) Then  Exit Do
			Loop
		End If
	End If
	Err.Raise en,,ed
End Function


 
'*************************************************************************
'  <<< [TextStreamDelegateToADODBStream] Class >>> 
'*************************************************************************
Class  TextStreamDelegateToADODBStream
	Public   aADODBStream
	Private  m_Line        '// as integer
	Public   WriteAbsPath  '// as string
	Public   CharSetEx     '// Empty, c.UTF_8_No_BOM
	Private  m_LineSeparator  '// for Property as vbCRLF or vbLF or Empty
	Private  m_ReadLineSeparator  '// as string

	Private Sub  Class_Initialize() : m_Line = 1 : m_LineSeparator = vbCRLF : End Sub
	Private Sub  Class_Terminate() : Me.Close : End Sub

	Public Function  Read( n )  : Read     = Me.aADODBStream.ReadText( n )  : Me.CountLine  Read : End Function
	Public Function  ReadAll()  : ReadAll  = Me.aADODBStream.ReadText( -1 ) : Me.CountLine  ReadAll : End Function

	Public Function  ReadLine()
		If m_ReadLineSeparator = "cut" Then
			ReadLine = Me.aADODBStream.ReadText( -2 )
			If Right( ReadLine, 1 ) = vbCR Then  ReadLine = Left( ReadLine, Len( ReadLine ) - 1 )
		Else
			ReadLine = Me.aADODBStream.ReadText( -2 ) + m_ReadLineSeparator
		End If
		m_Line = m_Line + 1
	End Function

	Public Sub  Skip( n )  : Me.aADODBStream.ReadText  n  : End Sub
	Public Sub  SkipLine() : Me.aADODBStream.SkipLine : End Sub

	Public Sub  Write( Text ) : Me.aADODBStream.WriteText  Text : Me.CountLine  Text : End Sub

	Public Sub  WriteLine( Text )
		If Right( Text, 1 ) = vbLF Then
			Me.aADODBStream.WriteText  Text, 0  '// 0=adWriteChar
			If m_LineSeparator = vbLF Then
				If Right( Text, 2 ) = vbCRLF Then  Me.LineSeparator = vbCRLF
			Else
				If Right( Text, 2 ) <> vbCRLF Then  Me.LineSeparator = vbLF
			End If
		Else
			Me.aADODBStream.WriteText  Text, 1  '// 1=adWriteLine
			m_Line = m_Line + 1
		End If
		Me.CountLine  Text
	End Sub

	Public Sub  WriteBlankLines( n )
		Dim i : For i=1 To n : Me.aADODBStream.WriteText  Me.LineSeparator, 0 : Next : m_Line = m_Line + n
	End Sub

	Public Sub  WriteLineDefault( ByVal Text )
		Text = Replace( Text, vbCRLF, vbLF )
		If m_LineSeparator = vbCRLF Then _
			Text = Replace( Text, vbLF, vbCRLF )
		Me.aADODBStream.WriteText  Text, 1  '// 1=adWriteLine
		Me.CountLine  Text
		m_Line = m_Line + 1
	End Sub

	Public Property Get  LineSeparator() : LineSeparator = m_LineSeparator : End Property
	Public Property Let  LineSeparator(x)
		Dim  c : Set c = g_VBS_Lib
		If x = vbCRLF Then
			Me.aADODBStream.LineSeparator = -1  '// -1=adCRLF
			m_ReadLineSeparator = ""
		ElseIf x = vbLF Then
			Me.aADODBStream.LineSeparator = 10  '// 10=adLF
			m_ReadLineSeparator = ""
		ElseIf x = c.CutLineSeparators Then
			Me.aADODBStream.LineSeparator = 10  '// 10=adLF
			m_ReadLineSeparator = "cut"
		Else
			Me.aADODBStream.LineSeparator = 10  '// 10=adLF
			m_ReadLineSeparator = vbLF
			x = Empty
		End If
		m_LineSeparator = x
	End Property

	Public Property Get  Line() : Line = m_Line : End Property
	Public Property Get  AtEndOfStream() : AtEndOfStream = aADODBStream.EOS : End Property

	Public Sub  CountLine( Text )
		If InStr( Text, vbLF ) = 0 Then  Exit Sub
		Dim  i : i = 1
		Do
			i = InStr( i, Text, vbLF )
			If i = 0 Then  Exit Sub
			m_Line = m_Line + 1
			i = i + 1
		Loop
	End Sub

	Public Function  Close()
		If not IsEmpty( Me.WriteAbsPath ) Then
			Dim  en,  ed,  n_retry,  o, t
			Dim  fo : fo = g_fs.GetParentFolderName( Me.WriteAbsPath )
			Dim  c : Set c = g_VBS_Lib

			If Me.CharSetEx = c.UTF_8_No_BOM Then
				Set o = Me.aADODBStream
					o.Position = 0
					o.Type = 1  '// adTypeBinary
					o.Position = 3  '// skip BOM
					t = o.Read()  '// read all after BOM
					o.Position = 0
					o.Write  t
					o.SetEOS
				o = Empty : t = Empty
			End If

			If not g_fs.FolderExists( fo ) Then  mkdir  fo

			n_retry = 0
			Do
				ErrCheck : On Error Resume Next

				Me.aADODBStream.SaveToFile  Me.WriteAbsPath, 2

				en = Err.Number : ed = Err.Description : On Error GoTo 0
				If en = 0 Then  Exit Do
				If en = 3004 Then  en = E_WriteAccessDenied
				If not IsWriteAccessDenied( en, Me.WriteAbsPath, c.File, n_retry ) Then  Exit Do
			Loop
			If en <> 0 Then  Err.Raise en,,ed
			Me.WriteAbsPath = Empty
		End If

		If not IsEmpty( aADODBStream ) Then  aADODBStream.Close
		aADODBStream = Empty
	End Function
End Class


 
'*************************************************************************
'  <<< [GetTempPath] >>> 
'*************************************************************************
Class  TempFileClass
	Public  m_FolderPath
	Public  m_LimitDate
End Class

Dim  g_TempFile


Function  GetTempPath( Param )
	Set c = g_VBS_Lib

	'//=== Delete old files
	If not g_fs.FolderExists( g_TempFile.m_FolderPath ) Then _
		mkdir  g_TempFile.m_FolderPath

	Set fo = g_fs.GetFolder( g_TempFile.m_FolderPath )
	For Each f in fo.Files
		If f.DateLastModified < g_TempFile.m_LimitDate Then
			path = f.Path
			n_retry = 0
			Do
				ErrCheck : On Error Resume Next
					g_fs.DeleteFile  path
				en = Err.Number : ed = Err.Description : On Error GoTo 0
				If not IsWriteAccessDenied( en, path, c.File, n_retry ) Then  Exit Do
			Loop
			If en <> 0 Then  Err.Raise en,,ed
		End If
	Next
	For Each f in fo.SubFolders
		If f.DateLastModified < g_TempFile.m_LimitDate Then
			path = f.Path
			n_retry = 0
			Do
				ErrCheck : On Error Resume Next
					g_fs.DeleteFolder  path
				en = Err.Number : ed = Err.Description : On Error GoTo 0
				If not IsWriteAccessDenied( en, path, c.Folder, n_retry ) Then  Exit Do
			Loop
			If en <> 0 Then  Err.Raise en,,ed
		End If
	Next


	'//=== path : Make unique path
	t = Now()
	param_abs = GetFullPath( Param, g_TempFile.m_FolderPath +"\"+ _
		Right( "0" & (Year(t) mod 100), 2 ) & _
		Right( "0" & Month(t), 2 )  &  Right( "0" & Day(t), 2 ) )

	t = Right( "0" & (Year(t) mod 100), 2 ) & _
		Right( "0" & Month(t), 2 )  &  Right( "0" & Day(t), 2 ) & "_" & _
		Right( "0" & Hour(t), 2 )  &  Right( "0" & Minute(t), 2 ) & "_"
	i = 1
	Do
		path = Replace( param_abs, "*", t & i )
		path = Replace( path, "?", i )
		If not exist( path ) Then  Exit Do
		i = i + 1
		If InStr( param_abs, "*" ) = 0  and  InStr( param_abs, "?" ) = 0 Then  Exit Do
	Loop
	GetTempPath = path
End Function


 
'*************************************************************************
'  <<< [get_TempFile] >>> 
'*************************************************************************
Sub  get_TempFile()
	If IsEmpty( g_TempFile ) Then
		Set g_TempFile = new TempFileClass : ErrCheck
		If IsDefined( "Setting_getTemp" ) Then
			Dim  out1, out2
			Setting_getTemp  out1, out2
			g_TempFile.m_FolderPath = out1
			g_TempFile.m_LimitDate = out2
		End If

		If IsEmpty( g_TempFile.m_FolderPath ) Then _
			g_TempFile.m_FolderPath = env( "%Temp%\Report" )
		If IsEmpty( g_TempFile.m_LimitDate ) Then _
			g_TempFile.m_LimitDate = DateAdd( "d", -2, Now() )

		If InStr( g_TempFile.m_FolderPath, "Temp" ) = 0 Then
			echo  "Not found ""Temp"" in temporary folder path in %Temp% or Setting_getTemp."
			echo  "Is this temporary folder path to delete? : " + g_TempFile.m_FolderPath
			echo  "Is this : " + g_TempFile.m_FolderPath
			pause
		End If
	End If
End Sub


 
'*************************************************************************
'  <<< [ReadAll] >>> 
'*************************************************************************
Function  ReadAll( FileStream )
	Dim  en, ed

	ReadAll = ""
	ErrCheck : On Error Resume Next
		ReadAll = FileStream.ReadAll()
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = E_EndOfFile Then  en = 0
	If en <> 0 Then  Err.Raise en,,ed
End Function


 
'*************************************************************************
'  <<< [ReadUnicodeFileBOM] >>> 
'*************************************************************************
Function  ReadUnicodeFileBOM( Path )
	Set c = g_VBS_Lib
	Const  adTypeBinary = 1
	Const  adReadLine = -2
	Const  adLF = 10

	If not g_fs.FileExists( Path ) Then  Exit Function
	size = g_fs.GetFile( Path ).Size
	If size <= 1 Then  ReadUnicodeFileBOM = c.No_BOM : Exit Function

	Set f = CreateObject( "ADODB.Stream" )
	f.Type = adTypeBinary
	f.Open
	ADODB_Stream_loadFromFile  f, Path
	Dim byte0 : byte0 = CByte( AscB( f.Read( 1 ) ) )
	Dim byte1 : byte1 = CByte( AscB( f.Read( 1 ) ) )
	Dim byte2
	If size = 2 Then
		byte2 = 0
	Else
		byte2 = CByte( AscB( f.Read( 1 ) ) )
	End If

	If byte0 = &hEF  and  byte1 = &hBB  and  byte2 = &hBF Then
		ReadUnicodeFileBOM = c.UTF_8    '// EF BB BF

	ElseIf byte0 = &hFF  and  byte1 = &hFE Then
		ReadUnicodeFileBOM = c.Unicode  '// FF FE

	ElseIf byte0 = &h3C  and  byte1 = &h3F  and  byte2 = &h78 Then  '// "<?x"

		Set f = CreateObject( "ADODB.Stream" )
		f.Charset = "UTF-8"
		f.Open
		f.LineSeparator = adLF
		f.LoadFromFile  Path

		line = f.ReadText( adReadLine )

		If InStr( line, "encoding=" ) > 0 Then
			If InStr( 1, line, "utf-8", 1 ) > 0 Then
				ReadUnicodeFileBOM = c.UTF_8_No_BOM
			Else
				ReadUnicodeFileBOM = c.No_BOM
			End If
		End If
	End If
	If IsEmpty( ReadUnicodeFileBOM ) Then
		If g_FileOptions.CharSet = "utf-8" Then
			ReadUnicodeFileBOM = c.UTF_8_No_BOM
		Else
			ReadUnicodeFileBOM = c.No_BOM
		End If
	End If
End Function


 
'********************************************************************************
'  <<< [ReadVBS_Comment] >>> 
'********************************************************************************
Function  ReadVBS_Comment( ByVal Path, ByVal StartTag, ByVal EndTag, Opt )
	If IsEmpty( Path ) Then  Path = WScript.ScriptFullName
	ReadVBS_Comment = ""
	is_in_tag = False
	is_in_tag_once = False

	Set jumps = GetTagJumpParams( Path )
	If not IsEmpty( jumps.Keyword ) Then
		StartTag = "["+  jumps.Keyword +"]"
		EndTag   = "[/"+ jumps.Keyword +"]"
		If jumps.Path = "" Then  jumps.Path = WScript.ScriptFullName
	End If

	Set file = OpenForRead( jumps.Path )
	Do Until  file.AtEndOfStream
		line = file.ReadLine()
		If not is_in_tag Then
			If InStr( line, StartTag ) > 0 Then _
				is_in_tag = True : is_in_tag_once = True
		Else
			If not IsEmpty( EndTag ) Then  If InStr( line, EndTag ) > 0 Then _
				Exit Function

			If Left( line, 1 ) <> "'" Then _
				Exit Function


			ReadVBS_Comment = ReadVBS_Comment + Mid( line, 2 ) + vbCRLF
		End If
	Loop
	If not is_in_tag_once Then _
		Raise  1, "<ERROR msg=""Not found"" keyword="""+ StartTag +""" path="""+ Path +"""/>"

	If is_in_tag  and  not IsEmpty( EndTag ) Then _
		Raise  1, "<ERROR msg=""Not found"" keyword="""+ EndTag +""" path="""+ Path +"""/>"
End Function


 
'********************************************************************************
'  <<< [WriteVBS_Comment] >>> 
'********************************************************************************
Sub  WriteVBS_Comment( ByVal Path, StartTag, EndTag, Text, Opt )
	If IsEmpty( Path ) Then  Path = WScript.ScriptFullName

	c_text = "'"+ Replace( Text, vbLF, vbLF + "'" )  '// Commented text
	c_text = Left( c_text, Len( c_text ) - 1 )

	vbs_path = GetTempPath( "Overwriting_*.vbs" )
	Set rep = StartReplace( Path, vbs_path, True )
	Do Until  rep.r.AtEndOfStream
		SplitLineAndCRLF  rep.r.ReadLine(), line, cr_lf
		If not is_in_tag Then
			If InStr( line, StartTag ) > 0 Then
				is_in_tag = True : is_in_tag_once = True
			End If
			rep.w.WriteLine  line + cr_lf
		Else
			If not IsEmpty( EndTag ) Then  If InStr( line, EndTag ) > 0 Then _
				is_in_tag = False : Exit Do

			If Left( line, 1 ) <> "'" Then _
				Exit Do
		End If
	Loop

	rep.w.WriteLine  c_text
	rep.w.WriteLine  line + cr_lf

	Do Until  rep.r.AtEndOfStream
		SplitLineAndCRLF  rep.r.ReadLine(), line, cr_lf
		rep.w.WriteLine  line + cr_lf
	Loop

	If not is_in_tag_once Then _
		Raise  1, "<ERROR msg=""Not found"" keyword="""+ StartTag +""" path="""+ Path +"""/>"

	If is_in_tag  and  not IsEmpty( EndTag ) Then _
		Raise  1, "<ERROR msg=""Not found"" keyword="""+ EndTag +""" path="""+ Path +"""/>"

	rep = Empty

	SafeFileUpdateEx  vbs_path, Path, Empty
End Sub


 
'*************************************************************************
'  <<< [ReadLineSeparator] >>> 
'*************************************************************************
Function  ReadLineSeparator( Path )
	Dim  f, t, i, size

	If not g_fs.FileExists( Path ) Then  Exit Function

	size = g_fs.GetFile( Path ).Size : If size > 1000 Then  size = 1000

	Set f = g_fs.OpenTextFile( Path,,, -2 )
	t = f.Read( size )
	i = InStr( t, vbLF )
	If i = 0 Then
		'// ReadLineSeparator = Empty
	ElseIf i = 1 Then
		ReadLineSeparator = vbLF
	ElseIf Mid( t, i-1, 2 ) = vbCRLF Then
		ReadLineSeparator = vbCRLF
	Else
		ReadLineSeparator = vbLF
	End If
End Function


 
'*************************************************************************
'  <<< [Txt2BinTxt] >>> 
'*************************************************************************
Sub  Txt2BinTxt( SrcPath, DstPath )
	RunTxt2BinTxtExe  """"+SrcPath+""" """+DstPath+""""
End Sub


Sub  RunTxt2BinTxtExe( CmdLineParam )
	Dim  r
	Dim  txt2bintxt_exe : txt2bintxt_exe = g_vbslib_ver_folder + "txt2bintxt.exe"

	If not g_fs.FileExists( txt2bintxt_exe ) Then _
		Err.Raise  1,, "not found txt2bintxt.exe in vbslib folder"

	r = RunProg( """"+txt2bintxt_exe+""" "+ CmdLineParam, Empty )
	If r<>0 Then  Err.Raise  1,, "error 0x" & Hex(r) & " in txt2bintxt.exe"
End Sub


 
'*************************************************************************
'  <<< [ConvertTextFileCharSet] >>> 
'*************************************************************************
Sub  ConvertTextFileCharSet( FromPath, FromCharSet, ToPath, ToCharSet )
	RunTxt2BinTxtExe  "/From:"+ FromCharSet +" """+SrcPath+""" /To:"+ ToCharSet +" """+DstPath+""""
End Sub


 
'*************************************************************************
'  <<< [WriteVBSLibHeader] >>> 
'*************************************************************************
Sub  WriteVBSLibHeader( OutFileStream, Opt )
	Dim  file, line

	Set file = g_fs.OpenTextFile( WScript.ScriptFullName,,,-2 )
	Do Until file.AtEndOfStream

		line = file.ReadLine()

		If InStr( line, "g_CommandPrompt =" ) > 0 and not IsEmpty( Opt ) Then
			If not IsEmpty( Opt.m_OverCommandPrompt ) Then
				line = "  g_CommandPrompt = " & Opt.m_OverCommandPrompt
			End If
		End If
		If InStr( 1, line, "main(", 1 ) > 0 Then Exit Do
		If InStr( 1, line, "main2(", 1 ) > 0 Then Exit Do

		OutFileStream.WriteLine  line
	Loop
End Sub


Class  WriteVBSLibHeader_Option
	Public  m_OverCommandPrompt
End Class


 
'*************************************************************************
'  <<< [WriteVBSLibFooter] >>> 
'*************************************************************************
Sub  WriteVBSLibFooter( OutFileStream, Opt )
	Dim  file,  line,  i,  is_out,  is_main

	For i = 1  To 7
		OutFileStream.WriteLine  ""
	Next

	is_out = False

	Set file = g_fs.OpenTextFile( WScript.ScriptFullName,,,-2 )
	Do Until file.AtEndOfStream

		line = file.ReadLine()

		If InStr( 1, line, "g_CommandPrompt =", 1 ) > 0 and not IsEmpty( Opt ) Then
			If not IsEmpty( Opt.CommandPromptMode ) Then
				i = InStr( line, "=" )
				line = Left( line, i ) +" " & Opt.CommandPromptMode
			End If
		End If

		If InStr( 1, line, "main(", 1 ) > 0 Then   is_main = True
		If InStr( 1, line, "main2(", 1 ) > 0 Then  is_main = True

		If InStr( line, "--- start of "+"vbslib include ---" ) > 0 Then
				'// "+" is for avoid "PartRep" tool
			If is_main Then
				is_out = True
			Else
				Exit Do
			End If
		End If

		If is_out Then _
			OutFileStream.WriteLine  line
	Loop
End Sub


Class  WriteVBSLibFooter_Option
	Public  CommandPromptMode
End Class


 
'*************************************************************************
'  <<< [GetFullPath] >>> 
'*************************************************************************
Function  GetFullPath( StepPath, ByVal BasePath )
	Dim  i, ii, i3, sep_ch, path, cur, is_root

	If IsNull( StepPath ) Then  GetFullPath = Null : Exit Function
	If IsEmpty( StepPath ) Then  Exit Function
	If IsEmpty( BasePath ) Then  BasePath = g_sh.CurrentDirectory
	If not IsFullPath( BasePath ) Then _
		Raise E_Others, "GetFullPath 's 2nd argument must be full path"
	If IsFullPath( StepPath ) Then  BasePath = Empty


	'//=== Joint and Replace to sep_ch
	sep_ch = GetFilePathSeparator( BasePath + StepPath )
	last_character = Right( BasePath, 1 )
	If last_character = "\"  or  last_character = "/"  or  IsEmpty( BasePath ) Then
		path = BasePath + StepPath
	Else
		path = BasePath + sep_ch + StepPath
	End If


	'// ...
	path = NormalizePath( path )


	'//=== Cut ... (SearchParent)
	If InStr( path, "..." ) > 0 Then
		i = InStr( path, sep_ch+"..."+sep_ch )
		If i = 0 Then
			If Left( path, 4 ) = "..."+sep_ch Then  i = 1
		Else
			i = i + 1
		End If
		If i > 0 Then
			Dim  ds_:Set ds_= new CurDirStack
			If i >= 2 Then  g_sh.CurrentDirectory = Left( path, i-2 )
			path = Mid( path, i+4 )
			Do
				cur = g_sh.CurrentDirectory
				is_root = ( InStr( cur, sep_ch ) = InStrRev( cur, sep_ch ) )
				If exist( path ) Then
					If is_root Then
						path = g_sh.CurrentDirectory + path
					Else
						path = g_sh.CurrentDirectory +"\"+ path
					End IF
					Exit Do
				End If
				If is_root Then  Err.Raise  E_PathNotFound
				g_sh.CurrentDirectory = ".."
			Loop
		End If
	End If


	If Right( path, 1 ) = ":" Then  path = path + sep_ch


	'(debug point) watch "path"

	GetFullPath = path
End Function


'//[GetAbsPath]
Function  GetAbsPath( StepPath, BasePath )
	GetAbsPath = GetFullPath( StepPath, BasePath )
	ThisIsOldSpec
End Function


 
'*************************************************************************
'  <<< [NormalizePath] >>> 
'*************************************************************************
Function  NormalizePath( ByVal Path )

	separator = GetFilePathSeparator( Path )


	'// Normalize separators
	If separator = "\" Then
		other_separator = "/"
	Else
		Assert  separator = "/"
		other_separator = "\"
	End If
	Path = Replace( Path, other_separator, separator )


	'// ...
	pos_root = GetRootSeparatorPosition( Path )


	'// Cut "\.\"
	Do
		i = InStr( Path, separator +"."+ separator )
		If i = 0 Then Exit Do
		Path = Left( Path, i ) + Mid( Path, i+3 )
	Loop
	While  Right( Path, 2 ) = separator+"."
		Path = Left( Path, Len(Path)-2 )
	WEnd
	While  Left( Path, 2 ) = "."+separator
		Path = Mid( Path, 3 )
	WEnd


	'// Cut "xxx\..\"
	pos_s1_s2 = 1  '// position of separator 1 to separator 2
	Do
		do_it = True

		pos_s1_s2 = InStr( pos_s1_s2, Path, separator+".."+separator )
		If pos_s1_s2 = 0 Then Exit Do

		pos_s0 = InStrRev( Path, separator, pos_s1_s2 - 1 )
		If pos_s0 < pos_root Then  Exit Do

		parent_name = Mid( Path,  pos_s0 + 1,  pos_s1_s2 - pos_s0 - 1 )
		If parent_name = ".." Then  do_it = False
		If parent_name = "" Then  do_it = False

		If do_it Then
			Path = Left( Path, pos_s0 ) + Mid( Path, pos_s1_s2 + 4 )
			pos_s1_s2 = pos_s0  '// Next of Separeor
			If pos_s0 = 0 Then Exit Do
		Else
			pos_s1_s2 = pos_s1_s2 + 3  '// Next of Separeor
		End If
	Loop


	'// Cut "\xxx\.."
	If Right( Path, 3 ) = separator + ".." Then
		pos_s1 = Len( Path ) - 2  '// position of separator 1
		If pos_s1 > pos_root Then
			pos_s0 = InStrRev( Path, separator, pos_s1 - 1 )
			If pos_s0 = pos_root Then
				If Mid( Path,  pos_s0 + 1,  pos_s1 - pos_s0 - 1 ) <> ".." Then
					If pos_s0 = 0 Then
						Path = "."
					Else
						Path = Left( Path, pos_root )
					End If
				End If
			ElseIf pos_s0 > pos_root Then
				If Mid( Path,  pos_s0 + 1,  pos_s1 - pos_s0 - 1 ) <> ".." Then
					Path = Left( Path, pos_s0 - 1 )
				End If
			End If
		End If
	End If


	'// Cut last "\"
	If Right( Path, 1 ) = "\" Then
		path_length = Len( Path )
		If path_length > pos_root Then
			Path = Left( Path, path_length - 1 )
		End If
	End If


	NormalizePath = Path
End Function


 
'*************************************************************************
'  <<< [GetStepPath] >>> 
' - AbsPath, BasePath, (return) as string
'*************************************************************************
Function  GetStepPath( AbsPath, ByVal BasePath )
	Dim  AbsPathU, BasePathU, path, sep_ch, i, ii

	If IsNull( AbsPath ) Then  GetStepPath = Null : Exit Function

	AbsPathU = UCase(AbsPath)
	If IsEmpty( BasePath ) Then _
		BasePath = g_sh.CurrentDirectory
	BasePathU = UCase( BasePath )

	sep_ch = GetFilePathSeparator( AbsPath )


	'// path = common parent folder path. The last character is not sep_ch
	path = BasePathU
	If Right( BasePathU, 1 ) = sep_ch Then  path = Left( BasePathU, Len(BasePathU)-1 )
	Do
		If path = Left( AbsPathU, Len(path) ) Then  Exit Do
		path = g_fs.GetParentFolderName( path )
	Loop
	If path = "" Then  GetStepPath = AbsPath : Exit Function
	If Right( path, 1 ) = sep_ch Then  path = Left( path, Len(path)-1 )
	'(debug point) watch "path"


	'// GetStepPath = step path without ..\
	GetStepPath = Mid( AbsPath, Len(path) + 2 )
	'(debug point) watch "GetStepPath"


	'// GetStepPath: Add "..\"
	path = Mid( BasePath, Len(path) + 2 )
	Do
		If path = "" Then Exit Do
		path = g_fs.GetParentFolderName( path )
		GetStepPath = ".." + sep_ch + GetStepPath
	Loop
	'(debug point) watch "GetStepPath"


	If GetStepPath = "" Then  GetStepPath = "."
	If Right( GetStepPath, 1 ) = sep_ch Then _
		GetStepPath = Left( GetStepPath, Len( GetStepPath ) - 1 )
End Function


 
'*************************************************************************
'  <<< [GetParentFullPath] >>> 
'*************************************************************************
Function  GetParentFullPath( Path )
	GetParentFullPath = g_fs.GetParentFolderName( g_fs.GetAbsolutePathName( Path ) )
End Function


 
'*************************************************************************
'  <<< [GetParentFoldersName] >>> 
'*************************************************************************
Function  GetParentFoldersName( FilePath, Level, SeparatorReplacedStr )
	full_path = g_fs.GetParentFolderName( g_fs.GetAbsolutePathName( FilePath ) )

	Assert  Level >= 1
	position = -1
	current_level = 0
	Do
		position_1 = InStrRev( full_path, "\", position )
		position_2 = InStrRev( full_path, "/", position )
		If position_1 > position_2 Then
			position = position_1
		Else
			position = position_2
		End If

		current_level = current_level + 1
		If current_level = Level Then _
			Exit Do
		If position = 0 Then _
			Exit Do
		position = position - 1
	Loop
	name = Mid( full_path, position + 1 )
	If not IsEmpty( SeparatorReplacedStr ) Then
		name = Replace( name, "\", SeparatorReplacedStr )
		name = Replace( name, "/", SeparatorReplacedStr )
	End If
	GetParentFoldersName = name
End Function


 
'*************************************************************************
'  <<< [GetRootSeparatorPosition] >>> 
'*************************************************************************
Function  GetRootSeparatorPosition( Path )

	separator = GetFilePathSeparator( Path )

	pos_separator = InStr( Path, separator )
	If pos_separator = 0 Then
		pos_root = 0
	ElseIf Mid( Path,  pos_separator + 1,  1 ) = separator Then  '// If double separator
		i = InStr( pos_separator + 2,  Path,  separator )
		If i > 0 Then
			If separator = "\" Then  '// UNC
				ii = InStr( i + 1,  Path,  separator )
				If ii > 0 Then  i = ii
			End If
			pos_root = i
		Else
			Path = Path + separator
			pos_root = Len( Path ) + 1
		End If
	Else
		If pos_separator >= 2 Then
			If Mid( Path,  pos_separator - 1,  1 ) = ":" Then
				pos_root = pos_separator
			Else
				pos_root = 0
			End If
		Else
			pos_root = pos_separator
		End If
	End If

	GetRootSeparatorPosition = pos_root
End Function


 
'*************************************************************************
'  <<< [GetFilePathSeparator] >>> 
'*************************************************************************
Function  GetFilePathSeparator( Path )
	Dim i  : i  = InStr( Path, "\" )
	Dim ii : ii = InStr( Path, "/" )

	If i > 0 Then
		If ii > 0 Then
			If i > ii Then  GetFilePathSeparator = "/" _
			Else            GetFilePathSeparator = "\"
		Else
			GetFilePathSeparator = "\"
		End If
	Else
		If ii > 0 Then  GetFilePathSeparator = "/" _
		Else            GetFilePathSeparator = "\"
	End If
End Function


 
'*************************************************************************
'  <<< [GetPathWithSeparator] >>> 
'*************************************************************************
Function  GetPathWithSeparator( Path )
	separator = GetFilePathSeparator( Path )
	If Right( Path, 1 ) = separator Then
		GetPathWithSeparator = Path
	Else
		GetPathWithSeparator = Path + separator
	End If
End Function


 
'*************************************************************************
'  <<< [ReplaceParentPath] >>> 
'*************************************************************************
Function  ReplaceParentPath( ByVal Path, FromSign, ToSign )
	separator = GetFilePathSeparator( Path )
	Path = NormalizePath( Path )
	pos_root = GetRootSeparatorPosition( Path )
	sub_path = Mid( Path,  pos_root + 1 )

	'// Set "parents_from"
	parents_from = ""
	Do
		parents_from = parents_from + FromSign + separator
		If StrCompHeadOf( sub_path, parents_from, Empty ) <> 0 Then
			parents_from = Mid( parents_from, Len( FromSign ) + 2 )
			Exit Do
		End If
	Loop

	'// Set "sub_path" from "parents_from" to "parents_to"
	If Len( parents_from ) >= 1 Then
		parents_to = Replace( parents_from, FromSign + separator, ToSign + separator )
		after_parents_from = Mid( sub_path, Len( parents_from ) + 1 )

		If after_parents_from = FromSign Then
			sub_path = parents_to + ToSign
		Else
			sub_path = parents_to + after_parents_from
		End If
	End If

	'// Replace one parent only
	If sub_path = FromSign Then
		sub_path = ToSign
	End If

	'// ...
	Path = Left( Path, pos_root ) + sub_path

	Assert  InStr( Path, FromSign ) = 0

	ReplaceParentPath = Path
End Function


 
'********************************************************************************
'  <<< [ReplaceRootPath] >>> 
'********************************************************************************
Function  ReplaceRootPath( BeforePath, BeforeRootFullPath, AfterRootFullPath, IsReplaceParent )
	If not IsFullPath( BeforeRootFullPath ) Then _
		Raise E_Others, "ReplaceRootPath 's 2nd argument must be full path"
	If not IsFullPath( AfterRootFullPath ) Then _
		Raise E_Others, "ReplaceRootPath 's 3rd argument must be full path"

	If IsFullPath( BeforePath ) Then
		step_path = GetStepPath( BeforePath, BeforeRootFullPath )
	Else
		step_path = BeforePath
	End If

	If IsReplaceParent Then
		step_path = ReplaceParentPath( step_path, "..", "_parent" )
	End If

	ReplaceRootPath = GetFullPath( step_path, AfterRootFullPath )
End Function


 
'*************************************************************************
'  <<< [SearchParent] >>> 
'*************************************************************************
Function  SearchParent( ByVal StepPath )
	current = g_sh.CurrentDirectory
	Set c = g_VBS_Lib

	If IsFullPath( StepPath ) Then
		If exist( StepPath ) Then  SearchParent = StepPath
		Exit Function
	End If

	i = 0
	Do
		i = InStr( i+1, current, "\" )
		If exist( StepPath ) Then
			If IsWildcard( StepPath ) Then
				ExpandWildcard  StepPath, c.File or c.Folder, folder, fnames
				SearchParent = folder +"\"+ fnames(0)
			Else
				SearchParent = GetFullPath( StepPath, Empty )
			End If
			Exit Function
		End If
		If i = 0 Then  Exit Function
		StepPath = "..\"+ StepPath
	Loop
End Function


 
'*************************************************************************
'  <<< [IsFullPath] >>> 
'*************************************************************************
Function  IsFullPath( Path )
	bs = InStr( Path, "\" )
	sl = InStr( Path, "/" )
	co = InStr( Path, ":" )
	If bs > 0 Then  If co > bs Then  co = 0
	If sl > 0 Then  If co > sl Then  co = 0

	If co > 0 Then
		If IsEmpty( g_IsFullPath_RegExp ) Then
			Set g_IsFullPath_RegExp = CreateObject( "VBScript.RegExp" )
			g_IsFullPath_RegExp.Pattern = "[a-zA-Z0-9]+:"
		End If
		Set matches = g_IsFullPath_RegExp.Execute( Path )
		If matches.Count >= 1 Then
			If matches(0).FirstIndex <> 0 Then
				co = 0
			End If
		End If
	End If

	IsFullPath = ( bs = co+1  or  sl = co+1 ) or ( Left( Path, 2 ) = "\\" )
End Function

Dim  g_IsFullPath_RegExp


'  <<< [IsAbsPath] >>> 
Function  IsAbsPath( Path )
	IsAbsPath = IsFullPath( Path )
	ThisIsOldSpec
End Function


 
'*************************************************************************
'  <<< [FindParent] >>> 
'*************************************************************************
Function  FindParent( TargetStepPath, StartFolderPath )
	Dim  base : base = GetFullPath( StartFolderPath, Empty )
	Dim  path

	Do
		path = base + "\" + TargetStepPath
		If g_fs.FileExists( path ) or g_fs.FolderExists( path ) Then  Exit Do
		base = g_fs.GetParentFolderName( base )
		If base = "" Then  Raise  E_PathNotFound, _
			 "<ERROR msg='No FindParent' target='" + TargetStepPath + "'/>"
	Loop
	FindParent = path
End Function


 
'*************************************************************************
'  <<< [GetTagJumpPath] >>> 
'*************************************************************************
Function  GetTagJumpPath( PathAndLine )
	ThisIsOldSpec  '// use GetTagJumpParams
	GetTagJumpPath = GetTagJumpParams( PathAndLine ).Path
End Function


 
'*************************************************************************
'  <<< [GetTagJumpLine] >>> 
'*************************************************************************
Function  GetTagJumpLine( PathAndLine )
	ThisIsOldSpec  '// use GetTagJumpParams
	GetTagJumpLine = GetTagJumpParams( PathAndLine ).Line
	If IsEmpty( GetTagJumpLine ) Then  GetTagJumpLine = 0
End Function


 
'*************************************************************************
'  <<< [GetTagJumpParams] >>> 
'*************************************************************************
Class  TagJumpParams
	Public  Path
	Public  LineNum  '// as integer or Empty
	Public  Keyword  '// as string or Empty
End Class

Function  GetTagJumpParams( PathAndFragment )  '// as TagJumpParams
	Dim  s
	Dim  ret,  file_name,  number_pos,  kakko_pos,  colon_pos,  fragment
	Dim  folder_len

	Set ret = new TagJumpParams
	file_name = g_fs.GetFileName( PathAndFragment )
	folder_len = Len( PathAndFragment ) - Len( file_name )  '// Length of folder path +"\"

	colon_pos = InStr( file_name, ":" )
	number_pos = InStr( file_name, "#" )
	If colon_pos > number_pos  and  number_pos > 0  Then  colon_pos  = 0
	If number_pos > colon_pos  and  colon_pos  > 0  Then  number_pos = 0

	If colon_pos > 0 Then
		fragment = Mid( file_name, colon_pos + 1 )
		If IsPlusIntOrZeroString( fragment ) Then
			ret.LineNum = CInt( fragment )
		Else
			ret.Keyword = fragment
		End If
		ret.Path = Left( PathAndFragment, folder_len + colon_pos - 1 )
		Set GetTagJumpParams = ret
		Exit Function
	End If

	If number_pos > 0 Then
		ret.Path = Left( PathAndFragment, folder_len + number_pos - 1 )
		ret.Keyword = Mid( file_name, number_pos + 1 )
		Set GetTagJumpParams = ret
		Exit Function
	End If

	If Right( PathAndFragment, 1 ) = ")" Then
		kakko_pos = InStrRev( PathAndFragment, "(" )
		s = Mid( PathAndFragment,  kakko_pos + 1 )
		s = Left( s, Len( s ) - 1 )
		If IsPlusIntOrZeroString( s ) Then
			ret.LineNum = CInt( s )
			ret.Path = Left( PathAndFragment, kakko_pos - 1 )
		Else
			ret.Path = PathAndFragment
		End If
		Set GetTagJumpParams = ret
		Exit Function
	End If

	ret.Path = PathAndFragment
	Set GetTagJumpParams = ret
End Function


 
'*************************************************************************
'  <<< [AddLastOfFileName] >>> 
'*************************************************************************
Function  AddLastOfFileName( BasePath, AddName )
	Dim  parent : parent = g_fs.GetParentFolderName( BasePath )
	Dim  ext    : ext    = g_fs.GetExtensionName( AddName )
	Dim  base   : base   = g_fs.GetBaseName( BasePath )

	If parent <> "" Then  If Right( parent, 1 ) <> "\" Then  parent = parent + "\"

	If ext = "" Then
		If Right( AddName, 1 ) = "." Then
			AddLastOfFileName = parent + base + Left( AddName, Len(AddName)-1 )
		Else
			AddLastOfFileName = parent + base + AddName +"."+ g_fs.GetExtensionName( BasePath )
		End If
	Else
		AddLastOfFileName = parent + base + AddName
	End IF
End Function


 
'*************************************************************************
'  <<< [CutLastOfFileName] >>> 
'*************************************************************************
Sub  CutLastOfFileName( in_out_Path, LastStr, Opt )
	Dim  base : base = g_fs.GetBaseName( in_out_Path )
	Dim  last_length : last_length = Len( LastStr )

	If StrComp( Right( base, last_length ), LastStr, StrCompOption( Opt ) ) = 0 Then
		base = Left( base, Len( base ) - last_length )

		Dim  parent : parent = g_fs.GetParentFolderName( in_out_Path )
		Dim  ext : ext = g_fs.GetExtensionName( in_out_Path )

		If parent <> "" Then  If Right( parent, 1 ) <> "\" Then  parent = parent + "\"

		If ext = "" Then
			in_out_Path = parent + base
		Else
			in_out_Path = parent + base +"."+ ext
		End If
	End If
End Sub



 
'*************************************************************************
'  <<< [GetPathLabels] >>> 
'*************************************************************************
Function  GetPathLabels( FilePaths )
	Dim  file_ubound : file_ubound = UBound( FilePaths )
	ReDim  ret( file_ubound ), path( file_ubound ), name( file_ubound )
	Dim  i,  ii,  is_all_diff,  is_no_parent,  num

	'// return  file name only
	is_all_diff = True
	For i=0 To file_ubound
		ret(i) = g_fs.GetFileName( FilePaths(i) )
		If i>0 Then  If ret(i) = ret(0) Then  is_all_diff = False
	Next
	If is_all_diff Then  GetPathLabels = ret : Exit Function

	For i=0 To file_ubound
		path(i) = GetParentFullPath( FilePaths(i) )
	Next

	'// return  folder name + file name
	Do
		is_all_diff = True
		For i=0 To file_ubound
			name(i) = g_fs.GetFileName( path(i) )
			If i>0 Then  If name(i) = name(0) Then  is_all_diff = False
		Next
		If is_all_diff Then
			For i=0 To file_ubound
				ret(i) = name(i) +"/"+ ret(i)
			Next
			GetPathLabels = ret : Exit Function
		End If

		is_no_parent = False
		For i=0 To file_ubound
			path(i) = g_fs.GetParentFolderName( path(i) )
			If path(i) = ""  or  Right( path(i), 1 ) = "\" Then  is_no_parent = True
		Next
		If is_no_parent Then  Exit Do
	Loop

	'// return  number + file name
	For i=0 To file_ubound : name(i) = Empty : Next
	num = 1
	For i=0 To file_ubound
		If IsEmpty( name(i) ) Then
			name(i) = num : ret(i) = num & "/" & ret(i)
			For ii = i+1 To file_ubound
				If path(ii) = path(i) Then  name(ii) = num : ret(ii) = num & "/" & ret(ii)
			Next
			num = num + 1
		End If
	Next
	GetPathLabels = ret : Exit Function
End Function


 
'*************************************************************************
'  <<< [CutCRLF] >>> 
'*************************************************************************
Function  CutCRLF( Str )
	CutCRLF = Replace( Str, vbCRLF, "" )
	CutCRLF = Replace( CutCRLF, vbLF, "" )
End Function


 
'*************************************************************************
'  <<< [StrCompLastOfFileName] >>> 
'*************************************************************************
Function  StrCompLastOfFileName( Path, LastStr, Opt )
	StrCompLastOfFileName = StrComp( Right( g_fs.GetBaseName( Path ), Len(LastStr) ), LastStr, StrCompOption( Opt ) )
End Function



 
'*************************************************************************
'  <<< [DesktopPath] >>> 
'*************************************************************************
Function  DesktopPath()
	DesktopPath = g_sh.SpecialFolders( "Desktop" )
End Function


 
'*************************************************************************
'  <<< [IsWildcard] >>> 
'*************************************************************************
Function  IsWildcard( ByVal path )
	IsWildcard = InStr( path, "?" ) <> 0 Or InStr( path, "*" ) <> 0
End Function


 
'*************************************************************************
'  <<< [ReplaceFileNameWildcard] >>> 
'*************************************************************************
Function  ReplaceFileNameWildcard( ByVal Path, ByVal FromStr, ByVal ToStr )
	Dim  ret, pos1, pos2, s1, s2

	CutLastOf  FromStr, "\", Empty
	CutLastOf  ToStr, "\", Empty


	'// replace folder
	pos1 = InStrRev( Path, "\" )
	If pos1 > 0 Then  '// if exist parent folder path
		pos2 = InStrRev( FromStr, "\" )
		If pos2 > 0 Then
			s2 = LCase( Left( FromStr, pos2 ) )
			If InStr( s2, "*" ) > 0 Then  Error
		Else
			s2 = LCase( FromStr ) +"\"
		End If

		s1 = LCase( Left( Path, pos1 ) )
		If s1 <> s2 Then
			If StrCompHeadOf( s1, FromStr +"\", Empty ) = 0 Then
				ReplaceFileNameWildcard = ToStr +"\"+ Mid( Path, Len(FromStr) + 2 )
			End If
			Exit Function
		End If

		Path    = Mid( Path,    pos1 + 1 )  '// change to file name
		FromStr = Mid( FromStr, pos1 + 1 )  '// change to file name

		'// if move to sub folder
		If pos2 = 0 Then  ReplaceFileNameWildcard = ToStr +"\"+ Path : Exit Function

		pos2 = InStrRev( ToStr, "\" )
		If pos2 > 0 Then
			ret = Left( ToStr, pos2 )
			ToStr = Mid( ToStr, pos2 + 1 )
		End If
	Else
		If InStr( FromStr, "\" ) > 0 Then  Exit Function
	End If


	'// replace file
	pos1 = InStr( FromStr, "*" )
	If pos1 = 1  and  Right( FromStr, 1 ) = "*"  and  Len( FromStr ) >= 3 Then  '// double wildcard

		FromStr = Mid( FromStr, 2, Len( FromStr ) - 2 )
		If InStr( 1, Path, FromStr, 1 ) = 0 Then  Exit Function

		If ToStr = "" Then  ReplaceFileNameWildcard = "" : Exit Function

		If Left(  ToStr, 1 ) <> "*" Then  Error
		If Right( ToStr, 1 ) <> "*" Then  Error
		ToStr   = Mid( ToStr,   2, Len( ToStr )   - 2 )

		ret = ret + Replace( Path, FromStr, ToStr, 1,-1,1 )

	ElseIf pos1 > 0 Then  '// single wildcard

		If InStr( pos1 + 1,  FromStr, "*" ) > 0 Then  Error

		If LCase( Left( Path, pos1 - 1 ) ) <> LCase( Left( FromStr, pos1 - 1 ) ) Then  Exit Function
		If LCase( Right( Path, Len(FromStr) - pos1 ) ) <> LCase( Mid( FromStr, pos1 + 1 ) ) Then  Exit Function

		If ToStr = "" Then  ReplaceFileNameWildcard = "" : Exit Function

		pos2 = InStr( ToStr, "*" )
		If pos2 = 0 Then  Error
		If InStr( pos2 + 1,  ToStr,  "*" ) > 0 Then  Error

		ret = ret + Left( ToStr, pos2 - 1 ) +_
					Mid( Path, pos1,  Len( Path ) - Len( FromStr ) + 1 ) +_
					Mid( ToStr, pos2 + 1 )

			 '// [sample]  1234567           12345         1234567890
			 '//     Path="lfXYZrt" FromStr="lf*rt" ToStr="left*right"

	Else  '// no wildcard
		If LCase( Path ) <> LCase( FromStr ) Then  Exit Function
		ret = ret + ToStr
	End If

	ReplaceFileNameWildcard = ret
End Function


 
'*************************************************************************
'  <<< [ArrayFromWildcard] >>> 
'*************************************************************************
Function  ArrayFromWildcard( WildcardPath )
	Set dic = new PathDictionaryClass
	dic.BasePath = g_sh.CurrentDirectory
	Set ArrayFromWildcard = dic

	If IsArray( WildcardPath ) Then
		For Each  path  In WildcardPath
			Set dic( path ) = Nothing
		Next
	ElseIf TypeName( WildcardPath ) = "PathDictionaryClass" Then
		Set ArrayFromWildcard = WildcardPath
	Else
		Set dic( WildcardPath ) = Nothing
	End If
End Function


 
'********************************************************************************
'  <<< [ArrayFromWildcard2] >>> 
'********************************************************************************
Function  ArrayFromWildcard2( WildcardPath )
	Set dic = new PathDictionaryClass
	Set ArrayFromWildcard2 = dic

	If TypeName( WildcardPath ) = "PathDictionaryClass" Then
		Set ArrayFromWildcard2 = WildcardPath
	Else
		Assert  not IsWildcard( g_fs.GetParentFolderName( WildcardPath ) )
		If IsWildcard( g_fs.GetFileName( WildcardPath ) ) Then
			dic.BasePath = GetParentFullPath( WildcardPath )
			Set dic( g_fs.GetFileName( WildcardPath ) ) = Nothing
		ElseIf g_fs.FolderExists( WildcardPath ) Then
			dic.BasePath = WildcardPath
			Set dic( "." ) = Nothing
		Else
			dic.BasePath = g_fs.GetParentFolderName( WildcardPath )
			Set dic( g_fs.GetFileName( WildcardPath ) ) = Nothing
		End If
	End If
End Function


 
'********************************************************************************
'  <<< [GetInputOutputFilePaths] >>> 
'********************************************************************************
Function  GetInputOutputFilePaths( in_InputPath, in_OutputPath, in_TemporaryBaseName )
	tmp_path = GetTempPath( in_TemporaryBaseName )

	If not IsEmpty( in_OutputPath ) Then
		in_OutputPath = GetFullPath( in_OutputPath, Empty )
	End If

	Set paths = ArrayFromWildcard2( in_InputPath )
	file_paths = paths.FilePaths

	ReDim  pairs( UBound( file_paths ) )
	i = 0
	For Each  step_path  In  file_paths

		Set pair = new InputOutputFilePathClass

		pair.InputPath = GetFullPath( step_path, paths.BasePath )

		If IsEmpty( in_OutputPath ) Then
			pair.OutputPath = tmp_path
			pair.IsOutputPathTemporary = True
		Else
			If file_paths(0) = g_fs.GetFileName( in_InputPath ) Then
				pair.OutputPath = in_OutputPath
			Else
				pair.OutputPath = GetFullPath( step_path, in_OutputPath )
			End If

			If pair.InputPath = pair.OutputPath Then
				pair.OutputPath = tmp_path
				pair.IsOutputPathTemporary = True
			Else
				pair.IsOutputPathTemporary = False
			End If
		End If
		Set pairs( i ) = pair
		i = i + 1
	Next
	GetInputOutputFilePaths = pairs
End Function


 
'********************************************************************************
'  <<< [InputOutputFilePathClass] >>> 
'********************************************************************************
Class  InputOutputFilePathClass
	Public  InputPath
	Public  OutputPath
	Public  IsOutputPathTemporary
End Class


 
'*************************************************************************
'  <<< [ExpandWildcard] >>> 
'*************************************************************************
Sub  ExpandWildcard( WildcardPath, ByVal Flags, out_FolderPath, out_StepPaths )
	Set c = g_VBS_Lib
	Set work = new ExpandWildcardWorkClass

	If IsArray( WildcardPath ) Then  wildcard_paths = WildcardPath _
	Else                             wildcard_paths = Array( WildcardPath )

	ReDim  temp_folder_paths( UBound( wildcard_paths ) )
	out_FolderPath = Empty
	ReDim  out_StepPaths( -1 )
	work.out_fast_ubound = UBound( out_StepPaths )
	If g_Vers("ExpandWildcard_Sort") Then  Set work.sort_box = new ArrayClass

	Set reg_exp_dic = CreateObject( "Scripting.Dictionary" )
		'// reg_exp_dic is dictionary of work.items, key= folder path
		'// work.items is ArrayClass of { re, re2, re, re2, ... }
	reg_exp_dic.CompareMode = 1


	'// Set "reg_exp_dic"
	For path_num = 0  To UBound( wildcard_paths )
		do_it = True
		path = wildcard_paths( path_num )
		Set re = Nothing : Set re2 = Nothing

		Set item = new ExpandWildcardItemClass
		'// Set "IsSubFolder" : This will be overwritten after here.
		If Flags and c.SubFolder Then
			item.IsSubFolder = True
		Else
			item.IsSubFolder = False
		End If

		SplitPathToSubFolderSign  path, sub_folder_sign, is_folder, Empty

		If is_folder Then
			If IsBitSet( Flags, c.File )  and  IsBitNotSet( Flags, c.Folder ) Then
				Flags = Flags and not ( c.File or c.Folder )
				Flags = Flags or c.Folder
			End If
		ElseIf Flags and c.SubFolderIfWildcard Then
			If g_fs.FolderExists( path ) Then
				If sub_folder_sign = "" Then
					path = path +"\*"
					item.IsSubFolder = True
				End If
			End If
		End If


		folder_path = GetParentFullPath( path )
		If not g_fs.FolderExists( folder_path ) Then
			If Flags and c.SubFolderIfWildcard Then
				If IsBitNotSet( Flags, c.NoError ) Then
					Raise  E_PathNotFound, "<ERROR msg=""Not found"" path="""+ _
						XmlAttr( path ) +"""/>"
				Else
					do_it = False
				End If
			End If
		End If
		file_name = g_fs.GetFileName( path )

		If file_name = "*" Then  '// Else If file_name = "*" Then  re = re2 = Empty
			If Flags and c.SubFolderIfWildcard Then
				item.IsSubFolder = True
			End If
		Else

			i = InStr( file_name, "*" )

			'//=== 
			If i = 0 Then
				If InStr( file_name, "?" ) = 0 Then
					Set re = new NameOnlyClass
					re.Name = file_name
					re.Delegate = LCase( file_name )

					If IsBitSet( Flags, c.SubFolderIfWildcard )  and  sub_folder_sign <> "*" Then
						If not exist( path ) Then
							If IsBitNotSet( Flags, c.NoError ) Then
								Raise  E_PathNotFound, "<ERROR msg=""Not found"" path="""+ _
									XmlAttr( path ) +"""/>"
							End If
							do_it = False
						End If
					End If
				End If


			'//=== //, re2=Empty
			ElseIf InStrRev( file_name, "*" ) = i Then  '// 
				If InStr( file_name, "?" ) = 0 Then

					'// Set "re"
					Set re = new StrMatchKey
					re.Keyword = LCase( file_name )  '// 
					re.IgnoreCase = True
					'// re2 = Empty

					If Flags and c.SubFolderIfWildcard Then
						If IsWildcard( file_name ) Then
							item.IsSubFolder = True
						Else
							item.IsSubFolder = False
						End If


						and_              = ( i = 0 )
						If and_ Then and_ = ( not exist( GetFullPath( file_name, folder_path ) ) )
						If and_ Then and_ = ( ( Flags and c.NoError ) = 0 )
						If and_ Then
							Raise  E_PathNotFound, "<ERROR msg=""Not found"" path="""+ _
								XmlAttr( path ) +"""/>"
						End If
					End If
				End If
			End If


			'//=== //
			If re is Nothing Then  '// 

				If Flags and c.SubFolderIfWildcard Then
					item.IsSubFolder = True
				End If

				If Right( path, 2 ) = ".*" Then  Set re2 = CreateObject("VBScript.RegExp")

				Set re = CreateObject("VBScript.RegExp")
				re.Global = True

				'// Add escape character "\" in "file_name"
				For Each  str  In Array( "\\", "\.", "\$", "\^", "\{", "\}", "\[", "\]", _
						"\(", "\)", "\|", "\+" )
					re.Pattern = str :  file_name = re.Replace( file_name, str )
				Next

				'// Replace in "file_name"
				re.Pattern = "\*" :  file_name = re.Replace( file_name, ".*" )
				re.Pattern = "\?" :  file_name = re.Replace( file_name, "." )

				'// Set "re"
				If Left( re.Pattern, 2 ) = ".*" Then
					re.Pattern = Mid( re.Pattern, 3 )
				Else
					re.Pattern = "^" + file_name
				End If
				re.Global = False
				re.IgnoreCase = True

				'// Set "re2"
				If not re2 is Nothing Then  '// re2 is for no extension file at any extension pattern
					re2.Pattern = Left( re.Pattern, Len( re.Pattern ) - 4 ) +"$"
						'// Cut last "\..*" in "file_name" and add "$"
					re2.Global = re.Global
					re2.IgnoreCase = re.IgnoreCase
				End If
			End If
		End If


		If do_it Then
			If sub_folder_sign = "*" Then
				item.IsSubFolder = True
			ElseIf sub_folder_sign = "." Then
				item.IsSubFolder = False
			End If


			If reg_exp_dic.Exists( folder_path ) Then
				Set work.items = reg_exp_dic.Item( folder_path )
			Else
				Set work.items = new ArrayClass
				Set reg_exp_dic.Item( folder_path ) = work.items
			End If

			Set item.re  = re
			Set item.re2 = re2
			work.items.Add  item
		End If
	Next


	'// Loop by "folder_path"
	For Each folder_path  In reg_exp_dic.Keys
		Set work.items = reg_exp_dic.Item( folder_path )
		sep_ch = GetFilePathSeparator( folder_path )
		work.folder_path = folder_path


		If Flags and c.ArrayOfArray Then
			prev_ubound = UBound( out_StepPaths )
		Else
			prev_ubound = work.out_fast_ubound
		End If



		'// ...
		ExpandWildcard_sub  work, Flags, out_StepPaths



		'// Set "out_FolderPath" : Common folder path
		If IsEmpty( out_FolderPath ) Then
			If Flags and c.AbsPath Then
				Select Case  Right( folder_path, 1 )
					Case sep_ch : new_folder_path = folder_path
					Case Else   : new_folder_path = folder_path + sep_ch
				End Select
				If Flags and c.ArrayOfArray Then
					For ii = prev_ubound + 1  To UBound( out_StepPaths )
						Set paths = out_StepPaths(ii)
						For i = 0  To paths.UBound_
							paths(i) = new_folder_path + paths(i)
						Next
					Next
				Else
					For i = prev_ubound + 1  To work.out_fast_ubound
						out_StepPaths(i) = new_folder_path + out_StepPaths(i)
					Next
				End If
			Else
				out_FolderPath = folder_path
			End If
		Else
			If StrCompHeadOf( folder_path, out_FolderPath, Empty ) <> 0 Then
				i=1
				Do
					If LCase( Mid( folder_path, i, 1 )) <> _
							LCase( Mid( out_FolderPath, i, 1 )) Then
						Exit Do
					End If
					i=i+1
				Loop
				new_folder_path = g_fs.GetParentFolderName( Left( out_FolderPath, i ) )
				If Right( new_folder_path, 1 ) = "\" Then _
					new_folder_path = Left( new_folder_path, Len( new_folder_path ) - 1 )
				step_path = Mid( out_FolderPath, Len( new_folder_path ) + 2 ) +"\"
				Assert  Len( new_folder_path ) > 0

				If Flags and c.ArrayOfArray Then
					For ii=0 To prev_ubound
						Set paths = out_StepPaths(ii)
						For i = 0  To paths.UBound_
							paths(i) = step_path + paths(i)
						Next
					Next
				Else
					For i=0 To prev_ubound
						out_StepPaths(i) = step_path + out_StepPaths(i)
					Next
				End If
				out_FolderPath = new_folder_path
			End If

			If folder_path <> out_FolderPath Then
				step_path = Mid( folder_path, Len( out_FolderPath ) + 2 ) +"\"
				If Flags and c.ArrayOfArray Then
					For ii = prev_ubound + 1  To UBound( out_StepPaths )
						Set paths = out_StepPaths(ii)
						For i = 0  To paths.UBound_
							paths(i) = step_path + paths(i)
						Next
					Next
				Else
					For i = prev_ubound + 1  To work.out_fast_ubound
						out_StepPaths(i) = step_path + out_StepPaths(i)
					Next
				End If
			End If
		End If
	Next


	If Flags and c.ArrayOfArray Then
	Else
		ReDim Preserve  out_StepPaths( work.out_fast_ubound )
	End If
End Sub


'//[ExpandWildcardWorkClass]
Class  ExpandWildcardWorkClass
	Public  items
	Public  is_root_folder
	Public  folder_path
	Public  first_out_count
	Public  out_fast_ubound
	Public  sort_box
End Class


'//[ExpandWildcardItemClass]
Class  ExpandWildcardItemClass
	Public  re
	Public  re2
	Public  IsSubFolder
End Class


'[ExpandWildcard_sub]
'- sort_box as ArrayClass of string. No pass data from caller
Sub  ExpandWildcard_sub( work, Flags, out_StepPaths )
	Set c = g_VBS_Lib

	If not g_fs.FolderExists( work.folder_path ) Then  Exit Sub


	If Flags and c.ArrayOfArray Then
		work.first_out_count = UBound( out_StepPaths ) + 1
		Assert  IsEmpty( work.sort_box )
		ReDim Preserve  out_StepPaths( UBound( out_StepPaths ) + work.items.Count )
		For i = work.first_out_count  To UBound( out_StepPaths )
			Set out_StepPaths(i) = new ArrayClass
		Next
	Else
		work.first_out_count = work.out_fast_ubound + 1
	End If


	'// Set "is_scan_sub_folder"
	is_scan_sub_folder = False
	is_all_file_path = True
	For i=0 To work.items.UBound_
		Set  item = work.items(i)

		If item.IsSubFolder Then
			is_scan_sub_folder = True
			is_all_file_path = False
		End If
		If TypeName( item.re ) <> "NameOnlyClass" Then
			is_all_file_path = False
		End If
	Next


	If is_all_file_path Then
		For i=0 To work.items.UBound_
			If UBound(out_StepPaths) <= work.out_fast_ubound Then _
				ReDim Preserve  out_StepPaths( ( work.out_fast_ubound + 100 ) * 4 )
			work.out_fast_ubound = work.out_fast_ubound + 1

			file_name = work.items(i).re.Name
			out_StepPaths( work.out_fast_ubound ) = file_name
		Next
	Else
		'// Set "folders"
		ReDim  folders(0)
		Set folders(0) = g_fs.GetFolder( work.folder_path )
		folders_fast_ubound = UBound( folders )
		folders_get_num = 0

		work.is_root_folder = True
		While  folders_get_num <= folders_fast_ubound

			If not is_all_file_path Then

				'// Loop by "SubFolders" not sub folders
				For Each folder  In  folders( folders_get_num ).SubFolders

					If is_scan_sub_folder Then

						'// Add "folder" to "folders"
						If UBound(folders) <= folders_fast_ubound Then _
							ReDim Preserve  folders( ( folders_fast_ubound + 100 ) * 4 )
						folders_fast_ubound = folders_fast_ubound + 1

						Set folders( folders_fast_ubound ) = folder
					End If



					'// Call "ExpandWildcard_sub2"
					If Flags and c.Folder Then
						ExpandWildcard_sub2  work, Flags, out_StepPaths, folder
					End If



				Next
				If not IsEmpty( work.sort_box ) Then
					QuickSort  work.sort_box, 0, work.sort_box.UBound_, _
						GetRef("NameCompare"), vbTextCompare

					i = work.out_fast_ubound
					If UBound(out_StepPaths) < i + work.sort_box.Count Then _
						ReDim Preserve  out_StepPaths( ( i + work.sort_box.Count + 100 ) * 4 )
					work.out_fast_ubound = i + work.sort_box.Count

					For Each o  In work.sort_box.Items
						i = i + 1 : out_StepPaths( i ) = o.Delegate
					Next
					work.sort_box.ToEmpty
				End If
			End If


			'// Call "ExpandWildcard_sub2"
			'// Loop by "Files" not sub folders
			If Flags and c.File Then
				For Each file  In folders( folders_get_num ).Files
					ExpandWildcard_sub2  work, Flags, out_StepPaths, file
				Next
			End If



			If not IsEmpty( work.sort_box ) Then
				QuickSort  work.sort_box, 0, work.sort_box.UBound_, _
					GetRef("NameCompare"), vbTextCompare

				i = work.out_fast_ubound
				If UBound(out_StepPaths) < i + work.sort_box.Count Then _
					ReDim Preserve  out_StepPaths( ( i + work.sort_box.Count + 100 ) * 4 )
				work.out_fast_ubound = i + work.sort_box.Count

				For Each o  In work.sort_box.Items
					i = i + 1 : out_StepPaths( i ) = o.Delegate
				Next
				work.sort_box.ToEmpty
			End If

			folders_get_num = folders_get_num + 1
			work.is_root_folder = False
		WEnd


		'// Change object to path
		n = Len( work.folder_path ) + 2
		If Flags and c.ArrayOfArray Then
			For ii = work.first_out_count  To UBound( out_StepPaths )
				Set paths = out_StepPaths(ii)
				For i = 0  To paths.UBound_
					If IsObject( paths( i ) ) Then
						paths( i ) = Mid( paths( i ).Path, n )
					Else
						paths( i ) = Mid( paths( i ), n )
					End If
				Next
			Next
		Else
			For i = work.first_out_count To work.out_fast_ubound
				If IsObject( out_StepPaths( i ) ) Then
					out_StepPaths( i ) = Mid( out_StepPaths( i ).Path, n )
				Else
					out_StepPaths( i ) = Mid( out_StepPaths( i ), n )
				End If
			Next
		End If
	End If
End Sub


'[ExpandWildcard_sub2]
Sub  ExpandWildcard_sub2( work, Flags, out_StepPaths, object )
	Set c = g_VBS_Lib

	'// Set "is_match", "match_num"
	is_match = True
	full_path = Empty
	For i=0 To work.items.UBound_
		If work.is_root_folder  or  work.items(i).IsSubFolder Then
			Set re  = work.items(i).re
			Set re2 = work.items(i).re2

			If re is Nothing Then
				match_num = i
			Else
				If IsEmpty( full_path ) Then
					full_path = object.Path
					fname = g_fs.GetFileName( full_path )
				End If
				is_match = ( not re2 is Nothing )
				If is_match Then is_match=( re2.Test( fname )  and  InStr( fname, "." ) = 0 )  'and
				If not is_match Then  'or
					If TypeName( re ) = "NameOnlyClass" Then
						is_match = ( LCase( fname ) = re.Delegate )
					Else
						is_match = re.Test( fname )
					End If
				End If
				If is_match Then  match_num = i : Exit For
			End If
		End If
	Next

	'// Add to "out_StepPaths" or "sort_box"
	If is_match Then
		If IsEmpty( work.sort_box ) Then
			If Flags and c.ArrayOfArray Then
				If IsEmpty( full_path ) Then
					out_StepPaths( match_num + work.first_out_count ).Add  object
				Else
					out_StepPaths( match_num + work.first_out_count ).Add  full_path
				End If
			Else
				If UBound(out_StepPaths) <= work.out_fast_ubound Then _
					ReDim Preserve  out_StepPaths( ( work.out_fast_ubound + 100 ) * 4 )
				work.out_fast_ubound = work.out_fast_ubound + 1

				If IsEmpty( full_path ) Then
					Set out_StepPaths( work.out_fast_ubound ) = object
				Else
					out_StepPaths( work.out_fast_ubound ) = full_path
				End If
			End If
		Else
			Set o = new NameOnlyClass
			If re is Nothing Then
				full_path = object.Path
				fname = g_fs.GetFileName( full_path )
			End If
			o.Name = fname
			o.Delegate = full_path
			work.sort_box.Add  o
		End If
	End If
End Sub


 
'********************************************************************************
'  <<< [new_PathDictionaryClass_withRemove] >>> 
'********************************************************************************
Function  new_PathDictionaryClass_withRemove( in_BasePath, in_RemovePathArray, in_DictionaryItem )
	Set dic = new PathDictionaryClass
	dic.BasePath = in_BasePath
	Set dic( "." ) = in_DictionaryItem
	If not IsEmpty( in_RemovePathArray ) Then
		For Each  path  In in_RemovePathArray
			dic.AddRemove  path
		Next
	End If
	Set new_PathDictionaryClass_WithRemove = dic
End Function


 
'*************************************************************************
'  <<< [new_PathDictionaryClass_fromXML] >>> 
'*************************************************************************
Function  new_PathDictionaryClass_fromXML( path_XML_Elements, AttributeName, BasePath )
	Set new_PathDictionaryClass_fromXML = new_PathDictionaryClass_fromXML_Ex( _
		path_XML_Elements, AttributeName, BasePath, new PathDictionaryOptionClass )
End Function


 
'*************************************************************************
'  <<< [new_PathDictionaryClass_fromXML_Ex] >>> 
' - Options_ as PathDictionaryOptionClass
'*************************************************************************
Function  new_PathDictionaryClass_fromXML_Ex( path_XML_Elements, AttributeName, BasePath, Options_ )
	Set dic = new PathDictionaryClass
	dic.BasePath = BasePath
	Set new_PathDictionaryClass_fromXML_Ex = dic

	If IsArray( path_XML_Elements ) Then
		If UBound( path_XML_Elements ) = -1 Then  Exit Function
	Else
		If path_XML_Elements.Length = 0 Then  Exit Function
	End If

	Set root = path_XML_Elements(0).ownerDocument.lastChild
	Set variables = new_LazyDictionaryClass( root )

	For Each  element  In path_XML_Elements
		path = element.getAttribute( AttributeName )
		If not IsNull( path ) Then
			path = variables( path )
			If Options_.IsReplaceParentPath Then _
				path = ReplaceParentPath( path, "..", "_parent" )
			Set dic( path ) = element
		End If

		For Each  except  In element.selectNodes( "Except" )
			except_paths = except.getAttribute( AttributeName )
			If not IsNull( except_paths ) Then
				except_paths = variables( except_paths )
				If Options_.IsReplaceParentPath Then _
					except_paths = ReplaceParentPath( except_paths, "..", "_parent" )
				For Each except_path  In ArrayFromCSV( except_paths )
					dic.AddRemove  except_path
				Next
			End If
		Next
	Next
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [PathDictionaryOptionClass] >>>> 
'-------------------------------------------------------------------------
Class  PathDictionaryOptionClass
	Public  IsReplaceParentPath

	Private Sub  Class_Initialize()
		IsReplaceParentPath = False
	End Sub
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [PathDictionaryClass] >>>> 
'-------------------------------------------------------------------------
Class  PathDictionaryClass
	Public  m_PathDictionary         '// as dictionary of Empty
	Public  FilesDictionary          '// as dictionary of Variant
	Public  FoldersDictionary        '// as dictionary of True, key=step path
	Public  DeleteFoldersDictionary  '// as dictionary of False, key=step path
	Public  FilesArray               '// as array of DicElem string(path)
	Public  FoldersArray             '// as array of DicElem string(path)
	Public  DeleteFoldersArray       '// as array of DicElem string(path)
	Public  RemoveFilesDictionary    '// as dictionary of False, key=step path
	Public  m_IsScaned         '// as boolean
	Public  m_IsFolderScaned     '// as boolean. Ignored if m_IsScaned = True
	Private m_BasePath         '// as string
	Public  RemovedObject      '// Item for ".AddRemoved"
	Public  IsNotFoundError    '// as boolean

	Private Sub  Class_Initialize()
		Set m_PathDictionary = CreateObject( "Scripting.Dictionary" )
		Set Me.FilesDictionary = CreateObject( "Scripting.Dictionary" )
		Set Me.FoldersDictionary = CreateObject( "Scripting.Dictionary" )
		Set Me.DeleteFoldersDictionary = CreateObject( "Scripting.Dictionary" )
		Me.FilesArray = Array( )
		Me.FoldersArray = Array( )
		m_IsScaned = True
		Me.BasePath = g_sh.CurrentDirectory
		Set Me.RemovedObject = new NameOnlyClass : Me.RemovedObject.Name = "Removed"
		Me.IsNotFoundError = True
	End Sub

	Public Property Set  Item( in_Key, NewItem )
		Set  m_PathDictionary( in_Key ) = NewItem
		m_IsScaned = False
	End Property

	Public Sub  AddRemove( in_Key )
		Set  m_PathDictionary( in_Key ) = Me.RemovedObject
		m_IsScaned = False
	End Sub

	Public Default Property Get  Item( in_Key )
		If m_PathDictionary.Exists( in_Key ) Then
			LetSet  Item, m_PathDictionary( in_Key )
		Else
			If not m_IsScaned Then  Me.Scan
			If Me.FilesDictionary.Exists( in_Key ) Then
				LetSet  Item, Me.FilesDictionary( in_Key )
			ElseIf IsFullPath( in_Key ) Then
				step_path = GetStepPath( in_Key, Me.BasePath )
				If Me.FilesDictionary.Exists( step_path ) Then
					LetSet  Item, Me.FilesDictionary( step_path )
				Else
					Err.Raise  9,,"<ERROR msg=""Not found the key"" key="""& in_Key &"""/>"
				End If
			Else
				Err.Raise  9,,"<ERROR msg=""Not found the key"" key="""& in_Key &"""/>"
			End If
		End If
	End Property

	Public Function  FileExists( in_Key )
		If not m_IsScaned Then  Me.Scan
		FileExists = Me.FilesDictionary.Exists( GetStepPath( in_Key, Me.BasePath ) )
	End Function

	Public Function  FolderExists( in_Key )
		If not m_IsScaned Then  Me.Scan
		If not m_IsFolderScaned Then  m_ScanFolder
		FolderExists = Me.FoldersDictionary.Exists( GetStepPath( in_Key, Me.BasePath ) )
	End Function

	Public Function  DeleteFolderExists( in_Key )
		If not m_IsScaned Then  Me.Scan
		If not m_IsFolderScaned Then  m_ScanFolder
		DeleteFolderExists = Me.DeleteFoldersDictionary.Exists( GetStepPath( in_Key, Me.BasePath ) )
	End Function

	Public Property Let  BasePath( x ) : m_BasePath = GetFullPath( x, Empty ) : End Property
	Public Property Get  BasePath() : BasePath = m_BasePath : End Property

	Public Sub  Add( Key, Item ) : m_PathDictionary.Add  Key, Item : End Sub
	Public Property Get  Count() : Count = m_PathDictionary.Count : End Property
	Public Property Get  Keys()  : Keys  = m_PathDictionary.Keys  : End Property
	Public Property Get  Items() : Items = m_PathDictionary.Items : End Property
	Public Property Get  CompareMode() : CompareMode = m_PathDictionary.CompareMode : End Property
	Public Property Let  CompareMode( x ) : m_PathDictionary.CompareMode = x : End Property
	Public Sub  Remove( Key ) : m_PathDictionary.Remove  Key : End Sub
	Public Sub  RemoveAll() : m_PathDictionary.RemoveAll : End Sub

	Public Property Get  FilePaths()
		FilePaths = Array( )
		If not m_IsScaned Then  Me.Scan
		DicElemArrayKeyToArr  Me.FilesArray, FilePaths '//[out]FilePaths
	End Property

	Public Property Get  FullPaths()
		FullPaths = Array( )
		FullPaths_Sub  FullPaths
	End Property
	Public Sub  FullPaths_Sub( out )
		If not m_IsScaned Then  Me.Scan

		base_path = GetPathWithSeparator( m_BasePath )

		ReDim  out( UBound( Me.FilesArray ) )
		For index = 0  To UBound( Me.FilesArray )
			path = Me.FilesArray( index ).Key
			If IsFullPath( path ) Then
				out( index ) = path
			Else
				out( index ) = base_path + path
			End If
		Next
	End Sub

	Public Property Get  FolderPaths()
		FolderPaths = Array( )
		If not m_IsScaned Then  Me.Scan
		If not m_IsFolderScaned Then  m_ScanFolder
		DicElemArrayKeyToArr  Me.FoldersArray, FolderPaths '//[out]FolderPaths
	End Property

	Public Property Get  DeleteFolderPaths()
		DeleteFolderPaths = Array( )
		If not m_IsScaned Then  Me.Scan
		If not m_IsFolderScaned Then  m_ScanFolder
		DicElemArrayKeyToArr  Me.DeleteFoldersArray, DeleteFolderPaths '//[out]DeleteFolderPaths
	End Property

	Public Function  SearchParent( in_Path )
		SearchParent = Dic_searchParent( m_PathDictionary, m_BasePath, in_Path )
	End Function

	Public Property Let  IsScaned( x )
		m_IsScaned = x
		m_IsFolderScaned = x
	End Property
	Public Property Get  IsScaned()
		IsScaned = m_IsScaned
	End Property

	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		xml_sub = GetTab(Level)+ "<"+TypeName(Me)+">"+ vbCRLF
		For Each key  In m_PathDictionary.Keys
			If not m_PathDictionary( key ) is Me.RemovedObject Then _
				xml_sub = xml_sub + GetTab(Level) + key + vbCRLF
		Next
		xml_sub = xml_sub + GetTab(Level) +"</"+TypeName(Me)+">"
	End Function


'*************************************************************************
'  <<< [PathDictionaryClass::Scan] >>> 
'*************************************************************************
Public Sub  Scan()
	Set c = g_VBS_Lib
	Set Me.RemoveFilesDictionary = CreateObject( "Scripting.Dictionary" )

	Me.FilesDictionary.RemoveAll

	DicToArr  m_PathDictionary, paths  '//[out] paths
	QuickSort  paths, 0, UBound( paths ), GetRef("PathNameCompare"), -1


	'// Set "Me.FilesDictionary"
	For Each  path_object  In paths
		do_it = True
		step_path = path_object.Name
		SplitPathToSubFolderSign  step_path, sub_folder_sign, is_folder, Empty
		full_path = GetFullPath( step_path, Me.BasePath )
		is_wildcard = IsWildcard( step_path )
		separator = GetFilePathSeparator( full_path )

		If ( g_fs.FolderExists( full_path )  and  not is_folder )  or _
				step_path = "."  or  step_path = ".."  or _
				is_wildcard  or  sub_folder_sign = "*" Then

			If sub_folder_sign <> "" Then
				wildcard = g_fs.GetParentFolderName( full_path ) + separator + _
					sub_folder_sign + separator + g_fs.GetFileName( step_path )
			Else
				If is_wildcard  and  Me.IsNotFoundError Then
					If not g_fs.FolderExists( g_fs.GetParentFolderName( full_path ) ) Then
						Raise  E_PathNotFound, "<ERROR msg=""Not found"" path="""+ _
							path_object.Name + """/>"
					End If
				End If
				wildcard = full_path
			End If
			If is_folder Then _
				wildcard = wildcard + separator

			If is_wildcard Then
				pos_asterisk = InStr( step_path, "*" )
				pos_separator = InStrRev( step_path, separator, pos_asterisk )
				If pos_separator = 0  and  path_object.Item Is Me.RemovedObject Then
					Me.RemoveFilesDictionary( step_path ) = False
					do_it = False
				Else
					step_path = Left( step_path, pos_separator )
				End If
			ElseIf sub_folder_sign = "*" Then
				step_path = Left( step_path, Len( step_path ) - Len( g_fs.GetFileName( step_path ) ) )
			Else  '// If "full_path" was folder path
				If Right( step_path, 1 ) <> separator Then
					If step_path = "." Then
						step_path = ""
					Else
						step_path = step_path + separator
					End If
				End If
			End If

			If do_it Then


				ExpandWildcard  wildcard, c.File or c.SubFolderIfWildcard or c.NoError, _
					folder, sub_step_paths


				For Each sub_step_path  In sub_step_paths
					If sub_step_path = "." Then
						file_step_path = step_path
						CutLastOf  file_step_path, separator, c.CaseSensitive
					Else
						file_step_path = step_path + sub_step_path
					End If
					If not Me.FilesDictionary.Exists( file_step_path ) Then
						Set Me.FilesDictionary( file_step_path ) = path_object.Item

						If path_object.Item Is Me.RemovedObject Then _
							Me.RemoveFilesDictionary( file_step_path ) = False
					End If
				Next
			End If
		ElseIf exist( full_path ) Then
			If not Me.FilesDictionary.Exists( step_path ) Then
				Set Me.FilesDictionary( step_path ) = path_object.Item

				If path_object.Item Is Me.RemovedObject Then _
					Me.RemoveFilesDictionary( step_path ) = False
			End If
		ElseIf Me.IsNotFoundError Then
			Raise  E_PathNotFound, "<ERROR msg=""Not found"" path="""+ path_object.Name + """/>"
		End If
	Next


	'// Remove in "Me.FilesDictionary"
	For Each  removing_key  In Me.RemoveFilesDictionary.Keys
		If IsWildcard( removing_key ) Then
			For Each key  In Me.FilesDictionary.Keys
				If IsMatchedWithWildcard( key, removing_key ) Then
					Me.FilesDictionary.Remove  key
				End If
			Next
		Else
			Me.FilesDictionary.Remove  removing_key
		End If
	Next


	'// Set "Me.FilesArray" : Sorted array
	DicToArr  Me.FilesDictionary, FilesArray  '//[out] Me.FilesArray
	QuickSort  FilesArray, 0, UBound( FilesArray ), GetRef("PathNameCompare"), +1

	m_IsScaned = True
	m_IsFolderScaned = False
End Sub


 
'*************************************************************************
'  <<< [PathDictionaryClass::m_ScanFolder] >>> 
'*************************************************************************
Private Sub  m_ScanFolder()
	Me.FoldersDictionary.RemoveAll
	Me.DeleteFoldersDictionary.RemoveAll

	For Each  file_path  In Me.FilesDictionary.Keys
		path = file_path
		Do
			path = g_fs.GetParentFolderName( path )
			If path = "" Then  Exit Do

			If Me.FoldersDictionary.Exists( path ) Then
				Exit Do
			End If

			Me.FoldersDictionary( path )= True
		Loop
	Next


	'// Set "Me.FoldersArray" : Sorted array
	DicToArr  Me.FoldersDictionary, FoldersArray  '//[out] Me.FoldersArray
	QuickSort  FoldersArray, 0, UBound( FoldersArray ), GetRef("PathNameCompare"), +1


	'// Set "remove_file_folders" from "Me.AddRemove()"
	Set remove_file_folders = CreateObject( "Scripting.Dictionary" )
	For Each  file_path  In Me.RemoveFilesDictionary.Keys
		path = file_path
		Do
			path = g_fs.GetParentFolderName( path )
			If path = "" Then  Exit Do

			If remove_file_folders.Exists( path ) Then
				Exit Do
			End If

			remove_file_folders( path )= True
		Loop
	Next


	'// "Me.DeleteFoldersDictionary" : Include sub folders
	'// Set "delete_folders"
	delete_folders = Array( )
	For Each  folder  In Me.FoldersArray
		If not remove_file_folders.Exists( folder.Name ) Then
			ReDim Preserve  delete_folders( UBound( delete_folders ) + 1 )
			Set delete_folders( UBound( delete_folders ) ) = folder

			Me.DeleteFoldersDictionary( folder.Name ) = False
		End If
	Next


	'// Set "Me.DeleteFoldersArray"
	'// Set "delete_folders"
	ReverseObjectArray  delete_folders,  delete_folders_reverse
	delete_folders = Array( )
	For Each  folder  In delete_folders_reverse
		path = g_fs.GetParentFolderName( folder.Name )

		If path = "" Then
			is_add = True
		ElseIf Me.DeleteFoldersDictionary.Exists( path ) Then
			is_add = False
		Else
			is_add = True
		End If

		If is_add Then
			ReDim Preserve  delete_folders( UBound( delete_folders ) + 1 )
			Set delete_folders( UBound( delete_folders ) ) = folder
		End If
	Next
	ReverseObjectArray  delete_folders,  DeleteFoldersArray


	m_IsFolderScaned = True
End Sub


 
' ### <<<< End of Class implement >>>> 
End Class


 
'*************************************************************************
'  <<< [NewDiffFilePaths] >>> 
'*************************************************************************
Function  NewDiffFilePaths( in_PathArray, in_RemovePathArray )
	Set dic_A_B = CreateObject( "Scripting.Dictionary" )
	i_max = UBound( in_PathArray )
	ReDim  dummys( i_max )
	ReDim  dic( i_max )

	For  i = 0  To  i_max
		Set dummys(i) = new PathDictionaryClass
		dummys(i).BasePath = in_PathArray(i)

		Set dic(i) = new_PathDictionaryClass_withRemove( _
			in_PathArray(i), in_RemovePathArray, dummys(i) )
	Next
	For  i = 0  To  i_max
		Dic_addFilePaths_fromPathDirectory  dic_A_B, i, i_max, dic(i).BasePath, dic(i), dummys(i)
	Next
	Set NewDiffFilePaths = dic_A_B
End Function


 
'*************************************************************************
'  <<< [GetItemsFromDiffFilePathsDictionary] >>> 
'*************************************************************************
Sub  GetItemsFromDiffFilePathsDictionary( in_DiffFilePathsDictionary, out_ItemArray )
	Set base_path_dic = CreateObject( "Scripting.Dictionary" )

	For Each  files  In  in_DiffFilePathsDictionary.Items
		For i = 0  To  files.UBound_
			If not files( i ) Is Nothing Then
				Set base_path_dic( files( i ).BasePath ) = files( i )
			End If
		Next

		If base_path_dic.Count = files.Count Then  Exit For
	Next

	DicItemToArr  base_path_dic, out_ItemArray
End Sub


 
'*************************************************************************
'  <<< [Dic_addFilePaths_fromPathDirectory] >>> 
'*************************************************************************
Function  Dic_addFilePaths_fromPathDirectory( in_out_Dictionary, in_SetIndex, in_MaxIndex, _
		in_BasePath, in_PathDictionary, in_Item )

	If IsEmpty( in_out_Dictionary ) Then
		Set in_out_Dictionary = CreateObject( "Scripting.Dictionary" )
		in_out_Dictionary.CompareMode = 1
	End If

	is_error_back_up = in_PathDictionary.IsNotFoundError
	in_PathDictionary.IsNotFoundError = False


	For Each  full_path  In  in_PathDictionary.FullPaths
		If in_PathDictionary( full_path )  is  in_Item Then

			'// Set "key"
			If IsEmpty( in_BasePath ) Then
				key = full_path
			Else
				key = GetStepPath( full_path, in_BasePath )
			End If


			'// Set "item_array"
			If in_out_Dictionary.Exists( key ) Then
				Set item_array = in_out_Dictionary( key )
			Else
				Set item_array = new ArrayClass
				Set in_out_Dictionary( key ) = item_array
				item_array.ReDim_  in_MaxIndex
				For index = 0  To in_MaxIndex
					Set item_array( index ) = Nothing
				Next
			End If

			Set item_array( in_SetIndex ) = in_Item
		End If
	Next

	in_PathDictionary.IsNotFoundError = is_error_back_up

	Set Dic_addFilePaths_fromPathDirectory = in_out_Dictionary
End Function


 
'*************************************************************************
'  <<< [Dic_addFilePaths_fromOtherPathDirectory] >>> 
'*************************************************************************
Function  Dic_addFilePaths_fromOtherPathDirectory( in_out_Dictionary, in_SetIndex, in_MaxIndex, _
		in_BasePath, in_NewItem, _
		in_OtherBasePath, in_OtherPathDictionary, in_OtherItem )

	If IsEmpty( in_out_Dictionary ) Then
		Set in_out_Dictionary = CreateObject( "Scripting.Dictionary" )
		in_out_Dictionary.CompareMode = 1
	End If

	Set new_path_directory = new PathDictionaryClass
	new_path_directory.BasePath = in_BasePath

	For Each  key  In  in_OtherPathDictionary.Keys
		If IsFullPath( key ) Then
			step_path = GetStepPath( key, in_OtherBasePath )
		Else
			step_path = key
		End If
		If Left( step_path, 2 ) <> ".." Then
			full_path = GetFullPath( step_path, in_BasePath )
			If exist( full_path ) Then
				If in_OtherPathDictionary( key )  is  in_OtherItem Then
					Set new_path_directory( full_path ) = in_NewItem
				Else
					Set new_path_directory( full_path ) = new_path_directory.RemovedObject
				End If
			End If
		End If
	Next


	Set Dic_addFilePaths_fromOtherPathDirectory = Dic_addFilePaths_fromPathDirectory( _
		in_out_Dictionary, in_SetIndex, in_MaxIndex, _
		in_BasePath,  new_path_directory,  in_NewItem )
End Function


 
'*************************************************************************
'  <<< [SplitPathToSubFolderSign] >>> 
'*************************************************************************
Sub  SplitPathToSubFolderSign( in_out_Path, out_SubFolderSign, out_IsFolder, out_Separator )

	root_pos = GetRootSeparatorPosition( in_out_Path )


	'// Set "out_IsFolder"
	right_1 = Right( in_out_Path, 1 )
	If right_1 = "\"  or  right_1 = "/" Then  '// If there is the last separator
		out_IsFolder = True

		path_length = Len( in_out_Path )
		If root_pos = path_length Then
			out_SubFolderSign = ""
			out_Separator = right_1

			Exit Sub
		End If

		in_out_Path = Left( in_out_Path, path_length - 1 )
	Else
		out_IsFolder = False
	End If


	'// Set "last_separator_position"
	back_slash_position = InStrRev( in_out_Path, "\" )
	slash_position = InStrRev( in_out_Path, "/" )
	If back_slash_position > slash_position Then
		last_separator_position = back_slash_position
	Else
		last_separator_position = slash_position
	End If

	If last_separator_position = 0 Then
		out_SubFolderSign = ""

		If right_1 = "/" Then
			out_Separator = "/"
		Else
			out_Separator = "\"
		End If

		Exit Sub
	End If

	out_Separator = Mid( in_out_Path, last_separator_position, 1 )

	If last_separator_position = root_pos Then
		Select Case  g_fs.GetFileName( in_out_Path )
			Case "."
				path_length = Len( in_out_Path )
				If path_length - 1 = root_pos Then
					in_out_Path = Left( in_out_Path, path_length - 1 )
				Else
					in_out_Path = Left( in_out_Path, path_length - 2 )
				End If
			Case Else
				out_SubFolderSign = ""
		End Select

		Exit Sub
	End If


	'// Set "last2_separator_position"
	back_slash_position = InStrRev( in_out_Path, "\", last_separator_position - 1 )
	slash_position = InStrRev( in_out_Path, "/", last_separator_position - 1 )
	If back_slash_position > slash_position Then
		last2_separator_position = back_slash_position
	Else
		last2_separator_position = slash_position
	End If


	'// Set "out_SubFolderSign"
	parent_name = Mid( in_out_Path, _
		last2_separator_position + 1, _
		last_separator_position - last2_separator_position - 1 )
	Select Case  parent_name
		Case  "*" : out_SubFolderSign = parent_name
		Case  "." : out_SubFolderSign = parent_name
		Case Else : out_SubFolderSign = ""
	End Select


	'// Set "in_out_Path"
	If out_SubFolderSign = "" Then
		Select Case  g_fs.GetFileName( in_out_Path )
			Case "."
				in_out_Path = Left( in_out_Path, Len( in_out_Path ) - 2 )
		End Select
	Else
		in_out_Path = Left( in_out_Path, last2_separator_position ) +_
			Mid( in_out_Path, last_separator_position + 1 )
	End If
End Sub


 
'*************************************************************************
'  <<< [GetSubFolders] >>> 
' argument
'  - folders : (out) array of folder pathes
'  - path : base folder path
'*************************************************************************
Sub  GetSubFolders( folders, ByVal path )
	ReDim  folders(-1)
	EnumSubFolders  folders, g_fs.GetFolder( path )
End Sub

Sub  EnumSubFolders( folders, fo )
	Dim  subfo

	ReDim Preserve  folders( UBound(folders) + 1 )
	folders( UBound(folders) ) = fo.Path

	For Each subfo in fo.SubFolders
		EnumSubFolders  folders, subfo
	Next
End Sub


 
'*************************************************************************
'  <<< [EnumFolderObject] >>> 
'*************************************************************************
Sub  EnumFolderObject( FolderPath, out_Folders )
	Dim  i_set, i_get, n, f

	ReDim  out_Folders(0)
	AssertExist  FolderPath
	Set out_Folders(0) = g_fs.GetFolder( FolderPath )
	i_set = 1 : i_get = 0

	While  i_get <= UBound( out_Folders )
		n = out_Folders( i_get ).SubFolders.Count
		ReDim Preserve  out_Folders( UBound( out_Folders ) + n )
		For Each f  In  out_Folders( i_get ).SubFolders
			Set out_Folders( i_set ) = f
			i_set = i_set + 1
		Next
		i_get = i_get + 1
	WEnd
End Sub


 
'*************************************************************************
'  <<< [EnumFolderObjectDic] >>> 
'*************************************************************************
Sub  EnumFolderObjectDic( FolderPath, Opt, out_Folders )
	Dim  folders,  step_pos,  fo

	EnumFolderObject  FolderPath,  folders   '// [out] folders
	step_pos = Len( folders(0).Path ) + 2

	Set out_Folders = CreateObject( "Scripting.Dictionary" )
	For Each fo  In folders  '// fo as Folder
		Set out_Folders( Mid( fo.Path, step_pos ) ) = fo
	Next
	Set out_Folders( "." ) = folders(0)
	out_Folders.Remove  ""
End Sub


 
'*************************************************************************
'  <<< [EnumFileObjectDic] >>> 
'*************************************************************************
Sub  EnumFileObjectDic( ByVal FolderOrPath, out_Files )
	Dim  files,  fi

	If VarType( FolderOrPath ) = vbString Then  Set FolderOrPath = g_fs.GetFolder( FolderOrPath )

	Set out_Files = CreateObject( "Scripting.Dictionary" )
	For Each fi  In FolderOrPath.Files
		Set out_Files( fi.Name ) = fi
	Next
End Sub


 
'*************************************************************************
'  <<< [IsMatchedWithWildcard] >>> 
'*************************************************************************
Function  IsMatchedWithWildcard( Path, WildCard )
	path_array = Array( Path )
	RemoveWildcardMatchedArrayItems  path_array, WildCard
	IsMatchedWithWildcard = ( UBound( path_array ) = -1 )
End Function


 
'*************************************************************************
'  <<< [RemoveWildcardMatchedArrayItems] >>> 
'*************************************************************************
Sub  RemoveWildcard( WildCard, in_out_PathArray )  '//[RemoveWildcard]
	RemoveWildcardMatchedArrayItems  in_out_PathArray, WildCard
	ThisIsOldSpec
End Sub

Sub  RemoveWildcardMatchedArrayItems( in_out_PathArray, WildCard )
	Dim  s, path, fname, i, n, wc, wc_len


	'//=== Check by with wildcard
	If Left( WildCard, 1 ) = "*" Then
		wc = LCase( Mid( WildCard, 2 ) ) : wc_len = Len( wc )
		n = UBound( in_out_PathArray )
		For i = 0 To  n
			path = in_out_PathArray(i)
			Do
				fname = g_fs.GetFileName( path )
				If LCase( Right( fname, wc_len ) ) = wc Then  in_out_PathArray(i) = Empty : Exit Do
				path = g_fs.GetParentFolderName( path )
				If path = "" Then Exit Do
			Loop
		Next


	'//=== Check by no wildcard
	Else
		wc = LCase( WildCard )
		n = UBound( in_out_PathArray )
		For i = 0 To n
			path = in_out_PathArray(i)
			Do
				fname = g_fs.GetFileName( path )
				If LCase( fname ) = wc Then  in_out_PathArray(i) = Empty : Exit Do
				path = g_fs.GetParentFolderName( path )
				If path = "" Then Exit Do
			Loop
		Next
	End If


	'//=== shrink the array
	n = 0
	For i = 0 To UBound( in_out_PathArray )
		If not IsEmpty( in_out_PathArray(i) ) Then _
			in_out_PathArray(n) = in_out_PathArray(i) : n = n + 1
	Next
	Redim Preserve  in_out_PathArray( n - 1 )
End Sub


 
'*************************************************************************
'  <<< [RemoveWildcard] >>> 
'*************************************************************************
Sub  RemoveWildcard( WildCard, fnames )
	Dim  s, path, fname, i, n, wc, wc_len

ThisIsOldSpec

	'//=== check by with wildcard
	If Left( WildCard, 1 ) = "*" Then
		wc = LCase( Mid( WildCard, 2 ) ) : wc_len = Len( wc )
		n = UBound( fnames )
		For i = 0 To  n
			path = fnames(i)
			Do
				fname = g_fs.GetFileName( path )
				If LCase( Right( fname, wc_len ) ) = wc Then  fnames(i) = Empty : Exit Do
				path = g_fs.GetParentFolderName( path )
				If path = "" Then Exit Do
			Loop
		Next


	'//=== check by no wildcard
	Else
		wc = LCase( WildCard )
		n = UBound( fnames )
		For i = 0 To n
			path = fnames(i)
			Do
				fname = g_fs.GetFileName( path )
				If LCase( fname ) = wc Then  fnames(i) = Empty : Exit Do
				path = g_fs.GetParentFolderName( path )
				If path = "" Then Exit Do
			Loop
		Next
	End If


	'//=== shrink the array
	n = 0
	For i = 0 To UBound( fnames )
		If not IsEmpty( fnames(i) ) Then  fnames(n) = fnames(i) : n = n + 1
	Next
	Redim Preserve  fnames( n - 1 )
End Sub


 
'*************************************************************************
'  <<< [SetIniFileTextValue] >>> 
'*************************************************************************
Function  SetIniFileTextValue( Text, SectionName, VariableName, Value, Option_ )
	SearchInIniFileText  Text, SectionName, VariableName, Option_, line  '//(out) line

	If line is Nothing Then _
		Raise  E_PathNotFound, "<ERROR msg=""Not found VariableName"" variable_name="""+ _
			VariableName+"""/>"

	SetIniFileTextValue = Left( Text, line.EqualPosition ) & " " & Value & _
		Mid( Text, line.OverPosition  )
End Function


 
'*************************************************************************
'  <<< [ParseIniFileLine] >>> 
'*************************************************************************
Function  ParseIniFileLine( in_Line )
	Set return = new ParsedIniFileLineClass
	a_line = Trim2( in_Line )
	If Left( a_line, 1 ) = "[" Then
		If Right( a_line, 1 ) = "]" Then
			a_line = Mid( a_line, 2, Len( a_line ) - 2 )
		Else
			a_line = Mid( a_line, 2, Len( a_line ) - 1 )
		End If
		return.Section = Trim( a_line )
	Else
		equal_position = InStr( a_line, "=" )
		If equal_position > 0 Then
			return.Name = Trim2( Left( a_line, equal_position - 1 ) )
			return.Value = Trim2( Mid( a_line, equal_position + 1 ) )
		End If
	End If
	Set ParseIniFileLine = return
End Function

Class  ParsedIniFileLineClass
	Public  Section
	Public  Name
	Public  Value
End Class


 
'*************************************************************************
'  <<< [GetIniFileTextValue] >>> 
'*************************************************************************
Function  GetIniFileTextValue( Text, SectionName, VariableName, Option_ )
	SearchInIniFileText  Text, SectionName, VariableName, Option_, line  '//(out) line

	If line is Nothing Then _
		Exit Function

	value = Trim2( Mid( Text, line.EqualPosition + 1, _
		line.OverPosition - line.EqualPosition ) )
	If IsNumeric( value ) Then  value = Eval( value )
	GetIniFileTextValue = value
End Function


 
'*************************************************************************
'  <<< [GetIniFileTextValues] >>> 
'*************************************************************************
Function  GetIniFileTextValues( Text, ByVal SectionName, VariableName, Option_ )
	Set ret = new ArrayClass
	Do
		SearchInIniFileText  Text, SectionName, VariableName, Option_, line  '//(out) line
		If line is Nothing Then _
			Exit Do

		value = Trim2( Mid( Text, line.EqualPosition + 1, _
			line.OverPosition - line.EqualPosition ) )
		If IsNumeric( value ) Then  value = Eval( value )

		ret.Add  value

		SectionName = line.OverPosition
	Loop
	GetIniFileTextValues = ret.Items
End Function


'*************************************************************************
'  <<< [SearchInIniFileText] >>> 
'*************************************************************************
Sub  SearchInIniFileText( Text, SectionName, VariableName, Option_, out_Line )
	Set c = g_VBS_Lib
	name_length = Len( VariableName )

	'// Set "section_start", "section_over"
	If IsEmpty( SectionName ) Then
		section_start = 1
		section_over = Len( Text ) + 1
	ElseIf VarType( SectionName ) = vbString Then
		If Left( SectionName, 1 ) = "[" Then
			section_keyword = SectionName
		Else
			section_keyword = "["+ SectionName +"]"
		End If
		section_start = InStrEx( Text, section_keyword, 0, c.LineHead or c.LineTail )
		section_over  = InStrEx( Text, "[", section_start + 1, c.LineHead )
		If section_over = 0 Then _
			section_over = Len( Text ) + 1
	Else
		Assert  IsNumeric( SectionName )
		section_start = SectionName
		section_over  = InStrEx( Text, "[", section_start + 1, c.LineHead )
		If section_over = 0 Then _
			section_over = Len( Text ) + 1
	End If

	Set line = new IniFileTextLineClass
	p = section_start
	Do
		p = InStrEx( Text, VariableName, p, c.LineHead or c.WholeWord )
		If p = 0  or  p >= section_over Then
			Set out_Line = Nothing
			Exit Sub
		End If

		q = InStr( p + name_length, Text, "=" )
		If Trim2( Mid( Text, p, q - p ) ) = VariableName Then
			line.HeadPosition = p
			line.EqualPosition = q
			Exit Do
		End If

		p = p + 1
	Loop

	line.OverPosition = InStr( line.EqualPosition, Text, vbCRLF )
	If line.OverPosition = 0 Then
		line.OverPosition = Len( Text ) + 1
	End If

	Set out_Line = line
End Sub


Class  IniFileTextLineClass
	Public  HeadPosition
	Public  OverPosition
	Public  EqualPosition
End Class


 
'*************************************************************************
'  <<< [RenumberIniFileData] >>> 
'*************************************************************************
Sub  RenumberIniFileData( ReadStream, WriteStream, StartNumber, IsPlusSpaceLineOnly )
	number = StartNumber
	is_space_line_previous = True

	Do Until  ReadStream.AtEndOfStream()
		line = ReadStream.ReadLine()

		equal_position = InStr( line, "=" )
		left_position  = InStr( line, "(" )
		right_position = InStr( line, ")" )
		If equal_position > 0  and  left_position > 0  and  right_position > 0  and _
			left_position < equal_position  and  right_position < equal_position  and _
			left_position < right_position  Then

			line = Left( line, left_position ) & number & Mid( line, right_position )
		End If

		WriteStream.WriteLine  line


		'// Plus "number"
		If IsPlusSpaceLineOnly Then
			If Trim( line ) = "" Then
				If is_space_line_previous = False Then
					number = number + 1
				End If
				is_space_line_previous = True
			Else
				is_space_line_previous = False
			End If
		Else
			number = number + 1
		End If
	Loop
End Sub


 
'*************************************************************************
'  <<< [ParseJSON] >>> 
'*************************************************************************
Function  ParseJSON( JSON_String )
	Set sc = CreateObject("ScriptControl")
	sc.Language = "JScript"
	sc.AddCode  "function parseJSON(s) { return eval('(' + s + ')'); }"
	Set ParseJSON = sc.CodeObject.parseJSON( JSON_String )
End Function


 
'*************************************************************************
'  <<< [MeltCSV] >>> 
'*************************************************************************
Function  MeltCSV( Line, in_out_Start )
	Dim  s, i, c

	i = in_out_Start

	If i=0 Then Exit Function


	'//=== Skip space character
	Do
		c = Mid( Line, i, 1 )
		If c<>" " and c<>vbTab Then Exit Do
		i = i + 1
	Loop

	Select Case  c

	 '//=== If enclosed by " "
	 Case """"
		s = ""
		Do
			i = i + 1
			c = Mid( Line, i, 1 )
			If c = "" Then Exit Do
			If c = """" Then
				i = i + 1
				c = Mid( Line, i, 1 )
				If c = """" Then  s = s + c  Else  Exit Do
			Else
				s = s + c
			End If
		Loop

		MeltCSV = s

		Do
			If c = "" Then  in_out_Start = 0 : Exit Function
			If c = "," Then  in_out_Start = i+1 : Exit Function
			i = i + 1
			c = Mid( Line, i, 1 )
		Loop


	 '//=== If no value
	 Case ","
		in_out_Start = i+1 : Exit Function
	 Case ""
		in_out_Start = 0 : Exit Function


	 '//=== If NOT enclosed by " "
	 Case Else
		Do
			If c = "" or c = "," Then Exit Do
			s = s + c
			i = i + 1
			c = Mid( Line, i, 1 )
		Loop

		MeltCSV = Trim2( s )

		If c = "" Then  in_out_Start = 0 : Exit Function
		If c = "," Then  in_out_Start = i+1 : Exit Function
	End Select
End Function


 
'*************************************************************************
'  <<< [CSVText] >>> 
'*************************************************************************
Function  CSVText( s )
	If InStr( s, """" ) = 0 and  InStr( s, "," ) = 0 and  InStr( s, vbCRLF ) = 0 and _
		 	Left( s, 1 ) <> " " and  Right( s, 1 ) <> " " Then
		CSVText = CStr( s )
	Else
		CSVText = """" + Replace( s, """", """""" ) + """"
	End If
End Function


 
'*************************************************************************
'  <<< [CSV_insert] >>> 
'*************************************************************************
Function  CSV_insert( CSV_Line, RightColumnNum0, Element )
	CSV_insert = CSV_insert_or_set( CSV_Line, RightColumnNum0, Element, True )
End Function


Function  CSV_insert_or_set( CSV_Line, RightColumnNum0, Element, IsInsert )
	If RightColumnNum0 < 0 Then  Error

	column_num = 0
	text_position = 1
	Do
		If column_num = RightColumnNum0 Then _
			Exit Do

		MeltCSV  CSV_Line, text_position  '//(in/out)text_position
		column_num = column_num + 1
		If text_position = 0 Then
			CSV_insert_or_set = CSV_Line +_
				String( RightColumnNum0 - column_num + 1, "," ) +_
				CSVText( Element )
			Exit Function
		End If
	Loop

	If Trim( CSV_Line ) = "" Then
		CSV_insert_or_set = CSVText( Element )
	Else
		If IsInsert Then
			CSV_insert_or_set = Left( CSV_Line, text_position - 1 ) + _
				CSVText( Element ) +","+ _
				Mid( CSV_Line, text_position )
		Else
			next_position = text_position
			MeltCSV  CSV_Line, next_position  '//(in/out)next_position

			If next_position = 0 Then
				CSV_insert_or_set = Left( CSV_Line, text_position - 1 ) + _
					CSVText( Element )
			Else
				CSV_insert_or_set = Left( CSV_Line, text_position - 1 ) + _
					CSVText( Element ) +","+ _
					Mid( CSV_Line, next_position )
			End If
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [CSV_set] >>> 
'*************************************************************************
Function  CSV_set( CSV_Line, ColumnNum0, Element )
	CSV_set = CSV_insert_or_set( CSV_Line, ColumnNum0, Element, False )
End Function


 
'*************************************************************************
'  <<< [CSV_remove] >>> 
'*************************************************************************
Function  CSV_remove( CSV_Line, ColumnNum0 )
	If ColumnNum0 < 0 Then  Error

	column_num = 0
	text_position = 1
	Do
		If column_num = ColumnNum0 Then _
			Exit Do

		MeltCSV  CSV_Line, text_position  '//(in/out)text_position
		column_num = column_num + 1
		If text_position = 0 Then
			If CSV_Line = "" Then
				CSV_remove = _
					String( ColumnNum0 - column_num + 1, "," )
			Else
				CSV_remove = CSV_Line +_
					String( ColumnNum0 - column_num, "," )
			End IF
			Exit Function
		End If
	Loop

	next_position = text_position
	MeltCSV  CSV_Line, next_position  '//(in/out)next_position

	If next_position = 0 Then
		If text_position = 1 Then
			CSV_remove = ""
		Else
			CSV_remove = Left( CSV_Line, text_position - 2 )
		End If
	Else
		CSV_remove = Left( CSV_Line, text_position - 1 ) + _
			Mid( CSV_Line, next_position )
	End If
End Function


 
'*************************************************************************
'  <<< [XmlAttr] >>> 
'*************************************************************************
Function  XmlAttr( s )
	XmlAttr = Replace( s, "&", "&amp;" )
	XmlAttr = Replace( XmlAttr, """", "&quot;" )
	XmlAttr = Replace( XmlAttr, "<", "&lt;" )
End Function


 
'*************************************************************************
'  <<< [XmlAttrA] >>> 
'*************************************************************************
Function  XmlAttrA( s )
	XmlAttrA = Replace( s, "&", "&amp;" )
	XmlAttrA = Replace( XmlAttrA, "'", "&apos;" )
	XmlAttrA = Replace( XmlAttrA, "<", "&lt;" )
End Function


 
'*************************************************************************
'  <<< [XmlText] >>> 
'*************************************************************************
Function  XmlText( s )
	XmlText = Replace( s, "&", "&amp;" )
	XmlText = Replace( XmlText, "<", "&lt;" )
	XmlText = Replace( XmlText, ">", "&gt;" )
End Function


 
'*************************************************************************
'  <<< [XmlTags] >>> 
'*************************************************************************
Function  XmlTags( Xml, Level )
	Dim  f : Set f = new StringStream

	f.SetString  Replace( Xml, vbTab, "  " )
	Do Until f.AtEndOfStream
		XmlTags = XmlTags + GetTab( Level ) + f.ReadLine() + vbCRLF
	Loop
	CutLastOf  XmlTags, vbCRLF, Empty
End Function


 
'*************************************************************************
'  <<< [GetTab] >>> 
'*************************************************************************
Function  GetTab( Level )
	GetTab = String( Level*2, " " )
End Function


 
'*************************************************************************
'  <<< [get_XMLDomConsts] >>> 
'*************************************************************************
Dim  g_XMLDomConsts

Function    get_XMLDomConsts()
	If IsEmpty( g_XMLDomConsts ) Then _
		Set g_XMLDomConsts = new XMLDomConsts
	Set get_XMLDomConsts =   g_XMLDomConsts
End Function


Class  XMLDomConsts
	Public  XMLDOMElement, XMLDOMText

	Private Sub  Class_Initialize()
		XMLDOMElement = 1
		XMLDOMText    = 3
	End Sub
End Class


 
'*************************************************************************
'  <<< [FilePathClass] >>> 
'*************************************************************************
Class  FilePathClass
	Public  Text
	Public  FilePath
End Class


 
'*************************************************************************
'  <<< [new_FilePathForString] >>> 
'*************************************************************************
Function  new_FilePathForString( Text )
	Set object = new FilePathClass
	object.Text = Text
	object.FilePath = WScript.ScriptFullName
	Set new_FilePathForString = object
End Function


 
'********************************************************************************
'  <<< [new_FilePathForFileInScript] >>> 
'********************************************************************************
Function  new_FilePathForFileInScript( Parameter )
	Set object = new FilePathClass
	object.Text = ReadVBS_Comment( Parameter, "["+"FileInScript.", "[/"+"FileInScript.", Empty )

	If IsEmpty( Parameter ) Then
		object.FilePath = WScript.ScriptFullName
	Else
		object.FilePath = GetFullPath( Parameter, Empty )
	End If
	Set new_FilePathForFileInScript = object
End Function


 
'*************************************************************************
'  <<< [GetFilePathString] >>> 
'*************************************************************************
Function  GetFilePathString( Path )
	If TypeName( Path ) = "FilePathClass" Then
		GetFilePathString = Path.FilePath
	Else
		GetFilePathString = Path
	End If
End Function


 
'*************************************************************************
'  <<< [LoadXML] >>> 
'*************************************************************************
Const  F_NoRoot = 1
Const  F_Str = &h8000

Function  LoadXML( PathOrStr, Opt )
	Set co = g_VBS_Lib
	Dim  xml, r, t, i, c, f, is_bom, proc_inst
	Const  start_tag = "<Dummy_Root_>"
	Const  end_tag = "</Dummy_Root_>"


	If Opt and co.InheritSuperClass Then
		Set base = GetHRefBase( PathOrStr, Array("SuperClass") )
		Set root = base.Linker.StackOfSourceXmlRootElem( 0 )

		For Each  destination_tag  In  root.selectNodes( "//*[@super_class]" )
			Set super = base.href( destination_tag.getAttribute( "super_class" ) )
			For Each  attribute  In  super.attributes
				If attribute.name <> "id"  and  attribute.name <> "name" Then
					destination_tag.setAttribute  attribute.name, attribute.value
				End If
			Next
			destination_tag.removeAttribute  "super_class"
		Next

		For Each  super  In  root.selectNodes( "..//SuperClass" )
			If not IsNull( super.firstChild ) Then
				Set sibling = super.nextSibling
				Set parent = super.parentNode
				If IsNull( super.getAttribute( "id" ) ) Then _
					parent.removeChild  super
				For Each  child  In  super.childNodes
					super.removeChild  child
					If sibling is Nothing Then
						parent.appendChild  child
					Else
						parent.insertBefore  child, sibling
					End If

					For Each  attribute  In  super.attributes
						If attribute.name <> "id"  and  attribute.name <> "name" Then
							child.setAttribute  attribute.name, attribute.value
						End If
					Next
				Next
			End If
		Next

		Set LoadXML = root
		Exit Function
	End If


	If TypeName( PathOrStr ) = "FilePathClass" Then
		t = PathOrStr.Text
	ElseIf Opt and co.StringData Then
		i=1 : Do : c = Mid( PathOrStr, i, 1 ) : If c<>" " and c<>vbTab Then Exit Do
		i=i+1 : Loop
		If (Opt and co.NoRootXML)  or  c <> "<" Then
			t = start_tag + PathOrStr + end_tag
		Else
			t = PathOrStr
		End If
	Else
		Set f = OpenForRead( PathOrStr )
		t = ReadAll( f )
'//    If Left( t, 2 ) = Chr(&h8145)+Chr(&hBF) Then  t = Mid( t, 3 ) : is_bom = True  '// BOM of UTF-8

		'// ?xml encoding
		If InStr( t, "<" ) = InStr( t, "<?xml" ) Then
			Set xml = CreateObject("MSXML2.DOMDocument")
			xml.resolveExternals = false
			xml.validateOnParse = false
			r = xml.load( PathOrStr )
		Else
			i=1 : Do : c = Mid( t, i, 1 ) : If c<>" " and c<>vbTab Then Exit Do
			i=i+1 : Loop  '// Mid( t, i, 1 ) = "<"
			If (Opt and co.NoRootXML) or c<>"<" Then  t = start_tag + t + end_tag
		End If
	End If

	If IsEmpty( xml ) Then
		Set xml = CreateObject("MSXML2.DOMDocument")
		xml.resolveExternals = false
		xml.validateOnParse = false
		r = xml.loadXML( t )
	End IF
	If not r Then
		If xml.parseError.errorCode = &HC00CE555 Then  '// If multi root element
			t = start_tag + t + end_tag
			r = xml.loadXML( t )
		End If
	End If
	If not r Then  Raise 1,"<ERROR msg=""This is not Unicode or not regular XML format"" PathOrStr="""+ XmlAttr( PathOrStr ) +_
		""" line="""& xml.parseError.line &""" line_pos="""& xml.parseError.linepos &""" reason="""+ Trim2( xml.parseError.reason ) +_
		"""><![CDATA["+ vbCRLF + xml.parseError.srcText + vbCRLF +"]]></ERROR>"
	Set LoadXML = xml.lastChild  '// If firstChild, <?xml> may be got.
End Function


 
'*************************************************************************
'  <<< [LoadXML_Cached] >>> 
'*************************************************************************
Function  LoadXML_Cached( PathOrStr, Opt )
	If TypeName( PathOrStr ) = "FilePathClass"  or  IsBitSet( Opt, g_VBS_Lib.StringData ) Then
		Set LoadXML_Cached = LoadXML( PathOrStr, Opt )
	Else
		full_path = GetFullPath( PathOrStr, Empty )
		If g_VBS_Lib.XML_DOM_ReadCache.Exists( full_path ) Then
			Set LoadXML_Cached = g_VBS_Lib.XML_DOM_ReadCache( full_path )
		Else
			Set LoadXML_Cached = LoadXML( PathOrStr, Opt )
			Set g_VBS_Lib.XML_DOM_ReadCache( full_path ) = LoadXML_Cached
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [OpenForReplaceXML] >>> 
'*************************************************************************
Function  OpenForReplaceXML( SrcPath, DstPath )
	echo  ">OpenForReplaceXML  """+ SrcPath +""", """& DstPath &""""
	Dim  ec : Set ec = new EchoOff

	Dim  o : Set o = new ReplaceXmlFile1
	o.SrcPath = SrcPath
	o.DstPath = DstPath
	If LCase( GetFullPath( SrcPath, Empty ) ) = LCase( GetFullPath( DstPath, Empty ) ) Then
		o.DstPath = Empty
	End If
	o.IsUserConfirm = False
	Set o.WriteSets = new ArrayClass
	Set OpenForReplaceXML = o
End Function


Class  ReplaceXmlFile1
	Public  SrcPath
	Public  DstPath
	Public  IsUserConfirm
	Public  WriteSets  ' as ArrayClass of ReplaceXmlFile1_Sets

	Public Sub  Write( XPath, Value )
		Dim  o : Set o = new ReplaceXmlFile1_Sets
		o.XPath = XPath
		o.Value = Value
		Me.WriteSets.Add  o
	End Sub

	Private Sub  Class_Terminate()
		If Err.Number = 0 Then
			Dim  en,ed : en = Err.Number : ed = Err.Description
			On Error Resume Next  '// This clears the error

				Me.Close

			ErrorCheckInTerminate  en
			On Error GoTo 0 : If en <> 0 Then  Err.Raise en,,ed  '// This sets en again
		End If
	End Sub

 Public Sub  Close()
	Dim  b, is_change, a, tmp_path, value, text, format, flags
	Dim  c : Set c = g_VBS_Lib

	format = ReadUnicodeFileBOM( Me.SrcPath )
	Dim  root : Set root = LoadXML( Me.SrcPath, Empty ) '// as IXMLDOMElement


	'//=== Confirm to user
	If Me.IsUserConfirm Then
		For Each a  In Me.WriteSets.Items
			value = XmlRead( root, a.XPath )
			If value = a.Value Then  Me.WriteSets.RemoveObject  a
		Next

		echo_line
		For Each a  In Me.WriteSets.Items
			echo  a.XPath +"="""+ a.Value +""""
		Next

		If Me.WriteSets.Count = 0 Then  Exit Sub

		If IsEmpty( Me.DstPath ) Then  tmp_path = Me.SrcPath  Else  tmp_path = Me.DstPath
		echo  "Update file:"+ tmp_path
		echo  "XML file's attribute will be modified"
		pause
		echo_line
	End If


	Dim  ec : Set ec = new EchoOff


	'//=== DOM object is modified
	is_change = False
	For Each a  In Me.WriteSets.Items
		b= XmlWrite( root, a.XPath, a.Value )
		is_change = is_change or b
	Next


	'//=== Save to file
	If is_change Then
		Dim  sf : Set sf = new_IsSafeFileUpdateStack( True )
		If format <> c.No_BOM Then  Dim  cs : Set cs = new_TextFileCharSetStack( format )
		If IsEmpty( Me.DstPath ) Then  tmp_path = Me.SrcPath  Else  tmp_path = Me.DstPath

		If root.ownerDocument.firstChild.nodeType = c.NODE_PROCESSING_INSTRUCTION Then
			If InStr( root.ownerDocument.firstChild.nodeValue, "encoding" ) > 0 Then
				Dim  s : s = root.ownerDocument.xml  '// This is not included <?xml encoding>
				Dim  xml_pos1 : xml_pos1 = InStr( s, "<?xml" )
				Dim  xml_pos2 : xml_pos2 = InStr( xml_pos1, s, "?>" )
				CreateFile  tmp_path,  Left( s, xml_pos1 - 1 ) +_
					"<?xml "+ root.ownerDocument.firstChild.nodeValue + Mid( s, xml_pos2 )
			Else
				CreateFile  tmp_path,  root.ownerDocument.xml
			End If
		Else
			CreateFile  tmp_path,  root.ownerDocument.xml
		End If
	End If
 End Sub
End Class

Class  ReplaceXmlFile1_Sets
	Public  XPath
	Public  Value
End Class



 
'*************************************************************************
'  <<< [OpenForAppendXml] >>> 
'*************************************************************************
Function  OpenForAppendXml( SrcPath, ByVal DstPath )
	Dim  ret,  xml_str,  pos
	Dim  is_dst_will_be_exist : is_dst_will_be_exist = not IsEmpty( DstPath )

	If IsEmpty( DstPath ) Then  DstPath = GetTempPath("OpenForAppendXml_*.xml")

	Set ret = new AppendXmlFile
	Set ret.Rep = StartReplace( SrcPath, DstPath, is_dst_will_be_exist )
	xml_str = ret.Rep.r.ReadAll()

	pos = InStrRev( xml_str, "<" )
	ret.CloseTag = Mid( xml_str, pos )

	xml_str = Left( xml_str, pos - 1 )
'//	pos = InStrRev( xml_str, vbLF )
'//	If pos = 0 Then
'//		ret.CR = vbCR
'//	ElseIf Mid( xml_str, pos - 1, 1 ) = vbCR Then
'//		ret.CR = vbCR
'//	Else
'//		ret.CR = ""
'//	End If

	ret.Rep.w.Write  xml_str
	Set OpenForAppendXml = ret
End Function


'//[AppendXmlFile]
Class  AppendXmlFile
	Public  Rep  '// as StartReplaceObj
	Public  CloseTag  '// as string
'//	Public  CR  '// string

	Public Sub  Write( Str ) : Me.Rep.w.Write  Str : End Sub
'//	Public Sub  WriteLine( LineStr ) : Me.Rep.w.WriteLine  LineStr + Me.CR : End Sub
	Public Sub  WriteLine( LineStr ) : Me.Rep.w.WriteLineDefault  LineStr : End Sub

	Public Sub  WriteXml( XmlStr )
		Dim  close_pos,  pos,  lf_pos,  close_tag,  root_tag,  xml_str
		close_pos = InStrRev( XmlStr, "<" )
		close_tag = Mid( XmlStr, close_pos )
		pos = InStr( close_pos, XmlStr, ">" )
		root_tag = "<" + Mid( close_tag, 3, pos - close_pos - 2 ) + ">"
		pos = InStr( XmlStr, root_tag )
		If pos = 0 Then  Raise  E_NotFoundSymbol, "<ERROR msg=""XML Root tag is not found"""+_
			" root_tag="""+ XmlAttr( root_tag ) +""" close_tag="""+ XmlAttr( close_tag ) +"""/>"
		pos = pos + Len( root_tag )
		lf_pos = InStr( pos, XmlStr, vbLF )
		If lf_pos > 0 Then
			If Trim2( Mid( XmlStr, pos, lf_pos - pos ) ) = "" Then  pos = lf_pos + 1
		End If
		xml_str = Mid( XmlStr, pos, close_pos - pos )
		Me.Rep.w.Write  xml_str
'//		If Right( xml_str, 1 ) <> vbLF Then  Me.Rep.w.WriteLine  Me.CR
		If Right( xml_str, 1 ) <> vbLF Then  Me.Rep.w.WriteLineDefault  ""
	End Sub

	Private Sub  Class_Terminate()
		Me.Rep.w.Write  Me.CloseTag
		Me.Rep.Finish
	End Sub
End Class


 
'*************************************************************************
'  <<< [XmlWriteEncoding] >>> 
'*************************************************************************
Sub  XmlWriteEncoding( RootXmlElement, CharSet )
	RootXmlElement.ownerDocument.insertBefore _
		RootXmlElement.ownerDocument.createProcessingInstruction( "xml", "version=""1.0"" encoding="""+ CharSet +"""" ), _
		RootXmlElement
End Sub

 
'*************************************************************************
'  <<< [XmlAttrDic] >>> 
'*************************************************************************
Function  XmlAttrDic( XmlElem )
	Dim  attr, dic
	Set dic = CreateObject( "Scripting.Dictionary" )
	For Each attr  In XmlElem.attributes
		dic( attr.name ) = attr.value
	Next
	Set XmlAttrDic = dic
End Function


 
'*************************************************************************
'  <<< [XmlWrite] >>> 
'*************************************************************************
Function  XmlWrite( BaseXmlElement, XPath, NewValue )
	XmlWrite = XmlWriteSub( BaseXmlElement, XPath, NewValue, True )
End Function

Function  XmlWriteSub( BaseXmlElement, XPath, NewValue, IsForWrite )
	Dim  nodes, node, is_attr, value, xpath2, text, i, attr_name, parent
	Dim  c : Set c = g_VBS_Lib
	Set xml = BaseXmlElement.ownerDocument

	If IsEmpty( XPath ) Then
		Set nodes = new ArrayClass : nodes.Add  BaseXmlElement
	Else
		Set nodes = BaseXmlElement.selectNodes( XPath )
	End If

	If BaseXmlElement.selectSingleNode( "/" ).lastChild.getAttribute("xml:space") = "preserve" Then
		is_space_preserve = True
	Else
		is_space_preserve = False  '// Not set Null
	End If


	'//=== Create new node
	If nodes.Length = 0 Then
		If IsEmpty( NewValue ) Then
			XmlWriteSub = False
		Else
			Dim  new_node, ii, i3, i4, i5, tag_name, name, is_last_node
			xpath2 = XPath

			Do
				xpath2 = g_fs.GetParentFolderName( xpath2 )
				If xpath2 = "" Then  Raise  E_NotFoundSymbol, "Root node can not be modified"
				Set nodes = BaseXmlElement.selectNodes( xpath2 )
				If nodes.Length > 0 Then  Exit Do
			Loop
			Set new_node = new ArrayClass  '// Cast from IXMLDOMNodeList to ArrayClass
			For Each node  In nodes
				new_node.Add  node
			Next
			Set nodes = new_node
			ReDim  new_node( nodes.UBound_ )
			ReDim  is_auto_space( nodes.UBound_ )

			ip = Len( xpath2 ) + 1  '// ip = index of path
			level = 0
			Do
				'//=== Set "tag_name", "is_attr", "is_last_node"
				ip2 = InStr( ip+1, XPath, "/" )
				If ip2 = 0 Then
					is_last_node = True
					xpath2 = XPath
					tag_name = Mid( XPath, ip+1 )
				Else
					is_last_node = (Len( Mid( XPath, ip2 ) ) = 1)
					xpath2 = Left( XPath, ip2 - 1 )
					tag_name = Mid( XPath, ip+1, ip2 - ip - 1 )
				End If
				is_attr = ( Left( tag_name, 1 ) = "@" )
				If is_attr Then  tag_name = Mid( tag_name, 2 )


				For i = 0  To  nodes.UBound_

					If not is_attr Then

						If level = 0 Then
							If is_space_preserve Then
								If InStr( nodes(i).xml, vbLF ) > 0 Then
									is_auto_space(i) = True
								Else
									is_auto_space(i) = False
								End If
							Else
								is_auto_space(i) = True
							End If
						End If


						'//=== Insert tab text for indent of xml element
						space_node = Empty
						If is_auto_space(i) Then
							i3 = 1 : Set node = nodes(i).parentNode.parentNode
							Do Until node is Nothing : Set node = node.parentNode : i3=i3+1 : Loop
							text = String( i3, vbTab )
							If nodes(i).hasChildNodes Then
								If nodes(i).lastChild.nodeType = c.NODE_TEXT Then
									If nodes(i).lastChild.previousSibling  is Nothing Then
										text = Empty
									Else
										text = vbTab
									End If
								Else
									text = vbCRLF + text
								End If
							Else
									text = vbCRLF + text
							End If
							If not IsEmpty( text ) Then
								Set space_node = xml.createTextNode( text )
								nodes(i).appendChild  space_node
							End If
						End If

						'//=== Insert xml element
						i3 = InStr( tag_name, "[" )
						If i3 = 0 Then
							Set new_node(i) = xml.createNode( c.NODE_ELEMENT, _
								tag_name, nodes(i).namespaceURI )
							If IsEmpty( space_node ) Then
								nodes(i).appendChild  new_node(i)
							Else
								XmlInsertAfter  space_node, new_node(i)
							End If
						Else
							i4 = InStr( i3, tag_name, "=" )
							name = Mid( tag_name, i3+2, i4-i3-2 )
							value = Mid( tag_name, i4+2, InStr( i4+2, tag_name, _
								Mid( tag_name, i4+1, 1 ) ) -i4-2 )
							tag_name = Left( tag_name, i3-1 )

							Set new_node(i) = nodes(i).appendChild( _
								xml.createElement( tag_name ) )
							new_node(i).setAttribute  name, value
						End If

						'//=== Insert tab text for indent of xml element
						If is_auto_space(i) Then
							i3 = 0 : Set node = new_node(i).parentNode.parentNode.parentNode
							Do Until node is Nothing : Set node = node.parentNode : i3=i3+1 : Loop
							nodes(i).appendChild  xml.createTextNode( vbCRLF + String( i3, vbTab ) )
						End If
					End If

					If is_last_node Then
						If is_attr Then
							'//=== Set xml attribute
							If IsEmpty( new_node(i) ) Then
								nodes(i).setAttribute  tag_name, NewValue
							Else
								new_node(i).setAttribute  tag_name, NewValue
							End If
						Else
							new_node(i).appendChild  xml.createTextNode( NewValue )
						End If
					End If

					If not IsEmpty( new_node(i) ) Then _
						Set nodes(i) = new_node(i)
				Next
				If is_last_node Then  Exit Do
				ip = ip2
				level = level + 1
			Loop
			XmlWriteSub = True
		End If


	'//=== Change xml attribute
	Else

		If nodes(0).nodeType = c.NODE_ATTRIBUTE Then
			i = InStr( XPath, "/@" )
			Set nodes = BaseXmlElement.selectNodes( Left( XPath, i-1 ) )
			attr_name = Mid( XPath, i+2 )
			is_attr = True
		End If


		XmlWriteSub = False
		For Each node  In nodes

			If is_attr Then

				value = node.getAttribute( attr_name )
				If IsEmpty( NewValue ) Then
					If not IsNull( value ) Then
						node.removeAttribute  attr_name
						XmlWriteSub = True
					End If
				Else
					If IsNull( value ) Then  value = Empty  '// for value <> NewValue
					If value <> NewValue Then
						If IsForWrite Then
							node.setAttribute  attr_name, NewValue
							XmlWriteSub = True
						Else
							'//=== Insert tab text for indent of xml element
							If node.nextSibling.nodeType = c.NODE_TEXT Then _
								node.nextSibling.nodeValue = _
									node.nextSibling.nodeValue + vbTab

							'//=== Insert xml element
							Set new_node = node.parentNode.appendChild(_
								node.ownerDocument.createElement( node.tagName ) )
							new_node.setAttribute  attr_name, NewValue

							'//=== Insert tab text for indent of xml element
							If not is_space_preserve Then
								i3 = 0 : Set parent = new_node.parentNode.parentNode.parentNode
								Do Until parent is Nothing : Set parent = parent.parentNode : i3=i3+1 : Loop
								node.parentNode.appendChild  node.ownerDocument.createTextNode(_
									vbCRLF + String( i3, vbTab ) )
							End If

							XmlWriteSub = True
							Exit For  '// xml element must be created one time
						End If
					End If
				End If

			Else

				Set text = node.selectSingleNode( "text()" )  '// as IXMLDOMText
				If text is Nothing Then
					If IsEmpty( NewValue ) or NewValue = "" Then
						XmlWriteSub = False
					Else
						node.appendChild  xml.createTextNode( NewValue )
						XmlWriteSub = True
					End If
				Else
					If IsEmpty( NewValue ) or NewValue = "" Then
						node.removeChild  text
						XmlWriteSub = True
					Else
						If text.nodeValue = NewValue Then
							XmlWriteSub = False
						Else
							text.nodeValue = NewValue
							XmlWriteSub = True
						End If
					End If
				End If

				'//=== remove parent, if IsEmpty( NewValue )
				If IsEmpty( NewValue ) Then

					Dim  deleting_depth,  current_depth
					xpath2 = GetXPath( BaseXmlElement, Array( ) )
					xpath2 = GetFullPath( XPath, xpath2 )
					deleting_depth = StrCount( xpath2, "/", 1, c.CaseSensitive )

					xpath2 = GetXPath( node, Array( ) )
					current_depth = StrCount( xpath2, "/", 1, c.CaseSensitive )

					Do
						If deleting_depth > current_depth Then  Exit Do
						Set text = node
						Set node = node.parentNode
						If node is Nothing Then  Exit Do
						If text.hasChildNodes Then  Exit Do

						If is_space_preserve Then
							If not IsNull( text.nextSibling ) Then
								Set sib = text.nextSibling
								If sib.nodeType = c.NODE_TEXT Then
									If Trim2( sib.nodeValue ) = "" Then
										node.removeChild  sib
									End If
								End If
							End If
						End If

						node.removeChild  text
						XmlWriteSub = True
						current_depth = current_depth - 1
					Loop
				End If
			End If
		Next
	End If
End Function


 
'*************************************************************************
'  <<< [XmlInsertAfter] >>> 
'*************************************************************************
Sub  XmlInsertAfter( in_ReferenceNode, in_NewNode )
	in_ReferenceNode.parentNode.insertBefore _
		in_NewNode, in_ReferenceNode.nextSibling
End Sub


'*************************************************************************
'  <<< [XmlRead] >>> 
'*************************************************************************
Function  XmlRead( BaseXmlElement, XPath )
	Dim  node
	Dim  c : Set c = g_VBS_Lib

	If IsEmpty( XPath ) Then
		Set node = BaseXmlElement
	Else
		Set node = BaseXmlElement.selectSingleNode( XPath )
		If node Is Nothing Then  Exit Function
	End If

	Select Case  node.nodeType
		Case  c.NODE_ATTRIBUTE
			XmlRead = node.value
		Case  c.NODE_ELEMENT
			Set node = node.selectSingleNode( "text()" )
			If not node is Nothing Then  XmlRead = node.nodeValue
		Case  c.NODE_TEXT
			XmlRead = node.nodeValue
	End Select
End Function


 
'*************************************************************************
'  <<< [XmlSelect] >>> 
'*************************************************************************
Function  XmlSelect( BaseXmlElement, XPath )
	Dim  slash_pos, kakko_pos, equal_pos, attribute_name, value
	Const  E_NoObject = 424

	Set XmlSelect = BaseXmlElement.selectSingleNode( XPath )
	If XmlSelect is Nothing Then

		'// Parse "element[@attribute_name='value']"
		slash_pos = InStrRev( XPath, "/" )
		kakko_pos = InStr( slash_pos + 1, XPath, "[" )
		If kakko_pos > 0 Then
			Assert  Mid( XPath, kakko_pos + 1, 1 ) = "@"
			equal_pos = InStr( kakko_pos + 2, XPath, "=" )

			attribute_name = Mid( XPath, kakko_pos + 2,  equal_pos - kakko_pos - 2 )

			value = Mid( XPath, equal_pos + 1 )
			Select Case  Left( value, 1 )
			 Case  """"
				Assert  Right( value, 2 ) = """]"
				value = Mid( value, 2, Len( value ) - 3 )
			 Case  "'"
				Assert  Right( value, 2 ) = "']"
				value = Mid( value, 2, Len( value ) - 3 )
			End Select

			XmlWriteSub  BaseXmlElement, Left( XPath, kakko_pos - 1 ) +"/@"+ attribute_name, _
				value, False
			Set XmlSelect = BaseXmlElement.selectSingleNode( XPath )
		Else
			XmlWriteSub  BaseXmlElement, XPath +"/@dummy", "dummy", False
			Set XmlSelect = BaseXmlElement.selectSingleNode( XPath )
			XmlSelect.removeAttribute  "dummy"
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [XmlSort] >>> 
'*************************************************************************
Sub  XmlSort( in_BaseXmlElement, in_XPath, in_CompareFunc, in_CompareFuncParam )
	Const  I_XMLDOMElement = 1
	Const  I_XMLDOMText = 3

	Set sorting_tags = in_BaseXmlElement.selectNodes( in_XPath )
	If sorting_tags.length <= 1 Then  Exit Sub
	Set compare_parameters = new_JoinedClass( in_CompareFunc, in_CompareFuncParam )


	'// Set "last_of_groups"
	Set last_of_groups = new ArrayClass
	For  tag_num = 0  To  sorting_tags.length - 2
		Set next_tag = sorting_tags( tag_num ).nextSibling
		If not next_tag is Nothing Then
			If next_tag.nodeType = I_XMLDOMText Then _
				Set next_tag = next_tag.nextSibling
		End If

		If next_tag is Nothing Then
			last_of_groups.Add  tag_num
		Else
			If next_tag.nodeType = I_XMLDOMElement Then
				If next_tag.tagName <> sorting_tags(0).tagName Then _
					last_of_groups.Add  tag_num
			End If
		End If
		tag_num = tag_num + 1
	Next
	last_of_groups.Add  sorting_tags.length - 1


	left_num = 0
	For Each  right_num  In  last_of_groups.Items  '// Loop of groups
		Set parent_node = sorting_tags( left_num ).parentNode


		'// Set "next_node"
		If right_num = sorting_tags.length Then
			next_node = Empty
		Else
			Set next_node = sorting_tags( right_num ).nextSibling
			If next_node is Nothing Then
				next_node = Empty
			Else
				If next_node.nodeType = I_XMLDOMText Then _
					Set next_node = next_node.nextSibling
			End If
		End If


		'// Set "tags"
		'// Remove sorting tags
		ReDim  tags( right_num - left_num )
		For  tag_num = right_num  To  left_num  Step -1
			Set tag_set = new JoinedClass
			Set tag_set.Left = sorting_tags( tag_num )
			Set tag_set.Right = tag_set.Left.nextSibling
			If not tag_set.Right is Nothing Then
				If tag_set.Right.nodeType <> I_XMLDOMText Then _
					Set tag_set.Right = Nothing
			End If

			Set tags( tag_num - left_num ) = tag_set

			parent_node.removeChild  tag_set.Left
			If not tag_set.Right is Nothing Then
				parent_node.removeChild  tag_set.Right
			End If
		Next


		'// ...
		QuickSort  tags, 0, UBound( tags ), GetRef( "XmlSortSub" ), compare_parameters


		'// Add sorted tags
		If IsEmpty( next_node ) Then
			For Each  tag_set  In tags
				parent_node.appendChild  tag_set.Left
				If not tag_set.Right is Nothing Then
					parent_node.appendChild  tag_set.Right
				End If
			Next
		Else
			For Each  tag_set  In tags
				parent_node.insertBefore  tag_set.Left, next_node
				If not tag_set.Right is Nothing Then
					parent_node.insertBefore  tag_set.Right, next_node
				End If
			Next
		End If


		'// Next
		left_num = right_num + 1
	Next
End Sub

Function  XmlSortSub( in_LeftTagSet, in_RightTagSet, in_CompareParameters )
	XmlSortSub = in_CompareParameters.Left( _
		in_LeftTagSet.Left,  in_RightTagSet.Left,  in_CompareParameters.Right )
End Function


 
'*************************************************************************
'  <<< [GetXPath] >>> 
'*************************************************************************
Function  GetXPath( ByVal DOMObject, ByVal ShowAttrs )
	Dim  attr, value, attr_length, ss
	Dim  c : Set c = g_VBS_Lib
	If IsEmpty( ShowAttrs ) Then  ShowAttrs = Array( "id", "name" )
	If not IsArray( ShowAttrs ) Then  ShowAttrs = Array( ShowAttrs )

	While  not DOMObject is Nothing
		Select Case  DOMObject.nodeType
			Case  c.NODE_ELEMENT

				'//=== GetXPath add attribute
				attr_length = 0
				ss = ""
				For Each attr  In ShowAttrs
					value = DOMObject.getAttribute( attr )
					If not IsNull( value ) Then
						If attr_length = 0 Then
							ss = "["
						Else
							ss = ss + " and "
						End IF
						ss = ss + "@"+ attr +"='"+ value +"'"
						attr_length = attr_length + 1
					End If
				Next
				If attr_length > 0 Then  ss = ss + "]"

				'//=== GetXPath add tagName
				GetXPath = "/"+ DOMObject.tagName + ss + GetXPath
				Set DOMObject = DOMObject.parentNode

			Case  c.NODE_DOCUMENT
				Exit Function

			Case  c.NODE_ATTRIBUTE
				GetXPath = "@"+ DOMObject.name
				Exit Function

		End Select
	WEnd
End Function


 
'*************************************************************************
'  <<< [ChangeToXml] >>> 
'*************************************************************************
Function  ChangeToXml( Stream )
	Select Case  TypeName( Stream )
		Case "ParentChildProcess" : Set ChangeToXml = Stream.m_InXML
		Case "String"             : Set ChangeToXml = LoadXML( Stream, g_VBS_Lib.StringData )
		Case Else                 : Set ChangeToXml = Stream
			'// Else = Nothing, IXMLDOMElement
	End Select
End Function


 
'*************************************************************************
'  <<< [XML_ReadCacheClass] >>> 
'*************************************************************************
Class  XML_ReadCacheClass
	Public Default Property Get  Item( a_URL )
		Const NODE_ELEMENT = 1
		Const NODE_ATTRIBUTE = 2

		Me.Get_Sub  a_URL, root, fragment  '//(out) root, fragment

		CutLastOf  fragment, "/", Empty
		Set element = root.selectSingleNode( fragment )
		If element is Nothing Then  Me.NotFoundError  a_URL
		If element.nodeType = NODE_ELEMENT Then
			Set text_object = element.selectSingleNode( "./text()")
			If text_object is Nothing Then  Me.NotFoundError  a_URL
			Item = text_object.text
		Else
			If element.nodeType <> NODE_ATTRIBUTE Then  Me.NotFoundError  a_URL
			Item = element.value
		End If
	End Property

	Sub Get_Sub( a_URL, root, fragment )
		number_position = InStr( a_URL, "#" )
		If number_position = 0 Then  Raise  1, "<ERROR msg=""No #"" URL="""+ a_URL +"""/>"
		full_path = GetFullPath( Left( a_URL, number_position - 1 ), Empty )
		fragment = Mid( a_URL, number_position + 1 )


		'// Set "root"
		If g_VBS_Lib.XML_DOM_ReadCache.Exists( full_path ) Then
			Set root = g_VBS_Lib.XML_DOM_ReadCache( full_path )
		Else
			If not g_fs.FileExists( full_path ) Then  Me.NotFoundError  a_URL
			Set root = LoadXML( full_path, Empty )
			Set g_VBS_Lib.XML_DOM_ReadCache( full_path ) = root
		End If
	End Sub


	Function GetArray( a_URL )
		Me.Get_Sub  a_URL, root, fragment  '//(out) root, fragment

		CutLastOf  fragment, "/", Empty
		Set nodes = root.selectNodes( fragment )
		ReDim  return_array( nodes.length - 1 )
		i = 0
		For Each  element  In nodes
			Set text_object = element.selectSingleNode( "./text()")
			If text_object is Nothing Then  Me.NotFoundError  a_URL
			return_array(i) = text_object.text
			i = i + 1
		Next
		GetArray = return_array
	End Function


	Sub  NotFoundError( a_URL )
		Raise  E_FileNotExist, "<ERROR msg=""Not found"" URL="""+ a_URL +"""/>"
	End Sub
End Class


 
'*************************************************************************
'  <<< [MultiTextXML_Class] >>> 
'*************************************************************************
Class  MultiTextXML_Class
	Function  GetText( a_URL )
		Set jumps = GetTagJumpParams( a_URL )
		full_path = GetFullPath( jumps.Path, Empty )


		'// Case of without "#" in URL
		If IsEmpty( jumps.Keyword ) Then
			If g_VBS_Lib.TextReadCache.Exists( full_path ) Then
				GetText = g_VBS_Lib.TextReadCache( full_path )
			Else
				If not g_fs.FileExists( full_path ) Then
					Raise  E_FileNotExist, "<ERROR msg=""Not found"" URL="""+_
						full_path +"""/>"
				End If
				GetText = ReadFile( full_path )
				g_VBS_Lib.TextReadCache( full_path ) = GetText
			End If

		'// Case of with "#" in URL
		Else
			'// Set "root"
			If g_VBS_Lib.XML_DOM_ReadCache.Exists( full_path ) Then
				Set root = g_VBS_Lib.XML_DOM_ReadCache( full_path )
			Else
				If not g_fs.FileExists( full_path ) Then
					Raise  E_FileNotExist, "<ERROR msg=""Not found"" URL="""+ full_path +_
						"#"+ jumps.Keyword +"""/>"
				End If
				Set root = LoadXML( full_path, Empty )
				Set g_VBS_Lib.XML_DOM_ReadCache( full_path ) = root
			End If


			'// Set "text"
			Set text_object = root.selectSingleNode("./Text[@id='"+ jumps.Keyword +"']/text()")
			If text_object is Nothing Then
				Raise  E_FileNotExist, "<ERROR msg=""Not found"" URL="""+ full_path +_
					"#"+ jumps.Keyword +"""/>"
			End If
			text = text_object.text


			'// Modify "text"
			left_of_text = Left( text, 5 )
			If left_of_text = vbLF +"----" Then
				text = Mid( text, InStr( 6, text, vbLF ) + 1 )
			ElseIf Left( left_of_text, 1 ) = vbLF Then
				text = Mid( text, 2 )
			ElseIf Left( left_of_text, 4 ) = "----" Then
				text = Mid( text, InStr( 5, text, vbLF ) + 1 )
			End If

			right_of_text = Right( text, 5 )
			text_length = Len( text )
			If right_of_text = "----"+ vbLF Then
				text = Left( text, InStrRev( text, vbLF, text_length - 5 ) )
			ElseIf Right( right_of_text, 4 ) = "----" Then
				text = Left( text, InStrRev( text, vbLF, text_length - 4 ) )
			End If

			GetText = Replace( text, vbLF, vbCRLF )
		End If
	End Function

	Function  IsExist( a_URL )
		Set jumps = GetTagJumpParams( a_URL )
		full_path = GetFullPath( jumps.Path, Empty )

		'// Case of without "#" in URL
		If IsEmpty( jumps.Keyword ) Then
			IsExist = g_fs.FileExists( full_path )

		'// Case of with "#" in URL
		Else
			If g_VBS_Lib.XML_DOM_ReadCache.Exists( full_path ) Then
				Set root = g_VBS_Lib.XML_DOM_ReadCache( full_path )
			Else
				IsExist = g_fs.FileExists( full_path )
				If IsExist Then
					Set root = LoadXML( full_path, Empty )
					Set g_VBS_Lib.XML_DOM_ReadCache( full_path ) = root
				End If
			End If
			If not IsEmpty( root ) Then
				Set text_object = root.selectSingleNode("./Text[@id='"+ jumps.Keyword +"']/text()")
				IsExist = not ( text_object is Nothing )
			End If
		End If
	End Function
End Class


 
'*************************************************************************
'  <<< [GetHRefBase] >>> 
'*************************************************************************
Function  GetHRefBase( Path, TargetTagNames )
	Dim  o : Set o = new HRefBase
	Set o.Linker = new LinkedXMLs
	If IsArray( TargetTagNames ) Then
		o.Linker.XmlTagNamesHavingIdName = TargetTagNames
	Else
		o.Linker.XmlTagNamesHavingIdName = Array( TargetTagNames )
	End If
	o.Linker.StartNavigation  Path, LoadXML( Path, Empty )
	Set GetHRefBase = o
End Function


Class  HRefBase
	Public  Linker  '// as LinkedXMLs

	Function  href( Path )
		Set href = Me.Linker.GetLinkTargetNode( Path )
	End Function

	Private Sub  Class_Terminate()
		Me.Linker.EndNavigation
	End Sub
End Class




 
'*-------------------------------------------------------------------------*
'* ### <<<< [LinkedXMLs] Class >>>> 
'*-------------------------------------------------------------------------*

Class  LinkedXMLs
	Public  XmlTagNamesHavingIdName  '// as array of string
	Public  Roots    '// as dictionary of root IXMLDOMElement cache. key is abs path
	Public  SourceXmlPath      '// as string
	Public  SourceXmlRootElem  '// as IXMLDOMElement
	Public  TargetLocation     '// as string
	Public  TargetXmlPath      '// as string
	Public  TargetXmlRootElem  '// as IXMLDOMElement

	Public  DicOfVisitedLocation      '// as dictionary of True. key is Location(path+"#"+name)
	Public  StackOfVisitedLocation    '// as ArrayClass of string
	Public  StackOfSourceXmlPath      '// as ArrayClass of string
	Public  StackOfSourceXmlRootElem  '// as ArrayClass of IXMLDOMElement

	Private Sub  Class_Initialize()
		Set Me.Roots = CreateObject( "Scripting.Dictionary" )
		Set Me.DicOfVisitedLocation     = CreateObject( "Scripting.Dictionary" )
		Set Me.StackOfVisitedLocation   = new ArrayClass
		Set Me.StackOfSourceXmlPath     = new ArrayClass
		Set Me.StackOfSourceXmlRootElem = new ArrayClass
	End Sub

	Public Sub  StartNavigation( in_SourceLocation, in_SourceXmlRootElem )
		Set source_jump = GetTagJumpParams( in_SourceLocation )
		Me.StackOfSourceXmlPath.Push  Me.SourceXmlPath
		Me.StackOfSourceXmlRootElem.Push  in_SourceXmlRootElem

		Me.SourceXmlPath = source_jump.Path
		Set Me.SourceXmlRootElem = in_SourceXmlRootElem
		Me.DicOfVisitedLocation.Add  in_SourceLocation,  True  '// Check mutual link
		Me.StackOfVisitedLocation.Push  in_SourceLocation
	End Sub

	Public Sub  EndNavigation()
		Dim  location : location = Me.StackOfVisitedLocation.Pop()
		Me.DicOfVisitedLocation.Remove  location

		Me.StackOfSourceXmlPath.Pop
		Me.StackOfSourceXmlRootElem.Pop
	End Sub


 
'*************************************************************************
'  <<< [LinkedXMLs::GetLinkTargetNode] >>> 
'*************************************************************************
Function  GetLinkTargetNode( in_TargetLocation )
	Dim  node, jump, name, e, err_msg, elem_name

	If IsEmpty( Me.XmlTagNamesHavingIdName ) Then _
		Raise  1, "XmlTagNamesHavingIdName is not specified."
	Assert  not IsEmpty( in_TargetLocation )
	Assert  not IsNull( in_TargetLocation )


	'//=== Me.TargetXmlRootElem: LoadXML( in_TargetLocation )
	'//=== name: in_TargetLocation after #
	If Left( in_TargetLocation, 1 ) = "#" Then
		Set Me.TargetXmlRootElem = Me.SourceXmlRootElem
		Me.TargetXmlPath = Me.SourceXmlPath
		name = Mid( in_TargetLocation, 2 )
		Me.TargetLocation = Me.SourceXmlPath +"#"+ name
	Else
		Set jump = GetTagJumpParams( in_TargetLocation )
		Me.TargetXmlPath = GetFullPath( jump.Path, GetParentFullPath( Me.SourceXmlPath ) )
		name = jump.Keyword : If IsEmpty( name ) Then  name = ""

		If Me.Roots.Exists( Me.TargetXmlPath ) Then
			Set Me.TargetXmlRootElem = Me.Roots.Item( Me.TargetXmlPath )
		Else
			If TryStart(e) Then  On Error Resume Next
				Set Me.TargetXmlRootElem = LoadXML( Me.TargetXmlPath, Empty )
				If Trying Then  Set Me.Roots.Item( Me.TargetXmlPath ) = Me.TargetXmlRootElem
			If TryEnd Then  On Error GoTo 0
			If e.num <> 0  Then  err_msg = e.desc : e.Clear
		End If

		If name = "" Then
			Me.TargetLocation = Me.TargetXmlPath
		Else
			Me.TargetLocation = Me.TargetXmlPath +"#"+ name
		End If
	End If


	'// //
	If name = "" Then  err_msg = "Link target name is not specified or # is not written"
	If not IsEmpty( err_msg ) Then
		Raise_sub  E_NotFoundSymbol, err_msg, in_TargetLocation
	End If


	'//=== //
	For Each elem_name  In Me.XmlTagNamesHavingIdName
		Set node = Me.TargetXmlRootElem.selectSingleNode( "//"+ elem_name +"[@id='"+ name +"']" )
		If node is Nothing Then  Set node = Me.TargetXmlRootElem.selectSingleNode( "//"+ elem_name +"[@name='"+ name +"']" )
		If not node is Nothing Then  Exit For
	Next
	If node is Nothing Then
		Raise_sub  E_NotFoundSymbol, "Link target is not found", in_TargetLocation
	End If


	'// Raise error, when mutual link
	If not IsEmpty( Me.DicOfVisitedLocation ) Then
		If Me.DicOfVisitedLocation.Exists( Me.TargetXmlPath +"#"+ name ) Then
			Raise_sub  E_NotFoundSymbol, "Operation can not be completed because mutual link", in_TargetLocation
		End If
	End If

	Set GetLinkTargetNode = node
End Function


 
'*************************************************************************
'  <<< [LinkedXMLs::Raise_sub] >>> 
'*************************************************************************
Private Sub  Raise_sub( ErrCode, ErrMessage, TargetLocation )
	'// When link navigation was started, Call Me.StartNavigation.
	Raise  ErrCode, "<ERROR msg="""+ ErrMessage +""" "+_
		"target_locaion="""+ XmlAttr( TargetLocation ) +""" from="""+ XmlAttr( Me.SourceXmlPath ) +_
		"""/>"
End Sub


 
End Class 
 
'*-------------------------------------------------------------------------*
'* ### <<<< Function call and include >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [call_vbs] >>> 
'*************************************************************************
Function  call_vbs( path, func, param )
	echo  ">call_vbs  """ & path & """, " & func
	call_vbs = call_vbs_t( path, func, param )
End Function


 
'*************************************************************************
'  <<< [SetupCallViaFile] >>> 
'*************************************************************************
Sub  SetupCallViaFile( SyncSendFilePath, SyncRecvFilePath, IsSyncWhenExit )
	If IsEmpty( g_CallViaFileSystem ) Then  Set g_CallViaFileSystem = new CallViaFileSystem
	Dim  o : Set o = g_CallViaFileSystem

	o.SyncSendFilePath = SyncSendFilePath
	o.SyncRecvFilePath = SyncRecvFilePath
	If IsEmpty( o.SyncSendFilePath ) or IsEmpty( o.SyncRecvFilePath ) Then
		Dim params : Set params = DicFromCmdLineOpt( GetCmdLine(), Array( "/SyncSendFile", "/SyncRecvFile" ) )
		If IsEmpty( o.SyncSendFilePath ) Then  o.SyncSendFilePath = params( "/SyncSendFile" )
		If IsEmpty( o.SyncRecvFilePath ) Then  o.SyncRecvFilePath = params( "/SyncRecvFile" )
	End If

	echo  ">SetupCallViaFile  """+ o.SyncSendFilePath +""", """+ o.SyncRecvFilePath +""""
	o.IsSyncWhenExit = IsSyncWhenExit
End Sub


Dim  g_CallViaFileSystem
Class  CallViaFileSystem
	Public  SyncSendFilePath
	Public  SyncRecvFilePath
	Public  IsSyncWhenExit

	Private Sub  Class_Terminate()
		If Me.IsSyncWhenExit Then
			Dim f : Set f = g_fs.CreateTextFile( Me.SyncSendFilePath, True, False )
			f.Write  "Finished"
			f.Close
		End If
	End Sub
End Class

 
'*************************************************************************
'  <<< [CallViaFile] >>> 
'*************************************************************************
Function  CallViaFile( ByVal Message )
	If IsEmpty( g_CallViaFileSystem ) Then  Raise  1, "Call SetupCallViaFile."
	Dim  o : Set o = g_CallViaFileSystem
	If IsEmpty( Message ) Then  Message = "Sync"
	echo  ">CallViaFile  """ & Message & """"
	Dim  ec : Set ec = new EchoOff
	CreateFile  o.SyncSendFilePath, Message
	CallViaFile = WaitForFile( o.SyncRecvFilePath )
End Function


 
'*************************************************************************
'  <<< [CallForEach0] >>> 
'*************************************************************************
Function  CallForEach0( Func, Collection )
	Dim  item

	If IsEmpty( Collection ) Then  Exit Function
	If IsArray( Collection ) Then
		For Each item  In Collection
			Func  item
		Next
	Else  '// Collection is single variable
		CallForEach1 = Func( Collection )
	End If
End Function


Function  CallForEach1( Func, Collection, Param1 )
	Dim  item

	If IsEmpty( Collection ) Then  Exit Function
	If IsArray( Collection ) Then
		For Each item  In Collection
			Func  item, Param1
		Next
	Else  '// Collection is single variable
		CallForEach1 = Func( Collection, Param1 )
	End If
End Function


Function  CallForEach2( Func, Collection, Param1, Param2 )
	Dim  item

	If IsEmpty( Collection ) Then  Exit Function
	If IsArray( Collection ) Then
		For Each item  In Collection
			Func  item, Param1, Param2
		Next
	Else  '// Collection is single variable
		CallForEach2 = Func( Collection, Param1, Param2 )
	End If
End Function


Function  CallForEach3( Func, Collection, Param1, Param2, Param3 )
	Dim  item

	If IsEmpty( Collection ) Then  Exit Function
	If IsArray( Collection ) Then
		For Each item  In Collection
			Func  item, Param1, Param2, Param3
		Next
	Else  '// Collection is single variable
		CallForEach2 = Func( Collection, Param1, Param2, Param3 )
	End If
End Function


Function  CallForEach4( Func, Collection, Param1, Param2, Param3, Param4 )
	Dim  item

	If IsEmpty( Collection ) Then  Exit Function
	If IsArray( Collection ) Then
		For Each item  In Collection
			Func  item, Param1, Param2, Param3, Param4
		Next
	Else  '// Collection is single variable
		CallForEach2 = Func( Collection, Param1, Param2, Param3, Param4 )
	End If
End Function


Sub  CallForEach_copy( FilePath, FromFolderPath, ToFolderPath )
	If VarType( FilePath ) <> vbString Then _
		Raise  13, "<ERROR msg=""File name is not string type"" type="""+ TypeName( FilePath ) +"""/>"

	If FilePath = "" Then  Exit Sub
	copy  GetFullPath( FilePath, GetFullPath( FromFolderPath, Empty ) ), _
		g_fs.GetParentFolderName( GetFullPath( FilePath, GetFullPath( ToFolderPath, Empty ) ) )
End Sub

Sub  CallForEach_move( FilePath, FromFolderPath, ToFolderPath )
	If VarType( FilePath ) <> vbString Then _
		Raise  13, "<ERROR msg=""File name is not string type"" type="""+ TypeName( FilePath ) +"""/>"

	If FilePath = "" Then  Exit Sub
	move  GetFullPath( FilePath, GetFullPath( FromFolderPath, Empty ) ), _
		g_fs.GetParentFolderName( GetFullPath( FilePath, GetFullPath( ToFolderPath, Empty ) ) )
End Sub

Sub  CallForEach_del( FilePath, FolderPath )
	If FilePath = "" Then  Exit Sub
	del  GetFullPath( FilePath, GetFullPath( FolderPath, Empty ) )
End Sub

Sub  CallForEach_AssertExist( FilePath, FolderPath )
	If FilePath = "" Then  Exit Sub
	AssertExist  GetFullPath( FilePath, GetFullPath( FolderPath, Empty ) )
End Sub

Sub  CallForEach_AssertNotExist( FilePath, FolderPath )
	If FilePath = "" Then  Exit Sub
	Assert  not exist( GetFullPath( FilePath, GetFullPath( FolderPath, Empty ) ) )
End Sub


 
'-------------------------------------------------------------------------
' ### <<<< [MakeFileClass] Class >>>> 
'-------------------------------------------------------------------------
Class  MakeFileClass
	Public  Name         ' as string
	Public  RuleGroups   ' as Dictionary of MakeRuleGroup. key=MakeRuleGroup::TargetPath and MakeRule::Sources
	Public  Variables    ' as LazyDictionaryClass
	Public  Delegate     ' as variant
	Public  DebugMode    ' as integer. c.OutDebugXML, c.BreakBeforeCommand( FileName ), c.Watch( Function )
	Public  DebugMode_Param1
	Public  c              ' as MakeFileClassConst

	Private Sub  Class_Initialize()
		Set Me.RuleGroups = CreateObject( "Scripting.Dictionary" )
		Me.RuleGroups.CompareMode = 1  '// NotCaseSensitive
		Set Me.Variables = new LazyDictionaryClass
		Set c = get_MakeFileClassConst()
	End Sub


 
'*************************************************************************
'  <<< [MakeFileClass::AddRule] >>> 
'*************************************************************************
Public Sub  AddRule( a_MakeRule )
	If IsArray( a_MakeRule ) Then
		For Each  rule  In a_MakeRule
			Me.AddRule  rule
		Next
		Exit Sub
	End If

	target_path = a_MakeRule.Target


	'//=== Add new "MakeRuleGroup" to "Me.RuleGroups"
	GetDicItem  Me.RuleGroups, target_path, group
	If IsEmpty( group ) Then
		Set o = new MakeRuleGroup
			o.TargetPath = target_path
			Set o.MakeFile = Me
		Set group = o
		Set Me.RuleGroups.Item( target_path ) = group
	End If


	'//=== Add new "MakeRuleSpec" to "group.Rules"
	Set o = new MakeRuleSpec
		Set o.Group = group
		Set o.Rule = a_MakeRule
		Set o.CommandFunc = a_MakeRule.Command
	Set rule = o
	Set a_MakeRule.Spec = rule
	group.Rules.Add  rule


	'//=== Add "src_group" to "rule.SourceFiles"
	For i_src=0 To UBound( a_MakeRule.Sources )
		src_path = a_MakeRule.Sources( i_src )
		GetDicItem  Me.RuleGroups, src_path, src_group  '//[out] src_group

		'//=== add source for timestamp
		If IsEmpty( src_group ) Then
			Set o = new MakeRuleGroup
				o.TargetPath = src_path
				Set o.MakeFile = Me
			Set src_group = o
			Set Me.RuleGroups( src_path ) = src_group
		End If


		'//=== mutual link source to target
		rule.SourceFiles.Add  src_group
		src_group.UserFiles.Add  rule
	Next
End Sub


 
'*************************************************************************
'  <<< [MakeFileClass::Make] >>> 
'*************************************************************************
Public Sub  Make()
	Set ds_= new CurDirStack

	echo  ">make"

	Set rules = new ArrayClass
	is_implicit = Me.GetIsImplicit()


	If Me.DebugMode and Me.c.OutDebugXML Then
		path = GetTempPath( "MakeFileDebug.xml" )
		Dim  f : Set f = OpenForWrite( path, g_VBS_Lib.Unicode )
		f.WriteLine  "<?xml version=""1.0"" encoding=""utf-16""?>"
		f.WriteLine  "<MakeFileClass current_folder="""+ g_sh.CurrentDirectory +""">"
	End If


	'//=== Expand wildcard (Implicit rules)
	If is_implicit Then
		expand_option = g_VBS_Lib.File  or  g_VBS_Lib.Folder  or  g_VBS_Lib.SubFolder

		Set rule_groups_back_up = Me.RuleGroups
		Set Me.RuleGroups = CreateObject( "Scripting.Dictionary" )
		For Each  target_path  In rule_groups_back_up.Keys
			Set group = rule_groups_back_up( target_path )
			If IsWildcard( target_path ) Then
				target_path_ex = Me.Variables( target_path )
				For Each rule_spec  In group.Rules.Items
					Set rule = rule_spec.Rule
					If UBound( rule.Sources ) >= 0 Then
						source_path = Me.Variables( rule.Sources(0) )
						ExpandWildcard  source_path, expand_option, folder, step_paths
						folder = GetStepPath( folder, Empty )
						For Each  step_path  In step_paths
							step_path_no_ext = ScanFromTemplate( step_path, _
								g_fs.GetFileName( source_path ), Array( "*" ), Empty )("*")
							Set o = new MakeRule
								o.Target = Replace( target_path_ex, "*", step_path_no_ext )
								o.Sources = rule.Sources
								o.Sources(0) = folder +"\"+ step_path
								Set o.Command = rule.Command
							Me.AddRule  o
						Next
						If UBound( step_paths ) = -1 Then
							If Me.DebugMode and Me.c.OutDebugXML Then
								f.WriteLine  "<MakeRuleNotMatch source_path="""+ _
									source_path +"""/>"
							End If
						End If
					End If
				Next
			Else
				Me.RuleGroups( target_path ) = group
			End If
		Next
	End If


	'//=== Set "Distance"
	For Each group  In Me.RuleGroups.Items  '// group as MakeRuleGroup
		If group.UserFiles.Count = 0 Then
			group.Distance = 0
			For Each rule  In group.Rules.Items
				For Each src  In rule.SourceFiles.Items
					src.Distance = 1
				Next
			Next
		End If
	Next
	dist = 1
	Do
		b = False
		For Each group  In RuleGroups.Items
			If group.Distance = dist Then
				For Each rule  In group.Rules.Items
					For Each src  In rule.SourceFiles.Items
						src.Distance = dist + 1
						b = True
					Next
				Next
			End If
		Next
		If not b Then  Exit Do
		dist = dist + 1
	Loop


	'//=== Scan timestamp
	For Each group  In Me.RuleGroups.Items
		If exist( group.TargetPath ) Then
			group.TargetTimeStamp = g_fs.GetFile( group.TargetPath ).DateLastModified
		Else
			group.TargetTimeStamp = Empty
		End If
	Next


	'//=== Compare timestamp
	For Each group  In RuleGroups.Items
		For Each rule  In group.Rules.Items
			rule.bDoCommand = False
			rule.Reason = Empty

			b=( not IsEmpty( rule.Rule ) )  'and
			If b Then b=( rule.Rule.Type_ <> group.c.Splitter )
			If b Then
				For Each src  In rule.SourceFiles.Items

					If IsEmpty( src.TargetTimeStamp ) Then
						'// Do Nothing
					ElseIf IsEmpty( group.TargetTimeStamp ) Then
						rule.bDoCommand = True
						rule.Reason = "No Target"
						rules.Add  rule
					ElseIf group.TargetTimeStamp < src.TargetTimeStamp Then
						rule.bDoCommand = True
						rule.Reason = "Timestamp"
						rules.Add  rule
					End If
				Next
				If rule.SourceFiles.Count = 0  and  not IsEmpty( rule.Rule ) Then
					If IsEmpty( rule.Group.TargetTimeStamp ) Then
						rule.bDoCommand = True
						rule.Reason = "NotExist"
						rules.Add  rule
					End If
				End If
			Else
				rule.Reason = "Splitter"
			End If
		Next
	Next


	'//=== Resolve dependency
	i_rule = 0
	While  i_rule < rules.Count
		For Each user  In rules( i_rule ).Group.UserFiles.Items  '// user as MakeRuleSpec
			If not user.bDoCommand Then
				user.bDoCommand = True
				user.Reason = "Dependency"
				rules.Add  user  '// Add to last. Loop count +1
			End If
		Next
		i_rule = i_rule + 1
	WEnd


	'//=== Do biggest priority command only
	For Each group  In RuleGroups.Items
		rule2 = Empty
		For Each rule  In group.Rules.Items
			If rule.bDoCommand  and  not IsEmpty( rule.Rule ) Then
				If not IsEmpty( rule2 ) Then
					If rule.Rule.Priority > rule2.Rule.Priority Then
						rule2.bDoCommand = False
						rule2.Reason = rule2.Reason + ", but Priority"
					Else
						rule.bDoCommand = False
						rule.Reason = rule.Reason + ", but Priority"
					End If
				End If
				Set rule2 = rule
			End If
		Next
	Next


	'//=== Output debug xml
	If Me.DebugMode and Me.c.OutDebugXML Then
		f.WriteLine  "<MakeFile name="""+ Me.Name +""" max_distance="""& dist &""">"
		i_group = 0
		For Each group  In Me.RuleGroups.Items
			If group.Rules.Count > 0 Then

				b_do_command = False : do_priority = Empty
				For Each rule  In group.Rules.Items
					If rule.bDoCommand Then _
						b_do_command = True : do_priority = rule.Rule.Priority
				Next

				f.WriteLine  "<MakeRuleGroup id="""& i_group &""" file_name="""+ g_fs.GetFileName( group.TargetPath ) +""" distance="""&_
					group.Distance &""" b_do_command="""& b_do_command &""" do_priority="""& do_priority &""">"
				f.WriteLine  "  <Target time_stamp="""& group.TargetTimeStamp &""" path="""+ group.TargetPath +"""/>"
				For Each rule  In group.Rules.Items
				 If not IsEmpty( rule.Rule ) Then
					f.WriteLine  "  <MakeRule b_do_command="""& rule.bDoCommand &""" reason="""+_
						rule.Reason +""" priority="""& rule.Rule.Priority &""">"
					i_rule = 0
					For Each src  In rule.SourceFiles.Items
						f.WriteLine  "    <Source time_stamp="""& src.TargetTimeStamp &""" path="""+ rule.Rule.Sources( i_rule ) +"""/>"
						i_rule = i_rule + 1
					Next
					f.WriteLine  "  </MakeRule>"
				 End If
				Next
				f.WriteLine  "</MakeRuleGroup>"
			End If
			i_group = i_group + 1
		Next
		f.WriteLine  "</MakeFileClass>"
		f = Empty
		start  path
		Pause
	End If


	'//=== Call the command
	While  dist >= 0
		For Each group  In Me.RuleGroups.Items
			If group.Distance = dist Then
				For Each rule  In group.Rules.Items
					If rule.bDoCommand Then
						Set ec = new EchoOff
						cd  rule.Rule.CurrentDirectory
						ec = Empty

						echo  ""
						echo  "make command for """+ rule.Rule.Target +""""

						Set rule.Rule.Variables = Me.Variables
						Me.Variables("${Target}") = rule.Rule.Target
						If UBound( rule.Rule.Sources ) = -1 Then
							Me.Variables("${Sources[0]}") = ""
						Else
							Me.Variables("${Sources[0]}") = rule.Rule.Sources(0)
						End If
						Me.Variables("${Sources}") = Replace( CmdLineFromStr( rule.Rule.Sources ), "\\", "\" )
						Me.Variables("${TargetNoExt}") = AddLastOfFileName( rule.Rule.Target, "." )

						If (Me.DebugMode and Me.c.Watch) and  not IsEmpty( Me.DebugMode_Param1 ) Then _
							Me.DebugMode_Param1  Me.Delegate, rule.Rule

						If (Me.DebugMode and Me.c.BreakBeforeCommand) Then _
							If ( IsEmpty( Me.DebugMode_Param1 ) or _
								InStr( rule.Rule.Target, Me.DebugMode_Param1 ) > 0 ) Then _
									Stop


						rule.CommandFunc  Me.Delegate, rule.Rule


						Me.Variables.Remove  "${Target}"
						Me.Variables.Remove  "${Sources[0]}"
						Me.Variables.Remove  "${Sources}"
						Me.Variables.Remove  "${TargetNoExt}"
						rule.Rule.Variables = Empty

						If (Me.DebugMode and Me.c.Watch) and  not IsEmpty( Me.DebugMode_Param1 ) Then _
							Me.DebugMode_Param1  Me.Delegate, rule.Rule
					End If
				Next
			End If
		Next
		dist = dist - 1
	WEnd


	'//=== Restore "Me.RuleGroups"
	If is_implicit Then
		Set Me.RuleGroups = rule_groups_back_up
	End If
End Sub


 
'*************************************************************************
'  <<< [MakeFileClass::GetIsImplicit] >>> 
'*************************************************************************
Function  GetIsImplicit()
	GetIsImplicit = False
	For Each group  In Me.RuleGroups.Items  '// group as MakeRuleGroup
		For Each rule_spec  In group.Rules.Items
			Set rule = rule_spec.Rule

			is_source_wildcard = False
			If UBound( rule.Sources ) >= 0 Then
				If IsWildcard( rule.Sources( 0 ) ) Then _
					is_source_wildcard = True
			End If

			If IsWildcard( rule.Target ) Then
				If is_source_wildcard Then
					GetIsImplicit = True
					Exit Function
				Else
					Error
				End If
			Else
				If is_source_wildcard Then _
					Error
			End If
		Next
	Next
End Function


 
End Class 


 
'-------------------------------------------------------------------------
' ### <<<< [MakeRule] Class >>>> 
'-------------------------------------------------------------------------
Class  MakeRule
	Public  Target     ' as string
	Public  Sources    ' as Array of string
	Public  Command    ' as Sub ()

	Public  Variables  ' as LazyDictionaryClass (Read Only)
	Public  CurrentDirectory  ' as string. abs path (Read Only)

	Public  Delegate ' as variant
	Public  Type_    ' as integer. 0 or Me.c.Splitter
	Public  Priority ' as number. Do biggest priority command only in same Target.

	Public  Spec     ' as MakeRuleSpec (Read Only)

	Public  c

	Private Sub  Class_Initialize()
		CurrentDirectory = g_sh.CurrentDirectory
		Set c = get_MakeFileClassConst()
		Priority = 1
	End Sub


	'//[MakeRule::NewestSource]
	Public Property Get  NewestSource()
		Dim  rule, newest_rule, i_src, i, rule_date, newest_date, b

		i = 0
		For Each rule  In Me.Spec.SourceFiles.Items
			If g_fs.FileExists( rule.TargetPath ) Then
				rule_date = g_fs.GetFile( rule.TargetPath ).DateLastModified

				b=( IsEmpty( newest_date ) )  'or
				If not b Then  b=( rule_date > newest_date )
				If b Then
					newest_date = rule_date
					i_src = i
				End If
			End If
			i = i + 1
		Next
		If not IsEmpty( i_src ) Then _
			NewestSource = Me.Sources( i_src )
	End Property


	'//[MakeRule::AllNewSource]
	Public Property Get  AllNewSource()  ' as Array
		Dim  group, i, srcs, target_date

		srcs = Array()
		If not g_fs.FileExists( Me.Spec.Group.TargetPath ) Then
			AllNewSource = Me.Sources
			Exit Property
		End If

		target_date = g_fs.GetFile( Me.Spec.Group.TargetPath ).DateLastModified
		i = 0
		For Each group  In Me.Spec.SourceFiles.Items
			If g_fs.FileExists( group.TargetPath ) Then
				If g_fs.GetFile( group.TargetPath ).DateLastModified > target_date Then
					ReDim Preserve  srcs( UBound( srcs ) + 1 )
					srcs( UBound( srcs ) ) = Me.Sources( i )
				End If
			End If
			i = i + 1
		Next
		AllNewSource = srcs
	End Property
End Class

 
'*************************************************************************
'  <<< [MakeRule_compare] >>> 
'*************************************************************************
Function  MakeRule_compare( TargetPath, SourcePath )
	If g_fs.FileExists( SourcePath ) Then
		If g_fs.FileExists( TargetPath ) Then
			MakeRule_compare = g_fs.GetFile( SourcePath ).DateLastModified > _
												 g_fs.GetFile( TargetPath ).DateLastModified
		Else
			MakeRule_compare = True
		End If
	Else
		MakeRule_compare = False
	End If
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [MakeRuleGroup] Class >>>> 
'-------------------------------------------------------------------------
Class  MakeRuleGroup
	Public  TargetPath       ' as string. abs path
	Public  TargetTimeStamp  ' as Date or Empty

	Public  Rules        ' as ArrayClass of MakeRuleSpec

	Public  Distance     ' as integer
	Public  UserFiles    ' as ArrayClass of MakeRuleSpec
	Public  MakeFile     ' as parnet of MakeFileClass

	Public  c

	Private Sub  Class_Initialize()
		Set Rules     = new ArrayClass
		Set UserFiles = new ArrayClass
		Set c = get_MakeFileClassConst()
	End Sub
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [MakeRuleSpec] Class >>>> 
'-------------------------------------------------------------------------
Class  MakeRuleSpec
	Public  Group        ' as parnet of MakeRuleGroup
	Public  Rule         ' as MakeRule
	Public  SourceFiles  ' as ArrayClass of MakeRuleGroup
	Public  CommandFunc  ' as Sub ( variant, MakeRuleSpec )
	Public  bDoCommand   ' as boolean
	Public  Reason       ' as string

	Private Sub  Class_Initialize()
		Set SourceFiles = new ArrayClass
	End Sub
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [MakeFileClassConst] Class >>>> 
'-------------------------------------------------------------------------
Dim  g_MakeFileClassConst

Function    get_MakeFileClassConst()  '// has_interface_of ClassI
	If IsEmpty( g_MakeFileClassConst ) Then _
		Set g_MakeFileClassConst = new MakeFileClassConst : ErrCheck
	Set get_MakeFileClassConst =   g_MakeFileClassConst
End Function

Class  MakeFileClassConst
	Public  Splitter, OutDebugXML, BreakBeforeCommand, Watch
	Private Sub  Class_Initialize() : Splitter = 1 : OutDebugXML = 1 : BreakBeforeCommand = 2
		Watch = 4 : End Sub
End Class


 
'*-------------------------------------------------------------------------*
'* ### <<<< Support of vbsool >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [CalculateVariables] >>> 
'*************************************************************************
Function  CalculateVariables( Formula, Variables )
	Dim  code,  key,  key2,  value,  prev_count
	Dim  not_out_vars : Set not_out_vars = CreateObject( "Scripting.Dictionary" )
	Dim  c : Set c = g_VBS_Lib

	If IsEmpty( Variables ) Then  Set Variables = CreateObject( "Scripting.Dictionary" )


	code = "Option Explicit" +vbCRLF+_
		 "Function  CalculateVariablesInside( Variables )" +vbCRLF


	'//=== Output value not referencing the variable
	For Each  key  In Variables.Keys
		LetSet  value, Variables.Item( key )  '//[out] value
		If IsObject( value ) Then
			code = code + "  Dim "+ key +" : Set "+ key +" = Variables.Item( """+ key +""" )"+ vbCRLF
		ElseIf IsNumeric( value ) Then
			code = code + "  Dim "+ key +" : "+ key +" = "& value & vbCRLF
		Else
			not_out_vars.Add  key, True
		End If
	Next


	'//=== Output value referencing the variable
	Do
		If not_out_vars.Count = 0 Then  Exit Do
		prev_count = not_out_vars.Count

		For Each  key  In not_out_vars.Keys
			LetSet  value, Variables.Item( key )  '//[out] value
			For Each  key2  In not_out_vars.Keys
				If InStrEx( value, key2, 1, c.WholeWord ) > 0 Then  Exit For
			Next
			If IsEmpty( key2 ) Then
				code = code + "  Dim "+ key +" : "+ key +" = "& value & vbCRLF
				not_out_vars.Remove  key
			End If
		Next

		If not_out_vars.Count = prev_count Then
			Raise  E_NotFoundSymbol, "<ERROR msg=""Mutual reference"" symbol="""+ _
				XmlAttr( DicKeyToCSV( not_out_vars ) ) +"""/>"
		End If
	Loop


	'//=== calculate
	code = code +_
				 "  CalculateVariablesInside = "+ Replace( Formula, "0x", "&h" ) +vbCRLF+_
				 "End Function" +vbCRLF

	Execute  code

	CalculateVariables = CalculateVariablesInside( Variables )
End Function


 
'*************************************************************************
'  <<< [ObjToXML] >>> 
'*************************************************************************
Function  ObjToXML( TagName, Objs, Opt )
	Dim  o
	Dim  out

	If not IsEmpty( TagName ) Then  out = "<" + TagName + ">" + vbCRLF
	If IsArray( Objs ) Then
		For Each o In Objs : If not IsEmpty(o) Then  ObjToXML1  o, out
		Next
	ElseIf TypeName( Objs ) = "ArrayClass" Then
		For Each o In Objs.Items : ObjToXML1  o, out : Next
	ElseIf IsObject( Objs ) Then
		ObjToXML1  Objs, out
	End If
	If not IsEmpty( TagName ) Then  out = out + "</" + TagName + ">" + vbCRLF
	ObjToXML = Left( out, Len( out ) - 2 )
End Function


Sub  ObjToXML1( Obj, Out )
	Dim en,ed

	Out = Out + "<" + TypeName( Obj )

	ErrCheck : On Error Resume Next
		ed = Obj.Name
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = 0 Then  Out = Out + " Name=""" & XmlAttr( Obj.Name ) & """"
	If en = 438 Then  en = 0
	If en <> 0 Then  Err.Raise en,,ed

	ErrCheck : On Error Resume Next
		ed = Obj.DefinePath
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = 0 Then  Out = Out + " DefinePath=""" & XmlAttr( Obj.DefinePath ) & """"
	If en = 438 Then  en = 0
	If en <> 0 Then  Err.Raise en,,ed

	Out = Out + "/>" + vbCRLF
End Sub


 
'*************************************************************************
'  <<< [get_Object] >>> 
'*************************************************************************
Function  get_Object( Name )
	Dim  en,ed

	ErrCheck : On Error Resume Next
		Dim  get_func : Set  get_func = GetRef( "get_" + Name )
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = 5 Then  Err.Raise  en,,ed + " : Not defined 'get_" + Name + "'"
	If en <> 0 Then  Err.Raise en,,ed

	Set  get_Object = get_func()
End Function


 
'*************************************************************************
'  <<< [get_ObjectFromFile] >>> 
'*************************************************************************
Function  get_ObjectFromFile( ModulePath, Name )
	Dim  f

	g_SrcPath = g_fs.GetAbsolutePathName( ModulePath )
	If g_is_debug Then  echo  ">include """ + g_SrcPath + """"
	Set f = g_fs.OpenTextFile( g_SrcPath,,,-2 )
	If g_is_debug Then
		ExecuteGlobal  "'// " + g_SrcPath +vbCRLF+ f.ReadAll()
	Else
		ExecuteGlobal  f.ReadAll()
	End If

	Dim  get_func : Set  get_func = GetRef( "get_" + Name )
	Set  get_ObjectFromFile = get_func()
End Function


 
'*************************************************************************
'  <<< [get_NameDelegator] >>> 
'*************************************************************************
Dim  g_NameDic : Set g_NameDic = CreateObject( "Scripting.Dictionary" )

Function  get_NameDelegator( Name, TrueName, InterfaceName )
	If g_NameDic.Exists( Name +"__"+ TrueName ) Then
		Set get_NameDelegator = g_NameDic.Item( Name +"__"+ TrueName +"_"+ InterfaceName )
		Exit Function
	End If

	Set  get_NameDelegator = new_X( InterfaceName + "_Delegator" ) : With get_NameDelegator
		.Name = Name
		.m_Delegate = TrueName  '// if validated was need.
		If not g_bNeedValidateDelegate Then _
			Set .m_Delegate = get_Object( TrueName )  '// if validated was not need.
	End With

	Set  g_NameDic.Item( Name +"__"+ TrueName +"_"+ InterfaceName ) = get_NameDelegator
End Function


Const  F_ValidateOnlyDelegate = &h40000000
Dim    g_bNeedValidateDelegate


Function  NameDelegator_getTrueName( m )
	If VarType( m.m_Delegate ) = vbString Then
		NameDelegator_getTrueName = m.m_Delegate
	Else
		NameDelegator_getTrueName = m.m_Delegate.TrueName
	End If
End Function


Sub  NameDelegator_validate( m, Flags )
	If VarType( m.m_Delegate ) = vbString Then
		Set m.m_Delegate = get_Object( m.m_Delegate )
	End If
	If ( Flags and F_ValidateOnlyDelegate ) = 0 Then _
		m.m_Delegate.Validate  Flags
End Sub


Function  NameDelegator_getXML( m )
	If VarType( m.m_Delegate ) = vbString Then
		NameDelegator_getXML = "<" + TypeName( m ) + _
			" Name='" + m.Name + "' TrueName='" + m.TrueName + "'/>"
	Else
		NameDelegator_getXML = "<" + TypeName( m ) + _
			" Name='" + m.Name + "' TrueName='" + m.TrueName + "'>" +vbCRLF+_
			m.m_Delegate.xml + vbCRLF + "</" + TypeName( m ) + ">"
	End If
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [NameOnlyClass] Class >>>> 
'-------------------------------------------------------------------------
Class  NameOnlyClass

	Public Name
	Public Delegate

	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		If not IsObject( Delegate ) Then
			If Delegate = "" Then
				xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Name='"+ _
					XmlAttrA( Name ) + "'/>"+ vbCRLF
			Else
				xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Name='"+ _
					XmlAttrA( Name ) + "'>"+ _
					GetEchoStr( Delegate ) +"</"+TypeName(Me)+">"+ vbCRLF
			End If
		Else
			xml_sub2 = GetEchoStr_sub( Delegate, Level )
			If InStr( xml_sub2, vbLF ) = Len( xml_sub2 ) Then
				CutLastOf  xml_sub2, vbCRLF, Empty
				If xml_sub2 = "" Then
					xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Name='"+ _
						XmlAttrA( Name ) + "'/>"+ vbCRLF
				Else
					xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Name='"+ _
						XmlAttrA( Name ) + "'>"+ _
						xml_sub2 +"</"+TypeName(Me)+">"+ vbCRLF
				End If
			Else
				xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Name='"+ _
					XmlAttrA( Name ) + "'>"+vbCRLF+ _
					 +vbCRLF+"</"+TypeName(Me)+">"+ vbCRLF
			End If
		End If
	End Function
End Class


'//[new_NameOnlyClass]
Function  new_NameOnlyClass( in_Name, in_Delegate )
	Set object = new NameOnlyClass
	object.Name = in_Name
	If IsObject( in_Delegate ) Then
		Set object.Delegate = in_Delegate
	Else
		object.Delegate = in_Delegate
	End If
	Set new_NameOnlyClass = object
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [DestroyCountClass] Class >>>> 
'-------------------------------------------------------------------------
Dim  g_DestroyCount

Class  DestroyCountClass
	Sub Class_Terminate()
		g_DestroyCount = g_DestroyCount + 1
	End Sub

	Public Function  xml_sub( in_Level )
		xml_sub = vbCRLF
	End Function
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [JoinedClass] Class >>>> 
'-------------------------------------------------------------------------
Class  JoinedClass

	Public Left
	Public Right

	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		xml_sub = GetTab(Level)+ "<"+TypeName(Me)+" Left='"+ GetEchoStr( Left ) + "'>"+vbCRLF+ _
			GetEchoStr( Right ) +vbCRLF+"</"+TypeName(Me)+">"+ vbCRLF
	End Function
End Class


 
'*************************************************************************
'  <<< [new_JoinedClass] >>> 
'*************************************************************************
Function  new_JoinedClass( in_Left, in_Right )
	Set  self = new JoinedClass

	If IsObject( in_Left ) Then _
		Set self.Left = in_Left _
	Else _
		self.Left = in_Left

	If IsObject( in_Right ) Then _
		Set self.Right = in_Right _
	Else _
		self.Right = in_Right

	Set new_JoinedClass = self
End Function


 
'-------------------------------------------------------------------------
'  <<< [FuncRedir] Class >>> 
'-------------------------------------------------------------------------

Sub  FuncRedir_add( GetFuncs, FuncName )
	If IsEmpty( GetFuncs ) Then  Set GetFuncs = new FuncRedir
	If IsDefined( FuncName ) Then _
		GetFuncs.m_Funcs.Add  GetRef( FuncName )
End Sub


Class  FuncRedir
	Public  m_Funcs  ' as ArrayClass of Function
	Public  m_CallLevel  ' as integer

	Private Sub  Class_Initialize()
		Set m_Funcs = new ArrayClass
		m_CallLevel = 0
	End Sub

	Public Function  CallFunction0()
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		CallFunction0 = m_Funcs( i )()
		m_CallLevel = m_CallLevel - 1
	End Function

	Public Function  CallFunction1( Param1 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		CallFunction1 = m_Funcs( i )( Param1 )
		m_CallLevel = m_CallLevel - 1
	End Function

	Public Function  CallFunction2( Param1, Param2 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		CallFunction2 = m_Funcs( i )( Param1, Param2 )
		m_CallLevel = m_CallLevel - 1
	End Function

	Public Function  CallFunction3( Param1, Param2, Param3 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		CallFunction3 = m_Funcs( i )( Param1, Param2, Param3 )
		m_CallLevel = m_CallLevel - 1
	End Function

	Public Function  CallFunction4( Param1, Param2, Param3, Param4 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		CallFunction4 = m_Funcs( i )( Param1, Param2, Param3, Param4 )
		m_CallLevel = m_CallLevel - 1
	End Function

	Public Sub  CallSub0()
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		Call  m_Funcs( i )()
		m_CallLevel = m_CallLevel - 1
	End Sub

	Public Sub  CallSub1( Param1 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		Call  m_Funcs( i )( Param1 )
		m_CallLevel = m_CallLevel - 1
	End Sub

	Public Sub  CallSub2( Param1, Param2 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		Call  m_Funcs( i )( Param1, Param2 )
		m_CallLevel = m_CallLevel - 1
	End Sub

	Public Sub  CallSub3( Param1, Param2, Param3 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		Call  m_Funcs( i )( Param1, Param2, Param3 )
		m_CallLevel = m_CallLevel - 1
	End Sub

	Public Sub  CallSub4( Param1, Param2, Param3, Param4 )
		m_CallLevel = m_CallLevel + 1
		Dim  i : i = m_Funcs.Count - m_CallLevel : If i < 0 Then  i = 0
		Call  m_Funcs( i )( Param1, Param2, Param3, Param4 )
		m_CallLevel = m_CallLevel - 1
	End Sub
End Class


 
'*************************************************************************
'  <<< [new_X] >>> 
'*************************************************************************
Function  new_X( Name )
	Dim en,ed

	ErrCheck : On Error Resume Next
		Dim  new_f : Set  new_f = GetRef( "new_" + Name )
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = 5 Then  Err.Raise  en,,ed + " : Not defined 'new_" + Name + "'"
	If en <> 0 Then  Err.Raise en,,ed

	Set  new_X = new_f()
End Function


 
'*************************************************************************
'  <<< [include_objs] >>> 
'*************************************************************************
Dim  g_included_paths : Set g_included_paths = CreateObject( "Scripting.Dictionary" )

Sub  include_objs( Wildcard, Flags, out_GetObjectFuncs )
	Dim  ds_:Set ds_= new CurDirStack
	Dim  folder_path, fname_key_s, folders, i, fo, f, fi, t, en, ed, e, wildcards
	Dim  fname_key, fname_keys, is_match, fname,  folder_fnames,  path,  is_done
	Dim  file_names,  file_path,  file_names_fast_ubound
	Dim  c : Set c = g_VBS_Lib
	Const  NotCaseSensitive = 1

	If ( Flags and c.NotToEmptyArray ) = 0 Then _
		ReDim  out_GetObjectFuncs(-1)

	is_done = False
	If not IsArray( Wildcard ) Then
	 If not IsWildcard( Wildcard )  and  g_fs.FileExists( Wildcard ) Then

		g_SrcPath = GetFullPath( Wildcard, Empty )

		If IsEmpty( g_included_paths.Item( g_SrcPath ) ) Then

			If g_is_debug Then  echo  ">include """ + g_SrcPath + """"
			ExecuteGlobal  "Sub  get_StaticObjects(a,b) : End Sub"

			Set fi = g_fs.OpenTextFile( g_SrcPath,,,-2 )
			If g_is_debug Then  t = "'// " + g_SrcPath +vbCRLF+ fi.ReadAll()  Else  t = fi.ReadAll()
			fi.Close
			g_sh.CurrentDirectory = g_fs.GetParentFolderName( g_SrcPath )

			If not IsEmpty( g_debug_vbs_path ) and _
			       InStr( g_SrcPath, g_debug_vbs_path ) > 0 Then
				InvestigateInterpretError2  g_SrcPath, en, ed
			Else
				If TryStart(e) Then  On Error Resume Next
					ExecuteGlobal  t  '// Interpret  g_SrcPath
				If TryEnd Then  On Error GoTo 0
				If e.Num = 1041 Then
					InvestigateInterpretError  g_SrcPath, e.Num, e.desc
				ElseIf e.Num <> 0 Then
					e.Raise
				End If
			End If

			ReDim Preserve  out_GetObjectFuncs( UBound( out_GetObjectFuncs ) + 1 )
			Set out_GetObjectFuncs( UBound( out_GetObjectFuncs ) ) = GetRef( "get_StaticObjects" )

			Set  g_included_paths.Item( g_SrcPath ) = out_GetObjectFuncs( UBound( out_GetObjectFuncs ) )
		Else
			ReDim Preserve  out_GetObjectFuncs( UBound( out_GetObjectFuncs ) + 1 )
			Set out_GetObjectFuncs( UBound( out_GetObjectFuncs ) ) = g_included_paths.Item( g_SrcPath )
		End If
		is_done = True
	 End If
	End If
	If not is_done Then
		ReDim  file_names(-1)
		file_names_fast_ubound = UBound( file_names )
		Set folder_fnames = CreateObject( "Scripting.Dictionary" )
		folder_fnames.CompareMode = NotCaseSensitive
		'// folder_fnames as dictionary of array of StrMatchKey, key= folder_path

		If IsArray( Wildcard ) Then  wildcards = Wildcard  Else  wildcards = Array( Wildcard )
		For Each path  In wildcards
			If g_fs.FileExists( path ) Then
				If UBound(file_names) <= file_names_fast_ubound Then _
				ReDim Preserve  file_names( ( file_names_fast_ubound + 100 ) * 4 )
				file_names_fast_ubound = file_names_fast_ubound + 1

				file_names( file_names_fast_ubound ) = path
				folder_path = Empty
			ElseIf g_fs.FolderExists( path ) Then
				folder_path = path : fname_key_s = "*_obj.vbs"
			Else
				folder_path = GetParentFullPath( path ) : fname_key_s = g_fs.GetFileName( path )
			End If
			If not IsEmpty( folder_path ) Then
				Set fname_key = new StrMatchKey
				fname_key.Keyword = LCase( fname_key_s )

				If IsEmpty( folder_fnames.Item( folder_path ) ) Then
					Set fname_keys = new ArrayClass
					Set folder_fnames.Item( folder_path ) = fname_keys
				Else
					Set fname_keys = folder_fnames.Item( folder_path )
				End If
				fname_keys.Add  fname_key
			End If
		Next

		ReDim Preserve  file_names( file_names_fast_ubound )
		For Each file_path  In file_names
			include_objs  file_path, Flags or c.NotToEmptyArray, out_GetObjectFuncs
		Next

		For Each folder_path  In folder_fnames.Keys
			Set fname_keys = folder_fnames.Item( folder_path )  '// as ArrayClass

			EnumFolderObject  folder_path, folders  '// [out] folders
			For Each fo  In folders
				For Each f  In fo.Files
					fname = f.Name  '// time( f.ShortName ) + time( f.DateLastModified ) = time( f.Name ) * 0.506/0.871
					is_match = False
					For Each fname_key  In fname_keys.Items
						If fname_key.IsMatch( fname ) Then  is_match = True
					Next
					If is_match Then

						include_objs  f.Path, Flags or c.NotToEmptyArray, out_GetObjectFuncs

					End If
				Next
			Next
		Next
	End If
	g_SrcPath = Empty
End Sub


 
'*************************************************************************
'  <<< [get_ObjectsFromFile] >>> 
'*************************************************************************
Sub  get_ObjectsFromFile( GetObjectFuncs, InterfaceName, out_Objs )
	If VarType( GetObjectFuncs ) = vbString Then
		Dim  create_funcs
		include_objs  GetObjectFuncs, Empty, create_funcs '// [out] create_funcs
		get_ObjectsFromFile_sub  create_funcs, InterfaceName, out_Objs
	Else
		get_ObjectsFromFile_sub  GetObjectFuncs, InterfaceName, out_Objs
	End If
End Sub

Sub  get_ObjectsFromFile_sub( GetObjectFuncs, InterfaceName, out_Objs )
	Dim  func, objs

	ReDim  out_Objs(-1)
	For Each func  In GetObjectFuncs
		objs = Empty
		Call  func( InterfaceName, objs ) '// [out] objs
		AddArrElem  out_Objs, objs
	Next
End Sub


 
'*************************************************************************
'  <<< [get_DefineInfoObject] >>> 
'*************************************************************************
Class  DefineInfoClass
	Public  FullPath
End Class

Sub  get_DefineInfoObject( in_out_Object, FullPath )
	If not IsEmpty( in_out_Object ) and  not g_bInvestigateInterpretError Then
		Raise 1, "<ERROR msg=""DefineInfoObject is double defined"" path1="""+ in_out_Object.FullPath +_
			 """ path2="""+ FullPath +"""/>"
	End If
	Set in_out_Object = new DefineInfoClass
	in_out_Object.FullPath = FullPath
End Sub


 
'*************************************************************************
'  <<< [new_ObjectFromStream] >>> 
'*************************************************************************
Sub  new_ObjectFromStream( out_Obj, ClassName, Stream )
	Dim  elem, obj
	Dim  stream_xml : Set stream_xml = ChangeToXml( Stream )

	out_Obj = Empty
	If stream_xml is Nothing Then  Exit Sub

	Set elem = stream_xml.selectSingleNode( ClassName )
	If not elem is Nothing Then
		Dim  i_attr : i_attr = InStr( ClassName, "[" )
		If i_attr = 0 Then
			Set out_Obj = new_X( ClassName )
		Else
			Set out_Obj = new_X( Left( ClassName, i_attr - 1 ) )
		End If
		out_Obj.loadXML  elem
	End If
End Sub


 
'*************************************************************************
'  <<< [new_ObjectsFromStream] >>> 
'*************************************************************************
Sub  new_ObjectsFromStream( out_Objs, ClassName, Stream )
	Dim  elem, obj, i_attr
	Dim  stream_xml : Set stream_xml = ChangeToXml( Stream )

	ReDim  out_Objs(-1)
	If stream_xml is Nothing Then  Exit Sub
	i_attr = InStr( ClassName, "[" )

	For Each elem  In stream_xml.selectNodes( ClassName )
		If i_attr = 0 Then
			Set obj = new_X( ClassName )
		Else
			Set obj = new_X( Left( ClassName, i_attr - 1 ) )
		End If
		obj.loadXML  elem
		ReDim Preserve  out_Objs( UBound( out_Objs ) + 1 )
		Set  out_Objs( UBound( out_Objs ) ) = obj
	Next
End Sub


 
'-------------------------------------------------------------------------
' ### <<<< [EventRespondersClass] >>>> 
'-------------------------------------------------------------------------

Class  EventRespondersClass
	Public  Responders

	Private Sub  Class_Initialize()
		ReDim  Responders(-1)
	End Sub

	Public Sub  Add( Func, Object, Priority )
		For Each responder  In Responders
			If responder.Object is Object Then
				Exit Sub
			End If
		Next

		Set responder = new EventResponder

		ReDim Preserve  Responders( UBound( Responders ) + 1 )
		Set Responders( UBound( Responders ) ) = responder
		responder.Priority = Priority
		Set responder.Func = Func
		Set responder.Object = Object

		ShakerSort  Responders,  0,  UBound( Responders ), _
			GetRef( "EventRespondersStaticClass_comparePriority" ), _
			Empty
	End Sub

	Public Sub  Remove( Object )
		For Each responder  In Responders
			If responder.Object is Object Then
				Exit For
			End If
		Next
		RemoveObjectArray  Responders, responder
	End Sub

	Public Sub  Calls( Caller, Args )
		For Each responder  In Responders
			responder.Func  responder.Object, Caller, Args
		Next
	End Sub
End Class

Class  EventResponder
	Public  Priority  '// as integer. Smallest fuction is called first.
	Public  Func      '// as 
	Public  Object
End Class

Function  EventRespondersStaticClass_comparePriority( Left, Right, Parameter )
	EventRespondersStaticClass_comparePriority = Left.Priority - Right.Priority
End Function


 
'*************************************************************************
'  <<< [DefaultFunction] >>> 
'*************************************************************************
Sub  DefaultFunction( Argument1 )
End Sub


 
'*************************************************************************
'  <<< [InvestigateInterpretError] >>> 
'*************************************************************************
Dim  g_debug_vbs_path
Dim  g_debug_vbs_err_num
Dim  g_bInvestigateInterpretError
Dim  g_IsDisableInvestigateInterpretError : g_IsDisableInvestigateInterpretError = False

Sub  InvestigateInterpretError( Path, en, ed )
	Dim  f, t

	If g_IsDisableInvestigateInterpretError Then  Exit Sub

	echo ""
	echo ">InvestigateInterpretError  """ + Path + """"
	g_bInvestigateInterpretError = True

	Set f = g_fs.OpenTextFile( Path,,,-2 ) : t = f.ReadAll() : f.Close
	Dim  en2, ed2
	ErrCheck : On Error Resume Next
		ExecuteGlobal  t
	en2 = Err.Number : ed2 = Err.Description : On Error GoTo 0

	If en2 = 0 Then
		Err.Raise  en,,"<ERROR msg='"+ ed +"' include_path="+vbCRLF+"'"+ g_SrcPath +_
			"'"+vbCRLF+"hint='Second ExecuteGlobal does not raise any error."+_
			"Write following code in main function and restart' add_code=" +vbCRLF+_
			"'g_debug_vbs_path = """ + Path + """'/>"
	End If

	echo  GetErrStr( en, ed )


	'// Try to display error line
	RunProg  "wscript.exe """ + Path + """", ""


	'// Error of Duplicate Name
	If en2 = 1041 Then
		Err.Raise  en,,"<ERROR msg='"+ ed +"' include_path="+vbCRLF+"'"+ g_SrcPath +_
			"'"+vbCRLF+"hint='Write following code in main function and restart' add_code=" +vbCRLF+_
			"'g_debug_vbs_path = """ + Path + """ : g_debug_vbs_err_num = 1041'/>"
	End If


	'// Try to break at error line ([attention] 2nd execute may different behavior)
	Set f = g_fs.OpenTextFile( Path,,,-2 ) : t = f.ReadAll() : f.Close
	ExecuteGlobal  "'// This is 2nd execute(include) from InvestigateInterpretError." +vbCRLF + t


	'// This is no new hint
	Err.Raise  en,,"<ERROR msg='"+ ed +"' include_path="+vbCRLF+"'"+ g_SrcPath + "'/>"
End Sub


 
'*************************************************************************
'  <<< [InvestigateInterpretError2] >>> 
'*************************************************************************
Sub  InvestigateInterpretError2( Path, en, ed )
	Dim  f, t

	If g_IsDisableInvestigateInterpretError Then  Exit Sub

	If g_debug_vbs_err_num = 1041 Then
		Stop
		InvestigateDuplicatedNameError  g_SrcPath, en, ed
		Stop
	ElseIf g_debug_vbs_err_num = -1041 Then
		Stop  ' This is 1st include. Next is ...
		g_debug_vbs_err_num = 1041
		Set f = g_fs.OpenTextFile( Path,,,-2 ) : t = f.ReadAll() : f.Close
		ExecuteGlobal  t  '// Interpret  g_SrcPath
	Else
		Set f = g_fs.OpenTextFile( Path,,,-2 ) : t = f.ReadAll() : f.Close
		ExecuteGlobal  t  '// Interpret  g_SrcPath
	End If
End Sub


 
'*************************************************************************
'  <<< [InvestigateDuplicatedNameError] >>> 
'*************************************************************************
Sub  InvestigateDuplicatedNameError( Path, en, ed )
	Dim  f, t, i, j, c

	Set f = g_fs.OpenTextFile( Path,,,-2 )
	Do Until f.AtEndOfStream
		t = f.ReadLine()
		i = InStr( t, "Class" )
		If i = 0 Then  i = InStr( t, "Dim" )
		If i > 0 Then
			i=i+1
			Do
				If Mid(t,i,1)=" " Then Exit Do
				If Mid(t,i,1)="" Then Error
				i=i+1
			Loop
			Do
				If Mid(t,i,1)<>" " Then Exit Do
				i=i+1
			Loop

			j=i
			Do
				c = Mid(t,j,1)
				If not( (c>="A" and c<="Z") or (c>="a" and c<="z") or (c>="0" and c<="9") or c="_" ) Then _
					Exit Do
				j=j+1
			Loop
			If j > i Then
				If InStr( t, "Class" ) > 0 Then
					c = "Class " + Mid( t, i, j-i ) + " : End Class"
				Else
					c = "Dim " + Mid( t, i, j-i )
				End If
				echo ">ExecuteGlobal  """ + c + """"
				ExecuteGlobal  c
			End If
		End If
	Loop
	f.Close

	Err.Raise  en,,"<ERROR msg='"+ ed +"' include_path="+vbCRLF+"'"+ g_SrcPath +_
		"'"+vbCRLF+"hint='It may be double include."+_
		"Write following code in main function and restart' add_code=" +vbCRLF+_
		"'g_debug_vbs_path = """ + Path + """ : g_debug_vbs_err_num = -1041'/>"
End Sub


 
'*************************************************************************
'  <<< [LetIf] >>> 
'*************************************************************************
Sub  LetIf( in_out_Variable, Value, PassCondition )
	If in_out_Variable = Value Then  Exit Sub
	If not PassCondition Then  Stop : Error
	in_out_Variable = Value
End Sub


 
'*************************************************************************
'  <<< [ParseOptionArguments] >>> 
'*************************************************************************
Sub  ParseOptionArguments( in_out_Options )
	If TypeName( in_out_Options ) = "Dictionary" Then _
		Exit Sub

	Const  NotCaseSensitive = 1

	If IsEmpty( in_out_Options ) Then
		in_out_Options = Array( )
	ElseIf not IsArray( in_out_Options ) Then
		in_out_Options = Array( in_out_Options )
	End If

	Set dic = CreateObject( "Scripting.Dictionary" )
	dic.CompareMode = NotCaseSensitive
	For Each  option_  In in_out_Options
		Select Case  VarType( option_ )
			Case vbObject  : Set dic( TypeName( option_ ) ) = option_
			Case vbInteger : dic( "integer" ) = option_
			Case vbLong    : dic( "integer" ) = option_
			Case vbByte    : dic( "integer" ) = option_
			Case vbSingle  : dic( "float" )   = option_
			Case vbDouble  : dic( "float" )   = option_
			Case Else      : dic( TypeName( option_ ) )  = option_
		End Select
	Next

	Set in_out_Options = dic
End Sub


 
'*************************************************************************
'  <<< [IsBitSet] >>> 
'*************************************************************************
Function  IsBitSet( Variable, ConstValue )
	If Variable and ConstValue Then
		IsBitSet = True
	Else
		IsBitSet = False
	End If
End Function


 
'*************************************************************************
'  <<< [IsAnyBitsSet] >>> 
'*************************************************************************
Function  IsAnyBitsSet( Variable, ConstValue )
	If Variable and ConstValue Then
		IsAnyBitsSet = True
	Else
		IsAnyBitsSet = False
	End If
End Function


 
'*************************************************************************
'  <<< [IsAllBitsSet] >>> 
'*************************************************************************
Function  IsAllBitsSet( Variable, ConstValue )
	If ( Variable and ConstValue ) = ConstValue Then
		IsAllBitsSet = True
	Else
		IsAllBitsSet = False
	End If
End Function


 
'*************************************************************************
'  <<< [IsBitNotSet] >>> 
'*************************************************************************
Function  IsBitNotSet( Variable, ConstValue )
	If not Variable and ConstValue Then
		IsBitNotSet = True
	Else
		IsBitNotSet = False
	End If
End Function


 
'*************************************************************************
'  <<< [IsAnyBitsNotSet] >>> 
'*************************************************************************
Function  IsAnyBitsNotSet( Variable, ConstValue )
	If not Variable and ConstValue Then
		IsAnyBitsNotSet = True
	Else
		IsAnyBitsNotSet = False
	End If
End Function


 
'*************************************************************************
'  <<< [IsAllBitsNotSet] >>> 
'*************************************************************************
Function  IsAllBitsNotSet( Variable, ConstValue )
	If ( not Variable and ConstValue ) = ConstValue Then
		IsAllBitsNotSet = True
	Else
		IsAllBitsNotSet = False
	End If
End Function


 
'*-------------------------------------------------------------------------*
'* ### <<<< Process >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [env] >>> 
'*************************************************************************
Function  env( s )
	Dim  i
	If IsEmpty( s ) Then Exit Function  '// for avoid to s=""
	If IsArray( s ) Then
		For i=0 To UBound( s )
			s(i) = env( s(i) )
		Next
		env = s
		Exit Function
	End If

	i = 1
	Do
		p1 = InStr( i, s, "%" )
		If p1 = 0 Then
			env = env + Mid( s, i )
			Exit Function
		End If

		env = env + Mid( s, i, p1 - i )
		If IsNumeric( Mid( s, p1 + 1, 1 ) ) Then
			env = env + "%"
			i = p1 + 1
		Else
			p2 = InStr( p1+1, s, "%" )
			If p2 = p1+1 Then
				env = env + "%"
			ElseIf p2 = 0 Then
				Raise  E_NotFoundSymbol, "<ERROR msg=""End of % is not found"" expression="""+_
					s +"""/>"
			Else
				p3 = InStr( p1+1, s, "(" )
				If p2 < p3  or  p3 = 0 Then
					symbol = Mid( s, p1+1, p2-p1-1 )
					value = GetVarSub( symbol, True )
				Else
					symbol = Mid( s, p1+1, p3-p1 )
					LetSet  value, GetVarSub( symbol, True )
					If TypeName( value ) = "VarWithParameterClass" Then
						p3 = p3 + 1
						p2 = InStr( p3, s, ")%" )
						If p2 = 0 Then
							Raise  E_NotFoundSymbol, "<ERROR msg=""Closing )% is"+_
								"Not found"" expression="""+ Str +"""/>"
						End If

						value = value.GetVar( Mid( s, p3, p2 - p3 ) )
						p2 = p2 + 1
					End If
				End If
				If IsArray( value ) Then
					For ii=0 To UBound( value )
						If IsEmpty( value(ii) ) Then
							Raise  E_NotFoundSymbol, "<ERROR msg="""+_
							"not found var symbol"" symbol="""+_
							symbol +""" array_index="""& ii &"""/>"
						End If
					Next

					If Mid( s, p2 ) <> "%" Then _
						Raise  E_NotFoundSymbol, "<ERROR msg=""Array value can not be cast to string""/>"
					env = value
					Exit Function
				End If
				If IsEmpty( value ) Then
					Raise  E_NotFoundSymbol, "<ERROR msg=""not found var symbol"" symbol="""+ symbol +"""/>"
				End If
				env = env & value
			End If
			i = p2 + 1
		End If
	Loop
End Function


 
'*************************************************************************
'  <<< [VarWithParameterClass] >>> 
'*************************************************************************
Class  VarWithParameterClass
	Public  VarNameAndStartBracket
	Public  GetVar  '// as function
End Class


 
'*************************************************************************
'  <<< [VarWithParameter_XML_FileClass_GetVar] >>> 
'*************************************************************************
Function  VarWithParameter_XML_FileClass_GetVar( Parameter )
	a_URL = env( Parameter )
	value = (new XML_ReadCacheClass)( a_URL )
	value = env( value )
	VarWithParameter_XML_FileClass_GetVar = value
End Function


 
'*************************************************************************
'  <<< [VarWithParameter_ArrayFromXML_FileClass_GetVar] >>> 
'*************************************************************************
Function  VarWithParameter_ArrayFromXML_FileClass_GetVar( Parameter )
	a_URL = env( Parameter )
	values = (new XML_ReadCacheClass).GetArray( a_URL )
	For i = 0 To UBound( values )
		values(i) = env( values(i) )
	Next
	VarWithParameter_ArrayFromXML_FileClass_GetVar = values
End Function


 
'*************************************************************************
'  <<< [VarWithParameter_SpecialFoldersClass_GetVar] >>> 
'*************************************************************************
Function  VarWithParameter_SpecialFoldersClass_GetVar( Parameter )
	identifier = env( Parameter )
	VarWithParameter_SpecialFoldersClass_GetVar = g_sh.SpecialFolders( identifier )
End Function


 
'*************************************************************************
'  <<< [start] >>> 
'*************************************************************************
Sub  start( cmdline )
	echo  ">start  " & cmdline
	cmdline = g_sh.ExpandEnvironmentStrings( cmdline )


	'//=== call a function
	If StrCompHeadOf( cmdline, "\?InCurrentProcessFunc\", Empty ) = 0 Then
		i = 1
		Dim  exe_path : exe_path = MeltCmdLine( cmdline, i )
		Dim func : Set func = GetRef( g_fs.GetFileName( exe_path ) )
		func  cmdline
		Exit Sub

	'//=== edit cmdline by g_CommandPrompt
	ElseIf InStr( cmdline, "cscript" ) > 0 Then
		Dim  f, line, i, ii, s, b_cscript

		i = 1
		Do
			s = MeltCmdLine( cmdline, i )
			Select Case  LCase( s )
				Case  "cscript", "cscript.exe" : b_cscript = True
				Case Else
					If b_cscript Then
						If Left( s, 1 ) <> "/" Then  Exit Do
					End If
			End Select
			s = Empty
			If i = 0 Then  Exit Do
		Loop

		If not IsEmpty( s ) Then
			i = 0
			Set f = OpenForRead( s )
			Do Until  f.AtEndOfStream
				line = f.ReadLine()
				ii = InStr( line, "g_CommandPrompt" )
				If ii > 0 Then  ii = InStr( ii, line, "=" )
				If ii > 0 Then
					ii = CInt( Mid( line, ii+1 ) )
					Exit Do
				End If
				ii = Empty
				i=i+1
				If i=100 Then  Exit Do
			Loop
			f = Empty

			If g_is64bitWindows Then
				cmdline = Replace( cmdline, "cscript",_
					g_sh.ExpandEnvironmentStrings( "%windir%\SysWOW64\cscript" ),1,1 )
			End If

			Select Case  ii
				Case 0:  cmdline = Replace( cmdline, "cscript", "wscript",1,1 )
				Case 1:  cmdline = "cmd /v:on /K ("+ cmdline +"& if ""!errorlevel!""==""21"" exit)"
				Case 2:  cmdline = "cmd /v:on /K ("+ cmdline +")"
			End Select
		End If
	End If


	'//=== create process
	Dim en,ed

	ErrCheck : On Error Resume Next

		g_sh.Run  cmdline,, FALSE

	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = E_WIN32_FILE_NOT_FOUND Then _
		Err.Raise en,,"<ERROR msg=""Execute file is not found or Not close by double quotation"+_
			""" cmdline="""+ XmlAttr( cmdline ) +"""/>"
	If en <> 0 Then  Err.Raise en,,ed
End Sub


 
'*************************************************************************
'  <<< [RunProg] >>> 
'*************************************************************************
Function  RunProg( ByVal cmdline, ByVal stdout_stderr_redirect )
	Dim  dbg_cmd, pr, i, echo_do

	cmdline = Trim2( cmdline )

	'// ChildProcess object for cscript (1)
	If Left( cmdline, 7 ) = "cscript" Then
		If TypeName( stdout_stderr_redirect ) = "ParentChildProcess" Then
			Set pr = stdout_stderr_redirect : stdout_stderr_redirect = ""
		Else
			Set pr = new_ParentProcess()
		End If
	End If


	'// Echo command line
	echo_do = True
	If IsNumeric( stdout_stderr_redirect ) Then
		Set c = g_VBS_Lib
		If stdout_stderr_redirect = c.NotEchoStartCommand Then  echo_do = False
		stdout_stderr_redirect = ""
	End If
	If g_EchoObj.m_bEchoOff Then  echo_do = False
	If g_is_debug Then  echo_do = True
	If echo_do Then
		echo_v  ">current dir = """ & g_sh.CurrentDirectory & """"
		If stdout_stderr_redirect = "" Then
			echo_v  ">RunProg  " & cmdline
		Else
			echo_v  ">RunProg  " & cmdline+" > """+stdout_stderr_redirect+""""
		End If
	End If


	'// env
	cmdline = g_sh.ExpandEnvironmentStrings( cmdline )


	If not IsEmpty( pr ) Then  '// call cscript

		'// Break in ChildProcess by "g_debug_process"
		If g_InterProcess.ProcessCallID.UBound_ = g_Err2.Break_ChildProcess_CallID.UBound_ Then
			For i=0 To g_InterProcess.ProcessCallID.UBound_-1
				If g_InterProcess.ProcessCallID(i) <> g_Err2.Break_ChildProcess_CallID(i) Then  Exit For
			Next
			If i = g_InterProcess.ProcessCallID.UBound_ Then
				If g_InterProcess.ProcessCallID(i) + 1 = g_Err2.Break_ChildProcess_CallID(i) Then
					If InStr( cmdline, "//x " ) = 0 Then _
						cmdline = "cscript //x "+ Mid( cmdline, InStr( cmdline, " " ) )
				End If
			End If
		End If

		'// avoid to stop by StdIn
		cmdline = cmdline + " /GUI_input:1"

		'// Use 32bit WSH
		If g_is64bitWindows Then
			If Left( cmdline, 7 ) = "cscript" Then
				cmdline = Replace( cmdline, "cscript",_
					g_sh.ExpandEnvironmentStrings( "%windir%\SysWOW64\cscript" ),1,1 )
			End If
		End If

		'// Set "/g_debug", "/debug"
		If InStr( cmdline, "//x " ) > 0 Then
			If InStr( cmdline, "/g_debug" ) = 0 Then _
				cmdline = cmdline + " /g_debug:1"
			If InStr( cmdline, "/debug:" ) = 0 Then _
				cmdline = cmdline + " /debug:1"
		End If

		'// ChildProcess object for cscript (2)
		i = InStr( cmdline, "/ChildProcess:0" )
		If i > 0 Then  '// without /ChildProcess
			Select Case  Mid( cmdline, i + Len( "/ChildProcess:0" ), 1 )
				Case " " : i = -1 : Case "" : i = -1
			End Select
		End If
		If i <= 0 Then  '// with /ChildProcess
			pr.OnCallInParent
			If i = 0 Then _
				cmdline = cmdline +" /ChildProcess:"+ pr.m_DateID +","+ pr.m_FileID
		End If
	End If


	'// Create new process
	Dim  ex
	Set ex = g_sh.Exec( cmdline )
	stdout_stderr_redirect = g_sh.ExpandEnvironmentStrings( stdout_stderr_redirect )
	RunProg = WaitForFinishAndRedirect( ex, stdout_stderr_redirect )
	echo  ""


	'// ChildProcess object for cscript (3)
	If not IsEmpty( pr ) Then  pr.OnReturnInParent  RunProg
End Function


 
'*************************************************************************
'  <<< [RunBat] >>> 
'*************************************************************************
Function  RunBat( BatchCommand, ByVal OutPath )
	Dim  BatPath : BatPath = GetTempPath( "RunBat_*.bat" )
	Dim  RetPath : RetPath = GetTempPath( "RunBatRet_*.txt" )
	Dim  b_tmp_out
	If IsEmpty( OutPath ) Then  OutPath = GetTempPath( "RunBatOut_*.txt" ) : b_tmp_out = True

	echo  g_sh.CurrentDirectory & ">"
	Dim  ec : Set ec = new EchoOff

	CreateFile  BatPath, BatchCommand +vbCRLF+ "@echo %errorlevel% > """+ RetPath +""""
	CreateFile  RetPath, "1"  '// default value

	del  OutPath
	ec = Empty  '// echo on in batch file
	RunProg  env("%windir%")+"\system32\cmd.exe /C cd """+ g_sh.CurrentDirectory +""" & ("""+_
		BatPath +""" 2>&1 )",  OutPath

	RunBat = CInt( ReadFile( RetPath ) )
	echo  "%errorlevel% = "& RunBat

	Set ec = new EchoOff
	del  BatPath
	del  RetPath
	IF b_tmp_out Then  del  OutPath
End Function


 
'*************************************************************************
'  <<< [RunBatAsync] >>> 
'*************************************************************************
Function  RunBatAsync( BatchCommand )
	Set self = new RunBatAsyncClass
	self.BatPath = GetTempPath( "RunBat_*.bat" )
	self.RetPath = GetTempPath( "RunBatRet_*.txt" )
	self.IsReceived = False

	echo  g_sh.CurrentDirectory & ">RunBatAsync"
	Dim  ec : Set ec = new EchoOff

	CreateFile  self.BatPath, BatchCommand +vbCRLF+ "@echo %errorlevel% > """+ self.RetPath +""""
	CreateFile  self.RetPath, "1"  '// default value

	ec = Empty  '// echo on in batch file
	Set self.Exec = g_sh.Exec( env("%windir%")+"\system32\cmd.exe "+_
		"/C cd """+ g_sh.CurrentDirectory +""" & ("""+_
		self.BatPath +""" 2>&1 )" )

	'// For debug
	'// exit_code = WaitForFinishAndRedirect( self.Exec, stdout_stderr_redirect ) : Pause

	Set RunBatAsync = self
End Function


'//[RunBatAsyncClass]
Class  RunBatAsyncClass
	Public  ExitCode

	Public Property Get  Status()
		Status = Exec.Status
	End Property

	Public Sub  DoEvents()
		If self.IsReceived Then  Exit Sub

		If Exec.Status = 1 Then
			self.ExitCode = CInt( ReadFile( self.RetPath ) )

			Set ec = new EchoOff
			del  self.BatPath
			del  self.RetPath

			self.IsReceived = True
		End If
	End Sub

	Public  Exec  '// as WshScriptExec
	Public  BatPath
	Public  RetPath
	Public  IsReceived
End Class


 
'*************************************************************************
'  <<< [WaitForFinishAndRedirect] >>> 
'*************************************************************************
Function  WaitForFinishAndRedirect( ex, path )
	Dim  f

	Const  output_is_nul    = 1
	Const  output_is_stdout = 2
	Const  output_is_file   = 3

	If path = "nul" Then
		output = output_is_nul
	ElseIf path = "" Then
		If g_EchoObj.m_bEchoOff Then
			output = output_is_nul
		Else
			output = output_is_stdout
		End If
	Else
		output = output_is_file
	End If


	If g_is_debug and IsEmpty( g_ChildHead ) Then  g_ChildHead = ">|"


	If output = output_is_file Then
		Set ec = new EchoOff
		Set f = OpenForWrite( path, Empty )
		ec = Empty
	End If

	Do While ex.Status = 0
		If output = output_is_nul Then
			Do Until ex.StdOut.AtEndOfStream : ex.StdOut.ReadLine : Loop
			Do Until ex.StdErr.AtEndOfStream : ex.StdErr.ReadLine : Loop
		ElseIf output = output_is_stdout Then
			EchoStream  ex.StdOut, WScript.StdOut, ex, g_ChildHead
			EchoStream  ex.StdErr, WScript.StdErr, ex, g_ChildHead
		Else
			Do Until ex.StdOut.AtEndOfStream
				line = ex.StdOut.ReadLine
				f.WriteLine  line
				echo  line
			Loop
			Do Until ex.StdErr.AtEndOfStream
				line = ex.StdErr.ReadLine
				f.WriteLine  line
				echo  line
			Loop
		End If
	Loop

	If output = output_is_nul Then
		Do Until ex.StdOut.AtEndOfStream : ex.StdOut.ReadLine : Loop
		Do Until ex.StdErr.AtEndOfStream : ex.StdErr.ReadLine : Loop
	ElseIf output = output_is_stdout Then
		EchoStream  ex.StdOut, WScript.StdOut, ex, g_ChildHead
		EchoStream  ex.StdErr, WScript.StdErr, ex, g_ChildHead
	Else
		Do Until ex.StdOut.AtEndOfStream
			line = ex.StdOut.ReadLine
			f.WriteLine  line
			echo  line
		Loop
		Do Until ex.StdErr.AtEndOfStream
			line = ex.StdErr.ReadLine
			f.WriteLine  line
			echo  line
		Loop
	End If

	WaitForFinishAndRedirect = ex.ExitCode
End Function


 
'*************************************************************************
'  <<< [EchoStream] echo supported No vbCRLF >>> 
'*************************************************************************
Dim  g_EchoStreamBuf
Sub  EchoStream( StreamIn, StreamOut, ex, Prompt )
	Dim  c, b, i

	Do Until StreamIn.AtEndOfStream
		c = StreamIn.Read(1)
		If c <> vbCR and c <> vbLF Then
			If g_EchoStreamBuf = "" Then  StreamOut.Write  Prompt
			g_EchoStreamBuf = g_EchoStreamBuf + c
		End If

		'// Support only pause
		If Left( g_EchoStreamBuf, 6 ) = "Press any" Then
			i = 0
			If g_EchoStreamBuf="Press any key to continue . . . " Then  i = 1
			If g_EchoStreamBuf=Left(g_PauseMsg,g_PauseMsgStone)+"*"+Chr(8) Then  i = 3
			If g_EchoStreamBuf=g_PauseMsg Then  i = 2
			If i > 0 Then
				StreamOut.Write  c
				If ex.Status = 0 Then
					If i < 3 Then
						WScript.StdIn.ReadLine  '// Waiting Enter from only main process
						If i = 1 Then
							ex.StdIn.Write  vbCR
							StreamIn.ReadLine
						Else
							ex.StdIn.Write  vbCRLF
						End If
					End If
				End If
				If not IsEmpty( g_Test ) Then  g_Test.WriteLogLine  g_EchoStreamBuf
				g_EchoStreamBuf = ""
				c = ""
			End If
		End If

		'// echo
		If c = vbLF Then
			StreamOut.Write  vbLF
			If not IsEmpty( g_Test ) Then  g_Test.WriteLogLine  g_EchoStreamBuf
			g_EchoStreamBuf = ""
		Else
			StreamOut.Write  c
		End If
	Loop
End Sub


 
'*************************************************************************
'  <<< [ArgumentExist] >>> 
'*************************************************************************
Function  ArgumentExist( name )
	Dim  key
	For Each key in WScript.Arguments.Named
		If key = name  Then  ArgumentExist = True : Exit Function
	Next
	ArgumentExist = False
End Function


 
'*************************************************************************
'  <<< [DicFromCmdLineOpt] >>> 
'*************************************************************************
Function  DicFromCmdLineOpt( CmdLine, ByVal OptionNames )
	Dim  i,  name,  value_type,  param,  length,  is_next_value
	Dim  params : params = ArrayFromCmdLine( CmdLine )
	Dim  options : Set options = CreateObject( "Scripting.Dictionary" )
	ReDim  value_types( UBound( OptionNames ) )
	Dim  colons : colons = Array( "", ":", "::" )
	Const  normal      = 0
	Const  space_value = 1
	Const  multi_value = 2


	'// value_types(i) : normal, space_value, multi_value
	'// OptionNames(i) : cut last of ":"
	For i=0 To UBound( OptionNames )
		name = OptionNames(i)
		If Right( name, 1 ) <> ":" Then
			value_types(i) = normal
		ElseIf Right( name, 2 ) <> "::" Then
			value_types(i) = space_value
			OptionNames(i) = Left( name, Len( name ) - 1 )
		Else
			value_types(i) = multi_value
			OptionNames(i) = Left( name, Len( name ) - 2 )
		End If
	Next


	'// options : return value of this function
	Set options( "no name" ) = new ArrayClass
	is_next_value = False
	For Each  param  In params
		If not is_next_value Then
			For i=0 To UBound( OptionNames )
				name = OptionNames(i)
				value_type = value_types(i)
				length = Len( name )

				If Left( param, length ) = name Then
					Select Case  Mid( param, length+1, 1 )
						Case  ":", "="
							If value_type <> multi_value Then
								options( Left( param, length ) + colons( value_type ) ) = Mid( param, length + 2 )
							Else
								If IsEmpty( options( name + colons( value_type ) ) ) Then _
									Set options( name + colons( value_type ) ) = new ArrayClass
								options( Left( param, length ) + colons( value_type ) ).Add  Mid( param, length + 2 )
							End If
						Case  ""
							If value_type <> normal Then
								is_next_value = True
							Else
								options( param ) = True
							End If
						Case Else
							If value_type <> multi_value Then
								options( Left( param, length ) + colons( value_type ) ) = Mid( param, length + 1 )
							Else
								If IsEmpty( options( name + colons( value_type ) ) ) Then _
									Set options( name + colons( value_type ) ) = new ArrayClass
								options( Left( param, length ) + colons( value_type ) ).Add  Mid( param, length + 1 )
							End If
					End Select
					Exit For
				End If
			Next
			If i = UBound( OptionNames ) + 1 Then
				options( "no name" ).Add  param
			End If
		Else
			If value_type = space_value Then
				options( name + colons( value_type ) ) = param
			Else
				If IsEmpty( options( name + colons( value_type ) ) ) Then _
					Set options( name + colons( value_type ) ) = new ArrayClass
				options( name + colons( value_type ) ).Add  param
			End If
			is_next_value = False
		End If
	Next
	Set DicFromCmdLineOpt = options
End Function


 
'*************************************************************************
'  <<< [ModifyCmdLineOpt] >>> 
'*************************************************************************
Function  ModifyCmdLineOpt( CmdLine, ByVal OptionName, NewOptionNameAndParam )
	Dim  params : params = ArrayFromCmdLine( CmdLine )
	Dim  param,  value_type,  length,  is_replaced,  is_next_replaced,  ret
	Dim  old_multi_value,  old_param
	Const  normal      = 0
	Const  space_value = 1
	Const  multi_value = 2


	'// value_type : normal, space_value, multi_value
	'// OptionName : cut last of ":"
	If InStr( OptionName, ":" ) = 0 Then
		value_type = normal
	ElseIf InStr( OptionName, "::" ) = 0 Then
		value_type = space_value
		length = InStr( OptionName, ":" )
		OptionName = Left( OptionName, length - 1 )
	Else
		value_type = multi_value
		length = InStr( OptionName, "::" )
		old_multi_value = Mid( OptionName, length + 2 )
		OptionName = Left( OptionName, length - 1 )
	End If


	'// ret : replace to NewOptionNameAndParam
	ret = ""
	is_replaced = False
	is_next_replaced = False
	For Each  param  In params
		If not is_next_replaced Then
			length = Len( OptionName )
			If Left( param, length ) = OptionName Then
				If value_type <> multi_value Then
					If not is_replaced Then
						If not IsEmpty( NewOptionNameAndParam ) Then _
							ret = ret +" "+ NewOptionNameAndParam
						If value_type = space_value Then  If Mid( param, length + 1, 1 ) = "" Then _
							is_next_replaced = True
						is_replaced = True
					Else
						ret = ret +" "+ param
					End If
				Else  '// value_type = multi_value
					If old_multi_value = "" Then
						ret = ret +" "+ NewOptionNameAndParam +" "+ param
						value_type = normal : is_replaced = True
					Else
						Set old_param = DicFromCmdLineOpt( param, Array( OptionName ) )
						If old_param( OptionName ) = True Then
							is_next_replaced = True
						Else
							If old_param( OptionName ) = old_multi_value Then
								If not IsEmpty( NewOptionNameAndParam ) Then _
									ret = ret +" "+ NewOptionNameAndParam
								value_type = normal : is_replaced = True
							Else
								ret = ret +" "+ param
							End IF
						End IF
					End If
				End If
			Else
				ret = ret +" "+ param
			End If
		Else
			If value_type = multi_value Then
				If param = old_multi_value Then
					If not IsEmpty( NewOptionNameAndParam ) Then _
						ret = ret +" "+ NewOptionNameAndParam
					value_type = normal : is_replaced = True
				Else
					ret = ret +" "+ OptionName +" "+ param
				End If
			End If
			is_next_replaced = False
		End If
	Next
	If not is_replaced Then
		If IsEmpty( NewOptionNameAndParam ) Then
			ModifyCmdLineOpt = CmdLine
		Else
			ModifyCmdLineOpt = NewOptionNameAndParam +" "+ CmdLine
		End If
	Else
		ModifyCmdLineOpt = Mid( ret, 2 )
	End If
End Function


 
'*************************************************************************
'  <<< [GetCommandLineOptionName] >>> 
'*************************************************************************
Function  GetCommandLineOptionName( OneParameter )
	Dim  i
	Dim  s : s = OneParameter

	s = Trim( s )
	If Left( s, 1 ) = "/" Then
		i = InStr( s, ":" )
		If i = 0 Then
			GetCommandLineOptionName = Mid( s, 2 )
		Else
			GetCommandLineOptionName = Mid( s, 2, i-2 )
		End If
	Else
		GetCommandLineOptionName = ""
	End If
End Function


 
'*************************************************************************
'  <<< [GetCommandLineOptionValue] >>> 
'*************************************************************************
Function  GetCommandLineOptionValue( OneParameter )
	Dim  i
	Dim  s : s = OneParameter

	s = Trim( s )
	i = InStr( s, ":" )
	If i > 0 Then
		GetCommandLineOptionValue = ArrayFromCmdLine( Mid( s, i+1 ) )(0)
	Else
		GetCommandLineOptionValue = ""
	End If
End Function


 
'*************************************************************************
'  <<< [ArrayFromCmdLine] >>> 
'*************************************************************************
Function  ArrayFromCmdLine( CmdLine )
	Dim  i, s, arr

	ReDim  arr(-1)
	i = 1
	Do
		ReDim Preserve  arr( UBound( arr ) + 1 )
		arr( UBound( arr ) ) = MeltCmdLine_sub( CmdLine, i, "DOS" )
		If i = 0 Then  ArrayFromCmdLine = arr : Exit Function
	Loop
End Function


 
'*************************************************************************
'  <<< [ArrayFromCmdLineWithoutOpt] >>> 
'*************************************************************************
Function  ArrayFromCmdLineWithoutOpt( CommandLine, ByVal OptionSigns )
	Dim  arr,  i, ii, j, is_opt

	If IsEmpty( OptionSigns ) Then  OptionSigns = Array( "-", "/" )

	arr = ArrayFromCmdLine( CommandLine )
	ii = 0
	For i=0 To UBound( arr )
		is_opt = False
		For j=0 To UBound( OptionSigns )
			If StrCompHeadOf( arr(i), OptionSigns(j), Empty ) = 0 Then  is_opt = True
		Next
		If not is_opt Then
			arr(ii) = arr(i)
			ii = ii + 1
		End If
	Next
	ReDim Preserve  arr(ii-1)
	ArrayFromCmdLineWithoutOpt = arr
End Function


 
'*************************************************************************
'  <<< [MeltCmdLine] >>> 
'*************************************************************************
Function  MeltCmdLine( CmdLine, in_out_Index1 )
	MeltCmdLine = MeltCmdLine_sub( CmdLine, in_out_Index1, "DOS" )
End Function

Function  MeltCmdLine_sub( CmdLine, in_out_Index1, ShellType )
	Dim  i, c, ret,  bs_count
	Dim  is_in_quot : is_in_quot = False
	Dim  is_skip : is_skip = False

	'// skip space
	Do
		c = Mid( CmdLine, in_out_Index1, 1 )
		If c <> " " Then  Exit Do
		in_out_Index1 = in_out_Index1 + 1
	Loop

	If c = "" Then  in_out_Index1 = 0 : Exit Function

	'// melt one parameter
	ret = ""
	Do
		Select Case  c

			Case  ""
				in_out_Index1 = 0
				Exit Do

			Case  """"
				is_in_quot = not is_in_quot

			Case  " "
				If is_in_quot Then
					ret = ret + c
				Else
					Do
						in_out_Index1 = in_out_Index1 + 1
						c = Mid( CmdLine, in_out_Index1, 1 )
						If c <> " " Then  Exit Do
					Loop
					If c = "" Then  in_out_Index1 = 0
					Exit Do
				End If

			Case  "\"
				If ShellType = "DOS" Then
					bs_count = 1
					Do
						in_out_Index1 = in_out_Index1 + 1
						c = Mid( CmdLine, in_out_Index1, 1 )

						If c <> "\" Then  Exit Do
						bs_count = bs_count + 1
					Loop
					If c = """" Then
						If bs_count Mod 2 = 1 Then
							ret = ret + String( ( bs_count - 1 ) / 2, "\" ) + """"
						Else
							ret = ret + String( bs_count / 2, "\" )
							is_in_quot = not is_in_quot
						End If
					Else
						ret = ret + String( bs_count, "\" )
						is_skip = True
					End If

				Else  '// bash
					in_out_Index1 = in_out_Index1 + 1
					c = Mid( CmdLine, in_out_Index1, 1 )
					Select Case  c
						Case  "\"
							ret = ret + "\"
						Case  """"
							ret = ret + """"
						Case Else
							ret = ret + "\" + c
					End Select
				End If

			Case Else
				ret = ret + c
		End Select

		If not is_skip Then
			in_out_Index1 = in_out_Index1 + 1
			c = Mid( CmdLine, in_out_Index1, 1 )
		Else
			is_skip = False
		End If
	Loop
	MeltCmdLine_sub = ret
End  Function


 
'*************************************************************************
'  <<< [CmdLineFromStr] >>> 
'*************************************************************************
Function  CmdLineFromStr( StrOrArray )
	Dim  arr : If IsArray( StrOrArray ) Then  arr = StrOrArray  Else  arr = Array( StrOrArray )
	Dim  ret, i, str,  back_sh_pos,  back_sh_count

	ret = ""
	For i=0 To UBound( arr )
		str = arr(i)
		If i >= 1 Then  ret = ret + " "

		'//  \" -> \\"
		back_sh_pos = 1
		Do
			back_sh_pos = InStr( back_sh_pos,  str, "\""" )
			If back_sh_pos = 0 Then  Exit Do
			back_sh_count = 1

			Do
				If back_sh_pos = 1 Then  Exit Do
				If Mid( str, back_sh_pos - 1, 1 ) <> "\" Then  Exit Do
				back_sh_pos = back_sh_pos - 1  '// pos of first \
				back_sh_count = back_sh_count + 1
			Loop

			str = Left( str, back_sh_pos - 1 ) + String( back_sh_count, "\" ) + _
					Mid( str, back_sh_pos )

			back_sh_pos = back_sh_pos + back_sh_count * 2 + 1
		Loop


		'// " -> \"
		str = Replace( str, """", "\""" )

		'// string -> "string"
		If InStrEx( str, Array( " ", "<", ">", "^", "|" ), 1, Empty ) > 0 Then _
			str = """"+ str +""""

		ret = ret + str
	Next
	CmdLineFromStr = ret
End Function


 
'*************************************************************************
'  <<< [GetWScriptArgumentsUnnamed] >>> 
'*************************************************************************
Function  GetWScriptArgumentsUnnamed()
	Set  args = new ArrayClass
	For index = 0  To WScript.Arguments.Unnamed.Count - 1
		args.Add  ParseWScriptArgumentQuotation( WScript.Arguments.Unnamed( index ) )
	Next
	Set GetWScriptArgumentsUnnamed = args
End Function


 
'*************************************************************************
'  <<< [GetWScriptArgumentsNamed] >>> 
'*************************************************************************
Function  GetWScriptArgumentsNamed( OptionName )
	GetWScriptArgumentsNamed = ParseWScriptArgumentQuotation( _
		WScript.Arguments.Named( OptionName ) )
End Function


 
'*************************************************************************
'  <<< [ParseWScriptArgumentQuotation] >>> 
'*************************************************************************
Function  ParseWScriptArgumentQuotation( ByVal Value )
	Set re = new_RegExp( "\\+'", False )
	Set matches = re.Execute( Value )
	index = 0
	out = ""
	If IsEmpty( Value ) Then  Exit Function  '// Return Empty

	For Each match  In matches

		'// Add a string before matched string
		length = match.FirstIndex - index
		out = out + Mid( Value, index + 1, length )
		index = index + length

		'// Add replaced matched string
		length = match.Length - 1
		If length mod 2 = 0 Then
			out = out + String( length / 2, "\" ) + "'"
		Else
			out = out + String( ( length - 1 ) / 2, "\" ) + """"
		End If
		index = index + match.Length
	Next

	ParseWScriptArgumentQuotation = out + Mid( Value, index + 1 )
End Function


 
'*************************************************************************
'  <<< [ArrayFromBashCmdLine] >>> 
'*************************************************************************
Function  ArrayFromBashCmdLine( CmdLine )
	Dim  i, s, arr

	ReDim  arr(-1)
	i = 1
	Do
		ReDim Preserve  arr( UBound( arr ) + 1 )
		arr( UBound( arr ) ) = MeltCmdLine_sub( CmdLine, i, "bash" )
		If i = 0 Then  ArrayFromBashCmdLine = arr : Exit Function
	Loop
End Function


 
'*************************************************************************
'  <<< [MeltBashCmdLine] >>> 
'*************************************************************************
Function  MeltBashCmdLine( CmdLine, in_out_Index1 )
	MeltBashCmdLine = MeltCmdLine_sub( CmdLine, in_out_Index1, "bash" )
End Function


 
'*************************************************************************
'  <<< [new_ParentProcess] >>> 
'*************************************************************************
Function  new_ParentProcess()
	Dim  c : Set c = g_VBS_Lib
	Dim  Me_ : Set Me_ = new ParentChildProcess
	Me_.m_TempPath = GetTempPath( "ChildProcess_?" )
	Me_.m_FileID = Mid( Me_.m_TempPath, InStrRev( Me_.m_TempPath, "_" ) + 1 )
	Me_.m_DateID = g_fs.GetFileName( g_fs.GetParentFolderName( Me_.m_TempPath ) )
	Dim  ec : Set ec = new EchoOff
	Set Me_.m_OutFile = OpenForWrite( Me_.m_TempPath +"\In.xml", c.Unicode )
	ec = Empty
	Me_.m_OutFile.WriteLine  "<ToChildProcess>"
	Me_.m_OutFile_LastLine = "</ToChildProcess>"
	Set new_ParentProcess = Me_
End Function


 
'*************************************************************************
'  <<< [ParentChildProcess] >>> 
'*************************************************************************
Class  ParentChildProcess
	Public  m_DateID    ' as string. e.g.:"090401"
	Public  m_FileID    ' as string. e.g.:"1"
	Public  m_TempPath  ' as string. Folder path creating communication files
	Public  m_InXML     ' as IXMLDOMElement or Empty. Root XML element
	Public  m_OutFile   ' as TextStream (Write)
	Public  m_OutFile_LastLine ' as string. "</ToChildProcess>" or  "</ToParentProcess>"
	Public  m_bFinished

	Sub  OnCallInParent()
		g_InterProcess.OnCallInParent  Me

		m_OutFile.WriteLine  m_OutFile_LastLine
		m_OutFile.Close
		m_OutFile = Empty
	End Sub

	Sub  OnReturnInParent( ReturnedValue )
		Dim  ec
		Dim  c : Set c = g_VBS_Lib
		Dim  in_path : in_path = m_TempPath +"\Out.xml"
		If g_fs.FileExists( in_path ) Then
			Set ec = new EchoOff
			Set m_InXML = LoadXML( in_path, Empty )
			ec = Empty
			g_InterProcess.OnReturnInParent  Me
		Else
			If ReturnedValue <> 21 Then
				If ReturnedValue = 22 Then
					Raise  ReturnedValue, "<SKIP/>"
				ElseIf ReturnedValue = 0 Then
					Raise  E_Others, "Error was raised in cscript(errorlevel=0)"
				Else
					Raise  ReturnedValue, "Error was raised in cscript"
				End If
			End If
		End If

		Set ec = new EchoOff
		Set m_OutFile = OpenForWrite( m_TempPath +"\In.xml", c.Unicode )
		ec = Empty
		m_OutFile.WriteLine  "<ToChildProcess>"
		m_OutFile_LastLine = "</ToChildProcess>"
	End Sub

	Sub Finish()
		If m_OutFile_LastLine = "</ToChildProcess>" Then
			If not IsEmpty( m_OutFile ) Then  m_OutFile.Close : m_OutFile = Empty
			Dim  temp_path : temp_path = env( "%TEMP%" )
			If temp_path <> "%TEMP%"  and  Left( m_TempPath, Len( temp_path ) ) = temp_path Then
				If Err.Number = 0 Then
					Dim ec : Set ec = new EchoOff
					del  m_TempPath
					ec = Empty
				Else
					g_fs.DeleteFolder  m_TempPath  '// don't del because Err.Number becomes 0(clear)
				End If
			End If
		Else  '// </ToParentProcess>
			g_InterProcess.OnReturnInChild  Me

			m_OutFile.WriteLine  m_OutFile_LastLine
			m_OutFile.Close
		End If
		m_bFinished = True
	End Sub

	Sub Class_Terminate()
		If not m_bFinished Then  Finish
	End Sub
End Class


 
'*************************************************************************
'  <<< [get_ChildProcess] >>> 
'*************************************************************************
Dim  g_ChildProcess

Function  get_ChildProcess()
	Set get_ChildProcess = g_ChildProcess
End Function

Sub  new_ChildProcess_ifChildProcess()
	Dim  opt : opt = WScript.Arguments.Named.Item("ChildProcess")
	If IsEmpty( opt ) or opt="0" Then  Set g_ChildProcess = Nothing : Exit Sub
	Dim xml : Set xml = CreateObject("MSXML2.DOMDocument")

	'// Initialize g_ChildProcess
	Dim  i : i = 1
	Dim  m : Set m = new ParentChildProcess
		m.m_DateID = MeltCSV( opt, i )
		m.m_FileID = MeltCSV( opt, i )
		m.m_TempPath = env( "%TEMP%\Report\" ) + m.m_DateID +"\ChildProcess_"+ m.m_FileID
		If Left( m.m_TempPath, 6 ) = "%TEMP%" Then  Error
		If not xml.load( m.m_TempPath +"\In.xml" ) Then  Error
		Set m.m_InXML = xml.lastChild
		Set m.m_OutFile = g_fs.CreateTextFile( m.m_TempPath +"\Out.xml", True, True )
			'// Can not use OpenForWrite. Because no g_AppKey
		m.m_OutFile.WriteLine  "<ToParentProcess>"
		m.m_OutFile_LastLine = "</ToParentProcess>"
		m.m_bFinished = True  '// Disable Class_Terminate but enable Finish
	Set g_ChildProcess = m
End Sub


 
'*************************************************************************
'  <<< [InterProcess] >>> 
'*************************************************************************
Dim  g_InterProcess

Class InterProcess
	Public  InterProcessDataArray '// as ArrayClass of InterProcessData

	Public  ProcessCallID  '// as Array Class of integer
	Public  InterProcessUserData  '// as string

	Private Sub  Class_Initialize()
		Set Me.InterProcessDataArray = new ArrayClass
		Set Me.ProcessCallID = new ArrayClass
		Me.ProcessCallID.Add  0
	End Sub

	Property Get  xml()
		xml = "<"+ TypeName( Me ) +" call_id='"+ Me.ProcessCallID.CSV + "' user_data='"& Me.InterProcessUserData &"'/>"
	End Property

	Public Sub  loadXML( XmlObject )
		Dim i,s  : i = 1
		Dim line : line = XmlObject.getAttribute( "call_id" )
		Me.ProcessCallID.ReDim_  -1
		Do
			s = MeltCSV( line, i )
			If not IsEmpty( s ) Then  Me.ProcessCallID.Add  CInt2( s )
			If i = 0 Then Exit Do
		Loop

		Me.InterProcessUserData = XmlObject.getAttribute( "user_data" )
	End Sub

	Public Sub  OnCallInParent( a_ParentProcess )
		Dim  obj
		For Each obj  In Me.InterProcessDataArray.Items
			obj.OnCallInParent  a_ParentProcess
		Next
		Me.ProcessCallID( Me.ProcessCallID.UBound_ ) = Me.ProcessCallID( Me.ProcessCallID.UBound_ ) + 1
		a_ParentProcess.m_OutFile.WriteLine  Me.xml

		If g_is_debug Then
			echo "<ChildProcess call_id='"+ Me.ProcessCallID.CSV +"' folder='"+ _
				a_ParentProcess.m_DateID +"\ChildProcess_"+ a_ParentProcess.m_FileID +"'/>"
		End If
	End Sub

	Public Sub  OnCallInChild( a_ChildProcess )
		Me.loadXML  a_ChildProcess.m_InXML.selectSingleNode( TypeName(Me) )
		Me.ProcessCallID.Add  0
	End Sub

	Public Sub  OnReturnInChild( a_ChildProcess )
		Dim  obj
		Me.ProcessCallID.ReDim_  Me.ProcessCallID.UBound_ - 1
		a_ChildProcess.m_OutFile.WriteLine  Me.xml
		For Each obj  In Me.InterProcessDataArray.Items
			obj.OnReturnInChild  a_ChildProcess
		Next
	End Sub

	Public Sub  OnReturnInParent( a_ParentProcess )
		Dim  obj, call_id_back

		Set call_id_back = new ArrayClass
		call_id_back.Copy  Me.ProcessCallID
		Me.loadXML  a_ParentProcess.m_InXML.selectSingleNode( TypeName(Me) )
		Me.ProcessCallID.Copy  call_id_back '// Me.ProcessCallID do not update by Child Process

		For Each obj  In Me.InterProcessDataArray.Items
			obj.OnReturnInParent  a_ParentProcess
		Next

		'// Error raised from sub process is raised from main process
		If g_Err2.Num <> 0 Then
			If g_Err2.Number = 21  Then
				Dim  pos : pos = g_Err2.PrevErrNestPos
				g_Err2.Clear  '// Pass code in exit code is not error
				g_Err2.PrevErrNestPos = pos  '// resume PrevErrNestPos
			Else
				g_Err2.Raise
			End If
		End If
	End Sub
End Class


Class  InterProcessData  '// defined_as_interface
	Public Sub  OnCallInParent( a_ParentProcess ) : End Sub
	Public Sub  loadXML( XmlObject ) : End Sub
	Public Sub  OnReturnInChild( a_ChildProcess ) : End Sub
	Public Sub  OnReturnInParent( a_ParentProcess ) : End Sub
End Class


 
'*************************************************************************
'  <<< [GetEditorCmdLine] >>> 
'  <<< [GetSearchOpenCmdLine] >>>
'*************************************************************************
Function  GetSearchOpenCmdLine( PathAndFragment )
	ThisIsOldSpec
	GetSearchOpenCmdLine = GetEditorCmdLine( Replace( PathAndFragment, "%d", "%L" ) )
End Function

Function  GetEditorCmdLine( PathAndFragment )
	Dim  cmd,  no_jump_cmd, jumps

	Set jumps = GetTagJumpParams( PathAndFragment )


	'//=== Check path
	If not g_fs.FileExists( jumps.Path ) Then _
		Raise  E_FileNotExist, "<ERROR msg='Not found file' path='"+ GetFullPath( jumps.Path, Empty ) +"'/>"


	'//=== Get command line template. Set to cmd, simple_cmd
	If not IsDefined( "Setting_getEditorCmdLine" ) Then
		cmd = """C:\Windows\notepad.exe"" ""%1"""
		no_jump_cmd = cmd
	Else
		If not IsEmpty( jumps.Keyword ) Then
			cmd = Setting_getEditorCmdLine( 3 )
			If InStr( cmd, "%2" ) = 0 Then  cmd = Empty
		End If
		If IsEmpty( cmd ) Then  cmd = Setting_getEditorCmdLine( 2 )

		no_jump_cmd = Setting_getEditorCmdLine( 1 )
		If IsEmpty( no_jump_cmd ) Then
			no_jump_cmd = Setting_getEditorCmdLine( 0 )
			If IsEmpty( no_jump_cmd ) Then
				no_jump_cmd = env( """%windir%" ) +"\notepad.exe"" ""%1"""
			Else
				no_jump_cmd = """" + no_jump_cmd + """ ""%1"""
			End If
		End If

		If IsEmpty( cmd ) Then  cmd = no_jump_cmd
	End If


	'//=== Replace command line
	If jumps.Keyword <> "" Then
		If InStr( cmd, "%2" ) > 0 Then
			cmd = Replace( cmd, "%2", jumps.Keyword )
		ElseIf InStr( cmd, "%L" ) > 0 Then
			If IsEmpty( jumps.LineNum ) Then _
				jumps.LineNum = GetLineOfSearchOpen( jumps.Path, jumps.Keyword )
			cmd = Replace( cmd, "%L", CStr( jumps.LineNum ) )
		End If
	ElseIf not IsEmpty( jumps.LineNum ) Then
		cmd = Replace( cmd, "%L", CStr( jumps.LineNum ) )
	Else
		cmd = no_jump_cmd
	End If

	GetEditorCmdLine = Replace( cmd, "%1", GetFullPath( jumps.Path, Empty ) )
End Function


Function  GetLineOfSearchOpen( Path, Name )
	Dim  f, line, i

	Set f = OpenForRead( Path )
	i = 1
	Do Until  f.AtEndOfStream
		line = f.ReadLine()
		If InStr( line, Name ) > 0 Then
			GetLineOfSearchOpen = i
			Exit Function
		End If
		i = i + 1
	Loop
	f = Empty
	GetLineOfSearchOpen = 1
End Function


 
'*************************************************************************
'  <<< [GetEditorCmdLine_DefaultParams] >>> 
'*************************************************************************
Function  GetEditorCmdLine_DefaultParams( i, ExeName, OptType2 )
	Dim  ret

	If i = 1 Then  ret = ExeName +" ""%1"""
	If i = 2 Then  ret = ExeName +" "+ OptType2
	If i = 3 Then  ret = ExeName +" ""%1#%2"""

	GetEditorCmdLine_DefaultParams = ret
End Function


 
'*************************************************************************
'  <<< [GetDiffCmdLine] >>> 
'*************************************************************************
Function  GetDiffCmdLine( PathA, PathB )
	Dim  jumps_a,  jumps_b,  cmd,  line_num

	If not IsDefined( "Setting_getDiffCmdLine" ) Then _
		ExecuteGlobal  "Function  Setting_getDiffCmdLine(t) : End Function"

	Set jumps_a = GetTagJumpParams( PathA )
	Set jumps_b = GetTagJumpParams( PathB )

	If not IsEmpty( jumps_a.Keyword ) and IsEmpty( jumps_a.LineNum ) Then _
		jumps_a.LineNum = GetLineOfSearchOpen( jumps_a.Path, jumps_a.Keyword )
	If not IsEmpty( jumps_b.Keyword ) and IsEmpty( jumps_b.LineNum ) Then _
		jumps_b.LineNum = GetLineOfSearchOpen( jumps_b.Path, jumps_b.Keyword )

	If not IsEmpty( jumps_a.LineNum ) Then
		cmd = Setting_getDiffCmdLine( 21 )
		line_num = jumps_a.LineNum
	ElseIf not IsEmpty( jumps_b.LineNum ) Then
		cmd = Setting_getDiffCmdLine( 22 )
		line_num = jumps_b.LineNum
	End If
	If IsEmpty( cmd ) Then  cmd = Setting_getDiffCmdLine( 2 )
	If IsEmpty( cmd ) Then
		cmd = Setting_getDiffCmdLine( 0 )
		If IsEmpty( cmd ) Then
			echo  "Diff  """ + PathA + """ """ + PathB + """"
			Exit Function
		Else
			cmd = """"+ cmd +""" ""%1"" ""%2"""
		End IF
	End IF

	cmd = Replace( cmd, "%1", jumps_a.Path )
	cmd = Replace( cmd, "%2", jumps_b.Path )
	cmd = Replace( cmd, "%L", line_num )

	GetDiffCmdLine = cmd
End Function


 
'*************************************************************************
'  <<< [GetDiffCmdLine3] >>> 
'*************************************************************************
Function  GetDiffCmdLine3( PathA, PathB, PathC )
	Dim  jumps_a,  jumps_b,  jumps_c,  cmd,  line_num

	If not IsDefined( "Setting_getDiffCmdLine" ) Then _
		ExecuteGlobal  "Function  Setting_getDiffCmdLine(t) : End Function"

	Set jumps_a = GetTagJumpParams( PathA )
	Set jumps_b = GetTagJumpParams( PathB )
	Set jumps_c = GetTagJumpParams( PathC )

	If not IsEmpty( jumps_a.Keyword ) and IsEmpty( jumps_a.LineNum ) Then _
		jumps_a.LineNum = GetLineOfSearchOpen( jumps_a.Path, jumps_a.Keyword )
	If not IsEmpty( jumps_b.Keyword ) and IsEmpty( jumps_b.LineNum ) Then _
		jumps_b.LineNum = GetLineOfSearchOpen( jumps_b.Path, jumps_b.Keyword )
	If not IsEmpty( jumps_c.Keyword ) and IsEmpty( jumps_c.LineNum ) Then _
		jumps_c.LineNum = GetLineOfSearchOpen( jumps_c.Path, jumps_c.Keyword )

	If not IsEmpty( jumps_a.LineNum ) Then
		cmd = Setting_getDiffCmdLine( 31 )
		line_num = jumps_a.LineNum
	ElseIf not IsEmpty( jumps_b.LineNum ) Then
		cmd = Setting_getDiffCmdLine( 32 )
		line_num = jumps_b.LineNum
	ElseIf not IsEmpty( jumps_c.LineNum ) Then
		cmd = Setting_getDiffCmdLine( 33 )
		line_num = jumps_c.LineNum
	End If
	If IsEmpty( cmd ) Then  cmd = Setting_getDiffCmdLine( 3 )
	If IsEmpty( cmd ) Then
		If IsEmpty( cmd ) Then
			cmd = Setting_getDiffCmdLine( 0 )
			If IsEmpty( cmd ) Then
				echo  "Diff  """ + PathA + """ """ + PathB + """ """ + PathC + """"
				Exit Function
			Else
				cmd = """"+ cmd +""" ""%1"" ""%2"""
			End IF
		End IF
		cmd = "cscript """+ g_vbslib_ver_folder +"diff3to2\diff3to2.vbs"" /DiffTool:"""+ _
			Setting_getDiffCmdLine( 0 ) +""" ""%1"" ""%2"" ""%3"""
	End If

	cmd = Replace( cmd, "%1", jumps_a.Path )
	cmd = Replace( cmd, "%2", jumps_b.Path )
	cmd = Replace( cmd, "%3", jumps_c.Path )
	cmd = Replace( cmd, "%L", line_num )

	GetDiffCmdLine3 = cmd
End Function


 
'*************************************************************************
'  <<< [GetDiffCmdLineMulti] >>> 
'*************************************************************************
Function  GetDiffCmdLineMulti( Files )
	Dim  op, cmd, i

	echo "--------------------------------------------------------"
	For i=0 To UBound( Files )
		echo (i+1) & ". " & Files(i)(0)
	Next
	op = CInt2( input( "Select number>" ) ) - 1
	echo "--------------------------------------------------------"


	Select Case UBound( Files(op)(1) )

		Case 1:  '// 2 files
			GetDiffCmdLineMulti = GetDiffCmdLine( _
				GetFullPath( Files(op)(1)(0) +"\"+ Files(op)(0), Empty ), _
				GetFullPath( Files(op)(1)(1) +"\"+ Files(op)(0), Empty ) )

		Case 2:  '// 3 files
			GetDiffCmdLineMulti = GetDiffCmdLine3( _
				GetFullPath( Files(op)(1)(0) +"\"+ Files(op)(0), Empty ), _
				GetFullPath( Files(op)(1)(1) +"\"+ Files(op)(0), Empty ), _
				GetFullPath( Files(op)(1)(2) +"\"+ Files(op)(0), Empty ) )

		Case Else
			Error
	End Select
End Function


 
'*************************************************************************
'  <<< [GetDiffCmdLine_DefaultParams] >>> 
'*************************************************************************
Function  GetDiffCmdLine_DefaultParams( i, ExeName )
	Dim  ret

	If i = 0 Then  ret = ExeName
	If i = 2 Then  ret = ExeName +" ""%1"" ""%2"""
	If i = 3 Then  ret = ExeName +" ""%1"" ""%2"" ""%3"""
	If i = 21 Then ret = ExeName +" -i=21 ""%1"" ""%2"""
	If i = 22 Then ret = ExeName +" -i=22 ""%1"" ""%2"""
	If i = 31 Then ret = ExeName +" -i=31 ""%1"" ""%2"" ""%3"""
	If i = 32 Then ret = ExeName +" -i=32 ""%1"" ""%2"" ""%3"""
	If i = 33 Then ret = ExeName +" -i=33 ""%1"" ""%2"" ""%3"""

	GetDiffCmdLine_DefaultParams = ret
End Function


 
'*************************************************************************
'  <<< [DiffCUI] >>> 
'*************************************************************************
Function  DiffCUI( FilePaths, ByVal FileLabels )
	Dim  file_num,  file_ubound,  compare_num, key,  options,  temp_path
	Dim  exist_count, exist_num1, exist_num2
	ReDim  compare_files(1)
	Dim  c : Set c = g_VBS_Lib

	file_ubound = UBound( FilePaths )
	If IsEmpty( FileLabels ) Then
		FileLabels = GetPathLabels( FilePaths )
	End If
	Assert  file_ubound + 1 >= 2  and  file_ubound + 1 <= 6

	Do
		echo_line
		echo  " 1. Open difference information by using text editor"
		exist_count = 0 : exist_num1 = Empty
		For file_num = 0  To file_ubound
			If g_fs.FileExists( FilePaths( file_num ) ) Then
				echo  " " & (file_num + 4) & ". Open file : " & FileLabels( file_num )
				exist_count = exist_count + 1
				If IsEmpty( exist_num1 ) Then  exist_num1 = file_num  Else  exist_num2 = file_num
			Else
				echo  " *. " & FileLabels( file_num ) & " file is not found"
			End If
		Next
		echo  " **. copy (source(4 and over) x 10 + destination(4 and over)"
		echo  "     e.g.: 45 = "+ FileLabels( 0 ) +" -> "+ FileLabels( 1 )
		echo  " 99. return"

		Do
			key = CInt2( input( "Input operation number >" ) )
			If key = 1  or  key = 99 Then  Exit Do
			If key >= 4  and  key <= 4 + file_ubound Then
				If g_fs.FileExists( FilePaths( key - 4 ) ) Then _
					Exit Do
			End If
			If key >= 44  and  key <= (file_ubound + 4) * 10 + 9 Then
				If key mod 10 >= 4  and  key mod 10 <= file_ubound + 4 Then _
					Exit Do
			End If
		Loop
		echo_line

		If key = 99 Then  Exit Do

		'// fc command
		If key = 1 Then
			If exist_count = 2 Then
				compare_files( 0 ) = exist_num1
				compare_files( 1 ) = exist_num2
			Else
				For compare_num = 1 To 2
					echo  "Select comparing No." & compare_num & " file"
					For file_num = 0  To file_ubound
						If g_fs.FileExists( FilePaths( file_num ) ) Then _
							echo  " " & (file_num + 4) & ". " & FileLabels( file_num ) & " file"
					Next
					Do
						key = CInt2( input( "Input number >" ) )
						If key >= 4  and  key <= file_ubound + 4 Then  Exit Do
					Loop
					compare_files( compare_num - 1 ) = key - 4
				Next
			End If

			temp_path = GetTempPath( "fc_*.txt" )

			If ReadUnicodeFileBOM( FilePaths( compare_files( 0 ) ) ) = c.Unicode Then
				options = " /N /U"
			Else
				options = " /N"
			End If
			Set ec = new EchoOff
			RunProg  "fc"+ options +" """+ FilePaths( compare_files( 0 ) ) +""" """+ _
				FilePaths( compare_files( 1 ) ) +"""", temp_path
			ec = Empty

			start  GetEditorCmdLine( temp_path )
			key = 1  '// goto next loop
		End If

		'// start Editor
		If key >= 4  and  key <= 4 + file_ubound Then
			start  GetEditorCmdLine( FilePaths( key - 4 ) )
		End If

		'// copy
		If key >= 44 Then
			compare_files( 0 ) = Fix( key / 10 ) - 4
			compare_files( 1 ) = key mod 10 - 4
			key = input( "Source: "+ FileLabels( compare_files( 0 ) ) +_
													 " : "+ FilePaths(  compare_files( 0 ) ) + vbCRLF +_
									 "Destination: "+ FileLabels( compare_files( 1 ) ) +_
													 " : "+ FilePaths(  compare_files( 1 ) ) + vbCRLF +_
									 "Do you overwrite copy?[Y/N]" )
			If key = "Y"  or  key = "y" Then
				copy_ren  FilePaths( compare_files( 0 ) ), FilePaths( compare_files( 1 ) )
			End If
		End If
	Loop
End Function


 
'*-------------------------------------------------------------------------*
'* ### <<<< Wait >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [Sleep] >>> 
'*************************************************************************
Sub  Sleep( ByVal msec )
	'// echo  ">Sleep  " & msec
	WScript.Sleep msec
End Sub


 
'*************************************************************************
'  <<< [WaitForFile] Wait for make the file >>> 
'*************************************************************************
Function  WaitForFile( Path )
	echo  ">WaitForFile  " & Path
	Set c = g_VBS_Lib

	'//=== Wait for file exists
	f = 0
	While g_fs.FileExists( Path ) = False
		WScript.Sleep 1000
		f=f+1 : If f=3 Then  WScript.Echo  ">WaitForFile  " & Path & " ..."
	Wend


	'//=== read as xml
	n_retry = 0
	Do
		ErrCheck : On Error Resume Next

			Set xml = CreateObject("MSXML2.DOMDocument")
			is_xml = ( xml.load( Path ) <> 0 )

		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If not IsWriteAccessDenied( en, Path, c.File, n_retry ) Then  Exit Do
	Loop
	If en <> 0 Then  Err.Raise en,,ed


	If is_xml Then

		WaitForFile = xml.lastChild.xml

	Else

		'//=== Open file supported lock
		n_retry = 0
		Do
			ErrCheck : On Error Resume Next

				Set f = g_fs.OpenTextFile( Path,,,-2 )

			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If not IsWriteAccessDenied( en, Path, c.File, n_retry ) Then  Exit Do
		Loop
		If en <> 0 Then  Err.Raise en,,ed


		'//=== Read file supported lock
		n_retry = 0
		Do
			ErrCheck : On Error Resume Next
				WaitForFile = f.ReadLine
			en = Err.Number : ed = Err.Description : On Error GoTo 0
			If en <> E_EndOfFile Then
				If en <> 0 Then  Err.Raise en,,ed
				Exit Do
			End If

			Sleep  250  '// wait for close of write process
			n_retry = n_retry + 1
			If n_retry = 4 Then  WaitForFile = "" : Exit Do
		Loop
		f = Empty
	End If


	'//=== Delete file
	del  Path
	While  g_fs.FileExists( Path )
		WScript.Sleep 200  '// Delete may have delay ?
	WEnd


	'//=== Raise error
	If Left( WaitForFile, 6 ) = "<ERROR" Then
		num = xml.lastChild.getAttribute( "num" )
		If IsNull( num ) Then
			Raise  1, WaitForFile
		Else
			Raise  CInt(num), WaitForFile
		End If
	End If
End Function


 
'*-------------------------------------------------------------------------*
'* ### <<<< Sound >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [Play] >>> 
'*************************************************************************
Sub  Play( Path )
	Player_validate '// g_Player

	With g_Player.m_Obj
		.URL = Path
		'// .PreviewMode = True  '// Cannot play movie because WSH does not have window.
		.Controls.Play
	End With
End Sub


 
'*************************************************************************
'  <<< [SystemSound] >>> 
'*************************************************************************
Sub  SystemSound( Sound )
	Const  base = "HKEY_CURRENT_USER\AppEvents\Schemes\Apps\"
	Const  current = "\.Current\"
	Const  E_PathNotFound = &h80070002

	Dim  en,ed, parent, reg_path, file_path

	For Each parent  In Array( ".Default", "Explorer", "devenv", "dexplore", "sapisvr" )
		reg_path = base + parent +"\"+ Sound + current
		ErrCheck : On Error Resume Next
			file_path = env( g_sh.RegRead( reg_path ) )
		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If en = 0 Then  Exit For
		If en <> E_PathNotFound Then  Err.Raise en,,ed
	Next
	If file_path <> "" and file_path <> reg_path  Then  Play  file_path
End Sub


 
'*************************************************************************
'  <<< [WaitForSound] >>> 
'*************************************************************************
Sub  WaitForSound( Timeout_msec )
	Player_validate '// g_Player

	Dim  i : i = CInt( Timeout_msec / 250 )
	If IsEmpty( Timeout_msec ) Then  i=9
	For i=i To 1 Step -1
		If g_Player.m_Obj.PlayState = 1 Then Exit For
		If g_Player.m_Obj.PlayState = 10 Then  Raise E_PathNotFound, _
			"<ERROR msg='Cannot play the file' path='" + g_Player.m_Obj.URL + "'/>"
		WScript.Sleep  250
		If IsEmpty( Timeout_msec ) Then  i=9
	Next
	g_Player.m_Obj.Controls.Stop
End Sub


 
'*************************************************************************
'  <<< [SetVolume] >>> 
'*************************************************************************
Sub  SetVolume( Volume )
	Player_validate '// g_Player
	g_Player.m_Obj.Settings.Volume = Volume
End Sub


 
'*************************************************************************
'  <<< [Player_validate] >>> 
'*************************************************************************
Sub  Player_validate()
	If IsEmpty( g_Player ) Then  Set g_Player = new Vbslib_Player
End Sub

Class  Vbslib_Player
	Public  m_Obj

	Private Sub  Class_Initialize()
		Set m_Obj = CreateObject( "WMPlayer.OCX" )
		m_Obj.Settings.Volume = 100
	End Sub

	Private Sub  Class_Terminate()
		Dim  i
		For i=1 To 12 '// 12 = 3second for sound effects. Music will stop.
			If m_Obj.PlayState = 1 or m_Obj.PlayState = 10 Then Exit For
			WScript.Sleep  250
		Next
	End Sub
End Class


 
'*-------------------------------------------------------------------------*
'* ### <<<< Variable, Array and collection >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [DicItem] >>> 
'*************************************************************************
Function  DicItem( Dic, Key )
	If not Dic.Exists( Key ) Then  Exit Function
	If IsObject( Dic.Item( Key ) ) Then  Set DicItem = Dic.Item( Key )  Else  DicItem = Dic.Item( Key )
End Function


 
'*************************************************************************
'  <<< [IfElse] >>> 
'*************************************************************************
Function  IfElse( Condition, TrueValue, FalseValue )
	If Condition Then
		IfElse = TrueValue
	Else
		IfElse = FalseValue
	End If
End Function


 
'*************************************************************************
'  <<< [IfElseObj] >>> 
'*************************************************************************
Function  IfElseObj( Condition, TrueObject, FalseObject )
	If Condition Then
		Set IfElse = TrueValue
	Else
		Set IfElse = FalseValue
	End If
End Function

 
'*************************************************************************
'  <<< [GetDicItem] >>> 
'*************************************************************************
Sub  GetDicItem( Dic, Key, Out )
	If not Dic.Exists( Key ) Then  Out = Empty : Exit Sub
	If IsObject( Dic.Item( Key ) ) Then
		Set Out = DicItem( Dic, Key )
	Else
		Out = DicItem( Dic, Key )
	End If
End Sub


 
'*************************************************************************
'  <<< [GetDicItemAsArrayClass] >>> 
'*************************************************************************
Function  GetDicItemAsArrayClass( Dic, Key )
	If IsEmpty( Dic.Item( Key ) ) Then
		Set GetDicItemAsArrayClass = new ArrayClass
		Set Dic.Item( Key ) = GetDicItemAsArrayClass
	Else
		Set GetDicItemAsArrayClass = Dic.Item( Key )
	End If
End Function


 
'*************************************************************************
'  <<< [DicItemOfItem] >>> 
'*************************************************************************
Function  DicItemOfItem( Dic, ByVal Key )
	Dim  item
	Dim  ref_keys : Set ref_keys = CreateObject( "Scripting.Dictionary" )

	Do
		GetDicItem  Dic, Key, item  '//[out] item

		If VarType( item ) = vbString Then
			Select Case  Left( item, 1 )
				Case "$"
					item = Mid( item, 2 )
					If Left( item, 1 ) = "{" Then
						item = Mid( item, 2 )
						If Right( item, 1 ) = "}" Then _
							item = Left( item, Len(item)-1 )
					End IF

				Case "%"
					item = Mid( item, 2 )
					If Right( item, 1 ) = "%" Then _
						item = Left( item, Len(item)-1 )

				Case Else
					DicItemOfItem = item : Exit Function

			End Select
		ElseIf IsObject( item ) Then
			Set DicItemOfItem = item : Exit Function
		Else
			DicItemOfItem = item : Exit Function
		End If

		If ref_keys.Exists( item ) Then _
			Raise  E_NotFoundSymbol, "<ERROR msg=""Mutual reference"" key="""+ XmlAttr( DicKeyToCSV( ref_keys ) ) +"""/>"

		ref_keys.Item( item ) = True

		If not Dic.Exists( item ) Then _
			Raise  E_NotFoundSymbol, "<ERROR msg=""Not found symbol"" key="""+ XmlAttr( item ) +"""/>"

		Key = item
	Loop
End Function


 
'*************************************************************************
'  <<< [Dic_addNewObject] >>> 
'*************************************************************************
Function  Dic_addNewObject( Dic, ClassName, ObjectName, bDuplicate )
	GetDicItem  Dic, ObjectName, Dic_addNewObject  '//[out] Dic_addNewObject
	If IsEmpty( Dic_addNewObject ) Then
		Set Dic_addNewObject = new_X( ClassName )
		Set Dic.Item( ObjectName ) = Dic_addNewObject
		If g_Vers("CutPropertyM") Then
			Dic_addNewObject.Name = ObjectName
		Else
			Dic_addNewObject.m_Name = ObjectName
		End If
	ElseIf not bDuplicate Then
		Raise  E_AlreadyExist, "<ERROR msg='Same name object is found' class='"&_
			ClassName & "' name='" & ObjectName & "'/>"
	End If
End Function


 
'*************************************************************************
'  <<< [Dic_addElem] >>> 
'*************************************************************************
Sub  Dic_addElem( in_out_Dic, Key, Item )
	If IsEmpty( Item ) Then
		If in_out_Dic.Exists( Key ) Then _
			in_out_Dic.Remove  Key
	ElseIf IsObject( Item ) Then
		Set in_out_Dic.Item( Key ) = Item
	Else
		in_out_Dic.Item( Key ) = Item
	End If
End Sub


 
'*************************************************************************
'  <<< [Dic_addPaths] >>> 
'*************************************************************************
Sub  Dic_addPaths( self, ByVal BasePath, StepPaths, Item )
	If BasePath <> "" Then
		If Right( BasePath, 1 ) <> "\" Then  BasePath = BasePath +"\"
	End If

	If IsObject( Item ) Then
		For Each  step_path  In StepPaths
			Set self( BasePath + step_path ) = Item
		Next
	Else
		For Each  step_path  In StepPaths
			self( BasePath + step_path ) = Item
		Next
	End If
End Sub


 
'*************************************************************************
'  <<< [DictionaryClass] >>> 
'*************************************************************************
Class  DictionaryClass
	Private  m_Dictionary
	Public   IsNotFoundSymbolError

	Private Sub  Class_Initialize()
		Set m_Dictionary = CreateObject( "Scripting.Dictionary" )
		Me.IsNotFoundSymbolError = True
	End Sub

	Public Default Property Get  Item( Key )
		If not m_Dictionary.Exists( Key ) Then
			If Me.IsNotFoundSymbolError Then  Err.Raise  9
		End If

		LetSet  Item, m_Dictionary( Key )
	End Property

	Public Sub  Add( Key, Item ) : m_Dictionary.Add  Key, Item : End Sub
	Public Property Get  Count() : Count = m_Dictionary.Count : End Property
	Public Property Get  Keys()  : Keys  = m_Dictionary.Keys  : End Property
	Public Property Let  Item( Key, NewItem ) : m_Dictionary( Key ) = NewItem : End Property
	Public Property Set  Item( Key, NewItem ) : Set  m_Dictionary( Key ) = NewItem : End Property
	Public Property Get  Items() : Items = m_Dictionary.Items : End Property
	Public Function  Exists( Key ) : Exists = m_Dictionary.Exists( Key ) : End Function
	Public Property Get  CompareMode() : CompareMode = m_Dictionary.CompareMode : End Property
	Public Property Let  CompareMode( x ) : m_Dictionary.CompareMode = x : End Property
	Public Sub  Remove( Key ) : m_Dictionary.Remove  Key : End Sub
	Public Sub  RemoveAll() : m_Dictionary.RemoveAll : End Sub
End Class


 
'********************************************************************************
'  <<< [FlatArray] >>> 
'********************************************************************************
Sub  FlatArray( out_FlatArray, NestedArray )
	count = 0
	For Each  a_item  In NestedArray
		If IsArray( a_item ) Then
			count = count + UBound( a_item ) + 1
		Else
			count = count + 1
		End If
	Next

	left_index = UBound( NestedArray )  '// Before "ReDim"
	right_index = count - 1

	If IsEmpty( out_FlatArray ) Then
		ReDim  out_FlatArray( count - 1 )
	Else
		ReDim Preserve  out_FlatArray( count - 1 )
	End If

	For left_index = left_index  To 0  Step -1
		If IsArray( NestedArray( left_index ) ) Then
			For child_index = UBound( NestedArray( left_index ) )  To 0  Step -1
				element = NestedArray( left_index )( child_index )
				out_FlatArray( right_index ) = element
					'// 
					'// 
				right_index = right_index - 1
			Next
		Else
			out_FlatArray( right_index ) = NestedArray( left_index )
			right_index = right_index - 1
		End If
	Next
End Sub


 
'********************************************************************************
'  <<< [LazyDictionaryClass] >>> 
'********************************************************************************
Class  LazyDictionaryClass
	Private  m_Dictionary
	Public   IsNotFoundSymbolError
	Public   DebugMode  '// as Empty or True
	Public   ReadOnly   '// as True, False or Empty(False)
	Public   MaxExpandCount  '// as integer

	Private Sub  Class_Initialize()
		Set m_Dictionary = CreateObject( "Scripting.Dictionary" )
		Me.IsNotFoundSymbolError = True
		Me.MaxExpandCount = 100
	End Sub


	Public Default Property Get  Item( Key )
		Item = Key

		expand_count = 0
		If Me.DebugMode Then
			echo_v  "<DictionaryEx operation=""Get"" key="""+ Key +""">"
		End If

		Do
			If not IsArray( Item ) Then
				m_Expand  Key, Item, Empty, expand_count  '//[in/out] Item
			Else
				a_array = Item
				For index = 0  To UBound( a_array )
					m_Expand  Key, a_array( index ), index, expand_count
				Next
				FlatArray  Item, a_array
			End If

			If IsArray( Item ) Then
				is_loop = False
				For Each  a_item  In Item
					If InStr( a_item, "${" ) > 0 Then
						is_loop = True
						Exit For
					End If
				Next
			Else
				is_loop = False
			End If

			If not is_loop Then  Exit Do
		Loop

		If Me.DebugMode Then
			echo_v  "</DictionaryEx>"
		End If
	End Property


	Public Sub  m_Expand( Key, Item, Index, expand_count )
		sub_expand_count = 0
		Do
			start_pos = InStr( Item, "${" )
			If start_pos = 0 Then
				escape_pos = InStr( Item, "$\" )
				If escape_pos = 0 Then
					Exit Do
				End If
			End If

			If Me.DebugMode Then
				If sub_expand_count = 0 Then
					If not IsEmpty( Index ) Then
						echo_v  "<Item index="""& Index &""">"
					End If
				End If
			End If

			If start_pos > 0 Then
				last_pos = InStr( start_pos, Item, "}" )
				If last_pos = 0 Then
					Raise  1, "<ERROR msg=""Not found ""}"" of ""${"" of value"" "+_
						"value="""+ Item +"""/>"
				End If

				start_pos = InStrRev( Item, "${", last_pos )
				child_key = Mid( Item,  start_pos,  last_pos - start_pos + 1 )
				If not m_Dictionary.Exists( child_key ) Then
					a_key = "%"+ Mid( child_key, 3, Len( child_key ) - 3 ) +"%"
					child_item = g_sh.ExpandEnvironmentStrings( a_key )
					If child_item = a_key Then
						Raise  9, "<ERROR msg=""Not found the variable"" "+_
							"key="""+ child_key +"""/>"
					End If
				Else
					LetSet  child_item, m_Dictionary( child_key )
				End If

				If IsObject( child_item ) Then
					Set Item = child_item
					Exit Sub
				End If

				expand_count = expand_count + 1
				sub_expand_count = sub_expand_count + 1
				If Item = child_key Then
					Item = child_item

					If Me.DebugMode Then
						If IsArray( Item ) Then
							echo_v  "<Expand count="""& expand_count &""">"+ _
								GetEchoStr( Item ) +"</Expand>"
						End If
					End If

					If IsNull( Item ) Then  Item = Empty
						'// Portable to standard dictionary

					If VarType( Item ) <> vbString Then _
						Exit Do
				Else
					Item = Replace( Item, child_key, child_item )
				End If

				If expand_count >= Me.MaxExpandCount Then
					Raise  1, "<ERROR msg=""May be circular reference"" "+_
						"key="""+ Key +""" key2="""+ child_key +"""/>"
				End If

				If Me.DebugMode Then
					echo_v  "<Expand count="""& expand_count &""">"+ Item +"</Expand>"
				End If
			Else '// escape_pos > 0
				If IsNull( Item ) Then
					Item = Empty  '// Portable to standard dictionary
					Exit Do
				End If

				Item = Replace( Item, "$\", "$" )
				expand_count = expand_count + 1

				If Me.DebugMode Then
					echo_v  "<Expand count="""& expand_count &""">"+ Item +"</Expand>"
				End If
				Exit Do
			End If
		Loop

		If Me.DebugMode Then
			If not IsEmpty( Index ) Then
				If sub_expand_count = 0 Then
					echo_v  "<Item index="""& Index &""">"+ _
						GetEchoStr( Item ) +"</Item>"
				Else
					echo_v  "</Item>"
				End If
			End If
		End If
	End Sub


	Public Property Let  Item( Key, NewItem )
		Me.CheckAsKey  Key
		If Me.DebugMode Then  Me.EchoSetForDebug  Key, NewItem
		If Me.ReadOnly Then
			Me.Add  Key, NewItem
		Else
			m_Dictionary( Key ) = NewItem
		End If
	End Property

	Public Property Set  Item( Key, NewItem )
		Me.CheckAsKey  Key
		If Me.DebugMode Then  Me.EchoSetForDebug  Key, NewItem
		If Me.ReadOnly Then
			Me.Add  Key, NewItem
		Else
			Set  m_Dictionary( Key ) = NewItem
		End If
	End Property

	Public Sub  Add( Key, Item )
		Me.CheckAsKey  Key
		If Me.DebugMode Then  Me.EchoSetForDebug  Key, NewItem
		m_Dictionary.Add  Key, Item
	End Sub

	Public Sub  AddDictionary( in_Dictionary )
		For Each  key  In  in_Dictionary.Keys
			m_Dictionary( key ) = in_Dictionary.Formula( key )
		Next
	End Sub


	Public Property Get  Formula( Key )
		If not m_Dictionary.Exists( Key ) Then
			If Me.IsNotFoundSymbolError Then  Err.Raise  9
		End If

		LetSet  Formula, m_Dictionary( Key )
	End Property


	Public Function  EachFormula( Template, VariableName, aArray )
		m_EachFormula  Template, VariableName, aArray, EachFormula
	End Function

	Public Sub  m_EachFormula( Template, VariableName, aArray, out_Array )
		ReDim  out_Array( UBound( aArray ) )
		For index = 0  To UBound( aArray )
			out_Array( index ) = Replace( Template, VariableName, aArray( index ) )
		Next
	End Sub


	Public Function  Each_( Template, VariableName, aArray )
		m_EachFormula  Template, VariableName, aArray, formulas
		m_EachEvaluate  formulas, Each_
	End Function

	Public Sub  m_EachEvaluate( Formulas, out_Array )
		ReDim  out_Array( UBound( Formulas ) )
		For index = 0  To UBound( Formulas )
			out_Array( index ) = Me.Item( Formulas( index ) )
		Next
	End Sub


	Public Function  IsKeyOnly( Key )
		IsKeyOnly = Left( Key, 2 ) = "${"  and  Right( Key, 1 ) = "}"  and  InStr( 3, Key, "${" ) = 0
	End Function

	Public Sub  CheckAsKey( Key )
		If not Me.IsKeyOnly( Key ) Then
			Raise  1, "<ERROR msg=""Key name must be start from ""${"" and end to ""}"""" "+_
				"key="""+ Key +"""/>"
		End If
	End Sub


	Public Function  EvaluateReverse( in_Text )
		Redim  word_pair_array( m_Dictionary.Count - 1 )
		i = 0
		For Each  a_name  In m_Dictionary.Keys
			Set word_pair = new NameOnlyClass
			word_pair.Name = m_Dictionary( a_name )
			word_pair.Delegate = a_name

			Set word_pair_array( i ) = word_pair
			i = i + 1
		Next
		ShakerSort  word_pair_array, 0, UBound( word_pair_array ), _
			GetRef("CompareByNameLength"), -1


		text = in_Text
		For i=0 To  UBound( word_pair_array )
			Set word_pair = word_pair_array(i)
			text = Replace( text,  word_pair.Name,  word_pair.Delegate )
		Next
		EvaluateReverse = text
	End Function


	Public Sub  EchoSetForDebug( Key, NewItem )
		echo_v  "<DictionaryEx operation=""Set"" key="""+ Key +""">"+ GetEchoStr( NewItem ) +_
			"</DictionaryEx>"
	End Sub


	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		xml_sub = GetTab(Level)+ "<"+TypeName(Me)+">"
		For Each key In m_Dictionary.Keys
			source_item = m_Dictionary( key )
			If VarType( source_item ) = vbString Then
				xml_sub = xml_sub + vbCRLF + GetTab(Level) + key +" = """+ source_item +""""
				If InStr( source_item, "${" ) > 0 Then
					If VarType( Me.Item( key ) ) = vbString Then
						xml_sub = xml_sub +" = """+ Me.Item( key ) +""""
					Else
						xml_sub = xml_sub +" = "+ GetEchoStr( Me.Item( key ) )
					End If
				End If
			Else
				xml_sub = xml_sub + vbCRLF + GetTab(Level) + key +" = "+ GetEchoStr( source_item )
			End If
		Next
		xml_sub = xml_sub + vbCRLF + GetTab(Level) +"</"+TypeName(Me)+">"+ vbCRLF
	End Function

	Public Sub  Echo()
		Set ec = new EchoOff
		path = GetTempPath( "LazyDictionaryEcho_*.txt" )
		CreateFile  path,  Me.xml_sub(0)
		ec = Empty
		start  GetEditorCmdLine( path )
	End Sub


	Public Property Get  Count() : Count = m_Dictionary.Count : End Property
	Public Property Get  Keys()  : Keys  = m_Dictionary.Keys  : End Property
	Public Property Get  Items() : Items = m_Dictionary.Items : End Property
	Public Function  Exists( Key ) : Exists = m_Dictionary.Exists( Key ) : End Function
	Public Property Get  CompareMode() : CompareMode = m_Dictionary.CompareMode : End Property
	Public Property Let  CompareMode( x ) : m_Dictionary.CompareMode = x : End Property
	Public Sub  Remove( Key )
		If m_Dictionary.Exists( Key ) Then  m_Dictionary.Remove  Key
	End Sub
	Public Sub  RemoveAll() : m_Dictionary.RemoveAll : End Sub
End Class


 
'*************************************************************************
'  <<< [new_LazyDictionaryClass] >>> 
'*************************************************************************
Function  new_LazyDictionaryClass( ByVal RootXML_Element )
	If TypeName( RootXML_Element ) = "FilePathClass" Then
		base_path = GetParentFullPath( RootXML_Element.FilePath )
		Set RootXML_Element = RootXML_Element.Text
	End If

	Assert  TypeName( RootXML_Element ) = "IXMLDOMElement"
	Set variables = new LazyDictionaryClass
	For Each  variable_tag  In RootXML_Element.selectNodes( ".//Variable" )
		type__ = variable_tag.getAttribute( "type" )
		If IsNull( type__ ) Then
			variables( variable_tag.getAttribute( "name" ) ) = variable_tag.getAttribute( "value" )
		ElseIf type__ = "FullPathType" Then
			value = variable_tag.getAttribute( "value" )
			variables( variable_tag.getAttribute( "name" ) ) = GetFullPath( value, base_path )
		End If
	Next
	Set new_LazyDictionaryClass = variables
End Function


 
'*************************************************************************
'  <<< [GetPathLazyDictionary] >>> 
'*************************************************************************
Dim  g_PathLazyDictionary

Function  GetPathLazyDictionary()
	If IsEmpty( g_PathLazyDictionary ) Then
		Set g_PathLazyDictionary = new LazyDictionaryClass

		Dim  path : path = g_vbslib_folder : CutLastOf  path, "\", Empty
		g_PathLazyDictionary("${scriptlib}") = path

		If WScript.ScriptName = "vbslib Prompt.vbs" Then
			g_PathLazyDictionary("${vbslib}") = g_fs.GetParentFolderName( g_vbslib_folder )
		End If
	End If
	Set GetPathLazyDictionary = g_PathLazyDictionary
End Function


 
'*************************************************************************
'  <<< [ObjectSetClass] >>> 
'*************************************************************************
Class  ObjectSetClass
	Public   Objects
	Private  m_NextID

	Private Sub  Class_Initialize()
		Set Me.Objects = CreateObject( "Scripting.Dictionary" )
		m_NextID = 0
	End Sub

	Public Sub  Add( Object )
		If Me.Exists( Object ) Then  Exit Sub

		Do
			m_NextID = m_NextID + 1
			If not Objects.Exists( CStr( m_NextID ) ) Then _
				Exit Do
		Loop
		Set Me.Objects( CStr( m_NextID ) ) = Object
	End Sub

	Function  Exists( Object )
		Exists = True
		For Each  item  In Me.Objects.Items
			If item is Object Then  Exit Function
		Next
		Exists = False
	End Function

	Public Sub  Remove( Object )
		For Each  key  In Me.Objects.Keys
			If Objects( key ) is Object Then
				Me.Objects.Remove  key
				Exit Sub
			End If
		Next
	End Sub

	Property Get  Count() : Count = Me.Objects.Count : End Property
	Property Get  Items() : Items = Me.Objects.Items : End Property
	Public Sub  RemoveAll() : Me.Objects.RemoveAll : End Sub
End Class


 
'*************************************************************************
'  <<< [LifeGroupCounterClass] >>> 
'*************************************************************************
Class  LifeGroupCounterClass
    Public  Group
    Public  CreatedMemberID
    Public  ReferenceCount
End Class


 
'*************************************************************************
'  <<< [LifeHandleClass] >>> 
'*************************************************************************
Class  LifeHandleClass
	Public  p  '// Target object of this handle

	Private Sub  Class_Terminate()
		If not IsEmpty( Me.p.LifeGroup.Group ) Then _
			Me.p.LifeGroup.Group.AddTerminated  Me.p
	End Sub
End Class


 
'*************************************************************************
'  <<< [LifeGroupClass] >>> 
'*************************************************************************
Class  LifeGroupClass
	Public   Name
	Public   Objects
	Public   TerminatedObjects
	Private  m_CreatedMemberID

	Private Sub  Class_Initialize()
		Set Me.Objects = CreateObject( "Scripting.Dictionary" )
		Set Me.TerminatedObjects = CreateObject( "Scripting.Dictionary" )
		m_CreatedMemberID = 0
	End Sub

	Public Sub  AddHandle( Handle, Object )
		If IsEmpty( Object.LifeGroup ) Then _
			Set Object.LifeGroup = new LifeGroupCounterClass

		If not IsEmpty( Object.LifeGroup.Group ) Then
			If not Object.LifeGroup.Group Is Me Then
				Err.Raise  1, "Already in other LifeGroup"
			End If

			If Object.LifeGroup.ReferenceCount = 0 Then
				Me.TerminatedObjects.Remove  Object.LifeGroup.CreatedMemberID
			End If
			Object.LifeGroup.ReferenceCount = Object.LifeGroup.ReferenceCount + 1
		Else
			Do
				m_CreatedMemberID = m_CreatedMemberID + 1
				If not Objects.Exists( m_CreatedMemberID ) Then _
					Exit Do
			Loop

			Set Me.Objects( m_CreatedMemberID ) = Object
			Set Object.LifeGroup.Group = Me

			Object.LifeGroup.CreatedMemberID = m_CreatedMemberID
			Object.LifeGroup.ReferenceCount = 1
		End If

		Set Handle.p = Object
	End Sub

	Public Function  Add( Object )
		Set Add = new LifeHandleClass
		Me.AddHandle  Add, Object
	End Function

	Public Sub  Remove( Object )
		If not Object.LifeGroup.Group Is Me Then  Exit Sub
		If not Me.Objects( Object.LifeGroup.CreatedMemberID ) Is Object Then  Exit Sub

		Object.DestroyReferences

		Me.Objects.Remove  Object.LifeGroup.CreatedMemberID
		Object.LifeGroup.Group = Empty
		Object.LifeGroup.CreatedMemberID = Empty
	End Sub

	Public Sub  AddTerminated( Object )
		If not Object.LifeGroup.Group Is Me Then  Exit Sub
		If not Me.Objects( Object.LifeGroup.CreatedMemberID ) Is Object Then  Exit Sub

		Object.LifeGroup.ReferenceCount = Object.LifeGroup.ReferenceCount - 1

		If Object.LifeGroup.ReferenceCount = 0 Then
			Set Me.TerminatedObjects( Object.LifeGroup.CreatedMemberID ) = Object

			If Me.TerminatedObjects.Count = Me.Objects.Count Then
				For Each  a_object  In Me.Objects.Items
					a_object.DestroyReferences
					a_object.LifeGroup = Empty
				Next
				Me.Objects.RemoveAll
				Me.TerminatedObjects.RemoveAll
			End If
		End If
	End Sub
End Class


 
'*************************************************************************
'  <<< [DestroyerClass] >>> 
'*************************************************************************
Class  DestroyerClass
	Public  Objects

	Private Sub  Class_Initialize()
		Set Me.Objects = new ObjectSetClass
	End Sub

	Private Sub  Class_Terminate()
		For Each  item  In Me.Objects.Objects.Items
			item.DestroyReferences
		Next
	End Sub

	Public Sub  Add( Object )
		Me.Objects.Add  Object
		Object.IsDestroyer = True
	End Sub

	Public Sub  Remove( Object )
		Me.Objects.Remove  Object
		Object.IsDestroyer = False
	End Sub
End Class


 
'*************************************************************************
'  <<< [CheckUnderDestroyer] >>> 
'*************************************************************************
Sub  CheckUnderDestroyer( Object )
	If not Object.IsDestroyer Then _
		Err.Raise 1,, "This object must be in DestroyerClass, because this object refers mutually."
End Sub


 
'*************************************************************************
'  <<< [DicTable] >>> 
'*************************************************************************
Function  DicTable( DataArray )
	Dim  i,  i_start,  i_over,  column_length,  dic,  arr

	Set arr = new ArrayClass


	'//=== column_length
	column_length = 0
	Do
		If IsEmpty( DataArray( column_length ) ) Then  Exit Do
		column_length = column_length + 1
	Loop
	If column_length = 0 Then  Error


	'//=== make dictionary
	i = column_length + 1
	While i <= UBound( DataArray )

		Set dic = CreateObject( "Scripting.Dictionary" )
		i_start = i
		i_over  = i + column_length
		While  i < i_over
			LetSetWithBracket  dic,  DataArray( i - i_start ),  DataArray( i )
			i = i + 1
		WEnd

		arr.Add  dic
	WEnd

	DicTable = arr.Items
End Function


 
'*************************************************************************
'  <<< [JoinDicTable] >>> 
'*************************************************************************
Function  JoinDicTable( DicTable1, KeyField, DicTable2 )
	Dim  row_num1,  row_num2,  row1,  row2,  key_value,  field,  table2
	Dim  nums2 : Set nums2 = CreateObject( "Scripting.Dictionary" )
	Dim  ret_table : Set ret_table = new ArrayClass


	'//=== make table2 from DicTable2
	If VarType( DicTable2(0) ) = vbString Then
		table2 = DicTable( DicTable2 )
	Else
		table2 = DicTable2
	End If


	'//=== nums2
	For row_num2 = 0  To UBound( table2 )
		Set row2 = table2( row_num2 )
		If row2.Exists( KeyField ) Then
			key_value = row2.Item( KeyField )
			If not IsEmpty( key_value ) Then _
				nums2( key_value ) = row_num2
		End If
	Next


	'//=== row1 LEFT JOIN row2 WHERE row1.KeyField = row2.KeyField
	For row_num1 = 0  To UBound( DicTable1 )
		If DicTable1( row_num1 ).Exists( KeyField ) Then

			'//=== if key_value
			Set row1 = DicTable1( row_num1 )
			key_value = row1( KeyField )
			If not IsEmpty( key_value ) Then

				'//=== init row1
				Set row1 = CreateObject( "Scripting.Dictionary" )
				Dic_add  row1, DicTable1( row_num1 )

				'//=== row2
				If nums2.Exists( key_value ) Then
					Set row2 = table2( nums2( key_value ) )

					'//=== JOIN
					Dic_add  row1, row2

					nums2( key_value ) = -1
				End If

				ret_table.Add  row1
			End If
		End If
	Next


	'//=== row1 RIGHT? JOIN row2 WHERE row1.KeyField = row2.KeyField
	For Each row_num2  In nums2.Items
		If row_num2 >= 0 Then
			Set row1 = CreateObject( "Scripting.Dictionary" )
			Dic_add  row1, table2( row_num2 )
			ret_table.Add  row1
		End If
	Next

	JoinDicTable = ret_table.Items
End Function


 
'*************************************************************************
'  <<< [Dic_add] >>> 
'*************************************************************************
Sub  Dic_add( in_out_Dic, PlusDic )
	For Each key  In PlusDic.Keys
		Dic_addElem  in_out_Dic, key, PlusDic.Item( key )
	Next
End Sub


 
'*************************************************************************
'  <<< [Dic_sub] >>> 
'*************************************************************************
Sub  Dic_sub( out_Dic, WholeDic, BaseDic, CompareFunc, CompareFuncParam )
	Dim  key, whole_item

	If IsEmpty( out_Dic ) Then
		Set out_Dic = CreateObject( "Scripting.Dictionary" )
	Else
		out_Dic.RemoveAll
	End If

	If IsEmpty( CompareFunc ) Then  Set CompareFunc = GetRef( "StdCompare" )

	For Each key  In WholeDic.Keys
		GetDicItem  WholeDic, key, whole_item  '//[out] whole_item
		If BaseDic.Exists( key ) Then
			If CompareFunc( whole_item, BaseDic.Item( key ), CompareFuncParam ) <> 0 Then
				Dic_addElem  out_Dic, key, whole_item
			End If
		Else
			Dic_addElem  out_Dic, key, whole_item
		End If
	Next

	For Each key  In BaseDic.Keys
		If not WholeDic.Exists( key ) Then _
			out_Dic( key ) = Empty
	Next
End Sub


 
'*************************************************************************
'  <<< [Dic_searchParent] >>> 
'*************************************************************************
Function  Dic_searchParent( in_Dictionay, in_BasePath, in_Path )

	If IsFullPath( in_Path ) Then
		input_full_path = in_Path
		separator = GetFilePathSeparator( in_Path )
	Else
		input_full_path = GetFullPath( in_Path, in_BasePath )
		separator = GetFilePathSeparator( in_BasePath )
	End If

	matched_length_max = 0
	For Each  key  In in_Dictionay.Keys
		If InStr( 1, input_full_path, key, 1 ) > 0 Then
			If IsFullPath( key ) Then
				If Right( key, 1 ) = separator Then
					full_key = key
				Else
					full_key = key + separator
				End If
			Else
				full_key = GetFullPath( key, in_BasePath ) + separator
			End If
			If StrCompHeadOf( input_full_path, full_key, Empty ) = 0 Then
				matched_length = Len( full_key )
				If matched_length > matched_length_max Then
					matched_length_max = matched_length
					Dic_searchParent = key
				End If
			End If
		End If
	Next
End Function


 
'*************************************************************************
'  <<< [Dic_searchChildren] >>> 
'*************************************************************************
Function  Dic_searchChildren( in_Dictionay, in_BasePath, in_Path, out_PathArray, in_Option )

	If IsFullPath( in_Path ) Then
		separator = GetFilePathSeparator( in_Path )
		input_full_path = in_Path + separator
	Else
		separator = GetFilePathSeparator( in_BasePath )
		input_full_path = GetFullPath( in_Path, in_BasePath ) + separator
	End If

	ReDim  out_PathArray( in_Dictionay.Count )
	count = 0

	For Each  key  In in_Dictionay.Keys
		If InStr( 1, key, in_Path, 1 ) > 0 Then
			If IsFullPath( key ) Then
				full_key = key
			Else
				full_key = GetFullPath( key, in_BasePath )
			End If
			If StrCompHeadOf( full_key, input_full_path, Empty ) = 0 Then
				out_PathArray( count ) = key
				count = count + 1
			End If
		End If
	Next

	ReDim Preserve  out_PathArray( count - 1 )
End Function


 
'*************************************************************************
'  <<< [IsSameDictionary] >>> 
'*************************************************************************
Function  IsSameDictionary( DicA, DicB, Option_ )
	Set c = g_VBS_Lib
	IsSameDictionary = True

	If ( Option_ and c.CompareOnlyExistB ) = 0 Then
		If DicA.Count <> DicB.Count Then
			IsSameDictionary = False
			If Option_ and c.StopIsNotSame Then  Stop
			Exit Function
		End If
	End If

	For Each  key  In DicB.Keys
		If not DicA.Exists( key ) Then
			IsSameDictionary = False
			If Option_ and c.StopIsNotSame Then  Stop
		ElseIf DicA( key ) <> DicB( key ) Then
			IsSameDictionary = False
			If Option_ and c.StopIsNotSame Then  Stop
		End If
	Next
End Function


 
'*************************************************************************
'  <<< [DicToArr] >>> 
'*************************************************************************
Sub  DicToArr( Dic, Arr )
	Dim  keys : keys = Dic.Keys()
	Dim  key, i

	ReDim  Arr( UBound( keys ) )
	i = 0
	For Each key in keys
		Set Arr(i) = new DicElem : ErrCheck
		Arr(i).m_Key = key
		If IsObject( Dic.Item(key) ) Then
			Set Arr(i).m_Item = Dic.Item(key)
		Else
			Arr(i).m_Item = Dic.Item(key)
		End If
		i=i+1
	Next
End Sub

Class  DicElem
	Public  Key
	Public  Item
	Public Property Get  m_Key()  : m_Key  = Key  : End Property
	Public Property Let  m_Key(x) : Key    = x    : End Property
	Public Property Get  m_Item() : m_Item = Item : End Property
	Public Property Let  m_Item(x): Item   = x    : End Property
	Public Property Set  m_Item(x): Set Item = x  : End Property
	Public Property Get  Name()   : Name   = Key  : End Property
	Public Property Let  Name(x)  : Key    = x    : End Property
	Public Property Get  m_Name() : m_Name = Key  : End Property
	Public Property Let  m_Name(x): Key    = x    : End Property
	Public Sub  Init( a_Key, a_Item )
		Key = a_Key
		If IsObject( a_Item ) Then  Set Item = a_Item  Else  Item = a_Item
	End Sub
	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		xml_sub = GetTab(Level)+ "<"+TypeName(Me)+">"""+ Key +""" : "+ GetEchoStr( Item ) +"</"+TypeName(Me)+">"+ vbCRLF
	End Function
End Class

Function  DicElemCompare( Left, Right, Param )
	DicElemCompare = StrComp( Left.m_Key, Right.m_Key, Param )
End Function


 
'*************************************************************************
'  <<< [DicKeyToArr] >>> 
'*************************************************************************
Sub  DicKeyToArr( Dic, out_Array )
	Dim  keys : keys = Dic.Keys()
	Dim  key, i

	ReDim  out_Array( UBound( keys ) )
	i = 0
	For Each key in keys
		out_Array(i) = key
		i=i+1
	Next
End Sub


 
'*************************************************************************
'  <<< [DicItemToArr] >>> 
'*************************************************************************
Sub  DicItemToArr( Dic, Arr )
	Dim  keys : keys = Dic.Keys()
	Dim  key, i

	ReDim  Arr( UBound( keys ) )
	i = 0
	For Each key in keys
		If IsObject( Dic.Item(key) ) Then
			Set Arr(i) = Dic.Item(key)
		Else
			Arr(i) = dic.Item(key)
		End If
		i=i+1
	Next
End Sub


 
'*************************************************************************
'  <<< [DicElemArrayKeyToArr] >>> 
'*************************************************************************
Sub  DicElemArrayKeyToArr( a_DicElemArray, out_Array )
	ReDim  out_Array( UBound( a_DicElemArray ) )

	For index = 0  To UBound( a_DicElemArray )
		out_Array( index ) = a_DicElemArray( index ).Key
	Next
End Sub


 
'*************************************************************************
'  <<< [DicKeyToCSV] >>> 
'*************************************************************************
Function  DicKeyToCSV( Dic )
	Dim  key

	DicKeyToCSV = ""
	For Each key  In Dic.Keys
		If DicKeyToCSV <> "" Then  DicKeyToCSV = DicKeyToCSV + ","
		DicKeyToCSV = DicKeyToCSV + key
	Next
End Function

 
'*************************************************************************
'  <<< [Dictionary_xml_sub] >>> 
'*************************************************************************
Function  Dictionary_xml_sub( m, Level )
	Dim  key, s, ss

	s = GetTab( Level )+ "<Dictionary count="""& m.Count &""">{" +vbCRLF
	For Each key  In  m.Keys
		ss = Trim( GetEchoStr_sub( m.Item( key ), Level+1 ) )
		CutLastOf  ss, vbCRLF, Empty
		ss = CSVText( ss )
		If Left( ss, 1 ) <> """"  and _
				( VarType( m.Item( key ) ) = vbString  or  IsObject( m.Item( key ) ) ) Then _
			ss = """"+ ss +""""

		s = s +GetTab( Level+1 )+""""+ key +""" : " + ss +","+ vbCRLF
	Next
	s = s + GetTab( Level )+ "}</Dictionary>" +vbCRLF
	Dictionary_xml_sub = s
End Function


 
'*************************************************************************
'  <<< [CopyArr] >>> 
'*************************************************************************
Sub  CopyArr( Dst, Src )
	If g_cut_old Then Stop  ' Do not Dim a().  Dim a,b : b = Array( 1, 2 ) : a = b

	If IsArray( Src ) Then
		Dim  i

		ReDim  Dst( UBound( Src ) )
		For i=UBound( Src ) To 0 Step -1
			If IsObject( Src(i) ) Then  Set Dst(i) = Src(i)  Else  Dst(i) = Src(i)
		Next
	Else
		ReDim  Dst(0)
		If IsObject( Src ) Then  Set Dst(0) = Src  Else  Dst(0) = Src
	End If
End Sub


 
'********************************************************************************
'  <<< [Add] >>> 
'********************************************************************************
Function  Add( Operand1, Operand2 )
	If IsArray( Operand1 )  and  IsArray( Operand2 ) Then
		ReDim  a_array( UBound( Operand1 ) + UBound( Operand2 ) + 1 )

		For index = 0  To UBound( Operand1 )
			a_array( index ) = Operand1( index )
		Next
		count1 = UBound( Operand1 ) + 1
		For index = 0  To UBound( Operand2 )
			a_array( index + count1 ) = Operand2( index )
		Next
		Add = a_array
	Else
		Add = Operand1 + Operand2
	End If
End Function


 
'*************************************************************************
'  <<< [AddArrElem] >>> 
'*************************************************************************
Sub  AddArrElem( Dst, Src )
	AddArrElemEx  Dst, Src, False
End Sub


 
'*************************************************************************
'  <<< [AddArrElemEx] >>> 
'*************************************************************************
Function  AddArrElemEx( Dst, Src, IsReturn )
	If TypeName( Dst ) = "Dictionary" Then
		Dim  key, obj

		If IsArray( Src ) Then
			For Each obj  In Src : If not IsEmpty( obj ) Then
				If IsObject( obj ) Then  Set Dst.Item( obj.Name ) = obj  Else  Dst.Item( obj ) = True
			End If : Next
		ElseIf TypeName( Src ) = "Dictionary" Then
			For Each key  In Src.Keys()
				If IsObject( Src.Item( key ) ) Then
					Set Dst.Item( key ) = Src.Item( key )
				Else
					Dst.Item( key ) = Src.Item( key )
				End If
			Next
		Else
			If IsObject( Src ) Then  Set Dst.Item( Src.Name ) = Src  Else  Dst.Item( Src.Name ) = True
		End If

		If IsReturn Then  Set AddArrElemEx = Dst
	Else
		Dim  i, n

		n = UBound( Dst ) + 1
		If IsArray( Src ) Then
			ReDim Preserve  Dst( n + UBound( Src ) )
			For i=UBound( Src ) To 0 Step -1
				If IsObject( Src(i) ) Then  Set Dst(n+i) = Src(i)  Else  Dst(n+i) = Src(i)
			Next
		ElseIf TypeName( Src ) = "ArrayClass" Then
			ReDim Preserve  Dst( n + Src.UBound_ )
			For i=Src.UBound_ To 0 Step -1
				If IsObject( Src(i) ) Then  Set Dst(n+i) = Src(i)  Else  Dst(n+i) = Src(i)
			Next
		ElseIf not IsEmpty( Src ) Then
			ReDim Preserve  Dst( n )
			If IsObject( Src ) Then  Set Dst(n) = Src  Else  Dst(n) = Src
		End IF

		If IsReturn Then  AddArrElemEx = Dst
	End If
End Function


 
'*************************************************************************
'  <<< [SearchInSimpleArray] >>> 
'*************************************************************************
Function  SearchInSimpleArray( InData, InArray, OutBaseNumOrArray, DefaultOut )
	Dim  i

	For i=0 To UBound( InArray )
		If InData = InArray(i) Then
			If IsArray( OutBaseNumOrArray ) Then
				SearchInSimpleArray = OutBaseNumOrArray(i)
			Else
				SearchInSimpleArray = i + OutBaseNumOrArray
			End If
			Exit Function
		End If
	Next
	SearchInSimpleArray = DefaultOut
End Function


 
'*************************************************************************
'  <<< [ArrayFromLines] >>> 
'*************************************************************************
Function  ArrayFromLines( lines )
	ReDim  arr(-1)  '// output array
	arr_fast_ubound = UBound( arr )

	If lines = "" Then
		ArrayFromLines = Array( )
		Exit Function
	End If

	Set file = new StringStream
	file.SetString  lines
	Do Until  file.AtEndOfStream()
		If UBound(arr) < arr_fast_ubound + 1 Then _
			ReDim Preserve  arr( ( arr_fast_ubound + 100 ) * 4 )
		arr_fast_ubound = arr_fast_ubound + 1

		arr( arr_fast_ubound ) = file.ReadLine()
	Loop
	ReDim Preserve  arr( arr_fast_ubound )
	ArrayFromLines = arr
End Function


 
'*************************************************************************
'  <<< [CSVFrom] >>> 
'*************************************************************************
Function  CSVFrom( a_Array )
	If IsEmpty( a_Array ) Then  CSVFrom = "" : Exit Function
	Dim  arr : Set arr = new ArrayClass : arr.Items = a_Array : CSVFrom = arr.CSV
End Function


 
'*************************************************************************
'  <<< [ArrayFromCSV] >>> 
'- return as ArrayClass
'*************************************************************************
Function  ArrayFromCSV( a_CSV )
	Dim  arr : Set arr = new ArrayClass
	arr.AddCSV  a_CSV, Empty
	ArrayFromCSV = arr.Items
End Function


 
'*************************************************************************
'  <<< [ArrayFromCSV_Int] >>> 
'*************************************************************************
Function  ArrayFromCSV_Int( a_CSV )
	Dim  arr : Set arr = new ArrayClass
	arr.AddCSV  a_CSV, vbInteger
	ArrayFromCSV_Int = arr.Items
End Function


 
'*************************************************************************
'  <<< [IsSame] >>> 
'*************************************************************************
Function  IsSame( in_Left, in_Right )
	IsSame = False
	If IsEmpty( in_Left ) Then
		IsSame = IsEmpty( in_Right )
	ElseIf IsNull( in_Left ) Then
		IsSame = IsNull( in_Right )
	ElseIf IsObject( in_Left ) Then
		If IsObject( in_Right ) Then
			IsSame = ( in_Left  is  in_Right )
		End If
	ElseIf IsArray( in_Left ) Then
		If IsArray( in_Right ) Then
			IsSame = IsSameArray( in_Left, in_Right )
		End If
	Else
		If IsEmpty( in_Right ) Then
		ElseIf IsNull( in_Right ) Then
		Else
			IsSame = ( in_Left  =  in_Right )
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [IsSameArray] >>> 
'*************************************************************************
Function  IsSameArray( Arr1, Arr2 )
	Dim  i, low, up

	If IsEmpty( Arr1 ) <> IsEmpty( Arr2 ) Then  IsSameArray = False : Exit Function
	If IsEmpty( Arr1 ) Then  IsSameArray = True : Exit Function

	If IsArray( Arr1 ) Then
		If IsArray( Arr2 ) Then
			If UBound( Arr1 ) <> UBound( Arr2 ) Then  IsSameArray = False : Exit Function
		Else
			If UBound( Arr1 ) <> UBound( Arr2.Items ) Then  IsSameArray = False : Exit Function
		End If
		low = LBound( Arr1 ) : up = UBound( Arr1 )
	Else
		If IsArray( Arr2 ) Then
			If UBound( Arr1.Items ) <> UBound( Arr2 ) Then  IsSameArray = False : Exit Function
		Else
			If UBound( Arr1.Items ) <> UBound( Arr2.Items ) Then  IsSameArray = False : Exit Function
		End If
		low = 0 : up = UBound( Arr1.Items )
	End If

	For i = low To up
		If Arr1(i) <> Arr2(i) Then  IsSameArray = False : Exit Function
	Next
	IsSameArray = True
End Function


 
'*************************************************************************
'  <<< [IsSameArrayOutOfOrder] >>> 
'*************************************************************************
Function  IsSameArrayOutOfOrder( ArrayA, ArrayB, Option_ )
	Set c = g_VBS_Lib

	'// Reset "array_A_dictionary"
	Set array_A_dictionary = CreateObject( "Scripting.Dictionary" )
	For Each  element_A  In ArrayA
		If array_A_dictionary.Exists( element_A ) Then  Error
		array_A_dictionary( element_A ) = False
	Next

	'// Set "True" as "element_B" in "array_A_dictionary"
	For Each  element_B  In ArrayB
		If not array_A_dictionary.Exists( element_B ) Then
			If Option_ and c.StopIsNotSame Then _
				Stop
			IsSameArrayOutOfOrder = False
			Exit Function
		End If
		array_A_dictionary( element_B ) = True
	Next

	'// List up not same elements
	Set not_in_B_array = new ArrayClass
	For Each  element_A  In array_A_dictionary.Keys
		If not array_A_dictionary( element_A ) Then _
			not_in_B_array.Add  element_A
	Next
	If not_in_B_array.Count > 0 Then
		If Option_ and c.StopIsNotSame Then _
			Stop
		IsSameArrayOutOfOrder = False
		Exit Function
	End If
	IsSameArrayOutOfOrder = True
End Function


 
'*************************************************************************
'  <<< [ReverseObjectArray] >>> 
'*************************************************************************
Sub  ReverseObjectArray( in_Array, out_Array )
	Dim    i, u
	u = UBound( in_Array )
	ReDim  out_Array( u )
	For i=0 To u
		Set out_Array(i) = in_Array(u-i)
	Next
End Sub


 
'*************************************************************************
'  <<< [RemoveObjectArray] >>> 
'*************************************************************************
Sub  RemoveObjectArray( in_out_Array, ElementObject )
	i = 1
	For Each obj  In in_out_Array
		If obj is ElementObject Then
			Exit For
		End If
		i = i + 1
	Next
	If not IsEmpty( obj ) Then
		i_max = UBound( in_out_Array )
		While  i < i_max
			Set in_out_Array( i - 1 ) = in_out_Array( i )
			i = i + 1
		WEnd
		ReDim Preserve  in_out_Array( i_max - 1 )
	End If
End Sub


 
'*************************************************************************
'  <<< [RemoveObjectsByNames] >>> 
'*************************************************************************
Sub  RemoveObjectsByNames( in_out_Array, Names )
	If IsArray( Names ) Then
		For Each  name  In Names
			RemoveObjectsByNames  in_out_Array, name
		Next
	Else
		If TypeName( Names ) = "IRegExp2" Then
			Set key = Names
		Else
			If IsObject( Names ) Then
				key = Names.Name
			Else
				key = Names
			End If
		End If

		left_index = -1
		first_ubound = UBound( in_out_Array )
		For right_index = 0  To first_ubound
			name = in_out_Array( right_index ).Name

			If IsObject( key ) Then  '// key as RegExp
				is_match = key.Test( name )
			Else
				is_match = ( name = key )
			End If

			If not is_match Then
				left_index = left_index + 1

				If left_index <> right_index Then
					Set in_out_Array( left_index ) = in_out_Array( right_index )
				End If
			End If
		Next
	End If

	ReDim Preserve  in_out_Array( left_index )
End Sub


 
'*************************************************************************
'  <<< [ArrayToNameOnlyClassArray] >>> 
'*************************************************************************
Function  ArrayToNameOnlyClassArray( aArray )
	Dim  o, i, arr

	ReDim  arr( UBound( aArray ) )
	For i = 0 To UBound( aArray )
		Set o = new NameOnlyClass
		o.Name = aArray( i )
		Set arr( i ) = o
	Next

	ArrayToNameOnlyClassArray = arr
End Function


 
'********************************************************************************
'  <<< [NameOnlyClassArrayToArray] >>> 
'********************************************************************************
Function  NameOnlyClassArrayToArray( aArray )
	ReDim  a_array( UBound( aArray ) )

	index = 0
	For Each  object  In aArray
		a_array( index ) = object.Name
		index = index + 1
	Next

	NameOnlyClassArrayToArray = a_array
End Function


 
'*************************************************************************
'  <<< [QuickSort_fromDic] >>> 
'dic as Scripting.Dictionary
'out_arr as [out] object array
'*************************************************************************
Sub  QuickSort_fromDic( dic, out_arr, compare_func, param )
	Dim    i, i_last, elem
	i_last = dic.Count - 1
	Redim  out_arr( i_last )

	i=0
	For Each elem  In dic.Items
		Set out_arr(i) = elem
		i = i + 1
	Next

	QuickSort  out_arr, 0, i_last, compare_func, param
End Sub


 
'*************************************************************************
'  <<< [QuickSort] >>> 
'*************************************************************************
Sub  QuickSort( arr, i_left, i_right, compare_func, param )
	Dim  pivot, i_pivot, i_big_eq, i_small, sw, n_min_count

	If i_left >= i_right Then Exit Sub  ' rule-b'

	i_pivot = ( i_left + i_right ) \ 2
	Set pivot = arr( i_pivot )


	'//== for debug
	' Const  watch_sort_id = 6  '//**********************************
	' Dim  sort_debug_id,  sort_debug_id2
	' g_SortDebugID = g_SortDebugID + 1
	' sort_debug_id = g_SortDebugID
	' Dim  i, sym, value
	' echo "QuickSort start (" & sort_debug_id & ") ----------------------"
	' For i = i_left To i_right
	'   QuickSort_Debug_getSym  arr, i, sym, value
	'   If i = i_pivot Then  value = value & " (pivot)"
	'   echo "(" & i & ") " & sym & " = " & value
	' Next
	' If sort_debug_id = watch_sort_id Then  Stop


	'//=== Split to [ arr(i_left) ][ smaller than ][ arr(i_pivot) ][ greater equal ][ arr(i_right) ]
	i_big_eq = i_left : i_small = i_right
	Do

		'// Plus i_big_eq.  Result is that ( *i_big_eq >= *i_pivot ).
		Do
			If compare_func( arr(i_big_eq), pivot, param ) >= 0 Then  Exit Do
			i_big_eq = i_big_eq + 1

			If i_big_eq > i_right Then
				If compare_func( pivot, arr( i_big_eq - 1 ), param ) < 0 Then
					Raise  1, "<Error msg=""It may be always returned minus value""/>"
				Else
					Raise  1, "<Error msg=""Null can not be compared""/>"
				End If
			End If
		Loop


		'// Minus i_small.  Result is that ( *i_pivot > *i_small ).
		Do
			If i_small < i_left Then
				If compare_func( pivot, arr( i_small + 1 ), param ) > 0 Then
					Raise  1, "<Error msg=""It may be always returned plus value""/>"
				End If

				Exit Do
			End If

			If compare_func( arr(i_small), pivot, param ) < 0 Then  Exit Do
			i_small = i_small - 1
		Loop


		'//== for debug
		' If sort_debug_id = watch_sort_id Then
		'   sort_debug_id2 = sort_debug_id2 + 1
		'   echo "QuickSort swap (" & sort_debug_id & "-" & sort_debug_id2 & ")-----------------"
		'   For i = i_left To i_right
		'     QuickSort_Debug_getSym  arr, i, sym, value
		'     If i = i_small   Then  value = value & " (i_small)"
		'     If i = i_big_eq     Then  value = value & " (i_big_eq)"
		'     If i = i_pivot   Then  value = value & " (i_pivot)"
		'     echo "(" & i & ") " & sym & " = " & value
		'   Next
		' End If


		'// Splitted
		If i_small < i_big_eq Then
			If i_left <= i_small Then
				Exit Do


		'// If *i_pivot is minimum Then  (4) collect minimums at left
			Else
				Set sw = arr(i_left) : Set arr(i_left) = arr(i_pivot) : Set arr(i_pivot) = sw
				i_big_eq = i_big_eq + 1
				n_min_count = n_min_count + 1


				i_small = i_right  '// i_small is iterater to same value as minimum
				Do
					If i_big_eq >= i_small Then  Exit Do

					'// while ( *i_big_eq == *i_left )  i_big_eq++
					If compare_func( arr(i_big_eq), pivot, param ) = 0 Then
						i_big_eq = i_big_eq + 1
						n_min_count = n_min_count + 1

					'// Swap *i_big_eq  and  *i_small
					Else
						Do
							If i_small <= i_big_eq Then  Exit Do
							If compare_func( arr(i_small), pivot, param ) = 0 Then
								Set sw = arr(i_small) : Set arr(i_small) = arr(i_big_eq) : Set arr(i_big_eq) = sw
								Exit Do
							End If
							i_small = i_small - 1
						Loop
						If i_small <= i_big_eq Then  Exit Do
					End If
				Loop
				Exit Do
			End If


		'// If i_big_eq < i_pivot < i_small Then  (1) Swap *i_big_eq and *i_small
		ElseIf i_big_eq < i_pivot  and  i_pivot < i_small Then
			Set sw = arr(i_big_eq) : Set arr(i_big_eq) = arr(i_small) : Set arr(i_small) = sw
			i_big_eq = i_big_eq + 1 : i_small = i_small - 1


		'// If i_big_eq = i_pivot < i_small Then  (2A) Rotate3 *i_small -> *i_pivot -> *(i_pivot+1);  i_pivot++
		ElseIf i_big_eq = i_pivot  and  i_pivot < i_small Then
			If i_pivot + 1 < i_small Then
				Set sw = arr(i_pivot+1) : Set arr(i_pivot+1) = arr(i_pivot)
				Set arr(i_pivot) = arr(i_small) : Set arr(i_small) = sw
				i_big_eq = i_big_eq + 1 : i_pivot = i_pivot + 1


		'// If i_big_eq = i_pivot  and  i_pivot+1 = i_small Then  (2B) Swap *i_big_eq and *i_small
		'// (If rotate3, The result is Not swaped)
			Else
				Set sw = arr(i_big_eq) : Set arr(i_big_eq) = arr(i_small) : Set arr(i_small) = sw
				i_big_eq = i_big_eq + 1
				Exit Do
			End If


		'// If i_big_eq < i_small < i_pivot Then  (3) Rotate3 *i_small -> *i_big_eq -> *i_pivot;  i_pivot--
		ElseIf i_big_eq < i_small  and  i_small < i_pivot Then
			Set sw = arr(i_pivot) : Set arr(i_pivot) = arr(i_big_eq)
			Set arr(i_big_eq) = arr(i_small) : Set arr(i_small) = sw
			i_big_eq = i_big_eq + 1 : i_small = i_small - 1 : i_pivot = i_pivot - 1


		Else
			Stop
		End If

	Loop


	'//== for debug
	' echo "QuickSort middle (" & sort_debug_id & ") ----------------------"
	' For i = i_left To i_right
	'   QuickSort_Debug_getSym  arr, i, sym, value
	'   If i = i_big_eq-1 Then  value = value & " (i_big_eq-1)"
	'   If i = i_big_eq   Then  value = value & " (i_big_eq)"
	'   echo "(" & i & ") " & sym & " = " & value
	' Next
	' If sort_debug_id = watch_sort_id Then  Stop


	QuickSort  arr, (i_left + n_min_count), i_big_eq-1, compare_func, param  ' rule-b
	QuickSort  arr, i_big_eq,  i_right, compare_func, param  ' rule-b


	'//== for debug
	' echo "QuickSort end (" & sort_debug_id & ")----------------------"
	' For i = i_left To i_right
	'   QuickSort_Debug_getSym  arr, i, sym, value
	'   echo "(" & i & ") " & sym & " = " & value
	' Next
	'If g_is_debug Then
	'  For  i_small = i_left  To i_right - 1
	'    If compare_func( arr(i_small), arr(i_small + 1), param ) > 0 Then  Error
	'  Next
	'End If
End Sub


'//== for debug
'Dim  g_SortDebugID
'Sub  QuickSort_Debug_getSym( Arr, Index, out_Symbol, out_Value )
'  out_Symbol = Index
'  out_Value  = Arr(Index).id
'End Sub


 
'*************************************************************************
'  <<< [ShakerSort_fromDic] >>> 
'dic as Scripting.Dictionary
'out_arr as [out] object array
'*************************************************************************
Sub  ShakerSort_fromDic( dic, out_arr, sign, compare_func, param )
	Dim    i, i_last, elem
	i_last = dic.Count - 1
	Redim  out_arr( i_last )

	If sign >= 0 Then
		i=0
		For Each elem  In dic.Items
			Set out_arr(i) = elem
			i = i + 1
		Next
	Else
		i=i_last
		For Each elem  In dic.Items
			Set out_arr(i) = elem
			i = i - 1
		Next
	End If

	ShakerSort  out_arr, 0, i_last, compare_func, param
End Sub


 
'*************************************************************************
'  <<< [ShakerSort_fromDicKey] >>> 
'*************************************************************************
Sub  ShakerSort_fromDicKey( dic, out_arr, Option_ )
	Dim    i, i_last, key, elem
	i_last = dic.Count - 1
	Redim  out_arr( i_last )

	i=0
	For Each key  In dic.Keys
		Set elem = new DicElem
		elem.Init  key, dic.Item( key )
		Set out_arr(i) = elem
		i = i + 1
	Next

	ShakerSort  out_arr, 0, i_last, GetRef( "DicElemCompare" ), 1
End Sub


 
'*************************************************************************
'  <<< [ShakerSortDicByKey] >>> 
'*************************************************************************
Sub  ShakerSortDicByKey( a_Dictionary )
	ShakerSort_fromDicKey  a_Dictionary, items, Empty  '//[out] items
	a_Dictionary.RemoveAll
	For Each  item  In items
		Set a_Dictionary( item.Key ) = item.Item
	Next
End Sub


 
'*************************************************************************
'  <<< [ShakerSortDicByKeyCompare] >>> 
'*************************************************************************
Sub  ShakerSortDicByKeyCompare( a_Dictionary, CompareFunc, CompareFuncParam )
	ShakerSort_fromDicKey_Sub  a_Dictionary, items, CompareFunc, CompareFuncParam  '//[out] items
	a_Dictionary.RemoveAll
	For Each  item  In items
		Set a_Dictionary( item.Key ) = item.Item
	Next
End Sub


 
'*************************************************************************
'  <<< [DelegateCompareClass] >>> 
'*************************************************************************
Class  DelegateCompareClass
	Public  CompareFunction
	Public  CompareFunctionParameter
End Class


 
'*************************************************************************
'  <<< [ShakerSort_fromDicKey_Sub] >>> 
'*************************************************************************
Sub  ShakerSort_fromDicKey_Sub( dic, out_arr, CompareFunc, CompareFuncParam )
	i_last = dic.Count - 1
	Redim  out_arr( i_last )

	i=0
	For Each key  In dic.Keys
		Set elem = new DicElem
		elem.Init  key, dic.Item( key )
		Set out_arr(i) = elem
		i = i + 1
	Next

	Set compare = new DelegateCompareClass
	Set compare.CompareFunction = CompareFunc
	LetSet compare.CompareFunctionParameter, CompareFuncParam

	ShakerSort  out_arr, 0, i_last, GetRef("DicKeyDelegateCompare"), compare
End Sub

Function  DicKeyDelegateCompare( Left, Right, Param )
	DicKeyDelegateCompare = Param.CompareFunction( _
		Left.Key, Right.Key, Param.CompareFunctionParameter )
End Function


 
'*************************************************************************
'  <<< [ShakerSort] >>> 
'*************************************************************************
Sub  ShakerSort( arr, ByVal i_left, ByVal i_right, compare_func, param )
	Dim  i_swap, i, sw

	Do
		i_swap = i_left+1
		For i=i_left+1 To i_right
			If compare_func( arr(i-1), arr(i), param ) > 0 Then
				Set sw = arr(i-1) : Set arr(i-1) = arr(i) : Set arr(i) = sw
				i_swap = i
			End If
		Next
		If i_swap = i_left+1 Then Exit Do
		i_right = i_swap-1

		i_swap = i_right-1
		For i=i_right-1 To i_left Step -1
			If compare_func( arr(i), arr(i+1), param ) > 0 Then
				Set sw = arr(i) : Set arr(i) = arr(i+1) : Set arr(i+1) = sw
				i_swap = i
			End If
		Next
		If i_swap = i_right-1 Then Exit Do
		i_left = i_swap+1
	Loop
End Sub


 
'*************************************************************************
'  <<< [StdCompare] >>> 
'*************************************************************************
Function  StdCompare( Left, Right, Param )
	StdCompare = -1

	If IsObject( Left ) Then
		If IsObject( Right ) Then
			If Left is Right Then
				StdCompare = 0
			End If
		End If
	ElseIf VarType( Left ) = vbString Then
		If VarType( Right ) = vbString Then
			StdCompare = StrComp( Left, Right, Param )
		End If
	ElseIf IsEmpty( Left ) Then
		If IsEmpty( Right ) Then
			StdCompare = 0
		Else
			StdCompare = 1
		End IF
	ElseIf VarType( Left ) = VarType( Right ) Then
		If Left = Right Then
			StdCompare = 0
		ElseIf Left > Right Then
			StdCompare = 1
		End If
	Else
		Error
	End If
End Function


 
'*************************************************************************
'  <<< [NameCompare] >>> 
'*************************************************************************
Function  NameCompare( Left, Right, Param )
	NameCompare = -1

	If IsObject( Left ) Then
		If IsObject( Right ) Then
			If g_Vers("CutPropertyM") Then
				NameCompare = StdCompare( Left.Name, Right.Name, Param )
			Else
				NameCompare = StdCompare( Left.m_Name, Right.m_Name, Param )
			End If
		End If
	ElseIf not IsObject( Right ) Then
		NameCompare = 0
	Else
		NameCompare = 1
	End If
End Function


 
'*************************************************************************
'  <<< [NumStringCompare] >>> 
'*************************************************************************
Function  NumStringCompare( LeftString, RightString, Param )
	char_index = 1  '// character index
	Do
		left_char  = Mid( LeftString,  char_index, 1 )  '// left  character
		right_char = Mid( RightString, char_index, 1 )  '// right character


		'// Case of short string
		If left_char = "" Then
			If right_char = "" Then
				NumStringCompare = 0
			Else
				NumStringCompare = -1
			End If
			Exit Function
		ElseIf right_char = "" Then
			NumStringCompare = +1
			Exit Function
		End If


		'// Case of not number and/or not number
		If not IsNumeric( left_char )  or  not IsNumeric( right_char ) Then
			NumStringCompare = StrComp( left_char, right_char, 1 )
			If NumStringCompare <> 0 Then _
				Exit Function

			'// Next loop
			char_index = char_index + 1

		'// Case of number and number
		Else
			num_index = char_index
			Do
				char_index = char_index + 1
				left_char  = Mid( LeftString,  char_index, 1 )
				If not IsNumeric( left_char ) Then
					left_str = Mid( LeftString, num_index, char_index - num_index )
					Exit Do
				End If
			Loop

			char_index = num_index
			Do
				char_index = char_index + 1
				right_char  = Mid( RightString,  char_index, 1 )
				If not IsNumeric( right_char ) Then
					right_str = Mid( RightString, num_index, char_index - num_index )
					Exit Do
				End If
			Loop

			left_number  = CInt2( left_str ) 
			right_number = CInt2( right_str ) 
			If left_number < right_number Then
				NumStringCompare = -1
			ElseIf left_number > right_number Then
				NumStringCompare = +1
			Else
				NumStringCompare = StrComp( left_str, right_str, 1 )
			End If
			If NumStringCompare <> 0 Then _
				Exit Function
		End If
	Loop
End Function


 
'*************************************************************************
'  <<< [NumStringNameCompare] >>> 
'*************************************************************************
Function  NumStringNameCompare( Left, Right, Param )
	If g_Vers("CutPropertyM") Then
		NumStringNameCompare = NumStringCompare( Left.Name, Right.Name, Param )
	Else
		NumStringNameCompare = NumStringCompare( Left.m_Name, Right.m_Name, Param )
	End If
End Function


 
'*************************************************************************
'  <<< [LengthCompare] >>> 
'*************************************************************************
Function  LengthCompare( Left, Right, Param )
	LengthCompare = -1

	If VarType( Left ) = vbString Then
		LengthCompare = ( Len( Left ) - Len( Right ) )
		If IsNumeric( Param ) Then
			If Param < 0 Then  LengthCompare = -LengthCompare
		End If
	Else
		Error
	End If
End Function


 
'*************************************************************************
'  <<< [LengthNameCompare] >>> 
'*************************************************************************
Function  LengthNameCompare( Left, Right, Param )
	If g_Vers("CutPropertyM") Then
		LengthNameCompare = LengthCompare( Left.Name, Right.Name, Param )
	Else
		LengthNameCompare = LengthCompare( Left.m_Name, Right.m_Name, Param )
	End If
End Function


 
'*************************************************************************
'  <<< [PathCompare] >>> 
'*************************************************************************
Function  PathCompare( Left, Right, ByVal Param )
	PathCompare = -1
	If IsEmpty( Param ) Then  Param = +1

	If VarType( Left ) = vbString Then
		left_2  = Left
		right_2 = Right
		For Each  symbol  In Array( "\", "/", "*" )
			left_2 = Replace( left_2, symbol, vbTab )
			right_2 = Replace( right_2, symbol, vbTab )
		Next

		PathCompare = StrComp( left_2, right_2, 1 ) * Param
	Else
		Error
	End If
End Function


 
'*************************************************************************
'  <<< [PathNameCompare] >>> 
'*************************************************************************
Function  PathNameCompare( Left, Right, Param )
	If g_Vers("CutPropertyM") Then
		PathNameCompare = PathCompare( Left.Name, Right.Name, Param )
	Else
		PathNameCompare = PathCompare( Left.m_Name, Right.m_Name, Param )
	End If
End Function


 
'*************************************************************************
'  <<< [CInt2] >>> 
' - no exception
'*************************************************************************
Function  CInt2( v )
	Dim  en, ed

	ErrCheck : On Error Resume Next
		CInt2 = CLng( v )
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = 13 Then  '// if sym is not number
		CInt2 = 0
	ElseIf en <> 0 Then  Err.Raise en,,ed  End If
End Function


 
'*************************************************************************
'  <<< [Trim2] >>> 
'*************************************************************************
Function  Trim2( v )
	Trim2 = LTrim2( RTrim2( v ) )
End Function


 
'*************************************************************************
'  <<< [LTrim2] >>> 
'*************************************************************************
Function  LTrim2( v )
	Dim  i

	For i = 1 To Len( v )
		Select Case  Mid( v, i, 1 )
			Case  " ", vbTab, vbCR, vbLF
			Case Else  Exit For
		End Select
	Next
	LTrim2 = Mid( v, i )
End Function


 
'*************************************************************************
'  <<< [RTrim2] >>> 
'*************************************************************************
Function  RTrim2( v )
	Dim  i

	For i = Len( v ) To 1 Step -1
		Select Case  Mid( v, i, 1 )
			Case  " ", vbTab, vbCR, vbLF
			Case Else  Exit For
		End Select
	Next
	RTrim2 = Left( v, i )
End Function


 
'*************************************************************************
'  <<< [sscanf] >>> 
'*************************************************************************
Function  sscanf( String, Format )
	Dim  percent_pos_in_format,  start_tag,  end_tag,  start_pos,  end_pos,  type_


	'//=== get percent_pos_in_format and type_
	percent_pos_in_format = InStr( Format, "%s" )
	If percent_pos_in_format > 0 Then
		type_ = "%s"
	Else
		percent_pos_in_format = InStr( Format, "%d" )
		If percent_pos_in_format > 0 Then
			type_ = "%d"
		Else
			Error
		End If
	End If


	'//=== search in String
	start_tag = Left( Format, percent_pos_in_format - 1 )
	end_tag   = Mid(  Format, percent_pos_in_format + 2 )

	start_pos = InStr( String, start_tag )
	If start_pos = 0 Then  Raise  E_NotFoundSymbol, "<ERROR msg=""Not found"" format="""+ Format +"""/>"
	start_pos = start_pos + percent_pos_in_format - 1

	If end_tag <> "" Then
		end_pos = InStr( start_pos, String, end_tag )
		If end_pos = 0 Then  Raise  E_NotFoundSymbol, "<ERROR msg=""Not found"" format="""+ Format +"""/>"
	Else
		end_pos = Len( String ) + 1
	End If


	'//=== pickup
	sscanf = Mid( String,  start_pos,  end_pos - start_pos )

	If type_ = "%d" Then  sscanf = CInt2( sscanf )
End Function


 
'*************************************************************************
'  <<< [ScanFromTemplate] >>> 
'*************************************************************************
Function  ScanFromTemplate( ScanString, TemplateString, KeywordArray, Option_ )
	Set ScanFromTemplate = ScanFromTemplate_Sub( Empty, _
		ScanString, TemplateString, KeywordArray, 1, True, _
		IsBitSet( Option_, g_VBS_Lib.WithDollarVariable ) )
End Function


Function  ScanFromTemplate_Sub( out, ScanString, TemplateString, KeywordArray, in_out_Position,_
		IsErrorNotFoundFirstText, IsReplaceEscapeDollarVariable )

	If IsEmpty( out ) Then _
		Set out = CreateObject( "Scripting.Dictionary" )
	template_text_position = 1
	msg_1 = "There is not template contexts at the replacing target"

	If UBound( KeywordArray ) = -1 Then
		Set ScanFromTemplate_Sub = out
		Exit Function
	End If


	'// Set "template_text", "template_text_length"
	ReDim  template_text(        UBound( KeywordArray ) + 1 )
	ReDim  template_text_length( UBound( KeywordArray ) + 1 )
	For keyword_num = 0  To UBound( KeywordArray )
		template_keyword_position = InStr( template_text_position,_
			TemplateString, KeywordArray( keyword_num ) )
		If template_keyword_position = 0 Then
			If InStr( TemplateString, KeywordArray( keyword_num ) ) > 0 Then
				Raise  1, "<ERROR msg=""Change the order of keyword"""+_
					" keyword_1="""& KeywordArray( keyword_num - 1 ) &""""+_
					" keyword_2="""& KeywordArray( keyword_num ) &"""/>"
			Else
				Raise  1, "<ERROR msg=""Not found keyword in the template"""+_
					" keyword="""& KeywordArray( keyword_num ) &"""/>"
			End If
		End If
		template_text( keyword_num ) = Mid( TemplateString, template_text_position, _
			template_keyword_position - template_text_position )
		If IsReplaceEscapeDollarVariable Then
			template_text( keyword_num ) = Replace( template_text( keyword_num ), "$\{", "${" )
			template_text( keyword_num ) = Replace( template_text( keyword_num ), "$\\", "$\" )
		End If
		template_text_length( keyword_num ) = Len( template_text( keyword_num ) )

		template_text_position = template_keyword_position + Len( KeywordArray( keyword_num ) )
	Next
	template_text( keyword_num ) = Mid( TemplateString, template_text_position )
	If IsReplaceEscapeDollarVariable Then
		template_text( keyword_num ) = Replace( template_text( keyword_num ), "$\{", "${" )
		template_text( keyword_num ) = Replace( template_text( keyword_num ), "$\\", "$\" )
	End If
	template_text_length( keyword_num ) = Len( template_text( keyword_num ) )


	'// Special Case
	If KeywordArray( 0 ) = TemplateString Then
		out( KeywordArray( 0 ) ) = ScanString
		in_out_Position = Len( ScanString )
		Set ScanFromTemplate_Sub = out
		Exit Function
	End If


	'// Update "in_out_Position" : Search "template_text"( last )
	p = in_out_Position
	For keyword_num = 0  To UBound( KeywordArray ) + 1
		p = InStr( p, ScanString, template_text( keyword_num ) )
		If p = 0 Then
			If IsErrorNotFoundFirstText Then
				Raise  1, "<ERROR msg="""+ msg_1 +"""/>"
			Else
				Set ScanFromTemplate_Sub = Nothing
				Exit Function
			End If
		End If
		p = p + template_text_length( keyword_num )
	Next
	in_out_Position = p


	'// Set "out" : Searches forword for scanning the shortest matched
	p2 = in_out_Position - template_text_length( UBound( KeywordArray ) + 1 )
	For keyword_num = UBound( KeywordArray )  To 0  Step -1
		n = template_text_length( keyword_num )

		p1 = InStrRev( ScanString, template_text( keyword_num ), p2 - 1 )
		out( KeywordArray( keyword_num ) ) = Mid( ScanString,  p1 + n, p2 - p1 - n )

		p2 = p1
	Next


	'// Set "out"( first ), "out"( last )
	If StrCompHeadOf( TemplateString, KeywordArray( 0 ), g_VBS_Lib.CaseSensitive ) = 0 Then
		out( KeywordArray( 0 ) ) = Left( ScanString, p1 )
	End If
	If UBound( KeywordArray ) >= 1 Then
		If StrCompLastOf( TemplateString, KeywordArray( UBound( KeywordArray ) ), _
				g_VBS_Lib.CaseSensitive ) = 0 Then
			value = Mid( ScanString, in_out_Position )
			out( KeywordArray( UBound( KeywordArray ) ) ) = value
			in_out_Position = Len( value )
		End If
	End If


	Set ScanFromTemplate_Sub = out
End Function


 
'*************************************************************************
'  <<< [ScanMultipleFromTemplate] >>> 
'*************************************************************************
Sub  ScanMultipleFromTemplate( ScanString, TemplateString, KeywordArray, Option_, out_Scaned )
	Set scaned_array = new ArrayClass
	position = 1
	If IsArray( KeywordArray ) Then
		Do
			Set scaned = ScanFromTemplate_Sub( Empty, ScanString, TemplateString, _
				KeywordArray, position, False, False )  '//(in/out) position

			If scaned is Nothing Then _
				Exit Do

			scaned_array.Add  scaned
		Loop
	Else
		keyword_array = Array( KeywordArray )
		Do
			Set scaned = ScanFromTemplate_Sub( Empty, ScanString, TemplateString, _
				keyword_array, position, False, False )  '//(in/out) position

			If scaned is Nothing Then _
				Exit Do

			For Each  value  In scaned.Items
				scaned_array.Add  value
			Next
		Loop
	End If
	out_Scaned = scaned_array.Items
End Sub


 
'*************************************************************************
'  <<< [InStrEx] >>> 
'*************************************************************************
Function  InStrEx( WholeString, ByVal Keyword, ByVal StartIndex, Opt )
	Dim  pos,  pos1, keyword_num
	Dim  c : Set c = g_VBS_Lib
	Dim  case_sensitive : case_sensitive = StrCompOption( Opt and c.CaseSensitive )
	Dim  is_reverse : is_reverse = ( ( Opt and c.Rev ) <> 0 )
	Dim  keyword_len,  keyword_lens

	If StartIndex = 0 Then
		If is_reverse Then  StartIndex = -1  Else  StartIndex = 1
	End If
	whole_string_length = Len( WholeString )


	'// Set "Keyword" array, "keyword_len" array
	If IsArray( Keyword ) Then
		ReDim  keyword_lens( UBound( Keyword ) )
		For keyword_num = 0  To UBound( Keyword )
			keyword_lens( keyword_num ) = Len( Keyword( keyword_num ) )
		Next
	Else
		keyword_lens = Array( Len( Keyword ) )
		Keyword = Array( Keyword )
	End If


	Do
		'// Set "pos" : Search
		If is_reverse Then
			pos = 0
			For keyword_num = 0  To UBound( Keyword )
				pos1 = InStrRev( WholeString, Keyword(keyword_num), StartIndex, case_sensitive )
				If pos1 > pos Then  pos = pos1 : keyword_len = keyword_lens( keyword_num )
			Next
			If pos = 0 Then  InStrEx = 0 : Exit Function
		Else
			pos = whole_string_length + 1
			For keyword_num = 0  To UBound( Keyword )
				pos1 = InStr( StartIndex, WholeString, Keyword(keyword_num), case_sensitive )
				If pos1 <> 0 Then  If pos1 < pos Then  pos = pos1 : keyword_len = keyword_lens( keyword_num )
			Next
			If pos = whole_string_length + 1 Then  InStrEx = 0 : Exit Function
		End If


		'// Check by "c.WholeWord", "c.LineHead", "c.LineTail"
		is_match = True
		If (Opt and c.WholeWord) Then
			If not IsWholeWord( WholeString, pos, keyword_len ) Then _
				is_match = False
		End If
		If (Opt and c.LineHead) Then
			If pos >= 3 Then
				If Mid( WholeString, pos - 2, 2 ) <> vbCRLF Then _
					is_match = False
			ElseIf pos = 2 Then
				is_match = False
			End If
		End If
		If (Opt and c.LineTail) Then
			If pos + keyword_len <= whole_string_length - 1 Then
				If Mid( WholeString, pos + keyword_len, 2 ) <> vbCRLF Then _
					is_match = False
			ElseIf pos + keyword_len = whole_string_length Then
				is_match = False
			End If
		End If
		If is_match Then _
			Exit Do


		'// Set next "StartIndex"
		If is_reverse Then
			StartIndex = pos - 1
		Else
			StartIndex = pos + 1
		End If
	Loop
	If Opt and c.LastNextPos Then  pos = pos + keyword_len
	InStrEx = pos
End Function


 
'*************************************************************************
'  <<< [IsWholeWord] >>> 
'*************************************************************************
Function  IsWholeWord( WholeString, StartIndex, SubWordLen )
	Dim  i, ch

	IsWholeWord = False
	For i=1 To 2
		If i = 1 Then
			If StartIndex <= 1 Then
				ch = 0
			Else
				ch = Asc( Mid( WholeString, StartIndex - 1, 1 ) )
			End If
		Else
			If StartIndex + SubWordLen > Len( WholeString ) Then
				ch = 0
			Else
				ch = Asc( Mid( WholeString, StartIndex + SubWordLen, 1 ) )
			End If
		End If

		If ch < &h30 Then
		ElseIf ch <= &h39 Then
			Exit Function
		ElseIf ch <  &h41 Then
		ElseIf ch <= &h5A Then
			Exit Function
		ElseIf ch <  &h61 Then
		ElseIf ch <= &h7A Then
			Exit Function
		End If
	Next
	IsWholeWord = True
End Function


 
'*************************************************************************
'  <<< [InStrLast] >>> 
'*************************************************************************
Function  InStrLast( Str, Keyword )
	Dim  i
	i = InStr( Str, Keyword )
	If i = 0 Then
		InStrLast = 0
	Else
		InStrLast = i + Len( Keyword )
	End If
End Function


 
'*************************************************************************
'  <<< [IsPlusIntOrZeroString] >>> 
'*************************************************************************
Function  IsPlusIntOrZeroString( aString )
	Dim  i, c

	For i=1 To Len( aString )
		c = Asc( Mid( aString, i, 1 ) )
		If c < &h30  or  c > &h39  Then  '// c<'0' or c>'9'
			IsPlusIntOrZeroString = False
			Exit Function
		End If
	Next
	IsPlusIntOrZeroString = True
End Function


 
'*************************************************************************
'  <<< [MeltQuot] >>> 
'*************************************************************************
Function  MeltQuot( Line, in_out_Start )
	Dim  i, j, c


	'//=== Skip to "
	i = in_out_Start
	Do
		c = Mid( Line, i, 1 )
		If c = "" Then  in_out_Start = 0 : Exit Function
		If c = """"  Then Exit Do
		i = i + 1
	Loop
	j = i + 1


	'//=== Search the end of "
	i = j
	Do
		c = Mid( Line, i, 1 )
		If c = "" Then  in_out_Start = 0 : Exit Do
		If c = """"  Then  in_out_Start = i + 1 : Exit Do
		i = i + 1
	Loop


	'//=== Get the string
	MeltQuot = Mid( Line, j, i - j )

End Function


 
'*************************************************************************
'  <<< [CreateGuid] >>> 
'*************************************************************************
Dim  g_TypeLib

Function  CreateGuid()
	If g_TestModeFlags and F_NotRandom Then
		g_TypeLib = g_TypeLib + 1
		CreateGuid = "00000000-0000-0000-0000-" & Right( "000000000000" & g_TypeLib, 12 )
	Else
		If IsEmpty( g_TypeLib ) Then  Set g_TypeLib = CreateObject("Scriptlet.TypeLib")
		CreateGuid = Mid( g_TypeLib.Guid, 2, 36 )
	End IF
End Function


 
'*************************************************************************
'  <<< [new_RegExp] >>> 
'*************************************************************************
Function  new_RegExp( Pattern, IsCaseSensitive )
	Set re = CreateObject("VBScript.RegExp")
	re.Pattern = Pattern
	re.MultiLine = True
	re.Global = True
	re.IgnoreCase = not IsCaseSensitive
	Set new_RegExp = re
End Function


 
'*************************************************************************
'  <<< [FindStringLines] >>> 
'*************************************************************************
Function  FindStringLines( InputString, ByVal RegExpObject, Option_ )

	If VarType( RegExpObject ) = vbString Then
		Set re = CreateObject("VBScript.RegExp")
		re.Pattern = RegExpObject
	Else
		Set re = RegExpObject
	End If

	If VarType( Option_ ) = vbBoolean Then
		is_match = Option_
	Else
		is_match = True
	End IF

	FindStringLines = ""

	left_ = 1
	Do
		right_ = InStr( left_, InputString, vbLF )
		If right_ = 0 Then
			line = Mid( InputString, left_ )
		Else
			line = Mid( InputString, left_, right_ - left_ + 1 )
		End If

		If re.Test( line ) = is_match Then
			FindStringLines = FindStringLines + line
		End If

		If right_ = 0 Then _
			Exit Do

		left_ = right_ + 1
	Loop
End Function


 
'*************************************************************************
'  <<< [SortStringLines] >>> 
'*************************************************************************
Function  SortStringLines( InputString, IsDuplicated )

	'// Set "lines"
	ReDim  lines(-1)
	lines_fast_ubound = UBound( lines )

	left_ = 1
	Do
		'// Set "line_object"
		Set line_object = new NameOnlyClass

		right_ = InStr( left_, InputString, vbLF )
		If right_ = 0 Then

			'// Set "line_feed"
			right1 = InStr( InputString, vbLF )
			If right1 = 0 Then
				SortStringLines = InputString
				Exit Function
			End If
			line_feed = vbLF
			If right1 >= 2 Then
				If Mid( InputString, right1 - 1, 1 ) = vbCR Then
					line_feed = vbCRLF
				End If
			End If

			'// ...
			line_object.Name = Mid( InputString, left_ ) + line_feed
		Else
			line_object.Name = Mid( InputString, left_, right_ - left_ + 1 )
		End If


		'// Add to "lines"
		If UBound( lines ) <= lines_fast_ubound Then _
			ReDim Preserve  lines( ( lines_fast_ubound + 100 ) * 4 )
		lines_fast_ubound = lines_fast_ubound + 1
		Set lines( lines_fast_ubound ) = line_object


		'// Exit
		If right_ = 0 Then _
			Exit Do

		left_ = right_ + 1
	Loop
	ReDim Preserve  lines( lines_fast_ubound )


	'// ...
	QuickSort  lines, 0, UBound( lines ), GetRef( "NameCompare" ), Empty


	'// Make return value
	If IsDuplicated Then
		For Each  line_object  In lines
			SortStringLines = SortStringLines + line_object.Name
		Next
	Else
		For Each  line_object  In lines
			If line_object.Name <> previous_line Then
				SortStringLines = SortStringLines + line_object.Name
				previous_line = line_object.Name
			End If
		Next
	End If
End Function


 
'*************************************************************************
'  <<< [ReplaceTextFile] >>> 
'*************************************************************************
Class  ReplaceItem
	Public  Src
	Public  Dst
End Class

Sub  new_ReplaceItem( objs, n )
	ThisIsOldSpec
	new_ReplaceItem_array  objs, n
End Sub

Sub  new_ReplaceItem_array( objs, n )
	Dim i:ReDim objs(n-1):For i=0 To n-1:Set objs(i)=new ReplaceItem :Next : ErrCheck
End Sub

Sub  ReplaceTextFile( SrcPath, TmpDstPath, bDstWillBeExist, ReplaceList, Opt )
	echo  ">ReplaceTextFile  """ & SrcPath & """, """ & TmpDstPath & """, " & bDstWillBeExist
	Dim rep, item, line, b

	b = False
	Set rep = StartReplace( SrcPath, TmpDstPath, bDstWillBeExist )
	Do Until rep.r.AtEndOfStream
		line = rep.r.ReadLine
		For Each  item  In ReplaceList
			If InStr( line, item.Src ) > 0 Then  b = True
			line = Replace( line, item.Src, item.Dst )
		Next
		rep.w.WriteLine  line
	Loop
	If b Then
		rep.Finish
	Else
		rep.ExitFinish  Empty
	End If
End Sub


 
'*************************************************************************
'  <<< [OpenForReplace] >>> 
'*************************************************************************
Function  OpenForReplace( SrcPath, DstPath )
	echo  ">OpenForReplace  """& SrcPath &""", """& DstPath &""""
	AssertExist  SrcPath
	Set OpenForReplace = new ReplaceTextFile1 : Set o = OpenForReplace
	o.DstPath = DstPath : If IsEmpty( DstPath ) Then  o.DstPath = SrcPath
	o.Text = ReadFile( SrcPath )
	o.BOM = ReadUnicodeFileBOM( SrcPath )
End Function


Class  ReplaceTextFile1
	Public  DstPath
	Public  Text
	Public  BOM

	Public Sub  Replace( FromText, ToText )  '//[ReplaceTextFile1::Replace]
		If IsObject( FromText ) Then  '// RegExp
			Me.Text = FromText.Replace( Me.Text, ToText )
		Else
			If InStr( ToText, FromText ) > 0 Then  '// if FromText is part of ToText
				If InStr( Me.Text, ToText ) > 0 Then  Exit Sub
			End If
			Me.Text = ReplaceTextFile1_replace( Me.Text, FromText, ToText )
		End If
	End Sub


	Public Sub  ReplaceRange( StartOfFromText, EndOfFromText, ToText )  '//[ReplaceTextFile1::ReplaceRange]
		start_pos = InStr( Me.Text, StartOfFromText )
		If start_pos = 0 Then  Exit Sub

		end_pos = InStr( start_pos + Len( StartOfFromText ),  Me.Text,  EndOfFromText )
		If end_pos = 0 Then  Raise  E_NotFoundSymbol, _
			"<ERROR msg=""Not found EndOfFromText"" tag="""+ EndOfFromText +"""/>"

		Me.Text = Left( Me.Text, start_pos - 1 ) + ToText + Mid( Me.Text, end_pos + Len( EndOfFromText ) )
	End Sub


	Private Sub  Class_Terminate()
		If Err.Number = 0 Then
			Dim  ec : Set ec = new EchoOff
			Dim  cs : Set cs = new_TextFileCharSetStack( Me.BOM )
			CreateFile  Me.DstPath,  Me.Text
		End If
	End Sub
End Class


Function  ReplaceTextFile1_replace( Text, From, To_ )
	ReplaceTextFile1_replace = Replace( Text, From, To_ )
End Function


 
'*************************************************************************
'  <<< [StartReplace] >>> 
'*************************************************************************
Function  StartReplace( SrcPath, TmpDstPath, IsDstWillBeExist )
	echo  ">StartReplace  """ & SrcPath & """, """ & TmpDstPath & """, " & IsDstWillBeExist
	Dim  ec : Set ec = new EchoOff : ErrCheck
	Dim  m : Set m = new StartReplaceObj : ErrCheck
	Dim  ls

	If IsEmpty( g_FileOptions.LineSeparator ) Then
		Set ls = new_TextFileLineSeparatorStack( g_VBS_Lib.KeepLineSeparators )
	End If
	m.Init1  SrcPath, TmpDstPath, IsDstWillBeExist
	Set StartReplace = m
End Function


 
'*************************************************************************
'  <<< [StartReplace2] >>> 
'*************************************************************************
Function  StartReplace2( SrcPath, MidPath, Flags, TmpDstPath, bDstWillBeExist )
	echo  ">StartReplace2  """ & SrcPath & """, """ & MidPath & """, """ & TmpDstPath & """, " & bDstWillBeExist
	Dim  ec : Set ec = new EchoOff : ErrCheck
	Dim  m : Set m = new StartReplaceObj : ErrCheck
	m.Init2  SrcPath, MidPath, Flags, TmpDstPath, bDstWillBeExist
	Set StartReplace2 = m
End Function


Dim  F_Txt2BinTxt : F_Txt2BinTxt = 2


Class  StartReplaceObj
	Public  SrcPath           '// as string
	Public  TmpDstPath        '// as string
	Public  IsDstWillBeExist  '// as boolean

	Public  MidPath      '// as string
	Public  Flags        '// as bitfield

	Public  r  '// as TextStream of SrcPath
	Public  w  '// as TextStream of TmpDstPath

	Private  IsFinished


'// [StartReplaceObj::Init1]
Public Sub  Init1( SrcPath, TmpDstPath, IsDstWillBeExist )
	Dim  en, ed, n_retry
	Dim  ec : Set ec = new EchoOff : ErrCheck
	Dim  c : Set c = g_VBS_Lib

	If IsDstWillBeExist = c.DstIsBackUp Then
		copy_ren  SrcPath, TmpDstPath
		Me.SrcPath = TmpDstPath
		Me.TmpDstPath = SrcPath
	Else
		Me.SrcPath = SrcPath
		Me.TmpDstPath = TmpDstPath
	End If
	Me.IsDstWillBeExist = IsDstWillBeExist

	Set Me.r = OpenForRead( Me.SrcPath )

	Dim  cs : Set cs = new_TextFileCharSetStack( ReadUnicodeFileBOM( Me.SrcPath ) )
	Set Me.w = OpenForWrite( Me.TmpDstPath, Empty )
	If g_FileOptions.LineSeparator = g_VBS_Lib.KeepLineSeparators Then  If ReadLineSeparator( Me.SrcPath ) = vbLF Then _
		Me.w.LineSeparator = vbLF
End Sub


'// [StartReplaceObj::Init2]
Public Sub  Init2( SrcPath, MidPath, Flags, TmpDstPath, IsDstWillBeExist )
	Init1  SrcPath, MidPath, IsDstWillBeExist
	Me.MidPath = MidPath
	Me.TmpDstPath = TmpDstPath
	Me.Flags = Flags or 1
End Sub


'// [StartReplaceObj::Finish]
Public Sub  Finish()
	Dim  ec : Set ec = new EchoOff : ErrCheck
	Me.r = Empty
	Me.w = Empty

	If not IsEmpty( Me.MidPath ) Then
		If Me.Flags and F_Txt2BinTxt Then
			Txt2BinTxt  Me.MidPath, Me.TmpDstPath
		Else
			copy_ren  Me.MidPath, Me.TmpDstPath
		End If
		del  Me.MidPath
	End If

	If Me.IsDstWillBeExist = False Then
		copy_ren  Me.TmpDstPath, Me.SrcPath
		del   Me.TmpDstPath
	End If
	IsFinished = True  '// Me.IsFinished
End Sub


'// [StartReplaceObj::ExitFinish]
Public Sub  ExitFinish( Opt )
	IsFinished = True  '// Me.IsFinished
	Class_Terminate
	If not IsEmpty( Me.MidPath ) Then  del  Me.MidPath
	del  Me.TmpDstPath
End Sub


'// [StartReplaceObj::Class_Terminate]
Private Sub  Class_Terminate()
	Dim  en,ed : en = Err.Number : ed = Err.Description
	Dim  c : Set c = g_VBS_Lib
	On Error Resume Next  ' This clears the error
		Me.r = Empty
		Me.w = Empty
		If en <> 0 and en <> 21 and  not IsFinished Then  '// Me.IsFinished
			If Me.IsDstWillBeExist = c.DstIsBackUp Then
				copy_ren  Me.SrcPath, Me.TmpDstPath
				del  Me.SrcPath
			Else
				del  Me.TmpDstPath
			End If
		End If
		ErrorCheckInTerminate  en
	If en = 0 and not IsFinished Then  NotCallFinish  '// Me.IsFinished
	On Error GoTo 0 : If en <> 0 Then  Err.Raise en,,ed
End Sub


End Class


 
'*************************************************************************
'  <<< [SplitLineAndCRLF] >>> 
'*************************************************************************
Sub  SplitLineAndCRLF( LineAndCRLF, out_Line, out_CRLF )
	If Right( LineAndCRLF, 2 ) = vbCRLF Then
		out_Line = Left( LineAndCRLF, Len( LineAndCRLF ) - 2 )
		out_CRLF = vbCRLF
	ElseIf Right( LineAndCRLF, 1 ) = vbLF Then
		out_Line = Left( LineAndCRLF, Len( LineAndCRLF ) - 1 )
		out_CRLF = vbLF
	Else
		out_Line = LineAndCRLF
		out_CRLF = ""
	End If
End Sub


 
'*************************************************************************
'  <<< [PassThroughLineFilters] >>> 
'*************************************************************************
Sub  PassThroughLineFilters( SrcPath, TmpDstPath, IsDstWillBeExist, Opt, FilterArray )
	Dim rep, line, check_line
	Dim is_not

	ThisIsOldSpec  '// Use "FindStringLines"

	echo  ">PassThroughLineFilters  """+ SrcPath +""", """& TmpDstPath &""""
	Dim  ec : Set ec = new EchoOff

	If IsEmpty( Opt ) Then  is_not = False  Else  is_not = not Opt

	Set rep = StartReplace( SrcPath, TmpDstPath, IsDstWillBeExist )
	Do Until rep.r.AtEndOfStream
		line = rep.r.ReadLine()

		If not is_not Then
			For Each  check_line  In FilterArray
				If InStr( line, check_line ) > 0 Then
					rep.w.WriteLine  line
					Exit For
				End IF
			Next
		Else
			For Each  check_line  In FilterArray
				If InStr( line, check_line ) > 0 Then  Exit For
			Next
			If IsEmpty( check_line ) Then _
				rep.w.WriteLine  line
		End If
	Loop
	rep.Finish
End Sub


 
'*************************************************************************
'  <<< [g_FileOptions] >>> 
'*************************************************************************
Dim  g_FileOptions
Class  FileOptionsClass
	Public Property Get  CharSet : CharSet = m_CharSet : End Property  '// as lower case string.  ADODB.Stream.Charset
	Public Property Get  CharSetEx : CharSetEx = m_CharSetEx : End Property  '// as Empty, c.UTF_8_No_BOM
	Public Property Get  LineSeparator : LineSeparator = m_LineSeparator : End Property  '// as ADODBConsts.Keep
	Public Property Get  IsSafeFileUpdate : IsSafeFileUpdate = m_IsSafeFileUpdate : End Property  '// as boolean

	Private  m_CharSet
	Private  m_CharSetEx
	Private  m_LineSeparator
	Private  m_IsSafeFileUpdate

	Public Sub  SetCharSetPrivate(x,y) : m_CharSet = x : m_CharSetEx = y : End Sub
	Public Sub  SetLineSeparatorPrivate(x) : m_LineSeparator = x : End Sub
	Public Sub  SetIsSafeFileUpdatePrivate(x) : m_IsSafeFileUpdate = x : End Sub
End Class


 
'*************************************************************************
'  <<< [new_TextFileCharSetStack] >>> 
'*************************************************************************
Function  new_TextFileCharSetStack( CharSet )
	Set new_TextFileCharSetStack = new TextFileCharSetStack
	new_TextFileCharSetStack.Set_  CharSet
End Function

Class  TextFileCharSetStack
	Public  m_Prev, m_PrevEx
	Private Sub  Class_Initialize() : m_Prev = g_FileOptions.CharSet : m_PrevEx = g_FileOptions.CharSetEx : End Sub
	Public  Sub  Set_( ByVal CharSet )
		Dim  c : Set c = g_VBS_Lib
		If CharSet = c.Unicode      Then  CharSet = "unicode"
		If CharSet = c.UTF_8        Then  CharSet = "utf-8"
		If CharSet = c.UTF_8_No_BOM Then  CharSet = "utf-8-no-bom"
		If CharSet = c.Shift_JIS    Then  CharSet = "shift_jis"
		If CharSet = c.EUC_JP       Then  CharSet = "euc_jp"
		If CharSet = c.ISO_2022_JP  Then  CharSet = "ISO-2022-JP"
		If CharSet = c.No_BOM       Then  CharSet = "shift_jis"

		If VarType( CharSet ) <> vbString Then  CharSet = Empty
		Select Case  LCase( CharSet )
			Case "utf-8-no-bom" : g_FileOptions.SetCharSetPrivate  "utf-8", c.UTF_8_No_BOM
			Case ""   : g_FileOptions.SetCharSetPrivate  Empty, Empty
			Case Else : g_FileOptions.SetCharSetPrivate  LCase( CharSet ), Empty
		End Select
	End Sub
	Private Sub Class_Terminate : g_FileOptions.SetCharSetPrivate  m_Prev, m_PrevEx : End Sub
End Class


 
'*************************************************************************
'  <<< [new_TextFileLineSeparatorStack] >>> 
'*************************************************************************
Function  new_TextFileLineSeparatorStack( LineSeparator )
	Set new_TextFileLineSeparatorStack = new TextFileLineSeparatorStack
	new_TextFileLineSeparatorStack.Set_  LineSeparator
End Function

Class  TextFileLineSeparatorStack
	Public  m_Prev
	Private Sub  Class_Initialize() : m_Prev = g_FileOptions.LineSeparator : End Sub
	Public  Sub  Set_( ByVal LineSeparator )
		Dim  c : Set c = g_VBS_Lib
		If LineSeparator = vbCRLF  Then  LineSeparator = c.CRLF
		If LineSeparator = vbLF    Then  LineSeparator = c.LF

		g_FileOptions.SetLineSeparatorPrivate  LineSeparator
	End Sub
	Private Sub Class_Terminate : g_FileOptions.SetLineSeparatorPrivate  m_Prev : End Sub
End Class


 
'*************************************************************************
'  <<< [new_IsSafeFileUpdateStack] >>> 
'*************************************************************************
Function  new_IsSafeFileUpdateStack( IsEnabled )
	Set new_IsSafeFileUpdateStack = new IsSafeFileUpdateStack
	new_IsSafeFileUpdateStack.Set_  IsEnabled
End Function

Class  IsSafeFileUpdateStack
	Public  m_Prev
	Private Sub  Class_Initialize() : m_Prev = g_FileOptions.IsSafeFileUpdate : End Sub
	Public  Sub  Set_( IsEnabled )
		g_FileOptions.SetIsSafeFileUpdatePrivate  IsEnabled
	End Sub
	Private Sub Class_Terminate : g_FileOptions.SetIsSafeFileUpdatePrivate  m_Prev : End Sub
End Class


 
'*************************************************************************
'  <<< [ConvertBinaryEmulated] >>> 
'*************************************************************************
Sub  ConvertBinaryEmulated( SrcPath, TmpDstPath, bDstWillBeExist, ExpectedBeforePath, ExpectedAfterPath )
	If not fc( SrcPath, ExpectedBeforePath ) Then
		Setting_openFolder  GetParentFullPath( SrcPath )
		Setting_openFolder  GetParentFullPath( ExpectedBeforePath )
		Raise  1,"<ERROR msg='converting source file is not same' src='"+ XmlAttrA( SrcPath ) +_
			"' expected='"+ XmlAttrA( ExpectedBeforePath ) +"'/>"
	End If

	If bDstWillBeExist Then
		copy_ren  ExpectedAfterPath, TmpDstPath
	Else
		copy_ren  ExpectedAfterPath, SrcPath
	End If
End Sub


 
'-------------------------------------------------------------------------
' ### <<<< [ArrayClass] Class >>>> 
'-------------------------------------------------------------------------

Class  ArrayClass
	Public  Items '// as array

	Private Sub  Class_Initialize() : ReDim  Items( -1 ) : End Sub

	Public Default Property Get  Item( i )
		If IsEmpty( Me.ItemFunc ) Then  LetSet  Item, Items(i)  Else  LetSet  Item, Me.ItemFunc( Me.Items, i )
	End Property
	Public Property Let  Item( i, Value ) : Items(i) = Value : End Property
	Public Property Set  Item( i, Object ) : Set Items(i) = Object : End Property
	Public  ItemFunc  '// as Function ( Items as Array, Name as variant ) as variant

		Public Property Get  m_Array() : ThisIsOldSpec : m_Array = Items : End Property
		'//Public Property Get  m_Array(i) '// Can not define. then Me.m_Array(i) can not be used.


	Public Sub  ToEmpty()
		ReDim  Items( -1 )
	End Sub


	'// [ArrayClass::NewIterator]
	Public Function  NewIterator()
		Set NewIterator = new ArrayClassIterator : With NewIterator
			Set .Collection = Me
			.Index = -1
			.LastIndex = UBound( Items )
		End With
	End Function


	'// [ArrayClass::ReDim_]
	Public Sub  ReDim_( UBoundValue )
		ReDim Preserve  Items( UBoundValue )
	End Sub


	'// [ArrayClass::Add]
	Public Sub  Add( Elem )
		ReDim Preserve  Items( UBound(Items) + 1 )
		If IsObject( Elem ) Then
			Set Items( UBound(Items) ) = Elem
		Else
			Items( UBound(Items) ) = Elem
		End If
	End Sub


	'// [ArrayClass::AddNewObject]
	Public Function  AddNewObject( ClassName, ObjectName )
		Set AddNewObject = new_X( ClassName )
		Me.Push  AddNewObject
		If g_Vers("CutPropertyM") Then
			AddNewObject.Name = ObjectName
		Else
			AddNewObject.m_Name = ObjectName
		End If
	End Function


	'// [ArrayClass::Push]
	Public Sub  Push( Elem ) : Add  Elem : End Sub


	'// [ArrayClass::Pop]
	Public Function  Pop()
		If IsObject( Items( UBound(Items) ) ) Then
			Set Pop = Items( UBound(Items) )
		Else
			Pop = Items( UBound(Items) )
		End If
		ReDim Preserve  Items( UBound(Items) - 1 )
	End Function


	'// [ArrayClass::Remove]
	Public Sub  Remove( i, n )
		Dim  x
		If IsObject( Items(i) ) Then
			For x=i To UBound(Items) - n
				Set Items( x ) = Items( x + n )
			Next
		Else
			For x=i To UBound(Items) - n
				Items( x ) = Items( x + n )
			Next
		End If
		ReDim Preserve  Items( UBound(Items) - n )
	End Sub


	'// [ArrayClass::RemoveObject]
	Public Function  RemoveObject( a_Object )
		Dim  i, j, u : u = UBound( Items )
		For i=0 To u
			If Items(i) is a_Object Then
				Set RemoveObject = a_Object
				For j=i+1 To u
					Set Items(j-1) = Items(j)
				Next
				ReDim Preserve  Items(u-1)
				Exit Function
			End If
		Next
		Set RemoveObject = Nothing
	End Function


	'// [ArrayClass::RemoveEmpty]
	Public Sub  RemoveEmpty()
		Dim  i, j, u : u = UBound( Items )

		For i=0 To u
			If IsEmpty( Items(i) ) Then  j=i : Exit For
		Next
		If IsEmpty( j ) Then  Exit Sub

		For i=i+1 To u
			If not IsEmpty( Items(i) ) Then
				If IsObject( Items(i) ) Then
					Set Items(j) = Items(i)
				Else
					Items(j) = Items(i)
				End If
				j=j+1
			End If
		Next
		ReDim Preserve  Items(j-1)
	End Sub


	'// [ArrayClass::Count]
	Public Property Get  Count() : Count = UBound(Items) + 1 : End Property
	Public Property Get  Length() : Length = UBound(Items) + 1 : End Property
	Public Property Get  UBound_() : UBound_ = UBound(Items) : End Property

	Public Sub  Echo() : ThisIsOldSpec : WScript.Echo  xml : End Sub
	Public Property Get  Value() : ThisIsOldSpec : Value = xml : End Property


	'// [ArrayClass::xml]
	Public Property Get  xml() : xml = xml_sub(0) : CutLastOf xml, vbCRLF, Empty : End Property
	Public Function  xml_sub( Level )
		Dim  s, i

		s = GetTab( Level ) +"<ArrayClass count="""& Count &""">"+vbCRLF
		For i=0 To UBound( Items )
			s = s + GetTab( Level+1 )+ "<Item id="""& i &""">"& Trim( GetEchoStr_sub( Items(i), Level+1 ) )
			CutLastOf  s, vbCRLF, Empty
			s = s + "</Item>" +vbCRLF
		Next
		s = s + GetTab( Level ) +"</ArrayClass>"+ vbCRLF
		xml_sub = s
	End Function


	'// [ArrayClass::CSV]
	Public Property Get  CSV()
		Dim  i, n
		For Each i In Me.Items
			If not IsEmpty( CSV ) Then  CSV = CSV + ","
			CSV = CSV & i
		Next
	End Property


	'// [ArrayClass::AddCSV]
	Public Sub  AddCSV( CSV, Type_ )
		Dim  s, i : i = 1
		If Trim2( CSV ) = "" Then  Exit Sub
		Do
			s = MeltCSV( CSV, i )
			If IsEmpty( s ) Then
				Me.Add  Empty
			Else
				Select Case  Type_
					Case vbInteger:  Me.Add  CInt2( s )
					Case Else:       Me.Add  s
				End Select
			End If
			If i = 0 Then Exit Do
		Loop
	End Sub


	'// [ArrayClass::Code]
	Public Property Get  Code()
		Dim  i, n
		For Each i In Me.Items
			If not IsEmpty( Code ) Then  Code = Code +", _"+ vbCRLF
			If VarType( i ) = vbString Then
				Code = Code & """" & Replace( i, """", """""" ) & """"
			Else
				Code = Code & i
			End If
		Next
	End Property


	'// [ArrayClass::Copy]
	Public Sub  Copy( SrcArr )
		If IsArray( SrcArr ) Then
			Items = SrcArr
		ElseIf TypeName( SrcArr ) = "ArrayClass" Then
			Items = SrcArr.Items
		Else
			Err.Raise  1
		End If
	End Sub


	'// [ArrayClass::AddElems]
	Public Sub  AddElems( SrcArr )
		If IsArray( SrcArr ) Then
			AddArrElem  Items, SrcArr
		ElseIf TypeName( SrcArr ) = "ArrayClass" Then
			AddArrElem  Items, SrcArr.Items
		Else
			Me.Add  SrcArr
		End If
	End Sub


	'// [ArrayClass::LookUpDic]
	Public Function  LookUpDic( Key, Value )
		Dim  dic
		For Each dic  In Items
			If dic.Item( Key ) = Value Then  Set LookUpDic = dic : Exit Function
		Next
		Set LookUpDic = Nothing
	End Function
End Class


'*************************************************************************
'  <<< [new_ArrayClass] >>>
'*************************************************************************
Function  new_ArrayClass( a_Array )
	Set  new_ArrayClass = new ArrayClass
	new_ArrayClass.AddElems  a_Array
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [ArrayClassIterator] Class >>>> 
'-------------------------------------------------------------------------

Class  ArrayClassIterator
	Public  Collection ' as ArrayClass
	Public  Index ' as integer
	Public  LastIndex ' as integer

	Private Sub  Class_Initialize()
		Me.Index = -1
		Me.LastIndex = -1
	End Sub

	Public Function  GetNext()
		Me.Index = Me.Index + 1
		LetSet  GetNext, Me.Collection( Me.Index )
	End Function

	Public Function  HasNext()
		HasNext = ( Index < LastIndex )
	End Function

	Public Function  GetNextOrSentinel( Sentinel )
		Me.Index = Me.Index + 1
		If Index > LastIndex Then
			LetSet  GetNextOrSentinel, Sentinel
		Else
			LetSet  GetNextOrSentinel, Me.Collection( Me.Index )
		End If
	End Function
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [ArrayDictionary] Class >>>> 
'-------------------------------------------------------------------------

Class  ArrayDictionary

	Public  Dic  ' as Dictionary of ArrayClass

	Private Sub  Class_Initialize
		Set  Me.Dic = CreateObject("Scripting.Dictionary")
	End Sub

	Public Sub  ToEmpty
		Me.Dic.RemoveAll
	End Sub

	Public Sub  Add( key, item )
		Dim  dic_item

		If Me.Dic.Exists( key ) Then
			Me.Dic.Item( key ).Add  item
		Else
			Set  dic_item = new ArrayClass : ErrCheck
			dic_item.Add  item
			Me.Dic.Add  key, dic_item
		End If
	End Sub

	Public Function  Count
		Dim  i
		Count = 0
		For Each i in Me.Dic.Items()
			Count = Count + i.Count
		Next
	End Function

	Public Sub  Echo
		Dim  i, n

		WScript.Echo  "--- ArrayDictionary ------------------------------"
		WScript.Echo  "key  count = " & Me.Dic.Count

		WScript.Echo  "item count = " & Count

		For Each i in Me.Dic.Keys()
			WScript.Echo  "key=""" & i & """"
			WScript.Echo  Me.Dic.Item(i).xml
		Next
		WScript.Echo ""
	End Sub

End Class


 
'*************************************************************************
'  <<< [TreeA_Class] >>> 
'*************************************************************************
Function  new_TreeA_Class( in_Item )
	Set new_TreeA_Class = new TreeA_Class
	Set new_TreeA_Class.p.Item = in_Item
End Function

Class  TreeA_Class
	Public  p  '// Root node as TreeA_NodeClass

	Private Sub  Class_Initialize()
		Set group = new LifeGroupClass
		group.AddHandle  Me, new TreeA_NodeClass
	End Sub

	Private Sub  Class_Terminate()
		If not IsEmpty( Me.p.LifeGroup.Group ) Then _
			Me.p.LifeGroup.Group.AddTerminated  Me.p
	End Sub

	Property Get  TreeNode() : Set TreeNode = Me.p : End Property

	Function  CreateNode() : Set CreateNode = Me.p.CreateNode() : End Function
	Function  AddNewNode( x ) : Set AddNewNode = Me.p.AddNewNode( x ) : End Function
	Property Get  ParentNode() : Set ParentNode = Nothing : End Property
	Property Get  ChildNodes( i ) : Set ChildNodes = Me.p.ChildNodes( i ) : End Property
	Property Get  ParentItems() : ParentItems = Me.p.ParentItems : End Property
	Property Get  ChildItems() : ChildItems = Me.p.ChildItems : End Property

	Property Get  Item() : Set Item = Me.p.Item : End Property
	Property Set  Item(x) : Set Me.p.Item = x : End Property

	Public Property Get  xml() : xml = p.xml : End Property
	Public Function  xml_sub( Level ) : xml_sub = p.xml_sub( Level ) : End Function
End Class


 
'*************************************************************************
'  <<< [TreeA_NodeClass] >>> 
'*************************************************************************
Class  TreeA_NodeClass
	Public  ParentNode  '// as TreeA_NodeClass
	Public  ChildNodes  '// as ArrayClass of TreeA_NodeClass
	Public  Item        '// as Object

	Public  LifeGroup   '// Root node only has "LifeGroup".

	Private Sub  Class_Initialize()
		Set ParentNode = Nothing
		Set Me.ChildNodes = new ArrayClass
		Set Me.Item = Nothing
	End Sub

	Public Sub  DestroyReferences()
		Me.ParentNode = Empty
		Me.ChildNodes = Empty
		Me.Item = Empty
	End Sub

	Property Get  TreeNode() : Set TreeNode = Me : End Property


	Function  AddNewNode( x )
		Set new_node = Me.CreateNode()
		Me.ChildNodes.Add  new_node
		Set new_node.ParentNode = Me
		Set new_node.Item = x
		Set AddNewNode = new_node
	End Function


	Function  CreateNode()  '// Not add to "ChildNodes" and "ParentNode"
		Set node = new TreeA_NodeClass
		Me.LifeGroup.Group.Add  node
		Set CreateNode = node
	End Function


	Property Get  ParentItems()
		Set an_array = new ArrayClass
		Set node = Me.ParentNode
		While  not node is Nothing
			an_array.Add  node.Item
			Set node = node.ParentNode
		WEnd
		ParentItems = an_array.Items
	End Property


	Property Get  ChildItems()
		Set an_array = new ArrayClass
		Me.EnumerateChildItems  an_array
		ChildItems = an_array.Items
	End Property
	Function  EnumerateChildItems( arg_Array )
		For Each  node  In  Me.ChildNodes.Items
			node.EnumerateChildItems  arg_Array
		Next
		arg_Array.Add  Me.Item
	End Function


	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		xml_sub = GetTab( Level )+ "<"+TypeName(Me)+">"

		xml_sub2 = GetEchoStr_sub( Item, Level )
		If InStr( xml_sub2, vbLF ) = Len( xml_sub2 ) Then
			xml_sub = xml_sub + Trim( xml_sub2 )
		Else
			xml_sub = xml_sub + vbCRLF + xml_sub2
		End If

		xml_sub2 = ""
		For Each  node  In  Me.ChildNodes.Items
			xml_sub2 = xml_sub2 + GetEchoStr_sub( node, Level )
		Next
		If xml_sub2 = "" Then
			CutLastOf  xml_sub, vbCRLF, Empty
		Else
			xml_sub = xml_sub + xml_sub2 + GetTab( Level )
		End If

		xml_sub = xml_sub +"</"+TypeName(Me)+">"+ vbCRLF
	End Function
End Class


 
'*************************************************************************
'  <<< [head] >>> 
'*************************************************************************
Function  head( Str, LineLength )
	Dim  pos : pos = 1
	Dim  line_num

	For line_num = 1  To LineLength
		pos = InStr( pos, Str, vbLF )
		If pos = 0 Then  head = Str : Exit For
		pos = pos + 1
	Next
	If line_num > LineLength Then  head = Left( Str, pos )

	'// cut last CR LF
	Do
		If Right( head, 2 ) = vbCRLF Then
			head = Left( head, Len( head ) - 2 )
		ElseIf Right( head, 1 ) = vbLF Then
			head = Left( head, Len( head ) - 1 )
		Else
			Exit Do
		End If
	Loop
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [StringStream] Class >>>> 
'-------------------------------------------------------------------------

Class  StringStream

	Public   Str, StrLen
	Public   NextLinePos
	Public   IsWithLineFeed
	Public   WritingLineFeed
	Private  m_ReadLine, m_WriteLine, m_bPrevIsWrite

	Private Sub  Class_Initialize()
		Me.IsWithLineFeed = False
		Me.WritingLineFeed = vbCRLF
	End Sub

	Public Property Get Line()
		If m_bPrevIsWrite Then  Line = m_WriteLine  Else  Line = m_ReadLine
	End Property

	Public  Sub  SetString( Str )
		If Me.IsWithLineFeed Then
			Me.Str = Str
		Else
			Me.Str = Replace( Str, vbCRLF, vbLF ) '// for supporting vbCRLF and vbLF
		End If
		Me.StrLen = Len( Me.Str )
		Me.NextLinePos = 1
		m_ReadLine = 1
		m_WriteLine = 1
	End Sub

	Public Function  ReadLine()
		i = InStr( Me.NextLinePos, Me.Str, vbLF )
		If i > 0 Then
			If Me.IsWithLineFeed Then
				ReadLine = Mid( Me.Str, Me.NextLinePos, i - Me.NextLinePos + 1 )
			Else
				ReadLine = Mid( Me.Str, Me.NextLinePos, i - Me.NextLinePos )
			End If
			Me.NextLinePos = i + 1
			If Me.NextLinePos > Me.StrLen Then
				Me.Str = Empty
				Me.NextLinePos = Empty
			End If
		Else
			ReadLine = Mid( Me.Str, Me.NextLinePos )
			Me.Str = Empty
			Me.NextLinePos = Empty
		End If
		m_ReadLine = m_ReadLine + 1
	End Function

	Public Function  ReadAll()
		If not IsEmpty( Me.StrLen ) Then  Raise 1, "Not supported"
		ReadAll = Me.Str
		Me.Str = Empty
	End Function

	Public Property Get AtEndOfStream : AtEndOfStream = IsEmpty( Me.Str ) : End Property
	Public Sub  Write( Str ) : Me.Str = Me.Str + Str : End Sub

	Public Sub  WriteLine( LineStr )
		Me.Str = Me.Str + LineStr + Me.WritingLineFeed
		m_WriteLine = m_WriteLine + 1
	End Sub
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [StrMatchKey] Class >>>> 
'-------------------------------------------------------------------------
Class  StrMatchKey

	Public Property Let  Keyword( s )
		m_Keyword = s
		m_LeftCount = InStr( s, "*" ) - 1
		If m_LeftCount > -1 Then
			m_LeftStr = Left( s, m_LeftCount )
			m_RightCount = Len( s ) - m_LeftCount - 1
			m_RightStr = Right( s, m_RightCount )
		Else
			m_LeftStr = Empty
			m_RightCount = Len( s )
			m_RightStr = s
		End If

		If InStr( m_LeftCount + 2, s, "*" ) > 0 Then _
			Raise  1,"Multi * can not be set"
	End Property

	Public Property Get  Keyword()
		Keyword = m_Keyword
	End Property


	Public Function  IsMatch( TestStr )
		'// m_Keyword must be low case
		IsMatch = False

		If LCase( Right( TestStr, m_RightCount ) ) = m_RightStr Then
			If m_LeftCount <= 0 Then
				If m_LeftCount = 0  or  Len( TestStr ) = m_RightCount Then
					IsMatch = True
				End If
			Else
				If LCase( Left( TestStr, m_LeftCount ) ) = m_LeftStr Then
					IsMatch = True
				End If
			End If
		End If
	End Function

	Public Function  IsMatchULCase( TestStr )
		IsMatchULCase = False

		If Right( TestStr, m_RightCount ) = m_RightStr Then
			If m_LeftCount <= 0 Then
				If m_LeftCount = 0  or  Len( TestStr ) = m_RightCount Then
					IsMatchULCase = True
				End If
			Else
				If Left( TestStr, m_LeftCount ) = m_LeftStr Then
					IsMatchULCase = True
				End If
			End If
		End If
	End Function

	Public Function  Test( TestStr )
		If IgnoreCase Then
			Test = Me.IsMatch( TestStr )
		Else
			Test = Me.IsMatchULCase( TestStr )
		End If
	End Function

	Public  m_Keyword
	Public  m_LeftCount
	Public  m_RightCount
	Public  m_LeftStr
	Public  m_RightStr
	Public  IgnoreCase
End Class


 
'*************************************************************************
'  <<< [StringReplaceSetClass] >>> 
'*************************************************************************
Class  StringReplaceSetClass

	Public  List
	Public  ReplaceCommand,  ReplaceRangeCommand
	Public  OneCommandLength

	Private Sub  Class_Initialize()
		ReDim  List(-1)
		Me.ReplaceCommand = 1
		Me.ReplaceRangeCommand = 2
		Me.OneCommandLength = 4
	End Sub


	Public Sub  Replace( FromText, ToText )
		u_bound = UBound( Me.List )
		ReDim Preserve  List( u_bound + Me.OneCommandLength )
		index = u_bound + 1
		Me.List( index + 0 ) = Me.ReplaceCommand
		Me.List( index + 1 ) = FromText
		Me.List( index + 2 ) = ToText
	End Sub


	Public Sub  ReplaceRange( StartOfFromText, EndOfFromText, ToText )
		u_bound = UBound( Me.List )
		ReDim Preserve  List( u_bound + Me.OneCommandLength )
		index = u_bound + 1
		Me.List( index + 0 ) = Me.ReplaceRangeCommand
		Me.List( index + 1 ) = StartOfFromText
		Me.List( index + 2 ) = EndOfFromText
		Me.List( index + 3 ) = ToText
	End Sub


	Public Function  DoReplace( InputText )
		text = InputText

		For index = 0  To UBound( Me.List )  Step Me.OneCommandLength
			Select Case  Me.List( index + 0 )

				Case  Me.ReplaceCommand
					FromText = Me.List( index + 1 )
					ToText   = Me.List( index + 2 )

					is_skip = False
					If InStr( ToText, FromText ) > 0 Then  '// if FromText is part of ToText
						If InStr( text, ToText ) > 0 Then
							is_skip = True
						End If
					End If
					If not is_skip Then _
						text = ReplaceTextFile1_replace( text, FromText, ToText )

				Case  Me.ReplaceRangeCommand
					StartOfFromText = Me.List( index + 1 )
					EndOfFromText   = Me.List( index + 2 )
					ToText          = Me.List( index + 3 )

					start_pos = InStr( text, StartOfFromText )
					If start_pos > 0 Then
						end_pos = InStr( start_pos + Len( StartOfFromText ),  text,  EndOfFromText )
						If end_pos = 0 Then  Raise  E_NotFoundSymbol, _
							"<ERROR msg=""Not found EndOfFromText"" tag="""+ EndOfFromText +"""/>"

						text = Left( text, start_pos - 1 ) + ToText + _
							Mid( text, end_pos + Len( EndOfFromText ) )
					End If

				Case Else
					Error
			End Select
		Next

		DoReplace = text
	End Function
End Class


 
'*************************************************************************
'  <<< [ToRegExpPattern] >>> 
'*************************************************************************
Function  ToRegExpPattern( NoEscapedString )
	ToRegExpPattern = Replace( NoEscapedString, "\", "\\" )
	ToRegExpPattern = Replace( ToRegExpPattern, "(", "\(" )
	ToRegExpPattern = Replace( ToRegExpPattern, ")", "\)" )
	ToRegExpPattern = Replace( ToRegExpPattern, "{", "\{" )
	ToRegExpPattern = Replace( ToRegExpPattern, "}", "\}" )
	ToRegExpPattern = Replace( ToRegExpPattern, "[", "\[" )
	ToRegExpPattern = Replace( ToRegExpPattern, "]", "\]" )
	ToRegExpPattern = Replace( ToRegExpPattern, ".", "\." )
	ToRegExpPattern = Replace( ToRegExpPattern, "$", "\$" )
	ToRegExpPattern = Replace( ToRegExpPattern, "^", "\^" )
	ToRegExpPattern = Replace( ToRegExpPattern, "|", "\|" )
	ToRegExpPattern = Replace( ToRegExpPattern, "*", "\*" )
	ToRegExpPattern = Replace( ToRegExpPattern, "+", "\+" )
	ToRegExpPattern = Replace( ToRegExpPattern, "?", "\?" )
End Function


 
'*************************************************************************
'  <<< [LenK] >>> 
'*************************************************************************
Function  LenK( Str )
	Dim  c, a, i, n_zen

	i = 1 : n_zen = 0
	Do
		c = Mid( Str, i, 1 )
		If c = "" Then  LenK = i - 1 + n_zen : Exit Function
		a = Asc( c )
		If a >= 256 or a < 0 Then  n_zen = n_zen + 1
		i = i + 1
	Loop
End Function


 
'*************************************************************************
'  <<< [CutRight] >>> 
'*************************************************************************
Function  CutRight( Str, Count )
	CutRight = Left( Str, Len( Str ) - Count )
End Function


 
'*************************************************************************
'  <<< [CutLastOf] >>> 
'*************************************************************************
Sub  CutLastOf( Str, LastStr, Opt )
	Dim  length : length = Len( LastStr )
	If StrComp( Right( Str, length ), LastStr, StrCompOption( Opt ) ) = 0 Then _
		Str = Left( Str, Len( Str ) - length )
End Sub


 
'*************************************************************************
'  <<< [StrCompHeadOf] >>> 
'*************************************************************************
Function  StrCompHeadOf( StrA, StrB, Opt )
	StrCompHeadOf = StrComp( Left( StrA, Len(StrB) ), StrB, StrCompOption( Opt ) )
End Function



 
'*************************************************************************
'  <<< [StrCompLastOf] >>> 
'*************************************************************************
Function  StrCompLastOf( StrA, StrB, Opt )
	StrCompLastOf = StrComp( Right( StrA, Len(StrB) ), StrB, StrCompOption( Opt ) )
End Function


 
'*************************************************************************
'  <<< [StrCount] >>> 
'*************************************************************************
Function  StrCount( ByVal Str, ByVal Keyword, StartPos, Opt )
	Dim  pos, key_length

	If StrCompOption( Opt ) = vbTextCompare Then  '// Opt is c.CaseSensitive
		Str = LCase( Str ) : Keyword = LCase( Keyword )
	End If

	StrCount = 0
	key_length = Len( Keyword )
	pos = StartPos
	Do
		pos = InStr( pos, Str, Keyword )
		If pos = 0 Then  Exit Function
		StrCount = StrCount + 1
		pos = pos + key_length
	Loop
End Function


 
'*************************************************************************
'  <<< [StrCompOption] >>> 
'*************************************************************************
Function  StrCompOption( Opt )
	If VarType( Opt ) = vbInteger Then
		If Opt and g_VBS_Lib.CaseSensitive Then
			StrCompOption = vbBinaryCompare
		Else
			StrCompOption = vbTextCompare
		End If
	Else
		StrCompOption = vbTextCompare
	End If
End Function


 
'*************************************************************************
'  <<< [AddIfNotExist] >>> 
'*************************************************************************
Function  AddIfNotExist( WholeStr, AddStr, Separator, Opt )
	elements = Split( WholeStr, Separator )
	For Each element  In elements
		If StrComp( element, AddStr, StrCompOption( Opt ) ) = 0 Then
			AddIfNotExist = WholeStr
			Exit Function
		End If
	Next
	AddIfNotExist = AddStr + Separator + WholeStr
End Function


 
'********************************************************************************
'  <<< [TestableNow] >>> 
'********************************************************************************
Dim  g_TestableNow  '// as Date or Empty

Function  TestableNow()
	If IsEmpty( g_TestableNow ) Then
		TestableNow = Now()
	Else
		TestableNow = g_TestableNow
	End If
End Function


 
'********************************************************************************
'  <<< [new_TestableNow] >>> 
'********************************************************************************
Function  new_TestableNow( a_Date )
	Set new_TestableNow = new TestableNowClass
	g_TestableNow = a_Date
End Function

Class  TestableNowClass
	Private Sub  Class_Terminate()
		g_TestableNow = Empty
	End Sub
End Class


 
'*************************************************************************
'  <<< [IsTimeOnlyDate] >>> 
'*************************************************************************
Function  IsTimeOnlyDate( aDateTime )
	IsTimeOnlyDate = Year( aDateTime ) = 1899  and  Month( aDateTime ) = 12  and  Day( aDateTime ) = 30
End Function

 
'*************************************************************************
'  <<< [DateAddStr] >>> 
'*************************************************************************
Function  DateAddStr( BaseDate, Plus )
	Dim  i, i2, c, d, flag, num, unit, i_over

	DateAddStr = BaseDate
	i=1
	i_over = Len( Plus ) + 1

	'//=== Skip spaces
	While Mid( Plus, i, 1 ) = " " : i=i+1 : WEnd

	'//=== Get flag
	flag = +1
	c = Mid( Plus, i, 1 )
	If c = "+" Then
		i=i+1
	ElseIf c = "-" Then
		flag = -1 : i=i+1
	End If

	Do

		'//=== Skip spaces
		While Mid( Plus, i, 1 ) = " " : i=i+1 : WEnd

		If i = i_over Then  Exit Do

		c = Mid( Plus, i, 1 )
		i2 = i

		'//=== change flag
		Do
			If c = "-" Then
				flag = -1
			ElseIf c = "+" Then
				flag = +1
			Else
				Exit Do
			End If
			i2=i2+1 : c = Mid( Plus, i2, 1 )
		Loop
		i = i2

		'//=== Get number or "Z"(W3C-DTF UTC)
		While c >= "0" and c <= "9" : i2=i2+1 : c = Mid( Plus, i2, 1 ) : WEnd
		If i2 = i Then
			If c <> "Z" Then  Err.Raise 1002
			num = 0
		Else
			num = CInt( Mid( Plus, i, i2 - i ) )
			i = i2
		End If

		'//=== Skip spaces
		While Mid( Plus, i, 1 ) = " " : i=i+1 : WEnd

		'//=== Get unit
		c = Mid( Plus, i, 1 )
		i2 = i
		While (c >= "a" and c <= "z") or (c >= "A" and c <= "Z") or (c = ":" or c = "/")
			i2=i2+1 : c = Mid( Plus, i2, 1 )
		WEnd

		Select Case  LCase( Mid( Plus, i, i2 - i ) )
			Case "year",  "years"  : unit = "yyyy"
			Case "month", "months" : unit = "m"
			Case "day",   "days"   : unit = "d"
			Case "hour",  "hours"  : unit = "h"
			Case "minute","minutes","min": unit = "n"
			Case "second","seconds","sec": unit = "s"

			Case ":", "/", "-"
				unit = Mid( Plus, i, 1 )

				While (c >= "0" and c <= "9") or (c=unit) : i2=i2+1 : c = Mid( Plus, i2, 1 ) : WEnd
				d = CDate( num & Mid( Plus, i, i2-i ) )
				If unit = ":" Then
					DateAddStr = DateAdd( "h", flag * Hour(d),   DateAddStr )
					DateAddStr = DateAdd( "n", flag * Minute(d), DateAddStr )
					DateAddStr = DateAdd( "s", flag * Second(d), DateAddStr )
				Else
					DateAddStr = DateAdd( "m", flag * Month(d), DateAddStr )
					DateAddStr = DateAdd( "d", flag * Day(d),   DateAddStr )
				End If
				unit = Empty

			Case "z"  '// Do nothing
			Case Else  Err.Raise  1,,"Bad unit"
		End Select
		i = i2

		'//=== Add Date
		If not IsEmpty( unit ) Then _
			DateAddStr = DateAdd( unit, flag * num, DateAddStr )

	Loop
End Function
 
'*************************************************************************
'  <<< [W3CDTF] >>> 
'*************************************************************************
Function  W3CDTF( SourceTimeDate )
	If VarType( SourceTimeDate ) = vbString Then
		Dim  c, s, tzd

		'//=== string to Date
		s = Trim2( SourceTimeDate )
		If Mid( s, 5, 1 ) = "-" Then
			c = Mid( s, 11, 1 )
			If c = "T" or c = " " Then
				If Mid( s, 17, 1 ) = ":" Then
					W3CDTF = CDate( Left( s, 10 ) +" "+ Mid( s, 12, 8 ) )
					If Mid( s, 20, 1 ) = "." Then
						c = InStr( 21, s, "+" )
						If c = 0 Then  c = InStr( 21, s, "-" )
						If c = 0 Then  c = InStr( 21, s, "Z" )
						If c = 0 Then  Err.Raise 1002
						s = Mid( s, c )
					Else
						s = Mid( s, 20 )
					End If
				Else
					W3CDTF = CDate( Left( s, 10 ) +" "+ Mid( s, 12, 5 ) )
					s = Mid( s, 17 )
				End If
				If s <> TimeZoneDesignator( Empty ) Then
					W3CDTF = DateAddStr( W3CDTF, TimeZoneDesignator( Empty ) +"+"+ MinusTZD( s ) )
				End If
			Else
				W3CDTF = CDate( s )
			End If
		Else
			W3CDTF = CDate( s +"/1/1" )
		End If

	Else
		'//=== Date to string
		W3CDTF = Right( "000"& Year( SourceTimeDate ), 4 ) +"-"+ Right( "0"& Month( SourceTimeDate ), 2 ) +"-"+ _
			Right( "0"& Day(    SourceTimeDate ), 2 ) +"T"+ Right( "0"& Hour(   SourceTimeDate ), 2 ) +":"+ _
			Right( "0"& Minute( SourceTimeDate ), 2 ) +":"+ Right( "0"& Second( SourceTimeDate ), 2 ) + _
			TimeZoneDesignator( Empty )
	End If
End Function


 
'*************************************************************************
'  <<< [CDateFromEMailDate] >>> 
'*************************************************************************
Function  CDateFromEMailDate( DateString )
	'// e.g.) Date: Fri, 25 Oct 2013 16:07:30 +0900

	Const  num_0_day  = 0
	Const  num_2_year = 2
	Const  num_3_time = 3

	line = Trim( DateString )
	items = Split( line, " " )

	offset = 0
	If items( offset ) = "Date:" Then  offset = offset + 1
	If Right( items( offset ), 1 ) = "," Then  offset = offset + 1

	month_str = items( offset + 1 )
	month_num = GetMonthNumberFromString( month_str )

	time_zone_str = items( offset + 4 )
	time_zone_str = Left( time_zone_str, Len( time_zone_str ) - 2 ) +_
		":"+ Right( time_zone_str, 2 )

	string_of_W3CDTF = _
		items( offset + num_2_year ) +"-"+ _
		Right( "0" & month_num, 2 ) +"-"+ _
		Right( "0" & items( offset + num_0_day ), 2 ) +"T"+ _
		Right( "0"+ items( offset + num_3_time ), 8 ) + _
		time_zone_str
		'// e.g. "2001-02-02T01:15:00+09:00"

	CDateFromEMailDate = W3CDTF( string_of_W3CDTF )
End Function


 
'*************************************************************************
'  <<< [GetMonthNumberFromString] >>> 
'*************************************************************************
Function  GetMonthNumberFromString( Str )
	index = InStr( g_VBS_Lib_MonthStrings, Str )
	If index = 0 Then  Raise  E_NotFoundSymbol, ""
	If Mid( g_VBS_Lib_MonthStrings, index - 1, 1 ) <> "," Then  Raise  E_NotFoundSymbol, ""
	If Mid( g_VBS_Lib_MonthStrings, index + Len( Str ), 1 ) <> "," Then  Raise  E_NotFoundSymbol, ""
	GetMonthNumberFromString = ( index + 2 ) / 4
End Function
Dim  g_VBS_Lib_MonthStrings : g_VBS_Lib_MonthStrings = ",Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec,"


 
'*************************************************************************
'  <<< [ConvertTimeZone] >>> 
'*************************************************************************
Function  ConvertTimeZone( DateTime, FromTZD, ByVal ToTZD )
	Dim  utc, local

	If IsEmpty( ToTZD ) Then  ToTZD = TimeZoneDesignator( Empty )

	'//=== convert W3CDTF
	If VarType( DateTime ) = vbString Then

		If Mid( FromTZD, 3, 1 ) = ":" Then  FromTZD = Left( FromTZD, 1 ) +"0"+ Mid( FromTZD, 2 ) '// +0:00 -> +00:00
		If Mid( ToTZD,   3, 1 ) = ":" Then  ToTZD   = Left( ToTZD,   1 ) +"0"+ Mid( ToTZD,   2 ) '// +0:00 -> +00:00

		If not IsEmpty( FromTZD ) Then
			If FromTZD <> Right( DateTime, Len( FromTZD ) ) Then
				Raise  1, "If DateTime was set W3CDTF, Set FromTZD to Empty or TZD of DateTime"
			End If
		End If

		local = W3CDTF( DateTime )
		utc = DateAddStr( local, MinusTZD( Empty ) )
		ConvertTimeZone = Left( W3CDTF( DateAddStr( utc, ToTZD ) ), 19 ) + ToTZD

	'//=== convert value as Date
	Else
		utc = DateAddStr( DateTime, MinusTZD( FromTZD ) )
		ConvertTimeZone = DateAddStr( utc, ToTZD )
	End If
End Function


 
'*************************************************************************
'  <<< [TimeZoneDesignator] >>> 
'*************************************************************************
Dim  g_TimeZoneDesignatorHere  '// as string

Function  TimeZoneDesignator( MinuteBias )
	If IsEmpty( MinuteBias ) Then
		If IsEmpty( g_TimeZoneDesignatorHere ) Then _
			g_TimeZoneDesignatorHere = TimeZoneDesignator( get_WMI( Empty, "Win32_TimeZone" ).Bias )
		TimeZoneDesignator = g_TimeZoneDesignatorHere
	Else
		If MinuteBias = 0 Then
			TimeZoneDesignator = "Z"
		ElseIf MinuteBias > 0 Then
			TimeZoneDesignator = "+" + Right( "0" & CInt( MinuteBias / 60 ), 2 ) +_
						":" + Right( "0" &   ( MinuteBias mod 60 ), 2 )
		Else
			TimeZoneDesignator = "-" + Right( "0" & CInt( -MinuteBias / 60 ), 2 ) +_
						":" + Right( "0" &   ( -MinuteBias mod 60 ), 2 )
		End If
	End If
End Function


 
'*************************************************************************
'  <<< [MinusTZD] >>> 
'*************************************************************************
Function  MinusTZD( MinuteBiasOrStr )
	If VarType( MinuteBiasOrStr ) = vbString Then
		Select Case  Left( MinuteBiasOrStr, 1 )
			Case  "-" : MinusTZD = "+" + Mid( MinuteBiasOrStr, 2 )
			Case  "Z" : MinusTZD = "Z"
			Case Else : MinusTZD = "-" + Mid( MinuteBiasOrStr, 2 )
		End Select
	Else
		MinusTZD = MinusTZD( TimeZoneDesignator( MinuteBiasOrStr ) )
	End If
End Function


 
'*************************************************************************
'  <<< [Bench] >>> 
'*************************************************************************
Dim  g_Bench_Elapseds  '// as array of Single
Dim  g_Bench_Counts  '// as array of integer
Dim  g_Bench_CurrentSectionNum  '// as integer
Dim  g_Bench_PrevTime  '// as Single

Sub  BenchStart()
	ReDim  g_Bench_Elapseds( 100 )
	ReDim  g_Bench_Counts( 100 )
	g_Bench_CurrentSectionNum = 0
	g_Bench_PrevTime = Timer()
End Sub


Sub  Bench( SectionNum )
	Dim  current_time : current_time = Timer()
	g_Bench_Elapseds( g_Bench_CurrentSectionNum ) = g_Bench_Elapseds( g_Bench_CurrentSectionNum ) +_
		current_time - g_Bench_PrevTime
	g_Bench_Counts( g_Bench_CurrentSectionNum ) = g_Bench_Counts( g_Bench_CurrentSectionNum ) + 1
	g_Bench_CurrentSectionNum = SectionNum
	g_Bench_PrevTime = current_time
End Sub


Sub  BenchEnd()
	Dim  i,  count,  elapsed,  max_section_num

	Bench  0

	count = 0
	elapsed = 0
	max_section_num = Empty
	For i=0 To UBound( g_Bench_Elapseds )
		elapsed = elapsed + g_Bench_Elapseds(i)
		count = count + g_Bench_Counts(i)
		If i > 0  and  not IsEmpty( g_Bench_Elapseds(i) ) Then
			If IsEmpty( max_section_num ) Then
				max_section_num = i
			ElseIf g_Bench_Elapseds(i) > g_Bench_Elapseds( max_section_num ) Then
				max_section_num = i
			End If
		End If
	Next

	echo_v  ""
	echo_v  "Benchmark Result:"
	echo_v  "Sampling Count = " & count
	echo_v  "Elapsed Time = " & elapsed & "(sec)"
	echo_v  "Max is Section " & max_section_num

	For i=1 To UBound( g_Bench_Elapseds )
		If not IsEmpty( g_Bench_Elapseds(i) ) Then
			echo_v  "Section " & i & ": "+ Right( "  " & g_Bench_Counts(i), 3 ) & "times   " &_
				 Right( "     " & FormatNumber( g_Bench_Elapseds(i), 3,,, False ), 10 ) & "(sec)"
		End If
	Next
End Sub


 
'*************************************************************************
'  <<< [get_WMI] >>> 
' http://msdn.microsoft.com/en-us/library/aa384463(VS.85).aspx
'   Constructing the URI Prefix for WMI Classes
'*************************************************************************
Function  get_WMI( ByVal ComputerName, ClassName )
	Dim  objs, obj, en,ed
	Set  objs = get_SWbemServicesEx_cimv2( ComputerName ).ExecQuery( "Select * from "+ ClassName )

	If GetOSVersion() <= 5.1 Then
		ErrCheck : On Error Resume Next
			For Each obj  In objs : Set get_WMI = obj : Exit For : Next
		en = Err.Number : ed = Err.Description : On Error GoTo 0
		If en = 424 Then  en = 0  '// If obj.Count = 0 Then en = 424
		If en <> 0 Then  Err.Raise en,,ed
	Else  '// Win7
		If objs.Count >= 1 Then  Set get_WMI = objs.ItemIndex(0)
	End If

	If not IsEmpty( get_WMI ) Then
		If get_WMI.Path_.Class <> ClassName Then  Err.Raise 424
	Else
		If IsEmpty( ComputerName ) Then  ComputerName = "."
		Set get_WMI = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & ComputerName & "\root\default:"+ ClassName )
	End If
End Function


 
'*************************************************************************
'  <<< [get_SWbemServicesEx_cimv2] >>> 
'*************************************************************************
Dim  g_SWbemServicesExs_cimv2  '// as Dictionary
Function   get_SWbemServicesEx_cimv2( ComputerName )
	Dim  computer_name : If IsEmpty( ComputerName ) Then  computer_name = "."  Else  computer_name = ComputerName
	If IsEmpty( g_SWbemServicesExs_cimv2 ) Then _
		Set g_SWbemServicesExs_cimv2 = CreateObject( "Scripting.Dictionary" )
	If g_SWbemServicesExs_cimv2.Exists( computer_name ) Then
		Set get_SWbemServicesEx_cimv2 = g_SWbemServicesExs_cimv2.Item( computer_name )
	Else
		Set get_SWbemServicesEx_cimv2 = GetObject("winmgmts:\\" & computer_name & "\root\cimv2")
		Set g_SWbemServicesExs_cimv2.Item( computer_name ) = get_SWbemServicesEx_cimv2
	End If
End Function


 
'*************************************************************************
'  <<< [SWbemObjectEx_xml_sub] >>> 
'*************************************************************************
Function  SWbemObjectEx_xml_sub( m, Level )
	Dim  member

	SWbemObjectEx_xml_sub = GetTab(Level)+ "<"+ m.Path_.Class +" Server='"+ m.Path_.Server +_
		"' TypeName='SWbemObjectEx' Service='WMI'"+ vbCRLF
	For Each member  In m.Properties_
		SWbemObjectEx_xml_sub = SWbemObjectEx_xml_sub + GetTab(Level+1)+ member.Name +"='"& member.Value &"'"+vbCRLF
	Next
	SWbemObjectEx_xml_sub = SWbemObjectEx_xml_sub + GetTab(Level)+ "/>"+ vbCRLF
End Function


 
'*************************************************************************
'  <<< [IRegExp2_xml_sub] >>> 
'*************************************************************************
Function  IRegExp2_xml_sub( m, Level )
	IRegExp2_xml_sub = m.Pattern
End Function


 
'*-------------------------------------------------------------------------*
'* ### <<<< System (safe part) >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [Shutdown] >>> 
'*************************************************************************
Sub  Shutdown( ByVal Operation, ByVal CountDownTimeSec )
	Const PowerOff       = "PowerOff"
	Const PowerOffHybrid = "PowerOffHybrid"
	Const Reboot         = "Reboot"
	Const Hibernate      = "Hibernate"
	Const Sleep          = "Sleep"
	Const Standby        = "Standby"

	Dim  box_vbs_template : box_vbs_template = g_vbslib_ver_folder +"SetSuspendState\ShutdownMsgBox.vbs"
	Dim  suspend_exe : suspend_exe = g_vbslib_ver_folder +"SetSuspendState\SetSuspendState.exe"
	Dim  box_vbs : box_vbs = GetTempPath( "ShutdownMsgBox.vbs" )
	Dim  box_out : box_out = g_fs.GetParentFolderName( box_vbs ) +"\"+ g_fs.GetBaseName( box_vbs ) +"_out.txt"

	If IsEmpty( Operation ) Then
		If IsDefined( "Setting_getShutdownOperation" ) Then
			Operation = Setting_getShutdownOperation()
		Else
			Operation = PowerOff
		End If
	End IF
	If IsEmpty( CountDownTimeSec ) Then
		If IsDefined( "Setting_getShutdownCountDownTimeSec" ) Then
			CountDownTimeSec = Setting_getShutdownCountDownTimeSec()
		Else
			CountDownTimeSec = 60
		End If
	End IF


	echo  ">shutdown "+ Operation +", "& CountDownTimeSec

	If Operation = "PowerOff"  and  GetOSVersion() >= 6.2 Then
		If RegRead( "HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Session Manager\Power\HiberbootEnabled" ) _
				= 1 Then
			Operation = PowerOffHybrid
		End If
	End If

	Dim  op, op2, str1, str2, i, r
	Select Case  Operation
		Case  PowerOff  : op = " /s" : str1 = "Shutdown"
			str2 = "Shutdown..."
		Case  PowerOffHybrid  : op = " /s /hybrid" : str1 = "Shutdown"
			str2 = "Shutdown..."
		Case  Reboot    : op = " /r" : str1 = "Reboot"
			str2 = "Rebooting"
		Case  Hibernate : op = " /h" : str1 = "Hibernate"
			str2 = "Hibernating..."
		Case  Sleep     : op2 = "/sleep"   : str1 = "Sleep"
			str2 = "Sleepling..."
		Case  Standby   : op2 = "/standby" : str1 = "Standby"
			str2 = "Standbying..."
		Case Else  Error
	End Select


	'//=== Count down
	If CountDownTimeSec > 0 Then
		Dim  ec : Set ec = new EchoOff
		del   box_out
		copy_ren  box_vbs_template, box_vbs
		ec = Empty

		i =  "After "& CountDownTimeSec &"second, "+ str1 +"."
		start  "wscript """+ box_vbs +""" """+ i + """ """+ WScript.ScriptName +""" "& CountDownTimeSec

		For i=CountDownTimeSec To 1 Step -1
			echo_v  i  :  WScript.Sleep  1000
			r = ReadFile( box_out )
			If r <> "" Then
				If r = "1"+vbCRLF  or  r = "-1"+vbCRLF Then  Exit For
				Set ec = new EchoOff : del  box_out :  del  box_vbs
				Raise 1, "<ERROR msg='User cancel'/>"
			End If
		Next
		echo_v  0
		Set ec = new EchoOff
		del  box_out :  del  box_vbs
	Else
		CountDownTimeSec = 0
		Set ec = new EchoOff
	End If


	'//=== Shutdown
	If Operation = Hibernate Then
		If not exist( suspend_exe ) Then
			If GetOSVersion() >= 6 Then
				g_sh.Run  "shutdown /h"
			Else
				Raise 1,"<ERROR msg='Not found file' path='"+suspend_exe+"'/>"
			End If
		Else
			g_sh.Run  """"+ suspend_exe + """ 1"
		End If
	ElseIf not IsEmpty( op ) Then
		g_sh.Run  "shutdown"+ op +" /f /t 5 /c """+ str2 +""""  '// 5 is wait for finish this for piped program
	Else
		If not exist( suspend_exe ) Then  Raise 1,"<ERROR msg='Not found file' path='"+suspend_exe+"'/>"
		g_sh.Run  """"+ suspend_exe + """ 0"
	End If


	'//=== Shutdown message
	echo_v  str2 +"..."
End Sub


 
'*************************************************************************
'  <<< [RegRead] >>> 
'*************************************************************************
Function  RegRead( Path )
	Dim  e
	If TryStart(e) Then  On Error Resume Next
		RegRead = g_sh.RegRead( Path )
	If TryEnd Then  On Error GoTo 0
	If e.Num = E_PathNotFound or e.Num = E_WIN32_FILE_NOT_FOUND Then
		e.Clear
	End If
	If e.Num <> 0 Then  e.Raise
End Function


 
'*************************************************************************
'  <<< [RegReadEx] >>> 
'*************************************************************************
Function RegReadEx( Path, RegType )
	GetRegRootKeyAndStepPath  Path, root_key, step_path  '//[out] root_key, step_path

	Set nv = CreateObject( "WbemScripting.SWbemNamedValueSet" )
	Set locator = CreateObject( "Wbemscripting.SWbemLocator" )

	nv.Add  "__ProviderArchitecture", RegType
	Set reg = locator.ConnectServer( "", "root\default", "", "", , , , nv ).Get( "StdRegProv" )

	Set in_params = reg.Methods_( "GetStringValue" ).InParameters
	in_params.hDefKey = root_key
	in_params.sSubKeyName = g_fs.GetParentFolderName( step_path )
	in_params.sValueName = g_fs.GetFileName( step_path )

	Set out_params = reg.ExecMethod_( "GetStringValue", in_params, , nv )

	RegReadEx = out_params.sValue
	If IsNull( RegReadEx ) Then  RegReadEx = Empty
End Function


 
'*************************************************************************
'  <<< [RegEnumKeys] >>> 
'*************************************************************************
Sub  RegEnumKeys( Path, out_Keys, Opt )
	Dim    keys, key, i, root_key, step_path, i_step_path

	If IsEmpty( g_StdRegProv ) Then _
		Set g_StdRegProv = GetObject("winmgmts:{impersonationLevel=impersonate}!root/default:StdRegProv")

	GetRegRootKeyAndStepPath  Path, root_key, step_path  '//[out] root_key, step_path

	Set keys = new ArrayClass
	RegEnumKeys_sub  Path, root_key, step_path, keys  '//[out] keys

	If not IsEmpty( Opt ) Then

		i_step_path = InStr( Path, "\" ) + 1
		i = 0
		Do
			If i > keys.UBound_ Then  Exit Do
			RegEnumKeys_sub  keys(i), root_key, Mid( keys(i), i_step_path ), keys  '//[out] keys
			i=i+1
		Loop
	End If

	out_Keys = keys.Items
End Sub


Sub  RegEnumKeys_sub( AbsPath, RootKey, StepPath, in_out_Keys )
	Dim  key_names, key_name

	g_StdRegProv.EnumKey  RootKey, StepPath, key_names  '//[out] key_names
	If IsNull( key_names ) Then  ReDim  key_names(-1)

	For Each key_name  In key_names
		in_out_Keys.Add  AbsPath +"\"+ key_name
	Next
End Sub


Sub  GetRegRootKeyAndStepPath( AbsPath, out_RootKey, out_StepPath )
	Dim  i

	i = InStr( AbsPath, "\" )
	If i = 0 Then  RaiseRegPathNotFound  AbsPath
	Select Case  Left( AbsPath, i - 1 )
		Case "HKEY_CLASSES_ROOT"   : out_RootKey = &h80000000
		Case "HKEY_CURRENT_USER"   : out_RootKey = &h80000001
		Case "HKEY_LOCAL_MACHINE"  : out_RootKey = &h80000002
		Case "HKEY_USERS"          : out_RootKey = &h80000003
		Case "HKEY_PERFORMANCE_DATA":out_RootKey = &h80000004
		Case "HKEY_CURRENT_CONFIG" : out_RootKey = &h80000005
		Case "HKEY_DYN_DATA"       : out_RootKey = &h80000006
		Case Else : RaiseRegPathNotFound  AbsPath
	End Select

	out_StepPath = Mid( AbsPath, i + 1 )
End Sub


Sub  RaiseRegPathNotFound( Path )
	Raise  E_PathNotFound, "<ERROR msg=""There is not specified path in registry"" path="""+ Path +"""/>"
End Sub

 
'*************************************************************************
'  <<< [RegEnumValues] >>> 
'*************************************************************************
Class  RegValueName
	Public  Name
	Public  Type_
End Class


Sub  RegEnumValues( Path, out_Values )
	Dim  reg, i, root_key, step_path, names, types

	If IsEmpty( g_StdRegProv ) Then _
		Set g_StdRegProv = GetObject("winmgmts:{impersonationLevel=impersonate}!root/default:StdRegProv")

	GetRegRootKeyAndStepPath  Path, root_key, step_path  '//[out] root_key, step_path

	g_StdRegProv.EnumValues  root_key, step_path, names, types
	If IsNull( names ) Then  ReDim  out_Values(-1) : Exit Sub

	ReDim  out_Values( UBound( names ) )
	For i=0 To UBound( names )

		Set  out_Values(i) = new RegValueName : ErrCheck

		out_Values(i).Name = names(i)

		Select Case  types(i)
			Case 1 : out_Values(i).Type_ = "REG_SZ"
			Case 2 : out_Values(i).Type_ = "REG_EXPAND_SZ"
			Case 3 : out_Values(i).Type_ = "REG_BINARY"
			Case 4 : out_Values(i).Type_ = "REG_DWORD"
			Case 7 : out_Values(i).Type_ = "REG_MULTI_SZ"
		End Select
	Next
End Sub


 
'*************************************************************************
'  <<< [RegExists] >>> 
'*************************************************************************
Function  RegExists( Path )
	Dim en,ed
	Const  E_PathNotFound = &h80070002

	ErrCheck : On Error Resume Next
		g_sh.RegRead  Path
	en = Err.Number : ed = Err.Description : On Error GoTo 0
	If en = E_PathNotFound Then  RegExists = False : Exit Function
	If en <> 0 Then  Err.Raise en,,ed
	RegExists = True
End Function


 
'*************************************************************************
'  <<< [LoadReg] >>> 
'*************************************************************************
Function  LoadReg( PathOrStr, Opt )
	Set LoadReg = new RegFileClass

	If VarType( Opt ) = vbString Then
		LoadReg.m_InterestKeys = Array( Opt )
		LoadReg.ReadFile  PathOrStr
	ElseIf IsArray( Opt ) Then
		LoadReg.m_InterestKeys = Opt
		LoadReg.ReadFile  PathOrStr
	ElseIf Opt and g_VBS_Lib.StringData Then
		Dim  f : Set f = new StringStream
		f.SetString  Opt
		LoadReg.ParseReg  f
	Else
		LoadReg.ReadFile  PathOrStr
	End If
End Function


 
'-------------------------------------------------------------------------
' ### <<<< [RegFileClass] Class >>>> 
'-------------------------------------------------------------------------
Class  RegFileClass

	Dim  m_Variables ' as Dictionary. key=registory key and variable name, value=value
	Function  RegRead( Path ) : RegRead = m_Variables.Item( Path ) : End Function

	Dim  m_InterestKeys ' as Array of string

	Private Sub  Class_Initialize
		Set m_Variables = CreateObject( "Scripting.Dictionary" )
	End Sub


 
'*************************************************************************
'  <<< [RegFileClass::ReadFile] >>> 
'*************************************************************************
Sub  ReadFile( Path )
	Me.ParseReg  OpenForRead( Path )
End Sub

Sub  ParseReg( Stream )
	Dim  line, i, current_key, name, type_name, value, b_enabled

	b_enabled = IsEmpty( m_InterestKeys )

	Do Until  Stream.AtEndOfStream
		line = Stream.ReadLine()

		If Left( line, 1 ) = "[" Then
			current_key = Trim2( Mid( line, 2 ) ) : CutLastOf  current_key, "]", Empty

			'// Set b_enabled
			If not IsEmpty( m_InterestKeys ) Then
				b_enabled = False
				For Each i  In m_InterestKeys
					If i = current_key Then  b_enabled = True : Exit For
				Next
			End If

		ElseIf b_enabled Then
			i = InStr( line, "=" )
			If i > 0 Then  line = Left( line, i-1 ) +" "+ Mid( line, i+1 )

			i = 1
			name = MeltCmdLine( line, i )
			If i > 0 Then
				type_name = "dword:"
				i = InStr( i, line, type_name )
				If i > 0 Then
					i = i + Len( type_name )
					m_Variables.Item( current_key +"\"+ name ) = CLng( "&h" + Mid( line, i ) )
				End If
			End If
		End If
	Loop
End Sub


 
End Class 


 
'*************************************************************************
'  <<< [LoadEnvVars] >>> 
'*************************************************************************
Sub  LoadEnvVars( Path_ofSetCommandLog, Option_ )
	Dim  f, line, key, b, i, envs
	Dim  c : Set c = g_VBS_Lib

	If TypeName( Option_ ) = "Dictionary" Then
		Set envs = Option_
		envs.RemoveAll
	Else
		Set envs = g_sh.Environment( "Process" )
		If (Option_ and c.Append) = 0 Then  ClearEnvVars
	End If

	If VarType( Path_ofSetCommandLog ) = vbString Then
		echo  ">LoadEnvVars  """+ Path_ofSetCommandLog +""""
		Dim  ec : Set ec = new EchoOff

		Set f = OpenForRead( Path_ofSetCommandLog )
		Do Until  f.AtEndOfStream
			line = f.ReadLine()
			i = InStr( line, "=" )
			envs.Item( Trim2( Left( line, i-1 ) ) ) = Mid( line, i+1 )
		Loop

	Else

		For Each key  In Path_ofSetCommandLog.Keys
			envs.Item( key ) = Path_ofSetCommandLog.Item( key )
		Next

	End If
End Sub


 
'*************************************************************************
'  <<< [ClearEnvVars] >>> 
'*************************************************************************
Sub  ClearEnvVars()
	echo  ">ClearEnvVars"

	Dim    envs : Set envs = g_sh.Environment( "Process" )
	ReDim  symbols( envs.Count - 1 )

	If not IsEmpty( g_CurrentVarStackSub ) Then
		For Each  env_line  In envs
			index_of_equal = InStr( env_line, "=" )
			If index_of_equal >= 2 Then
				env_name = Left( env_line, index_of_equal - 1 )
				env_value = Mid( env_line, index_of_equal + 1 )
				g_CurrentVarStackSub.SetPrevValue1_IfNotExist  env_name, env_value
			End If
		Next
	End If


	'//=== Listup variables
	Dim  line, i, n
	n = -1
	For Each line  In envs
		i = InStr( line, "=" )
		If i >= 2 Then
			n=n+1
			symbols( n ) = Left( line, i-1 )
		End If
	Next


	'//=== for all ADODB.Stream objects, when environment variables was removed
	Set g_ADODB_Stream_Cache = CreateObject( "ADODB.Stream" )


	'//=== Remove variables
	ReDim Preserve  symbols( n )
	For Each line  In symbols
		envs.Remove  line
	Next
End Sub

Dim g_ADODB_Stream_Cache


 
'*************************************************************************
'  <<< [SetVarInBatchFile] >>> 
'*************************************************************************
Sub  SetVarInBatchFile( Path, Symbol, Value )
	Set ec = new EchoOff

	AssertExist( Path )
	text = ReadFile( Path )


	Set re = CreateObject("VBScript.RegExp")
	re.IgnoreCase = True
	re.MultiLine = True
	re.Pattern = "^ *set *"+ ToRegExpPattern( Symbol ) +"="

	Set matches = re.Execute( text )
	If matches.Count = 0 Then
		Raise  E_NotFoundSymbol, "<ERROR msg=""There is not the environment"" var="""+_
			Symbol +""" path="""+ GetFullPath( Path, Empty ) +"""/>"
	End If

	Set match = matches(0)
	equal_position = match.FirstIndex + match.Length

	end_of_line = InStr( equal_position, text, vbCRLF )

	text = Left( text, equal_position ) & Value & Mid( text, end_of_line )


	CreateFile  Path, text
End Sub


 
'*************************************************************************
'  <<< [GetPythonInstallPath] >>> 
'*************************************************************************
Dim  g_PythonExePath
Dim  g_TargetPythonVersionMin
Dim  g_TargetPythonVersionMax

Function  GetPythonInstallPath()
	If IsEmpty( g_PythonExePath ) Then
		Dim  keys,  keys_tmp,  key,  ver,  best_ver,  python_ver_key

		Set keys = new ArrayClass
		RegEnumKeys  "HKEY_LOCAL_MACHINE\SOFTWARE\Python\PythonCore", keys_tmp, Empty
		keys.AddElems  keys_tmp
		RegEnumKeys  "HKEY_CURRENT_USER\SOFTWARE\Python\PythonCore", keys_tmp, Empty
		keys.AddElems  keys_tmp

		For Each key  In keys.Items
			If RegExists( key +"\InstallPath\" ) Then
				ver = CSng( g_fs.GetFileName( key ) )

				If not IsEmpty( g_TargetPythonVersionMin ) Then
					If ver < g_TargetPythonVersionMin Then  ver = Empty
				End If
				If not IsEmpty( g_TargetPythonVersionMax ) Then
					If ver > g_TargetPythonVersionMax Then  ver = Empty
				End If
				If not IsEmpty( ver ) Then
					If ver > best_ver Then  best_ver = ver : python_ver_key = key
				End If
			End If
			'// memo: key +"\Modules\" is remained unless Python was uninstalled
		Next

		If IsEmpty( best_ver ) Then _
			Raise  1, "Python is not registed in registry"

		g_PythonExePath = RegRead( python_ver_key +"\InstallPath\" )
		CutLastOf  g_PythonExePath, "\", Empty
	End If
	GetPythonInstallPath = g_PythonExePath
End Function


 
'*************************************************************************
'  <<< [IsInstallPython] >>> 
'*************************************************************************
Dim  g_IsInstallPython

Function  IsInstallPython()
	If IsEmpty( g_IsInstallPython ) Then
		Dim  path,  e

		If TryStart(e) Then  On Error Resume Next
			path = GetPythonInstallPath()
		If TryEnd Then  On Error GoTo 0
		g_IsInstallPython = ( e.num = 0 )
		e.Clear
	End If
	IsInstallPython = g_IsInstallPython
End Function


 
'*************************************************************************
'  <<< [SetTargetPythonVersion] >>> 
'*************************************************************************
Sub  SetTargetPythonVersion( MinVer, MaxVer )
	g_PythonExePath = Empty
	g_IsInstallPython = Empty
	g_TargetPythonVersionMin = MinVer
	g_TargetPythonVersionMax = MaxVer
End Sub


 
'*************************************************************************
'  <<< [GetPerlVersion] >>> 
'*************************************************************************
Function  GetPerlVersion( Options )
	temporary_path = GetTempPath( "GetPerlVersion.txt" )
	Set ec = new EchoOff

	If TryStart(e) Then  On Error Resume Next

		RunProg  "perl -v", temporary_path

	If TryEnd Then  On Error GoTo 0
	If e.num = &h80070002  Then
		perl_exe = Setting_getPerlPath()
		If not exist( perl_exe ) Then
			Raise  e.num, "Not found perl or not set path to perl."+_
				"Install ActivePerl or others."
		End If
		e.Clear

		set_ "PATH", AddIfNotExist( env("%PATH%"), _
			g_fs.GetParentFolderName( perl_exe ), ";", Empty )
		RunProg  "perl -v", temporary_path
	End If
	If e.num <> 0 Then  e.Raise

	Set numbers = ScanFromTemplate( ReadFile( temporary_path ), _
		"This is perl ${Major}, version ${Minor}, subversion ${Sub} (", _
		Array( "${Major}", "${Minor}", "${Sub}" ), Empty )
	GetPerlVersion = numbers("${Major}") +"."+ _
		numbers("${Minor}") +"."+ numbers("${Sub}")

	del  temporary_path
End Function


 
'*-------------------------------------------------------------------------*
'* ### <<<< Error, Err2 >>>> 
'*-------------------------------------------------------------------------*


 
'*************************************************************************
'  <<< [Finish] >>> 
'*************************************************************************
Sub  Finish()
	ThisIsOldSpec
	Stop:WScript.Quit  9
End Sub


 
'*************************************************************************
'  <<< [Error] >>> 
'*************************************************************************
Sub  Error()
	Raise  1,"General Error"
End Sub


 
'*************************************************************************
'  <<< [OrError] Stop:OrError >>> 
'*************************************************************************
Sub  OrError()
	If g_debug <= 0 Then  Raise  1,"Stop:OrError"
End Sub


 
'*************************************************************************
'  <<< [Warning] >>> 
'*************************************************************************

'//[g_WarningMSec]
g_WarningMSec = 300

Sub  Warning( Description )
	echo_v  "<WARNING msg="""& Description &"""/>"
	Sleep  g_WarningMSec
	Err.Raise  E_TestPass
End Sub


 
'*************************************************************************
'  <<< [Assert] >>> 
'*************************************************************************
Sub  Assert( Condition )
	If Condition Then '// This is not same behavior as "If not Condition Then Fail"
	Else
		Fail
	End If
End Sub


 
'*************************************************************************
'  <<< [AssertExist] >>> 
'*************************************************************************
Sub  AssertExist( Path )
	path_string = GetFilePathString( Path )
	Set c = g_VBS_Lib
	If not exist_ex( path_string, c.CaseSensitive ) Then
		If IsFullPath( path_string ) Then
			s = path_string
		Else
			s = path_string +""""+ vbCRLF +" full_path="""+ GetFullPath( path_string, Empty )
		End If

		If exist( path_string ) Then
			echo_v  "<WARNING msg=""Not same upper lower case"" call="""+ s + """"+ vbCRLF +_
				" name="""+ GetCaseSensitiveFullPath( path_string ) +"""/>"
		Else
			Raise  E_PathNotFound, _
				"<ERROR msg=""Not found file or folder"" path="""+ s +"""/>"
		End If
	End If
End Sub


 
'*************************************************************************
'  <<< [AssertValue] >>> 
'*************************************************************************
Sub  AssertValue( Name, Value, PassValue, CompareOption )
	If IsArray( PassValue ) Then
		AssertValue_sub  Name, Value, PassValue, CompareOption
	Else
		AssertValue_sub  Name, Value, Array( PassValue ), CompareOption
	End If
End Sub

Sub  AssertValue_sub( Name, ByVal Value, PassValues, CompareOption )
	Dim  value2, pass_value, i
	Dim  c : Set c = g_VBS_Lib
	Dim  b_case_sensitive : b_case_sensitive = ( ( CompareOption and c.CaseSensitive ) <> 0 )


	'//=== get value
	If IsObject( Value ) Then
		If Value Is c.EnvVar Then  Value = GetVar( Name )
	End If


	If CompareOption and c.Range Then

		'//=== compare value range
		value2 = CInt2( Value )
		For i=0 To UBound( PassValues ) Step 2
			If value2 >= CInt2( PassValues( i ) ) and  value2 <= CInt2( PassValues( i+1 ) ) Then  Exit Sub
		Next

	Else

		'//=== compare a value or values
		If not b_case_sensitive Then  value2 = LCase( Value )  Else  value2 = Value
		For Each pass_value  In PassValues
			If b_case_sensitive Then
				If value2 = pass_value Then  Exit Sub
			Else
				If value2 = LCase( pass_value ) Then  Exit Sub
			End If
		Next
	End If
	ErrorValue  Name, Value
End Sub


 
'*************************************************************************
'  <<< [ErrorValue] >>> 
'*************************************************************************
Sub  ErrorValue( Name, Value )
	Raise  1, "<ERROR msg=""Value is not expected"" name="""+ Name + """ now_value="""& Value &"""/>"
End Sub


 
'*************************************************************************
'  <<< [Err2] >>> 
'*************************************************************************
Class Err2  '// has_interface_of InterProcessData

	Public  Num           ' Err.Number
	Public  Property Get  Number() : Number = Num : End Property
	Public  Property Let  Number(x) : Num = x : End Property
	Public  Desc          ' Err.Description (Error Message)
	Public  Property Get  Description() : Description = Desc : End Property
	Public  Property Let  Description(x) : Desc = x : End Property
	Public  Source       ' Err.Source
	Public  Queue        ' as string
	Public  ChildProcess_CallID  ' as ArrayClass of integer. g_InterProcess.ProcessCallID in Error
	Public  ErrID        ' count of (num <> 0) in each first Copy after Clear
	Public  RaiseID      ' count of (num <> 0) in Copy
	Public  CopiedErrID  ' as integer
	Public  Prev         ' as Err2
	Public  Break_ChildProcess_CallID  ' as array of integer
	Public  BreakErrID   ' as integer
	Public  BreakRaiseID ' as integer
	Public  a_NestPos       ' as NestPos
	Public  BreakNestPos    ' as Array of integer
	Public  ErrNestPos      ' as Array of integer
	Public  PrevErrNestPos  ' as Array of integer

	Private Sub  Class_Initialize()
		Me.Num = 0 : Me.Desc = "" : Me.Queue = ""
		Me.ErrID = 0 : Me.RaiseID = 0 : Set Me.ChildProcess_CallID = new ArrayClass
		Me.BreakErrID = 0 : Me.BreakRaiseID = 0 : Set Me.Break_ChildProcess_CallID = new ArrayClass
		Set Me.a_NestPos = new NestPos
		Me.BreakNestPos = Array( )
		Me.ErrNestPos = Array( )
		Me.PrevErrNestPos = Array( )
	End Sub

	Public Sub  OnSuccessFinish()
		If Err.Number = 0 and Me.Num <> 0 Then
			echo  Me.ErrStr
			Me.Clear

			On Error Resume Next
			Err.Raise  1,, "<ERROR msg='Script finished before Err2.Clear or Raise the error'/>"
			Me.Copy  Err
			Me.ErrID = Me.ErrID - 0.5
			If get_ChildProcess() is Nothing  Then
				echo  g_ConnectDebugMsgV5 + vbCRLF +String( 70, "-" )+ vbCRLF + _
					""""+ WScript.ScriptName +""" /g_debug:"& Me.ErrID & vbCRLF + String( 70, "-" )
			Else
				If Me.ChildProcess_CallID.UBound_ = -1 Then
					Me.ChildProcess_CallID.Copy  g_InterProcess.ProcessCallID
					Me.ChildProcess_CallID.Pop
				End If
			End If
		ElseIf Err.Number <> 0 Then
			Me.PrevErrNestPos = Me.a_NestPos.PosArr
		End If
	End Sub

	Public Sub  OnErrorFinish()
		If Err.Number <> 0 and Me.Num = 0 Then  Me.Copy Err
		If Me.Num <> 0 Then
			If get_ChildProcess() is Nothing  Then
				echo  Me.GetDebugHintPart()
			Else
				If Me.ChildProcess_CallID.UBound_ = -1 Then
					Me.ChildProcess_CallID.Copy  g_InterProcess.ProcessCallID
					Me.ChildProcess_CallID.Pop
				End If
			End If
		End If
	End Sub

	Public Sub  Copy( err )
		Me.Num = err.Number
		Me.Desc = err.Description
		Me.Source = err.Source
		If Me.Num <> 0 Then
			Me.RaiseID = Me.RaiseID + 1 : If Me.RaiseID = 1 Then Me.ErrID = Me.ErrID + 1
			If UBound( Me.ErrNestPos ) = -1 Then  Me.ErrNestPos = Me.a_NestPos.PosArr
		End If
	End Sub

	Public Sub  CopyAndClear( out_Err2Copy )
		Set out_Err2Copy = new Err2
		out_Err2Copy.Copy  Me
		If Me.num <> 0 Then _
			Me.CopiedErrID = Me.ErrID
		Me.Clear
	End Sub

	Public Sub  EnqueueAndClear()
		If Me.Num = 0 Then  Exit Sub
		Dim  s : s = Me.ErrStr()
		s = Left( s, Len( s ) - 2 ) + " g_debug='"& Me.ErrID
		Dim  tree, process : GetDebugHintData  tree, process
		If not IsEmpty( tree ) Then  s = s +"' g_debug_tree='Array( "& tree &" )"
		If not IsEmpty( process ) Then  s = s +"' g_debug_process='"& process
		s = s + "'/>"
		Me.Queue = Me.Queue + s + vbCRLF
		Me.Clear
	End Sub

	Public Function  DequeueAll()
		DequeueAll = Me.Queue
		Me.Queue = ""
	End Function

	Public Function  GetErrStr() : ThisIsOldSpec : GetErrStr = Me.ErrStr : End Function
	Public Property Get  ErrStr() : ErrStr = GetRef("GetErrStr")( Me.Num, Me.Desc ) : End Property

	Public Property Get  DebugHint()
		DebugHint = Me.GetDebugHintPart()
		DebugHint = DebugHint + vbCRLF + GetRef("GetErrStr")( Me.Num, Me.Desc )
	End Property

	Public Function  GetDebugHintPart()
		Dim  tree, process : GetDebugHintData  tree, process

		GetDebugHintPart = g_ConnectDebugMsgV5 + vbCRLF +String( 70, "-" )+ vbCRLF + _
			""""+ WScript.ScriptName +""" /g_debug:"& Me.ErrID
		If not IsEmpty( tree ) Then _
			GetDebugHintPart = GetDebugHintPart +","& Replace( tree, " ", "" )
		If not IsEmpty( process ) Then _
			GetDebugHintPart = GetDebugHintPart +";"& process

		If Me.CopiedErrID = Me.ErrID - 1  and  not IsEmpty( Me.CopiedErrID ) Then _
			GetDebugHintPart = GetDebugHintPart + vbCRLF +_
			"'//Raise before CopyAndClear for founding the place rasing the error"
		GetDebugHintPart = GetDebugHintPart + vbCRLF + String( 70, "-" )
	End Function

	Public Sub  GetDebugHintData( out_Tree, out_Process )
		If UBound( Me.ErrNestPos ) >= 0  and  UBound( Me.PrevErrNestPos ) >= 0  Then
			Dim  i, tree
			For i=0 To UBound( Me.ErrNestPos ) - 1
				If Me.ErrNestPos(i) <> Me.PrevErrNestPos(i) Then  Exit For
			Next
			If i > 0 Then
				tree = Me.ErrNestPos : ReDim Preserve  tree(i-1)
				out_Tree = ""
				For i=0 To UBound( tree )
					out_Tree = out_Tree & tree(i) &", "
				Next
				CutLastOf  out_Tree, ", ", Empty
			End If
		End If
		If Me.ChildProcess_CallID.UBound_ = 0 Then
			out_Process = Me.ChildProcess_CallID(0)
		ElseIf Me.ChildProcess_CallID.UBound_ >= 1 Then
			out_Process = Me.ChildProcess_CallID.CSV
		End If
	End Sub

	Public Sub OverRaise( e_num, e_desc )
		Me.Num = e_num : Me.Desc = e_desc : Me.Raise
	End Sub

	Public Sub  Raise()
		If Me.Num = 0 Then
			Err.Raise 1  '// Check caller function using debugger.
		Else
			Err.Raise Me.Num, Me.Source, Me.Desc  '// Re-throw previous error.
			 '// Check caller function using debugger.
			 '// Write "g_debug" = 1 (=Me.Num) or (=Me.Num+0.5) in SetupVbslibParameters function.
		End If
	End Sub

	Public Sub  Clear()
		If Me.Num <> 0 Then
			Me.PrevErrNestPos = Me.ErrNestPos
			Me.Num = 0 : Me.Desc = "" : Me.RaiseID = 0 : Me.ChildProcess_CallID.ToEmpty
			Me.ErrNestPos = Array( )
			Me.Prev = Empty
			'//echo_v "<TryStartEnd func='Err2:Clear' prev_err_nest_pos='"& UBound( Me.PrevErrNestPos ) &"'/>"
		End If
	End Sub


	Property Get  xml()
		xml = "<"+ TypeName( Me ) +" number="""& Me.Num &""" description="""+_
			XmlAttr( Me.Desc ) +""" source="""+ XmlAttr( Me.Source ) +""" err_id="""& Me.ErrID &_
			""" raise_id="""& Me.RaiseID &""" sub_process_call_id="""+ Me.ChildProcess_CallID.CSV +_
			""" break_err_id="""& Me.BreakErrID &""" break_raise_id="""& Me.BreakRaiseID &_
			""" break_sub_process_call_id="""+ Me.Break_ChildProcess_CallID.CSV +_
			""" a_nest_pos="""+ CSVFrom( Me.a_NestPos.PosArr ) +""" break_nest_pos="""+ CSVFrom( Me.BreakNestPos ) +_
			""" err_nest_pos="""+ CSVFrom( Me.ErrNestPos ) +""" prev_err_nest_pos="""+ CSVFrom( Me.PrevErrNestPos ) +"""/>"
	End Property

	Public Sub  loadXML( XmlObject )
		Me.Num = CInt2( XmlObject.getAttribute( "number" ) )
		Me.Desc = XmlObject.getAttribute( "description" )
		Me.Source = XmlObject.getAttribute( "source" )
		Me.ErrID = CSng( XmlObject.getAttribute( "err_id" ) )
		Me.RaiseID = CInt2( XmlObject.getAttribute( "raise_id" ) )
		Me.ChildProcess_CallID.ToEmpty
		Me.ChildProcess_CallID.AddCSV  XmlObject.getAttribute( "sub_process_call_id" ), vbInteger
		Me.BreakErrID = CSng( XmlObject.getAttribute( "break_err_id" ) )
		Me.BreakRaiseID = CInt2( XmlObject.getAttribute( "break_raise_id" ) )
		Me.Break_ChildProcess_CallID.ToEmpty
		Me.Break_ChildProcess_CallID.AddCSV  XmlObject.getAttribute( "break_sub_process_call_id" ), vbInteger
		Me.a_NestPos.PosArr = ArrayFromCSV_Int( XmlObject.getAttribute( "a_nest_pos" ) )
		Me.BreakNestPos = ArrayFromCSV_Int( XmlObject.getAttribute( "break_nest_pos" ) )
		Me.ErrNestPos = ArrayFromCSV_Int( XmlObject.getAttribute( "err_nest_pos" ) )
		Me.PrevErrNestPos = ArrayFromCSV_Int( XmlObject.getAttribute( "prev_err_nest_pos" ) )
	End Sub


'// has_interface_of InterProcessData
	Public Sub  OnCallInParent( a_ParentProcess )   : a_ParentProcess.m_OutFile.WriteLine Me.xml : End Sub
	Public Sub  OnReturnInChild( a_ChildProcess )   :  a_ChildProcess.m_OutFile.WriteLine Me.xml : End Sub
	Public Sub  OnReturnInParent( a_ParentProcess ) : Me.loadXML  a_ParentProcess.m_InXML.selectSingleNode( TypeName(Me) ) : End Sub
End Class


 
'*************************************************************************
'  <<< [Raise] >>> 
'*************************************************************************
Sub  Raise( ErrNum, Description )
	Dim  e : Set e = g_Err2

	If g_is_debug Then  Set e.Prev = new Err2 : e.Prev.Copy  e

	e.Num = ErrNum
	e.Source = "ERROR"
	e.Desc = Description
	e.RaiseID = e.RaiseID + 1 : If e.RaiseID = 1 Then e.ErrID = e.ErrID + 1

	Err.Raise  e.Num, e.Source, e.Desc
			'// When g_debug > 0 (not called On Error Resume Next),
			'// Error message was output at standard error output (e.g. Command Prompt)
			'// When Pass function of vbslib was called,
			'// C:\Folder\Sample.vbs(0, 1) ERROR: Pass. : Like this was output.
End Sub

Dim  g_IsDefinedRaise : g_IsDefinedRaise = True


 
'*************************************************************************
'  <<< [SetErrBreak] >>> 
'*************************************************************************
Sub  SetErrBreak( ErrID, RaiseID )
	g_Err2.BreakErrID = ErrID
	g_Err2.BreakRaiseID = RaiseID
End Sub


 
'*************************************************************************
'  <<< [NestPos] >>> 
'*************************************************************************
Class  NestPos
	Public  PosArr

	Private Sub Class_Initialize()
		Redim  PosArr(0)
		PosArr(0) = 0
	End Sub

	Public Sub  Start()
		Dim  u : u = UBound( PosArr )
		PosArr(u) = PosArr(u) + 1
		Redim Preserve  PosArr(u+1)
		PosArr(u+1) = 0
	End Sub

	Public Sub  End_()
		Redim Preserve  PosArr( UBound( PosArr ) - 1 )
	End Sub
End Class


 
'*************************************************************************
'  <<< [NotCallFinish] >>> 
'*************************************************************************
Sub  NotCallFinish()
	Err.Raise  1,,"<ERROR msg=""Not call Finish""/>"
End Sub


 
'*************************************************************************
'  <<< [ErrorCheckInTerminate] >>> 
'*************************************************************************
Sub  ErrorCheckInTerminate( ErrNumberAtStartOfClassTerminate )
	If ( Err.Number <> 0  and _
	     Err.Number <> 21 )  and _
	   ( ErrNumberAtStartOfClassTerminate = 0  or _
	     ErrNumberAtStartOfClassTerminate = 21 ) Then
		echo_v  GetErrStr( Err.Number, Err.Description + " in Class_Terminate" )
		Stop
		If g_is_cscript_exe  and  not g_IsAutoTest Then  Pause
		ErrNumberAtStartOfClassTerminate = Err.Number
	End If
End Sub


 
'*************************************************************************
'  <<< [TryStart] >>> 
'*************************************************************************
Function TryStart( e )
	Dim  i, b
	Set  e = g_Err2

	If Err.Number <> 0 or e.Num <> 0 Then
		Stop  '// Check before error here.
		Raise  1,"<ERROR msg=""TryStart can not be called, when error was raised. "+_
			"Look at the document of TryStart"">" +vbCRLF+_
			e.Desc +vbCRLF +"</ERROR>"
	End If

	e.a_NestPos.Start

	If IsEmpty( e.BreakErrID ) Then
		TryStart = True
	Else

		'//=== g_debug_tree (=e.BreakNestPos)
		b=( not IsEmpty( e.BreakNestPos ) )  'and
		If b Then b=( UBound( e.a_NestPos.PosArr ) <= UBound( e.BreakNestPos ) + 1 )
		If b Then
			For i=0 To UBound( e.a_NestPos.PosArr ) - 1
				If e.BreakNestPos(i) <> e.a_NestPos.PosArr(i) Then  Exit For
			Next
			If i = UBound( e.a_NestPos.PosArr ) Then
				TryStart = False
			Else
				TryStart = True
			End If

		'//=== g_debug (=e.BreakErrID)
		Else
			If e.ErrID = e.BreakErrID - 1 Then
				TryStart = False
			Else
				TryStart = True
			End If
		End If
	End If

'//echo_v  e.xml
'//echo_v  "<TryStartEnd func='TryStart' nest='"& UBound( e.a_NestPos.PosArr ) &"' return='"& TryStart &"'/>"
End Function


 
'*************************************************************************
'  <<< [Trying] >>> 
'*************************************************************************
Function Trying()
	Trying = (Err.Number=0)
	If not Trying Then
		If g_Err2.ErrID = g_Err2.BreakErrID - 1.5 Then _
			Stop  '// Check caller function using debugger.
	End If
End Function


 
'*************************************************************************
'  <<< [TryEnd] >>> 
'*************************************************************************
Function TryEnd()
' Do not have parameters.
' Because "If TryEnd(e) Then On Error Goto 0" cannot get error, if e is not Dim.

	Dim  e : Set e = g_Err2

	If Err.Number <> 0 Then
		e.Copy Err

		If e.ErrID = e.BreakErrID Then
			TryEnd = False
		Else
			TryEnd = True
		End If

		If e.ErrID = e.BreakErrID - 0.5 Then  Stop
			'// [g_debug+0.5] Check caller function using debugger.
	Else
		If g_Err2.Num <> 0 Then _
			echo  "<WARNING msg=""Error catched by TryEnd inside here TryEnd must call g_Err2.Clear inside or must raise again""/>"
		TryEnd = True
	End If

	e.a_NestPos.End_

'//echo_v  e.xml
'//echo_v  "<TryStartEnd func='TryEnd' nest='"& (UBound( e.a_NestPos.PosArr )+1) &"' return='"& TryEnd &"'/>"
'//If IsSameArray( e.a_NestPos.PosArr, Array( 1,1,2,1,1 ) ) Then  Stop
End Function


 
'*************************************************************************
'  <<< [ErrCheck] >>> 
'*************************************************************************
Sub  ErrCheck()
	If Err.Number <> 0 Then  If g_Err2.Num = 0  Then  g_Err2.Copy Err : g_Err2.Raise
End Sub


 
'*************************************************************************
'  <<< [chk_exist_in_lib] >>> 
' comment
'  - If there is not path in vbslib folder, raise error of E_FileNotExist.
'*************************************************************************
Sub  chk_exist_in_lib( ByVal path )
	If not exist( g_vbslib_ver_folder + path ) Then  Err.Raise  E_FileNotExist,, _
		"Not found """ + g_vbslib_ver_folder + path + """"
End  Sub


 
'*************************************************************************
'  <<< [SetStartSectionTree] >>> 
'*************************************************************************
Sub  SetStartSectionTree( ByVal Sections )
	If VarType( Sections ) = vbString Then
		Sections = Replace( Sections, ",", " >" )
		echo  ">SetStartSectionTree  "+ Sections
		Sections = Replace( Sections, ">", "," )
	Else
		echo  ">SetStartSectionTree  "+ Replace( GetEchoStr( Sections ), ",", " >" )
	End If
	If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
	If not IsArray( Sections ) Then  If Sections = "" Then  Exit Sub

	Dim g : Set g = g_SkipSectionGlobal
	Dim  arr : arr = g.CurrentSectionNames.Items
	If IsArray( Sections ) Then
		AddArrElem  arr, Sections
	Else
		AddArrElem  arr, ArrayFromCSV( Sections )
	End If

	g.SelectedSectionTree = arr
End Sub


 
'-------------------------------------------------------------------------
' ### <<<< [SectionTree] Class >>>> 
'-------------------------------------------------------------------------
Class  SectionTree
	Public  SkipSectionA    '// as SkipSection
	Public  LevelOnCreated  '// as integer
	Public  IsStartRetTrue  '// as boolean

	Private Sub  Class_Initialize()
		Set Me.SkipSectionA = new SkipSection
		If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
		Dim g : Set g = g_SkipSectionGlobal
		g.IsEcho = False
		Me.LevelOnCreated = g.CurrentLevel
		Me.IsStartRetTrue = False
	End Sub

	Private Sub  Class_Terminate()
		Dim g : Set g = g_SkipSectionGlobal

		If Err.Number <> 0 Then
			Dim  s, n
			For Each  n  In g.CurrentSectionNames.Items : s = s + n + " > " : Next
			If not IsEmpty( s ) Then
				echo  ""
				echo  "Section> " + Left( s, Len(s)-3 )
			End If
		End If

		While  g.CurrentLevel > Me.LevelOnCreated
			Me.End_
		WEnd
	End Sub

	Function  Start( Label )  '// [SectionTree::Start]
		Dim g : Set g = g_SkipSectionGlobal
		g.CurrentSectionNames.Add  Label
		Start = Me.SkipSectionA.Start()
		Me.IsStartRetTrue = Start
		If Start Then
			Dim  s, n
			For Each  n  In g.CurrentSectionNames.Items : s = s + n + " > " : Next
			echo  ""
			echo  "Section> " + Left( s, Len(s)-3 )
			echo  "<Section tree="""+ g.CurrentSectionNames.CSV +""">"
			g.OnStart  g.CallbackObject
		End If
		If not Start Then  g.CurrentLevel = g.CurrentLevel + 1
	End Function

	Sub  End_()  '// [SectionTree::End_]
		Dim g : Set g = g_SkipSectionGlobal
		g.CurrentSectionNames.Pop
		Me.SkipSectionA.End_
		If Me.IsStartRetTrue Then  echo  "</Section>"
	End Sub

	Public Property Get  xml() : xml=xml_sub(0) : CutLastOf xml,vbCRLF,Empty : End Property
	Public Function  xml_sub( Level )
		Dim g : Set g = g_SkipSectionGlobal
		xml_sub = GetTab(Level)+ "<"+TypeName(Me)+_
			" CurrentSectionNames="""+ XmlAttrA( g.CurrentSectionNames.CSV ) +"""/>"+ vbCRLF
	End Function
End Class



 
'-------------------------------------------------------------------------
' ### <<<< [SkipSection] Class >>>> 
'-------------------------------------------------------------------------
Dim  g_SkipSectionGlobal

'//[SkipSectionGlobal]
Class  SkipSectionGlobal
	Public  CurrentLevel        '// as integer. -1=root
	Public  CurrentSecNums      '// as ArrayClass of integer
	Public  CurrentSectionNames '// as ArrayClass of string
	Public  SkipToSecNums       '// as ArrayClass of integer
	Public  SelectedSectionTree '// as array of string
	Public  BreakSecNums        '// as ArrayClass of integer
	Public  IsEcho              '// as boolean
	Public  IsDontPause         '// as boolean
	Public  OnStart             '// as function( Me.CallbackObject )
	Public  CallbackObject      '// as variant

	Private Sub  Class_Initialize()
		Me.CurrentLevel = -1
		Set Me.CurrentSecNums = new ArrayClass : Me.CurrentSecNums.Add  0
		Set Me.CurrentSectionNames = new ArrayClass
		Set Me.SkipToSecNums  = new ArrayClass
		Me.SelectedSectionTree = Array( )
		Set Me.BreakSecNums   = new ArrayClass
		Me.IsEcho = True
		Me.IsDontPause = False
		Set Me.OnStart = GetRef( "DefaultFunction" )
	End Sub
End Class


Function  NotSkipSection()
	ThisIsOldSpec
	Dim  section : Set section = new SkipSection : ErrCheck : section.IsNoCheck = True
	If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
	Dim  g : Set g = g_SkipSectionGlobal
	If g.CurrentLevel >= 0 Then  section.End_
	NotSkipSection = section.Start()
End Function


Sub  SkipToSection( Section )
	If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
	Dim  g : Set g = g_SkipSectionGlobal
	g.SkipToSecNums.ToEmpty
	g.SkipToSecNums.AddElems  Section
End Sub


Sub  SetBreakAtSection( Num )
	If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
	Dim  g : Set g = g_SkipSectionGlobal
	g.BreakSecNums.ToEmpty
	g.BreakSecNums.AddElems  Num
End Sub


'//[GetSkipSectionGlobal]
Function  GetSkipSectionGlobal()
	If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
	Set GetSkipSectionGlobal = g_SkipSectionGlobal
End Function


Class  SkipSection
	Public  StartLevel
	Public  IsNoCheck
	Public  IsStartSection
	Public  SelectedEndCount  '// as integer

	Private Sub  Class_Initialize()
		If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
		Me.StartLevel = g_SkipSectionGlobal.CurrentLevel
		Me.IsStartSection = False
		Me.SelectedEndCount = 0
	End Sub

	Private Sub  Class_Terminate()
		If Me.IsNoCheck Then  Exit Sub
		Dim  en,ed : en = Err.Number : ed = Err.Description
		On Error Resume Next  '// This clears error

		If en=0 or en=21 Then
			If g_SkipSectionGlobal.CurrentLevel <> Me.StartLevel Then
				Raise 1,"<ERROR msg=""It is not match SkipSection::Start and SkipSection::End_""/>"
			ElseIf Me.SelectedEndCount = 0 Then
				If UBound( g_SkipSectionGlobal.SelectedSectionTree ) >= 0 Then
					Raise 1,"<ERROR msg=""Not execute any section or not pass""/>"
					g_SkipSectionGlobal.SelectedSectionTree = Array( )
				End If
			End If
		End If

		ErrorCheckInTerminate  en
		On Error GoTo 0 : If en <> 0 Then  Err.Raise en,,ed  '// This sets en again
	End Sub

	Function  Start()  '// [SkipSection::Start]
		Dim  i, n, g : Set g = g_SkipSectionGlobal

		g.CurrentLevel = g.CurrentLevel + 1
		If g.CurrentSecNums.UBound_ < g.CurrentLevel + 1  Then  g.CurrentSecNums.Add  0
		g.CurrentSecNums( g.CurrentLevel ) = g.CurrentSecNums( g.CurrentLevel ) + 1
		g.CurrentSecNums( g.CurrentLevel + 1 ) = 0

		If g.SkipToSecNums.UBound_ >= 0 Then
			If g.SkipToSecNums.UBound_ < g.CurrentLevel Then _
				n =  g.SkipToSecNums.UBound_  Else  n = g.CurrentLevel
			For i=0 To n
			 If g.CurrentSecNums(i) > g.SkipToSecNums(i) Then  i = n + 1 : Exit For
			 If g.CurrentSecNums(i) < g.SkipToSecNums(i) Then  Exit For
			Next
			Start = ( i > n )
			If not Start Then  g.CurrentLevel = g.CurrentLevel - 1
		ElseIf UBound( g.SelectedSectionTree ) >= 0 Then
			n = g.CurrentLevel
			For i=0 To n
			 If g.CurrentSectionNames(i) <> g.SelectedSectionTree(i) Then  Exit For
			Next
			Start = ( i > n )
			If not Start Then  g.CurrentLevel = g.CurrentLevel - 1
		Else
			Start = True
		End If

		If Start Then
			n = CStr( g.CurrentSecNums(0) )
			For i=1 To g.CurrentLevel : n = n +","& g.CurrentSecNums(i) : Next
			If g.CurrentLevel >= 1 Then  n = "Array("+ n +")"
			If g.IsEcho Then  echo  "<Section num='"+ n +"'>"
		End If

		If g.BreakSecNums.UBound_ >= 0 Then
			If g.BreakSecNums.UBound_ = g.CurrentLevel Then
				For i=0 To g.CurrentLevel
					If g.CurrentSecNums(i) <> g.BreakSecNums(i) Then  Exit For
				Next
				If i = g.BreakSecNums.Count Then  Stop  '// Look at caller function
			End If
		End If

		Me.IsStartSection = Start
	End Function

	Sub  End_()  '// [SkipSection::End_]
		If IsEmpty( g_SkipSectionGlobal ) Then  Set g_SkipSectionGlobal = new SkipSectionGlobal
		Dim  g : Set g = g_SkipSectionGlobal
		If g.IsEcho Then  echo  "</Section>"
		If g.CurrentLevel < 0 Then _
			Raise 1, "<ERROR msg=""Start corresponding SkipSection::End_ is not called or there is not End If before End_""/>"
		g.CurrentLevel = g.CurrentLevel - 1

		If Me.IsStartSection Then
			Me.SelectedEndCount = Me.SelectedEndCount + 1
			Me.IsStartSection = False
			If Err.Number = 0  and  g.CurrentLevel + 1 = UBound( g.SelectedSectionTree ) Then
				echo  "The section set by SetStartSectionTree was finished."
				If not g_IsAutoTest Then  Pause
				g.SelectedSectionTree = Array( )
			End If
		End If
	End Sub
End Class


 
'-------------------------------------------------------------------------
' ### <<<< [FinObj] Class >>>> 
'-------------------------------------------------------------------------
Class  FinObj
	Public  m_Vars  ' as Dictionay
	Public  m_FinallyFunc

	Private Sub Class_Initialize
		Set m_Vars = CreateObject("Scripting.Dictionary")
	End Sub

	Public Sub  SetFunc( FuncName )
		Set m_FinallyFunc = GetRef( FuncName )
	End Sub

	Public Sub  SetVar( Name, Var )
		If IsObject( Var ) Then  Set m_Vars.Item( Name ) = Var _
		Else                         m_Vars.Item( Name ) = Var
	End Sub

	Private Sub Class_Terminate()
		If not IsEmpty( m_FinallyFunc ) Then
			Dim  en, ed : en = Err.Number : ed = Err.Description
			m_FinallyFunc  m_Vars
			ErrorCheckInTerminate  en
		End If
	End Sub
End Class


 
